@@ -14,9 +14,10 @@
 define('io.ox/office/baseframework/app/basecontroller',
     ['io.ox/office/tk/utils',
      'io.ox/office/tk/keycodes',
+     'io.ox/office/tk/dialogs',
      'io.ox/office/tk/object/triggerobject',
      'io.ox/office/tk/object/baseobject'
-    ], function (Utils, KeyCodes, TriggerObject, BaseObject) {
+    ], function (Utils, KeyCodes, Dialogs, TriggerObject, BaseObject) {
 
     'use strict';
 
@@ -44,8 +45,7 @@ define('io.ox/office/baseframework/app/basecontroller',
      *  The application that has created this controller instance.
      *
      * @param {Object} [initOptions]
-     *  A map with options controlling the behavior of this controller. The
-     *  following options are supported:
+     *  Optional parameters:
      *  @param {Number} [initOptions.updateDelay=50]
      *      The delay for the debounced Controller.update() method, in
      *      milliseconds. Must not be less than 50.
@@ -63,45 +63,28 @@ define('io.ox/office/baseframework/app/basecontroller',
                     set: function () { _.defer(function () { app.quit(); }); }
                 },
 
+                // disabled during import, enabled afterwards
+                'app/imported': {
+                    enable: function () { return app.isImportFinished(); }
+                },
+
                 'document/download': {
-                    // enabled in read-only mode
-                    set: function (format, options) { return app.downloadDocument(format, options); }
+                    // enabled in read-only mode, disabled during import
+                    parent: 'app/imported',
+                    set: function () { return app.downloadDocument(); }
                 },
 
                 'document/print': {
-                    // enabled in read-only mode
+                    // enabled in read-only mode, disabled during import
+                    parent: 'app/imported',
                     set: function () { return app.print(); },
                     shortcut: { keyCode: 'P', ctrlOrMeta: true }
                 },
 
                 'document/sendmail': {
-                    // enabled in read-only mode
+                    // enabled in read-only mode, disabled during import
+                    parent: 'app/imported',
                     set: function () { return app.sendMail(); }
-                },
-
-                // toggle the search pane
-                'view/searchpane': {
-                    // enabled in read-only mode
-                    get: function () {
-                        return app.getWindow().search.active;
-                    },
-                    set: function (state) {
-                        var win = app.getWindow();
-                        state = _.isBoolean(state) ? state : !this.getValue();
-                        if (state) {
-                            if (win.search.active) {
-                                win.nodes.searchField.focus();
-                            } else {
-                                win.search.open();
-                            }
-                        } else {
-                            win.search.close();
-                            app.getView().grabFocus();
-                        }
-                    },
-                    // shortcuts always enable the search pane (no toggling)
-                    shortcut: [{ keyCode: 'F', ctrlOrMeta: true, value: true }, { keyCode: 'F3', value: true }],
-                    focus: 'never' // do not grab focus after setter
                 }
             },
 
@@ -111,8 +94,8 @@ define('io.ox/office/baseframework/app/basecontroller',
             // want to commit their value before the controller will reset it.
             updateDelay = Utils.getIntegerOption(initOptions, 'updateDelay', 50, 50),
 
-            // cached item values during a complex update
-            resultCache = {},
+            // cached item states, mapped by item keys
+            stateCache = {},
 
             // number of item setters currently running (recursion counter)
             runningSetters = 0,
@@ -121,72 +104,39 @@ define('io.ox/office/baseframework/app/basecontroller',
             keyShortcuts = {},
 
             // shortcut definitions for 'keypress', mapped by char code (for performance)
-            charShortcuts = {};
+            charShortcuts = {},
+
+            // dummy item singleton, used e.g. for undefined parent items
+            dummyItem = null;
 
         // class Item ---------------------------------------------------------
 
         var Item = BaseObject.extend({ constructor: function (key, definition) {
 
-            var // self reference
-                item = this,
-                // global enabled state of the item
-                enabled = true,
-                // parent items whose values/states are needed to resolve the own value/state
+            var // parent items whose values/states are needed to resolve the own value/state
                 parentKeys = Utils.getOption(definition, 'parent'),
-                // handler for enabled state (default to constant true if missing)
-                enableHandler = Utils.getFunctionOption(definition, 'enable', true),
-                // handler for value getter (default to identity to forward parent value)
+                // handler for enabled state (default to function that returns true)
+                enableHandler = Utils.getFunctionOption(definition, 'enable', _.constant(true)),
+                // handler for value getter (default to identity to forward first parent value)
                 getHandler = Utils.getFunctionOption(definition, 'get', _.identity),
                 // handler for value setter
                 setHandler = Utils.getFunctionOption(definition, 'set'),
-                // whether to block the GUI while setter is running
-                busy = Utils.getBooleanOption(definition, 'busy', false),
                 // behavior for returning browser focus to application
-                focus = Utils.getStringOption(definition, 'focus', 'direct'),
-                // additional user data
-                userData = Utils.getOption(definition, 'userData');
+                itemPreserveFocus = Utils.getBooleanOption(definition, 'preserveFocus', false),
+                // custom focus target node or callback handler
+                itemFocusTarget = Utils.getOption(definition, 'focusTarget');
 
             BaseObject.call(this);
 
             // private methods  - - - - - - - - - - - - - - - - - - - - - - - -
 
-            function getParentItem(parentKey) {
-                if (!(parentKey in items)) {
-                    Utils.warn('BaseController.Item.getParentItem(): item "' + key + '" refers to unknown parent "' + parentKey + '"');
-                }
-                return items[parentKey] || null;
-            }
-
             /**
-             * Returns whether the parent items of this item are all enabled.
+             * Returns the state descriptor of the specified parent item.
              */
-            function areParentsEnabled() {
-                return _(parentKeys).all(function (parentKey) {
-                    var parentItem = getParentItem(parentKey);
-                    return !parentItem || parentItem.isEnabled();
-                });
-            }
-
-            /**
-             * Returns the values of all parents of this item.
-             */
-            function getParentValues() {
-                return _(parentKeys).map(function (parentKey) {
-                    var parentItem = getParentItem(parentKey);
-                    return parentItem ? parentItem.getValue() : undefined;
-                });
-            }
-
-            function getAndCacheResult(type, handler) {
-
-                var // get or create a result object in the cache
-                    result = resultCache[key] || (resultCache[key] = {});
-
-                // if the required value does not exist yet, resolve it via the passed handler
-                if (!(type in result)) {
-                    result[type] = _.isFunction(handler) ? handler.apply(item, getParentValues()) : handler;
-                }
-                return result[type];
+            function getParentState(parentKey) {
+                if (parentKey in items) { return items[parentKey].getState(); }
+                Utils.warn('BaseController.Item.getParentState(): item "' + key + '" refers to unknown parent "' + parentKey + '"');
+                return dummyItem.getState();
             }
 
             // methods  - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -202,30 +152,96 @@ define('io.ox/office/baseframework/app/basecontroller',
             };
 
             /**
-             * Returns whether this item is effectively enabled, by looking at
-             * the own state, and by asking the enable handler of the item.
+             * Returns a raw descriptor for the current state of this item.
+             *
+             * @returns {Object}
+             *  A state descriptor with the following properties:
+             *  - {Boolean|Null} parentsEnabled
+             *      The enabled state of all parents of this item. The value
+             *      null represents the undetermined state (no 'enable'
+             *      callback handlers found at the parents, or no parents
+             *      defined).
+             *  - {Boolean|Null} enabled
+             *      The enabled state of this item. The value null represents
+             *      the undetermined state (no 'enable' callback handlers
+             *      found).
+             *  - {Array} parentValues
+             *      The values (results of all getter callbacks) of the parent
+             *      items.
+             *  - {Array} value
+             *      The value (result of the getter callback) of the item.
+             */
+            this.getState = function () {
+
+                // return cached state
+                if (key in stateCache) { return stateCache[key]; }
+
+                var // the states of all parent items
+                    parentStates = _.map(parentKeys, getParentState),
+                    // determine whether all parent items are enabled
+                    parentsEnabled = _.all(parentStates, function (state) { return state.enabled; }),
+                    // all parent values, as array
+                    parentValues = _.pluck(parentStates, 'value'),
+                    // create the state descriptor of this item
+                    itemState = {
+                        parentStates: parentStates,
+                        parentsEnabled: parentsEnabled,
+                        parentValues: parentValues,
+                        enabled: false,
+                        value: undefined
+                    };
+
+                // insert the state descriptor into the cache before resolving enabled state and value
+                stateCache[key] = itemState;
+
+                // resolve and store the value first (enable callback may want to query the own value)
+                itemState.value = getHandler.apply(this, parentValues);
+
+                // resolve and store the enabled state
+                itemState.enabled = parentsEnabled && (enableHandler.apply(this, parentValues) === true);
+
+                // return the state descriptor
+                return itemState;
+            };
+
+            /**
+             * Returns whether all parent items are enabled.
+             *
+             * @returns {Boolean}
+             *  False, if at least one parent item is disabled, otherwise true.
+             */
+            this.areParentsEnabled = function () {
+                return this.getState().parentsEnabled;
+            };
+
+            /**
+             * Returns whether this item is effectively enabled.
+             *
+             * @returns {Boolean}
+             *  False, if at least one parent item is disabled, or if the
+             *  enable handler of this item has returned false; otherwise true.
              */
             this.isEnabled = function () {
-                return getAndCacheResult('enable', (enabled && areParentsEnabled()) ? enableHandler : false);
+                return this.getState().enabled;
+            };
+
+            /**
+             * Returns the current value of the specified parent item.
+             *
+             * @param {Number} [index=0]
+             *  The index of the parent item (if multiple parent items have
+             *  been specified for this item).
+             */
+            this.getParentValue = function (index) {
+                var parentState = this.getState().parentStates[_.isNumber(index) ? index : 0];
+                return parentState ? parentState.value : undefined;
             };
 
             /**
              * Returns the current value of this item.
              */
             this.getValue = function () {
-                return getAndCacheResult('value', getHandler);
-            };
-
-
-            /**
-             * Returns the user data that has been registered with the
-             * definition of this item.
-             *
-             * @returns {Any}
-             *  The registered user data if existing, otherwise undefined.
-             */
-            this.getUserData = function () {
-                return userData;
+                return this.getState().value;
             };
 
             /**
@@ -237,15 +253,8 @@ define('io.ox/office/baseframework/app/basecontroller',
              *  The new value of the item.
              *
              * @param {Object} [options]
-             *  A map with options controlling the behavior of this method. All
-             *  options will be passed to the item setter function. The
-             *  following special options are supported:
-             *  @param {Boolean} [options.preserveFocus=false]
-             *      If set to true, the browser focus will not be modified,
-             *      regardless of the focus mode configured for this item.
-             *  @param {HTMLElement} [options.source]
-             *      An optional DOM source element that has caused the change.
-             *      This may influence the way how to modify the browser focus.
+             *  Optional parameters. See method BaseController.executeItem()
+             *  for details.
              *
              * @returns {jQuery.Promise}
              *  The Promise of a Deferred object that will be resolved or
@@ -257,87 +266,87 @@ define('io.ox/office/baseframework/app/basecontroller',
              */
             this.execute = function (value, options) {
 
-                var // the result Deferred object
-                    def = null;
+                var // the resulting promise
+                    promise = null,
+                    // whether the setter runs asynchronous code
+                    pending = false,
+                    // focus options after executing the item
+                    focusOptions = _.extend({}, options);
 
-                // do nothing if item is disabled
+                // mix item focus options and passed focus options
+                focusOptions.preserveFocus = itemPreserveFocus || Utils.getBooleanOption(options, 'preserveFocus', false);
+                focusOptions.focusTarget = Utils.getOption(options, 'focusTarget', itemFocusTarget);
+
+                // bind focus target callback function to the item instance
+                if (_.isFunction(focusOptions.focusTarget)) {
+                    focusOptions.focusTarget = _.bind(focusOptions.focusTarget, this);
+                }
+
+                // do nothing if item is disabled (but accept undetermined state)
                 if (this.isEnabled()) {
 
                     // execute the set handler
                     runningSetters += 1;
                     if (_.isFunction(setHandler)) {
-                        def = setHandler.call(this, value, options);
+                        promise = setHandler.call(this, value);
                     }
 
-                    // convert result of the setter to a Deferred object
-                    def = $.when(def);
-                    def.always(function () {
+                    // convert result of the setter callback to a promise
+                    promise = $.when(promise);
+                    promise.always(function () {
                         runningSetters -= 1;
                     });
 
-                    // disable this item and show busy screen if setter is still running
-                    if (def.state() === 'pending') {
-                        if (busy) {
-                            app.getView().enterBusy({ delay: 500 });
-                        } else {
-                            enabled = false;
-                            self.update();
-                        }
+                    // show busy screen if setter is running asynchronously
+                    // (but not for modal dialogs which have their own busy blocker)
+                    pending = promise.state() === 'pending';
+                    if (pending && !Dialogs.isModalDialogOpen()) {
+                        app.getView().enterBusy({ delay: 500 });
                     }
                 } else {
                     // item is disabled
-                    def = $.Deferred().reject();
+                    promise = $.Deferred().reject();
                 }
 
                 // focus back to application
-                switch (focus) {
-                case 'direct':
-                    grabApplicationFocus(options);
-                    break;
-                case 'wait':
-                    def.always(function () {
-                        // execute in a timeout, needed for dialogs which are closed *after* resolve/reject
-                        app.executeDelayed(function () { grabApplicationFocus(options); });
+                if (!focusOptions.preserveFocus) {
+                    promise.always(function () {
+                        if (pending) {
+                            // execute in a timeout, needed for dialogs which are closed *after* resolve/reject
+                            app.executeDelayed(_.bind(self.grabFocus, self, focusOptions));
+                        } else {
+                            self.grabFocus(focusOptions);
+                        }
                     });
-                    break;
-                case 'never':
-                    break;
-                default:
-                    Utils.warn('BaseController.Item.execute(): unknown focus mode "' + focus + '" for item "' + key + '"');
                 }
 
-                // post processing after the setter is finished
-                Utils.logSelenium('time=' + _.now(), 'key=' + key, 'state=' + def.state());
-                if (def.state() === 'pending') {
-                    def.always(function () {
-                        if (busy) {
-                            app.getView().leaveBusy();
-                        } else {
-                            enabled = true;
-                        }
+                // post-processing after the setter is finished
+                Utils.logSelenium('time=' + _.now(), 'key=' + key, 'state=' + promise.state());
+                if (pending) {
+                    promise.always(function () {
+                        app.getView().leaveBusy();
                         self.update();
-                        Utils.logSelenium('time=' + _.now(), 'key=' + key, 'state=' + def.state());
+                        Utils.logSelenium('time=' + _.now(), 'key=' + key, 'state=' + promise.state());
                     });
                 } else {
                     self.update();
                 }
 
-                return def.promise();
+                return promise;
             };
 
             // initialization - - - - - - - - - - - - - - - - - - - - - - - - -
 
             // always use an array of parent keys (as strings)
             parentKeys =
-                _.isArray(parentKeys) ? _(parentKeys).filter(_.isString) :
+                _.isArray(parentKeys) ? _.filter(parentKeys, _.isString) :
                 _.isString(parentKeys) ? [parentKeys] :
                 null;
 
             // destroy all class members on destruction
             this.registerDestructor(function () {
-                definition = item = parentKeys = null;
+                definition = parentKeys = null;
                 enableHandler = getHandler = setHandler = null;
-                userData = null;
             });
 
         }}); // class Item
@@ -351,26 +360,8 @@ define('io.ox/office/baseframework/app/basecontroller',
         /**
          * Clears all cached item results.
          */
-        function clearResultCache() {
-            resultCache = {};
-        }
-
-        /**
-         * Moves the browser focus back to the application pane.
-         */
-        function grabApplicationFocus(options) {
-
-            var preserveFocus = Utils.getBooleanOption(options, 'preserveFocus', false),
-                source = Utils.getOption(options, 'source');
-
-            // Bug 28214: Focus back to application if source GUI element is hidden now.
-            // If the source GUI element is hidden now, IE has already changed the
-            // 'document.activeElement' property, so it cannot be used to detect visibility.
-            // Therefore, all group instances now insert the source DOM node into the value
-            // options when triggering the 'group:change' event.
-            if (!preserveFocus || (source && !$(source).is(Utils.REALLY_VISIBLE_SELECTOR))) {
-                app.getView().grabFocus();
-            }
+        function clearStateCache() {
+            stateCache = {};
         }
 
         /**
@@ -404,7 +395,7 @@ define('io.ox/office/baseframework/app/basecontroller',
                 if (runningSetters === 0) {
                     // call value resolver function, or take constant value
                     value = _.isFunction(definition.value) ? definition.value(self.getItemValue(shortcut.key)) : definition.value;
-                    self.executeItem(shortcut.key, value);
+                    self.executeItem(shortcut.key, value, { sourceType: 'keyboard' });
                 }
 
                 // stop event propagation unless specified otherwise
@@ -417,7 +408,7 @@ define('io.ox/office/baseframework/app/basecontroller',
             case 'keydown':
                 // process all shortcut definitions for the key code in the passed event
                 if (event.keyCode in keyShortcuts) {
-                    _(keyShortcuts[event.keyCode]).each(function (shortcut) {
+                    _.each(keyShortcuts[event.keyCode], function (shortcut) {
                         // check if the additional modifier keys match the shortcut definition
                         if (KeyCodes.matchModifierKeys(event, shortcut.definition)) {
                             executeItemForShortcut(shortcut);
@@ -429,7 +420,7 @@ define('io.ox/office/baseframework/app/basecontroller',
                 // process all shortcut definitions for the char code in the passed
                 // event, but ignore 'keypress' events in text fields
                 if ((event.charCode in charShortcuts) && !$(event.target).is('input,textarea')) {
-                    _(charShortcuts[event.charCode]).each(executeItemForShortcut);
+                    _.each(charShortcuts[event.charCode], executeItemForShortcut);
                 }
                 break;
             }
@@ -457,7 +448,7 @@ define('io.ox/office/baseframework/app/basecontroller',
             // (hack for bug 27528 will fail).
             if (_.browser.IE && _.isFunction(windowNode[0].attachEvent)) {
                 windowNode[0].attachEvent('onkeydown', function (event) {
-                    var result = keyHandler(_(event).extend({ target: event.srcElement }));
+                    var result = keyHandler(_.extend(event, { target: event.srcElement }));
                     // modify the key code to prevent the browser default action
                     if (result === false) { event.keyCode = 0; }
                     return result;
@@ -502,29 +493,27 @@ define('io.ox/office/baseframework/app/basecontroller',
          *  @param {Function} [definition.enable]
          *      Predicate function returning true if the item is enabled, and
          *      false otherwise. If one or more parent items have been
-         *      specified (see above) and any of them returned false, the item
-         *      is disabled already, and this function will not be called
-         *      anymore. Defaults to a function that always returns true.
+         *      specified (see above) and at least one of them returned false,
+         *      the item is in disabled state already, and this function will
+         *      not be called anymore. If the item does not specify an enable
+         *      handler, it defaults to the value true, so the enabled state is
+         *      dependent on the parent items only.
          *  @param {Function} [definition.get]
          *      Getter function returning the current value of the item. Can be
          *      omitted for one-way action items (actions without a return
          *      value). If one ore more parent items have been specified (see
          *      above), the cached return values of their getters will be
          *      passed to this getter, in the same order as specified. May
-         *      return null to indicate an ambiguous state. May return
-         *      undefined to indicate that calculating the value is not
-         *      applicable, not possible, not implemented, etc. In the case of
-         *      an undefined return value, the current state of the controls in
-         *      the view components will not be changed. Defaults to a function
-         *      that returns undefined; or, if a parent item has been
-         *      registered, that returns its cached value directly.
+         *      return null to indicate an ambiguous state. Defaults to a
+         *      function that returns undefined; or, if parent items have been
+         *      registered, that returns the value of the first parent.
          *  @param {Function} [definition.set]
          *      Setter function changing the value of an item to the first
          *      parameter of the setter. Can be omitted for read-only items.
          *      Defaults to an empty function. May return a Deferred object or
          *      a Promise to indicate that the setter executes asynchronous
-         *      code. This has effect on the properties 'definition.busy' and
-         *      'definition.focus', see below.
+         *      code. In this case, the application window will be blocked and
+         *      a busy indicator will be shown while the setter is running.
          *  @param {Object|Array} [definition.shortcut]
          *      One or multiple keyboard shortcut definitions. If the window
          *      root node of the application receives a 'keydown' or 'keypress'
@@ -604,28 +593,19 @@ define('io.ox/office/baseframework/app/basecontroller',
          *          be a descendant node of a DOM node matching the selector.
          *          Otherwise, the event will be ignored completely, and the
          *          browser will execute its default action.
-         *  @param {Boolean} [definition.busy=false]
-         *      If set to true, the GUI of the application will be blocked and
-         *      a busy indicator will be shown while the setter of this item
-         *      is running (indicated by a pending Deferred object returned by
-         *      the setter). Does not have any effect for synchronous setters.
-         *  @param {String} [definition.focus='direct']
-         *      Determines how to return the browser focus to the application
-         *      pane after executing the setter function of this item. The
-         *      following values are supported:
-         *      - 'direct': (default) The focus will return directly after the
-         *          setter function has been executed, regardless of the return
-         *          value of the setter.
-         *      - 'never': The focus will not be returned to the application
-         *          pane. The setter function is responsible for application
-         *          focus handling.
-         *      - 'wait': The controller will wait until the Deferred object
-         *          returned by the setter function gets resolved or rejected,
-         *          and then sets the browser focus to the application pane.
-         *  @param {Any} [definition.userData]
-         *      Additional user data that will be provided by the method
-         *      'Item.getUserData()'. Can be used in all item getter and setter
-         *      methods.
+         *  @param {Boolean} [definition.preserveFocus=false]
+         *      Whether to return the browser focus to the application pane
+         *      after executing the setter function of this item (the default
+         *      behavior). If set to true, the current browser focus will not
+         *      be changed. Otherwise, the focus will return to the application
+         *      pane after the setter function has finished (will wait for
+         *      asynchronous setters before returning the focus).
+         *  @param {HTMLElement|jQuery|Object|Function} [options.focusTarget]
+         *      A DOM node that will receive the browser focus after the item
+         *      has been executed, an object that provides a grabFocus()
+         *      method, or a callback function that returns a focus target
+         *      dynamically. Default focus target is the view. Has no effect,
+         *      if the option 'preserveFocus' is set to true.
          *
          * @returns {BaseController}
          *  A reference to this controller instance.
@@ -667,7 +647,7 @@ define('io.ox/office/baseframework/app/basecontroller',
          *  A reference to this controller instance.
          */
         this.registerDefinitions = function (definitions) {
-            _(definitions).each(function (definition, key) {
+            _.each(definitions, function (definition, key) {
                 this.registerDefinition(key, definition);
             }, this);
             return this;
@@ -683,14 +663,29 @@ define('io.ox/office/baseframework/app/basecontroller',
         this.update = app.createDebouncedMethod(function () { return self; }, function () {
 
             // collect states of all matching items in resultCache
-            clearResultCache();
-            _(items).each(function (item) {
-                item.isEnabled();
-                item.getValue();
-            });
+            clearStateCache();
 
-            // notify all listeners
-            self.trigger('change:items', resultCache);
+            if (_.browser.IE) {
+
+                // early break out, 'getState' is only important for hud,
+                // but freezing to long can kill 'realtime' and the browser itself,
+                // so we cancel it after 1sec.
+                // (more than 3sec could also mean 5sec. or more, because we have to check later)
+
+                var startTime = _.now();
+                _.find(items, function(item) {
+                    item.getState();
+                    if ((_.now() - startTime) > 3000) { return true; }
+                });
+            } else {
+                _.invoke(items, 'getState');
+            }
+
+            // notify all listeners (with locked GUI refresh for performance,
+            // view elements are listening to controller items to change their visibility)
+            app.getView().lockPaneLayout(function () {
+                self.trigger('change:items', stateCache);
+            });
 
         }, { delay: updateDelay, maxDelay: updateDelay * 5 });
 
@@ -705,7 +700,7 @@ define('io.ox/office/baseframework/app/basecontroller',
          */
         this.isItemEnabled = function (key) {
             // do not clear the cache if currently running in a set handler
-            if (runningSetters === 0) { clearResultCache(); }
+            if (runningSetters === 0) { clearStateCache(); }
             return (key in items) && items[key].isEnabled();
         };
 
@@ -721,7 +716,7 @@ define('io.ox/office/baseframework/app/basecontroller',
          */
         this.getItemValue = function (key) {
             // do not clear the cache if currently running in a set handler
-            if (runningSetters === 0) { clearResultCache(); }
+            if (runningSetters === 0) { clearStateCache(); }
             return (key in items) ? items[key].getValue() : undefined;
         };
 
@@ -736,16 +731,36 @@ define('io.ox/office/baseframework/app/basecontroller',
          *  The new value of the item.
          *
          * @param {Object} [options]
-         *  A map with options controlling the behavior of this method. All
-         *  options will be passed to the item setter function. The following
-         *  special options are supported:
+         *  Optional parameters:
+         *  @param {String} [options.sourceType='custom']
+         *      The cause of the item invocation. Can be one of the following
+         *      types:
+         *      - 'click': The item has been executed due to a button click, or
+         *          another comparable GUI action such as a tap on a touch
+         *          device.
+         *      - 'keyboard': The item has been executed due to a standard
+         *          keyboard event, e.g. pressing the ENTER key on buttons or
+         *          text fields, pressing the SPACE key on buttons, or pressing
+         *          the TAB key in modified text fields; or a keyboard shortcut
+         *          as defined for the respective controller item.
+         *      - 'custom': (default) Any other cause.
          *  @param {Boolean} [options.preserveFocus=false]
          *      If set to true, the browser focus will not be modified after
          *      executing the item, regardless of the focus mode configured for
          *      that item.
-         *  @param {HTMLElement} [options.source]
-         *      An optional DOM source element that has caused the change. This
-         *      may influence the way how to modify the browser focus.
+         *  @param {HTMLElement|jQuery|Object|Function} [options.focusTarget]
+         *      A DOM node that will receive the browser focus after the item
+         *      has been executed, an object that provides a grabFocus()
+         *      method, or a callback function that returns a focus target
+         *      dynamically. Overrides the default focus target defined for the
+         *      executed item (option 'focusTarget' in the item definition).
+         *      Default focus target is the view. Has no effect, if the option
+         *      'preserveFocus' is set to true.
+         *  @param {jQuery.Event} [options.changeEvent]
+         *      Set by the framework when executing a controller item after
+         *      activating a GUI control element. Represents the event object
+         *      of the 'group:change' event as triggered by instances of the
+         *      Group class.
          *
          * @returns {jQuery.Promise}
          *  The Promise of a Deferred object that will be resolved or rejected
@@ -760,29 +775,77 @@ define('io.ox/office/baseframework/app/basecontroller',
             // execute setter of existing item
             if (key in items) {
                 // do not clear cache if called recursively from another setter
-                if (runningSetters === 0) { clearResultCache(); }
+                if (runningSetters === 0) { clearStateCache(); }
                 return items[key].execute(value, options);
             }
 
             // item does not exist: return focus to application
-            grabApplicationFocus(options);
+            this.grabFocus(options);
             return $.Deferred().reject();
         };
 
+        /**
+         * Moves the browser focus back to the application pane, or to a custom
+         * focus target node, if specified in the passed options.
+         *
+         * @param {Object} [options]
+         *  Optional parameters. See method BaseController.executeItem() for
+         *  details.
+         *
+         * @returns {BaseController}
+         *  A reference to this instance.
+         */
+        this.grabFocus = function (options) {
+
+            var preserveFocus = Utils.getBooleanOption(options, 'preserveFocus', false),
+                focusTarget = Utils.getOption(options, 'focusTarget'),
+                changeEvent = Utils.getOption(options, 'changeEvent'),
+                sourceNode = Utils.getOption(changeEvent, 'sourceNode');
+
+            // keep current focus, if 'preserveFocus' is set
+            // Bug 28214: Focus back to application if source GUI element is hidden
+            // now. If the source GUI element is hidden now, IE has already changed
+            // the 'document.activeElement' property, so it cannot be used to detect
+            // visibility. Therefore, all group instances now insert the source DOM
+            // node into the 'group:change' event.
+            if (preserveFocus && (!sourceNode || $(sourceNode).is(':visible'))) {
+                return this;
+            }
+
+            // resolve callback function to focus target
+            if (_.isFunction(focusTarget)) {
+                focusTarget = focusTarget.call(self, Utils.getStringOption(options, 'sourceType', 'custom'));
+            }
+
+            // move focus to specified target, or to the view
+            if (_.isObject(focusTarget) && _.isFunction(focusTarget.grabFocus) && _.isFunction(focusTarget.isVisible) && focusTarget.isVisible()) {
+                focusTarget.grabFocus();
+            } else if (((focusTarget instanceof $) || (focusTarget instanceof HTMLElement)) && $(focusTarget).is(':visible')) {
+                $(focusTarget).focus();
+            } else {
+                app.getView().grabFocus();
+            }
+            return this;
+        };
+
         // initialization -----------------------------------------------------
 
+        // dummy item singleton, used e.g. for undefined parent items
+        dummyItem = new Item('\x00');
+
         // register item definitions
         this.registerDefinitions(items);
 
         // update once after import (successful and failed)
-        app.on('docs:import:after', function () {
+        app.onImport(function () {
             initializeKeyHandler();
+            self.listenTo(app.getView(), 'refresh:layout', function () { self.update(); });
             self.update();
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            _(items).invoke('destroy');
+            _.invoke(items, 'destroy');
             self = items = null;
         });
 
