@@ -21,7 +21,7 @@ define('io.ox/office/baseframework/app/extensionregistry',
     var // which file formats are editable (maps format identifier to Booleans)
         EDITABLE_FILE_FORMATS = {
             ooxml: true,
-            odf: Utils.getBooleanOption(Config.get(), 'odfsupport', false)
+            odf: Utils.getBooleanOption(Config.get(), 'odfsupport', true)
         },
 
         // the extension suffix for files with an internal error
@@ -33,7 +33,7 @@ define('io.ox/office/baseframework/app/extensionregistry',
             text: {
                 module: 'io.ox/office/text',
                 requires: 'text',
-                editable: !_.browser.Android,
+                editable: !_.browser.Android || Utils.supportedChromeVersionOnAndroidForText(),
                 extensions: {
                     docx: { format: 'ooxml' },
                     docm: { format: 'ooxml', macros: true },
@@ -41,9 +41,9 @@ define('io.ox/office/baseframework/app/extensionregistry',
                     dotm: { format: 'ooxml', template: true, macros: true },
                     odt:  { format: 'odf',   macros: true },
                     ott:  { format: 'odf',   template: true, macros: true },
-                    doc:  { format: 'odf',   macros: true, convert: true },
-                    dot:  { format: 'odf',   template: true, templateLocked: true, macros: true, convert: true },
-                    rtf:  { format: 'odf',   convert: true }
+                    doc:  { format: 'ooxml',   macros: true, convert: true },
+                    dot:  { format: 'ooxml',   template: true, templateLocked: true, macros: true, convert: true },
+                    rtf:  { format: 'ooxml',   convert: true }
                 }
             },
 
@@ -60,7 +60,9 @@ define('io.ox/office/baseframework/app/extensionregistry',
                     xls:  { format: 'ooxml', macros: true, convert: true },
                     xlt:  { format: 'ooxml', template: true, templateLocked: true, macros: true, convert: true },
                     xla:  {                  macros: true }, // add-in
-                    xlsb: { format: 'ooxml', macros: true, convert: true }
+                    xlsb: { format: 'ooxml', macros: true, convert: true },
+                    ods:  { format: 'ooxml', macros: true, convert: true },
+                    ots:  { format: 'ooxml', template: true, templateLocked: true, macros: true, convert: true },
                 }
             },
 
@@ -88,8 +90,6 @@ define('io.ox/office/baseframework/app/extensionregistry',
             preview: {
                 extensions: {
                     pdf:  {},
-                    ods:  { format: 'odf', macros: true }, // spreadsheet
-                    ots:  { format: 'odf', template: true, macros: true },
                     odg:  { format: 'odf', macros: true }, // drawing
                     otg:  { format: 'odf', template: true, macros: true }, // drawing
                     odi:  { format: 'odf' }, // image
@@ -214,7 +214,7 @@ define('io.ox/office/baseframework/app/extensionregistry',
      */
     ExtensionRegistry.getViewableExtensions = function () {
         var extensions = [];
-        _(fileExtensionMap).each(function (settings, extension) {
+        _.each(fileExtensionMap, function (settings, extension) {
             if (settings.viewable) {
                 extensions.push(extension);
             }
@@ -301,6 +301,28 @@ define('io.ox/office/baseframework/app/extensionregistry',
         return Utils.getBooleanOption(extensionSettings, 'editable', false) && !Utils.getBooleanOption(extensionSettings, 'native', false);
     };
 
+    /**
+     * Returns whether the file with the passed name is a file that will be
+     * edited in the OfficeOpenXML file format (either natively, or by
+     * conversion from a file format not supported natively).
+     *
+     * @param {String} fileName
+     *  The file name (case-insensitive).
+     *
+     * @param {String} [editModule]
+     *  If specified, must match the module name of the edit application
+     *  registered for the extension of the passed file name. If omitted, the
+     *  edit application module name for the extension will be ignored.
+     *
+     * @returns {Boolean}
+     *  Whether the specified file will be edited in the OfficeOpenXML file
+     *  format.
+     */
+    ExtensionRegistry.isOfficeOpenXML = function (fileName, editModule) {
+        var extensionSettings = ExtensionRegistry.getExtensionSettings(fileName, editModule);
+        return Utils.getStringOption(extensionSettings, 'format') === 'ooxml';
+    };
+
     /**
      * Returns whether the file with the passed name is a file that will be
      * edited in the OpenDocument file format (either natively, or by
@@ -430,7 +452,7 @@ define('io.ox/office/baseframework/app/extensionregistry',
     // static initialization ==================================================
 
     // process the configuration of all modules
-    _(FILE_EXTENSION_CONFIGURATION).each(function (moduleConfiguration, documentType) {
+    _.each(FILE_EXTENSION_CONFIGURATION, function (moduleConfiguration, documentType) {
 
         var // the capability required to edit the file
             editable = Utils.getBooleanOption(moduleConfiguration, 'editable', false),
@@ -445,7 +467,7 @@ define('io.ox/office/baseframework/app/extensionregistry',
         applicationMap[documentType] = { editable: editAvailable };
 
         // process all extensions registered for the module
-        _(moduleConfiguration.extensions).each(function (extensionSettings, extension) {
+        _.each(moduleConfiguration.extensions, function (extensionSettings, extension) {
 
             var // the file format of files with the current extension
                 format = Utils.getStringOption(extensionSettings, 'format', ''),
@@ -457,13 +479,13 @@ define('io.ox/office/baseframework/app/extensionregistry',
             }
 
             // initialize all properties of the current extension
-            fileExtensionMap[extension] = _({
+            fileExtensionMap[extension] = _.extend({
                 type: documentType,
                 viewable: previewAvailable,
                 editable: editAvailable && Utils.getBooleanOption(EDITABLE_FILE_FORMATS, format, false),
                 module: module,
                 native: native
-            }).extend(extensionSettings);
+            }, extensionSettings);
 
             // initialize all properties of the current extension with internal error
             // suffix (native only) - these files are not viewable in OX Preview
