@@ -35,12 +35,17 @@ define('io.ox/office/baseframework/model/modelobject',
      *  The application instance containing this attributed model object.
      *
      * @param {Object} [initOptions]
-     *  A map with additional options for this instance. The following options
-     *  are supported:
+     *  Optional parameters:
      *  @param {Boolean} [initOptions.silent=false]
      *      If set to true, this instance will never trigger any change events.
-     *      Thus, specific sub classes may chose to be always silent although
+     *      Specific sub classes may chose to be permanently silent although
      *      their base class derives from this class.
+     *  @param {Boolean} [initOptions.forceTrigger=false]
+     *      If set to true, this instance will trigger even during loading the
+     *      document. Specific sub classes may chose to trigger all events although
+     *      their base class derives from this class.
+     *      This option requires, that the option 'silent' is set to  false (its
+     *      default value).
      */
     function ModelObject(app, initOptions) {
 
@@ -115,17 +120,23 @@ define('io.ox/office/baseframework/model/modelobject',
             // extend with Events mix-in
             Events.extend(this);
 
-            // overwrite the trigger() method with a method that triggers
-            // events only if the document has been imported already
-            this.trigger = (function (triggerFunc) {
-                return function () {
-                    if (app.isImportFinished()) {
-                        this.trigger = triggerFunc; // back to original method
-                        triggerFunc.apply(this, arguments);
-                    }
-                    return this;
-                };
-            }.call(this, this.trigger));
+            // disable triggering until import is finished. If 'forceTrigger'
+            // is set, triggering happens also in the loading phase.
+            if (!Utils.getBooleanOption(initOptions, 'forceTrigger', false)) {
+
+                // overwrite the trigger() method with a method that triggers
+                // events only if the document has been imported already
+                this.trigger = (function (triggerFunc) {
+                    return function () {
+                        if (app.isImportFinished()) {
+                            this.trigger = triggerFunc; // back to original method
+                            triggerFunc.apply(this, arguments);
+                        }
+                        return this;
+                    };
+                }.call(this, this.trigger));
+
+            }
 
             // destroy the Events mix-in on destruction of this instance
             this.registerDestructor(_.bind(this.events.destroy, this.events));
