@@ -15,22 +15,17 @@ define('io.ox/office/baseframework/view/baseview',
     ['io.ox/core/notifications',
      'io.ox/office/tk/utils',
      'io.ox/office/tk/keycodes',
+     'io.ox/office/tk/forms',
      'io.ox/office/tk/object/triggerobject',
      'io.ox/office/baseframework/view/pane',
      'gettext!io.ox/office/main',
      'io.ox/office/baseframework/view/nodetracking',
      'less!io.ox/office/baseframework/view/basestyle',
      'less!io.ox/office/baseframework/view/docs-icons'
-    ], function (Notifications, Utils, KeyCodes, TriggerObject, Pane, gt) {
+    ], function (Notifications, Utils, KeyCodes, Forms, TriggerObject, Pane, gt) {
 
     'use strict';
 
-    var // a helper node to workaround selection problems in IE (bug 28515, bug 28711)
-        focusHelperNode = $('<div>', { contenteditable: true }).addClass('focus-helper');
-
-    // insert the focus helper into the DOM
-    Utils.insertHiddenNodes(focusHelperNode);
-
     // global functions =======================================================
 
     /**
@@ -49,7 +44,7 @@ define('io.ox/office/baseframework/view/baseview',
     function getMarginFromValue(margin) {
 
         if (_.isObject(margin)) {
-            return _({ left: 0, right: 0, top: 0, bottom: 0 }).extend(margin);
+            return _.extend({ left: 0, right: 0, top: 0, bottom: 0 }, margin);
         }
 
         if (!_.isNumber(margin)) {
@@ -70,8 +65,7 @@ define('io.ox/office/baseframework/view/baseview',
      *      all registered view panes. This event will be triggered after
      *      inserting new view panes into this view, or content nodes into the
      *      application pane, after showing/hiding view panes, while and after
-     *      the browser window is resized, and when the method
-     *      BaseView.refreshPaneLayout() has been called manually.
+     *      the browser window is resized.
      *
      * @constructor
      *
@@ -83,12 +77,16 @@ define('io.ox/office/baseframework/view/baseview',
      * @param {Object} [initOptions]
      *  Additional options to control the appearance of the view. The following
      *  options are supported:
+     *  @param {String} [initOptions.classes]
+     *      Additional CSS classes to be added to the root node of the
+     *      application window.
      *  @param {Function} [initOptions.initHandler]
-     *      A callback handler function called to initialize the contents of
-     *      this view instance after construction.
-     *  @param {Function} [initOptions.deferredInitHandler]
-     *      A callback handler function called to initialize more contents of
-     *      this view instance, after the document has been imported.
+     *      A callback handler function called to initialize this view instance
+     *      after construction.
+     *  @param {Function} [initOptions.initGuiHandler]
+     *      A callback handler function called to initialize the graphical
+     *      elements of this view instance, after the document has been
+     *      imported.
      *  @param {Function} [initOptions.grabFocusHandler]
      *      A function that has to implement moving the browser focus somewhere
      *      into the application pane. Used in the method BaseView.grabFocus()
@@ -124,26 +122,32 @@ define('io.ox/office/baseframework/view/baseview',
         var // self reference
             self = this,
 
-            // CSS classes to be added at the window root node
-            windowNodeClasses = 'io-ox-office-main ' + app.getName().replace(/[.\/]/g, '-') + '-main',
-
             // moves the browser focus into a node of the application pane
             grabFocusHandler = Utils.getFunctionOption(initOptions, 'grabFocusHandler'),
 
-            // centered application pane
-            appPane = null,
+            // the application window instance (ox.ui.Window)
+            win = null,
+
+            // the DOM node of the application window body
+            winBodyNode = null,
+
+            // root node of the application pane (remaining space for document contents)
+            appPaneNode = null,
 
             // root container node for invisible document contents
-            hiddenRootNode = $('<div>').addClass('abs app-hidden-root'),
+            hiddenRootNode = $('<div class="abs app-hidden-root">'),
 
             // root container node for visible document contents (may be scrollable)
-            contentRootNode = $('<div>').addClass('abs app-content-root'),
+            contentRootNode = $('<div class="abs app-content-root">'),
 
             // container node for application contents
-            appContentNode = $('<div>').addClass('app-content').appendTo(contentRootNode),
+            appContentNode = $('<div class="app-content">').appendTo(contentRootNode),
 
             // busy node for the application pane
-            appBusyNode = $('<div>').addClass('abs app-busy'),
+            appBusyNode = $('<div class="abs app-busy">'),
+
+            // the temporary container for all nodes while application is hidden
+            tempNode = $('<div>'),
 
             // all fixed view panes, in insertion order
             fixedPanes = [],
@@ -154,8 +158,11 @@ define('io.ox/office/baseframework/view/baseview',
             // whether refreshing the pane layout is currently locked
             layoutLocks = 0,
 
-            // the temporary container for all nodes while application is hidden
-            tempNode = $('<div>').addClass(windowNodeClasses),
+            // whether refreshing the pane layout is currently running
+            layoutRunning = false,
+
+            // whether refreshing the pane layout was requested while it was locked
+            layoutPending = false,
 
             // margins of overlay panes to the borders of the application pane
             overlayMargin = getMarginFromValue(Utils.getOption(initOptions, 'overlayMargin', 0)),
@@ -166,8 +173,8 @@ define('io.ox/office/baseframework/view/baseview',
             // whether the application is hidden explicitly
             viewHidden = false,
 
-            // cached notification, shown when application becomes visible
-            lastNotification = null,
+            // cached yell options, shown when application becomes visible
+            pendingYellOptions = null,
 
             // the timer waiting to fade in the blocker element in busy mode
             blockerFadeTimer = null;
@@ -178,14 +185,9 @@ define('io.ox/office/baseframework/view/baseview',
 
         // private methods ----------------------------------------------------
 
-        /**
-         * Adjusts the positions of all view pane nodes.
-         */
-        function refreshPaneLayout() {
+        function silentRefreshPaneLayout() {
 
-            var // the root node of the application pane
-                appPaneNode = appPane.getNode(),
-                // current offsets representing available space in the application window
+            var // current offsets representing available space in the application window
                 offsets = { top: 0, bottom: 0, left: 0, right: 0 };
 
             function updatePane(pane) {
@@ -194,8 +196,8 @@ define('io.ox/office/baseframework/view/baseview',
                     visible = pane.isVisible(),
                     transparent = pane.isTransparent(),
                     position = pane.getPosition(),
-                    vertical = Utils.isVerticalPosition(position),
-                    leading = Utils.isLeadingPosition(position),
+                    vertical = pane.isVerticalPosition(),
+                    leading = pane.isLeadingPosition(),
                     sizeFunc = _.bind(vertical ? paneNode.outerHeight : paneNode.outerWidth, paneNode),
                     sizeAttr = vertical ? 'height' : 'width',
                     paneOffsets = _.clone(offsets);
@@ -211,9 +213,11 @@ define('io.ox/office/baseframework/view/baseview',
                     }
                 }
 
-                // set pane position, adjust global offsets
+                // set pane position, and let pane update its internal size
+                // (this may even change the pane size again due to internal refresh)
                 paneNode.css(paneOffsets);
                 if (visible) {
+                    pane.layout();
                     offsets[position] += sizeFunc();
                 }
 
@@ -223,38 +227,80 @@ define('io.ox/office/baseframework/view/baseview',
                 }
             }
 
-            // do nothing if refreshing is currently locked, or the window is hidden, or import still running
-            if ((layoutLocks > 0) || !self.isVisible() || !app.isImportFinished()) { return; }
-
             // prevent recursive processing
-            layoutLocks += 1;
+            if (layoutRunning) { return; }
+            layoutRunning = true;
 
             // update fixed view panes
-            _(fixedPanes).each(updatePane);
+            _.each(fixedPanes, updatePane);
 
             // update the application pane node
             appPaneNode.css(offsets);
 
             // skip margins for overlay panes
-            _(offsets).each(function (offset, pos) { offsets[pos] += overlayMargin[pos]; });
+            _.each(offsets, function (offset, pos) { offsets[pos] += overlayMargin[pos]; });
 
             // update overlay view panes
-            _(overlayPanes).each(updatePane);
+            _.each(overlayPanes, updatePane);
 
-            // increased above to prevent recursive processing
-            layoutLocks -= 1;
+            // leave recursion guard
+            layoutRunning = false;
+        }
 
-            // notify listeners
-            Utils.LayoutLogger.info('type="refresh:layout"');
+        /**
+         * Adjusts the positions of all view pane nodes.
+         */
+        function refreshPaneLayout() {
+
+            // do nothing if the window is hidden (but refresh if import runs (fast loading))
+            if (!self.isVisible()) {
+                return;
+            }
+
+            // do nothing if refreshing is currently locked, but set the pending flag
+            if (layoutLocks > 0) {
+                layoutPending = true;
+                return;
+            }
+
+            // refresh all panes, and notify listeners
+            silentRefreshPaneLayout();
             self.trigger('refresh:layout');
         }
 
+        /**
+         * Refreshes the view layout after a view pane has been modified.
+         */
+        var paneResizeHandler = (function () {
+
+            var // whether to really refresh the layout in the deferred callback
+                refreshLayout = false;
+
+            function registerRefresh() {
+                // do nothing when view panes are changing during layout updates
+                if (!layoutRunning) { refreshLayout = true; }
+            }
+
+            function executeRefresh() {
+                if (refreshLayout) {
+                    refreshLayout = false;
+                    refreshPaneLayout();
+                }
+            }
+
+            return app.createDebouncedMethod(registerRefresh, executeRefresh);
+        }());
+
         /**
          * Handles closed notification alert boxes and returns the browser
          * focus to the application.
          */
-        function removeNotificationHandler() {
-            app.executeDelayed(function () { self.grabFocus(); });
+        function notificationRemovedHandler(event) {
+            // return focus to application after the notification closes
+            // bug 33735: but only if focus is inside the alert box (e.g. after mouse click)
+            if (Utils.containsFocus(event.currentTarget)) {
+                app.executeDelayed(function () { self.grabFocus(); });
+            }
         }
 
         /**
@@ -262,23 +308,75 @@ define('io.ox/office/baseframework/view/baseview',
          * 'error', the message will be stored internally and automatically
          * shown again, after the application has been hidden and shown.
          *
-         * @param {Object} notification
-         *  The notification message data. Supports all options also supported
-         *  by the static method 'Notifications.yell()'.
+         * @param {Object} yellOptions
+         *  The notification message data. See method BaseView.yell() for
+         *  details.
          */
-        function showNotification(notification) {
-            if (notification.type === 'error') {
+        function showNotification(yellOptions) {
+
+            var // the notification DOM element
+                yellNode = null;
+
+            if (yellOptions.type === 'error') {
                 // Bug 28554: no auto-close for error messages
-                _.extend(notification, { duration: -1 });
+                yellOptions.duration = -1;
                 // remember error message, show again after switching applications
-                lastNotification = notification;
+                pendingYellOptions = yellOptions;
             } else {
-                lastNotification = null;
+                pendingYellOptions = null;
             }
-            Notifications.yell(notification);
+
+            // create and show the notification DOM node
+            yellNode = Notifications.yell(yellOptions);
+
+            // wait for deferred initialization of the DOM node
+            yellNode.on('notification:appear', function () {
+
+                var // the message node as target for additional contents
+                    messageNode = yellNode.find('.message'),
+                    // options for a static hyperlink shown below the message text
+                    linkOptions = Utils.getObjectOption(yellOptions, 'hyperlink'),
+                    // the URL for the static hyperlink
+                    linkUrl = Utils.getStringOption(linkOptions, 'url'),
+                    // the display text for the static hyperlink
+                    linkLabel = Utils.getStringOption(linkOptions, 'label'),
+                    // options for an action link shown below the message text
+                    actionOptions = Utils.getObjectOption(yellOptions, 'action'),
+                    // the key of the controller item for an action link
+                    actionKey = Utils.getStringOption(actionOptions, 'itemKey');
+
+                // append a static URL to the alert
+                if (linkUrl) {
+                    messageNode.append('<div><a href="' + linkUrl + '" target="_blank" tabindex="0">' + (linkLabel || _.noI18n(linkUrl)) + '</a></div>');
+                }
+
+                // do not insert an action link, if the controller item is missing or disabled
+                if (!actionKey || !app.getController().isItemEnabled(actionKey)) { return; }
+
+                var // the icon class for the link
+                    actionIcon = Utils.getStringOption(actionOptions, 'icon'),
+                    // the label text for the link
+                    actionLabel = Utils.getStringOption(actionOptions, 'label'),
+                    // the anchor element to be inserted into the alert node
+                    anchorNode = $('<a href="#">');
+
+                // insert the icon and label to the anchor node
+                if (actionIcon) { anchorNode.append(Forms.createIconMarkup(actionIcon)); }
+                if (actionLabel) { anchorNode.append(Forms.createSpanMarkup(actionLabel)); }
+
+                // execute the controller item on click (unbind the listener when quitting the application)
+                self.listenTo(anchorNode, 'click tap', function (event) {
+                    event.preventDefault();
+                    Notifications.yell.close();
+                    app.getController().executeItem(actionKey);
+                });
+
+                // insert the link into the alert node
+                messageNode.append($('<div>').append(anchorNode));
+            });
 
             // set focus back to the document container after the notification node is closed
-            $('.io-ox-alert').on('notification:removed', removeNotificationHandler);
+            yellNode.on('notification:removed', notificationRemovedHandler);
         }
 
         /**
@@ -290,20 +388,24 @@ define('io.ox/office/baseframework/view/baseview',
             if (viewHidden) { return; }
 
             // bug 29348: show pending notification also, if tempNode is empty
-            if (lastNotification && app.isImportFinished()) {
-                showNotification(lastNotification);
+            if (pendingYellOptions && app.isImportFinished() && self.isVisible()) {
+                showNotification(pendingYellOptions);
             }
 
             // move all application nodes from temporary storage into view
             if (tempNode.children().length > 0) {
-                app.getWindowNode().append(tempNode.children());
+            	// change tabindex-attribute to re-activate tabbing through the application
+				Utils.changeTabState(tempNode, {tabable: true, childrenOfNode: true});
+                winBodyNode.append(tempNode.children());
+            }
 
-                // do not update GUI and grab focus while document is still being imported
-                if (app.isImportFinished()) {
+            // do not update GUI and grab focus while document is still being imported
+            if (app.isImportFinished()) {
+                _.defer(function () {
                     refreshPaneLayout();
                     app.getController().update();
                     self.grabFocus();
-                }
+                });
             }
         }
 
@@ -313,11 +415,66 @@ define('io.ox/office/baseframework/view/baseview',
         function windowHideHandler() {
             // move all application nodes from view to temporary storage
             if (tempNode.children().length === 0) {
-                tempNode.append(app.getWindowNode().children());
+            	// change tabindex-attribute to prevent tabbing through the hidden application
+                tempNode.append(winBodyNode.children());
+                Utils.changeTabState(tempNode, {tabable: false, childrenOfNode: true});
             }
         }
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
+
+        /**
+         * Prepares the view for importing the document. Sets the application
+         * to busy state, and invokes the initialization handler passed to the
+         * constructor.
+         *
+         * @internal
+         *  Invoked from the import process of the application. DO NOT call
+         *  from external code.
+         *
+         * @returns {BaseView}
+         *  A reference to this instance.
+         */
+        this.hideBeforeImport = _.once(_.bind(function () {
+
+            // set window to busy state
+            app.getWindow().busy();
+            this.hide();
+
+            // invoke the initialization handler passed to the constructor
+            Utils.getFunctionOption(initOptions, 'initHandler', $.noop).call(self);
+
+            return this;
+        }, this));
+
+        /**
+         * Initializes the view after importing the document. Invokes the GUI
+         * initialization handler passed to the constructor, and sets the
+         * application to idle state.
+         *
+         * @internal
+         *  Invoked from the import process of the application. DO NOT call
+         *  from external code.
+         *
+         * @returns {BaseView}
+         *  A reference to this instance.
+         */
+        this.showAfterImport = function () {
+
+            // show the application window
+            this.show();
+
+            // invoke the GUI initialization handler passed to the constructor
+            Utils.getFunctionOption(initOptions, 'initGuiHandler', $.noop).call(self);
+
+            // immediately refresh all view panes to reduce flicker effects
+            silentRefreshPaneLayout();
+
+            this.leaveBusy();
+            return this;
+        };
+
+        // public methods -----------------------------------------------------
 
         /**
          * Moves the browser focus into the application pane. Calls the
@@ -333,7 +490,7 @@ define('io.ox/office/baseframework/view/baseview',
                 } else if (contentFocusable) {
                     contentRootNode.focus();
                 } else {
-                    this.getAppPaneNode().find('[tabindex]').first().focus();
+                    Forms.grabFocus(this.getAppPaneNode());
                 }
             }
             return this;
@@ -388,7 +545,7 @@ define('io.ox/office/baseframework/view/baseview',
          *  The DOM node of the application pane.
          */
         this.getAppPaneNode = function () {
-            return appPane.getNode();
+            return appPaneNode;
         };
 
         /**
@@ -493,6 +650,17 @@ define('io.ox/office/baseframework/view/baseview',
             return this;
         };
 
+        /**
+         * Adjusts the positions of all view pane nodes.
+         *
+         * @returns {BaseView}
+         *  A reference to this instance.
+         */
+        this.refreshPaneLayout = function () {
+            refreshPaneLayout();
+            return this;
+        };
+
         /**
          * Inserts new DOM nodes into the container node of the application
          * pane.
@@ -507,7 +675,8 @@ define('io.ox/office/baseframework/view/baseview',
          */
         this.insertContentNode = function (contentNode) {
             appContentNode.append(contentNode);
-            return this.refreshPaneLayout();
+            refreshPaneLayout();
+            return this;
         };
 
         /**
@@ -519,7 +688,8 @@ define('io.ox/office/baseframework/view/baseview',
          */
         this.removeAllContentNodes = function () {
             appContentNode.empty();
-            return this.refreshPaneLayout();
+            refreshPaneLayout();
+            return this;
         };
 
         /**
@@ -562,31 +732,20 @@ define('io.ox/office/baseframework/view/baseview',
 
             // insert the pane
             (pane.isOverlay() ? overlayPanes : fixedPanes).push(pane);
-            app.getWindowNode().append(pane.getNode());
+            (viewHidden ? tempNode : winBodyNode).append(pane.getNode());
 
             // refresh pane layout when the pane or its contents are changed
-            pane.on('pane:show pane:resize pane:layout', refreshPaneLayout);
-            refreshPaneLayout();
+            pane.on('pane:show pane:hide pane:resize', paneResizeHandler);
+            paneResizeHandler();
 
             return this;
         };
 
-        /**
-         * Adjusts the positions of all view pane nodes.
-         *
-         * @returns {BaseView}
-         *  A reference to this instance.
-         */
-        this.refreshPaneLayout = function () {
-            refreshPaneLayout();
-            return this;
-        };
-
         /**
          * Prevents refreshing the pane layout while the specified callback
          * function is running. After the callback function has finished, the
          * pane layout will be adjusted once by calling the method
-         * BaseApplication.refreshPaneLayout().
+         * BaseView.refreshPaneLayout().
          *
          * @param {Function} callback
          *  The callback function that will be executed synchronously while
@@ -602,7 +761,10 @@ define('io.ox/office/baseframework/view/baseview',
             layoutLocks += 1;
             callback.call(context);
             layoutLocks -= 1;
-            refreshPaneLayout();
+            if (layoutLocks === 0) {
+                if (layoutPending) { refreshPaneLayout(); }
+                layoutPending = false;
+            }
             return this;
         };
 
@@ -614,7 +776,7 @@ define('io.ox/office/baseframework/view/baseview',
          * new contents into these elements.
          *
          * @param {Object} [options]
-         *  A map with additional options. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.showFileName=false]
          *      If set to true, the file name will be shown in the top-left
          *      corner of the blocker element.
@@ -651,7 +813,7 @@ define('io.ox/office/baseframework/view/baseview',
             this.leaveBusy();
 
             // enter busy state, and extend the blocker element
-            app.getWindow().busy(null, null, function () {
+            win.busy(null, null, function () {
 
                 var // the initialization handler
                     initHandler = Utils.getFunctionOption(options, 'initHandler'),
@@ -697,13 +859,13 @@ define('io.ox/office/baseframework/view/baseview',
                         .appendTo(cancelNode);
 
                     // register a keyboard handler for the ESCAPE key
-                    app.getWindowNode().on('keydown', busyKeydownHandler);
-                    app.getWindow().one('idle', function () {
-                        app.getWindowNode().off('keydown', busyKeydownHandler);
+                    winBodyNode.on('keydown', busyKeydownHandler);
+                    win.one('idle', function () {
+                        winBodyNode.off('keydown', busyKeydownHandler);
                     });
 
                     // make the blocker focusable for keyboard input
-                    blockerNode.attr('tabindex', 0).focus();
+                    blockerNode.attr('tabindex', 1).focus();
 
                     // show the Cancel button after a delay
                     _.delay(function () { cancelNode.show().find('.btn').focus(); }, 5000);
@@ -729,6 +891,20 @@ define('io.ox/office/baseframework/view/baseview',
             return this;
         };
 
+        /**
+         * updates the progress-bar of the window blocker.
+         * when invoked repeatedly, it will only update the progress
+         * every 500 milliseconds.
+         *
+         * @param {Number} progress
+         *  value between 0 and 1
+         */
+        this.updateBusyProgress = _.throttle(function (progess) {
+            if (win.nodes.blocker.is(':visible')) {
+                win.busy(progess);
+            }
+        }, 500);
+
         /**
          * Leaves the busy state of the application. Hides the window blocker
          * element covering the entire GUI of the application.
@@ -744,7 +920,7 @@ define('io.ox/office/baseframework/view/baseview',
                 blockerFadeTimer = null;
             }
 
-            app.getWindow().idle();
+            win.idle();
             return this;
         };
 
@@ -753,40 +929,102 @@ define('io.ox/office/baseframework/view/baseview',
          * Otherwise, the last alert banner will be cached until the
          * application becomes visible again.
          *
-         * @param {String} type
-         *  The type of the alert banner. Supported types are 'success',
-         *  'info', 'warning', and 'error'.
-         *
-         * @param {String} message
-         *  The message text shown in the alert banner.
-         *
-         * @param {String} [headline]
-         *  An optional headline shown above the message text.
+         * @param {Object} yellOptions
+         *  The settings for the alert banner:
+         *  @param {String} [yellOptions.type='info']
+         *      The type of the alert banner. Supported types are 'success',
+         *      'info', 'warning', and 'error'.
+         *  @param {String} [yellOptions.headline]
+         *      An optional headline shown above the message text.
+         *  @param {String} yellOptions.message
+         *      The message text shown in the alert banner.
+         *  @param {Number} []yellOptions.duration]
+         *      The time to show the alert banner, in milliseconds; or -1 to
+         *      show a permanent alert. Default duration is dependent on the
+         *      alert type: permanent for error alerts, and specific durations
+         *      defined by the core toolkit for all other types.
+         *  @param {Object} [yellOptions.hyperlink]
+         *      An arbitrary static hyperlink that will be shown below the
+         *      message text, with the following properties:
+         *      @param {String} yellOptions.hyperlink.url
+         *          The URL for the hyperlink.
+         *      @param {String} [yellOptions.hyperlink.label]
+         *          The display text shown as hyperlink. If omitted, the URL
+         *          will be displayed.
+         *  @param {Object} [yellOptions.action]
+         *      A descriptor for an action link shown below the message text.
+         *      The link will be bound to an arbitrary item of the application
+         *      controller. If specified, must be an object with the following
+         *      properties:
+         *      @param {String} yellOptions.action.itemKey
+         *          The key of a controller item that will be bound to the
+         *          link. The link will be shown if the controller item is
+         *          enabled, and upon clicking the link, the controller item
+         *          will be executed.
+         *      @param {String} [yellOptions.action.icon]
+         *          The class name of an icon shown before the link label. MUST
+         *          be an icon from FontAwesome, internal OX Documents bitmap
+         *          icons are NOT supported.
+         *      @param {String} yellOptions.action.label
+         *          A label text for the link.
          *
          * @returns {BaseView}
          *  A reference to this instance.
          */
-        this.yell = function (type, message, headline) {
-            var notification = { type: type, message: message, headline: headline };
+        this.yell = function (yellOptions) {
+
+            // add default options
+            yellOptions = _.extend({ type: 'info' }, yellOptions);
+
+            // only show notification if the application is active, otherwise cache for later use
             if (this.isVisible()) {
-                showNotification(notification);
+                showNotification(yellOptions);
             } else {
-                lastNotification = notification;
+                pendingYellOptions = yellOptions;
             }
             return this;
         };
 
         // initialization -----------------------------------------------------
 
+        // initialize class members
+        win = app.getWindow();
+        winBodyNode = app.getWindowNode();
+
+        // add CSS classes to the window root node, and the temporary container node
+        (function () {
+
+            var // all CSS classes (always add the application name)
+                windowNodeClasses = 'io-ox-office-main ' + app.getName().replace(/[.\/]/g, '-') + '-main',
+                // additional CSS classes from options
+                classes = Utils.getStringOption(initOptions, 'classes');
+
+            // add the classes passed through options
+            if (classes) { windowNodeClasses += ' ' + classes; }
+
+            // add application-specific classes to the window body node, and the temporary container node
+            winBodyNode.addClass(windowNodeClasses);
+            tempNode.addClass(windowNodeClasses);
+        }());
+
         // add the temporary container for hidden state into the DOM
         Utils.insertHiddenNodes(tempNode);
 
-        // create the application pane, and insert the container nodes
-        appPane = new Pane(app, 'app', { classes: 'app-pane' });
-        appPane.getNode().append(hiddenRootNode, contentRootNode, appBusyNode.hide());
+        // log layout events
+        Utils.LayoutLogger.logEvent(this, 'refresh:layout');
 
-        // add the main application pane to the application window
-        app.getWindowNode().addClass(windowNodeClasses).append(appPane.getNode());
+        // create the application pane, and insert the container nodes
+        appPaneNode = $('<div class="app-pane">');
+        appPaneNode.append(hiddenRootNode, contentRootNode, appBusyNode.hide());
+        winBodyNode.append(appPaneNode);
+
+        // keep application in DOM while application is hidden, applications
+        // may want to access element geometry in background tasks
+        this.listenTo(win, 'show', windowShowHandler);
+        this.listenTo(win, 'hide', windowHideHandler);
+
+        // listen to browser window resize events when the application window is visible
+        app.registerWindowResizeHandler(refreshPaneLayout);
 
         // initialize content node from passed options
         contentRootNode.toggleClass('scrolling', Utils.getBooleanOption(initOptions, 'contentScrollable', false));
@@ -794,75 +1032,54 @@ define('io.ox/office/baseframework/view/baseview',
 
         // make the content root node focusable for global navigation with F6 key
         if (contentFocusable) {
-            contentRootNode.addClass('f6-target').attr('tabindex', 0);
+            contentRootNode.addClass('f6-target').attr('tabindex', 1);
         }
 
-        // listen to browser window resize events when the application window is visible
-        app.registerWindowResizeHandler(refreshPaneLayout);
+        // update view and pane layout after import
+        app.onImport(windowShowHandler);
 
-        // keep application in DOM while application is hidden, applications
-        // may want to access element geometry in background tasks
-        app.getWindow().on({ show: windowShowHandler, hide: windowHideHandler });
+        // show notification alerts when import fails
+        app.onImportFailure(function (result) {
 
-        // after construction, call initialization handler
-        app.on('docs:init', function () {
-            self.lockPaneLayout(function () {
-                Utils.getFunctionOption(initOptions, 'initHandler', $.noop).call(self);
-            });
-        });
+            var cause = Utils.getStringOption(result, 'cause', 'unknown'),
+                headline = Utils.getStringOption(result, 'headline', gt('Load Error')),
+                message = Utils.getStringOption(result, 'message', gt('An error occurred while loading the document.'));
 
-        // after import, call deferred initialization handler, and update view and controller
-        app.on('docs:import:after', function () {
-            self.lockPaneLayout(function () {
-                Utils.getFunctionOption(initOptions, 'deferredInitHandler', $.noop).call(self);
-            });
-            windowShowHandler();
+            // special case if a server component is not working correctly
+            if (cause === 'bad server component') {
+                headline = gt('Server Error');
+                message = gt('A Documents server component is not working. Please contact the server administrator.');
+            }
+
+            // be silent in server timeout situations
+            if (cause !== 'timeout') {
+                self.yell({ type: 'error', headline: headline, message: message });
+            }
         });
 
         // safely destroy all image nodes, and prevent focusing destroyed application after yelling
-        app.getWindow().on('quit', function () {
-            app.destroyImageNodes(app.getWindowNode());
-            $('.io-ox-alert').off('notification:removed', removeNotificationHandler);
+        win.on('quit', function () {
+            app.destroyImageNodes(winBodyNode);
+            app.destroyImageNodes(tempNode);
+            $('.io-ox-alert').off('notification:removed', notificationRemovedHandler);
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
+
             _.invoke(fixedPanes, 'destroy');
             _.invoke(overlayPanes, 'destroy');
 
-            hiddenRootNode.remove();
-            appPane.destroy();
             tempNode.remove();
+            appPaneNode.remove();
 
-            grabFocusHandler = appPane = fixedPanes = overlayPanes = tempNode = null;
+            app = initOptions = self = grabFocusHandler = win = winBodyNode = appPaneNode = null;
+            hiddenRootNode = contentRootNode = appContentNode = appBusyNode = tempNode = null;
+            fixedPanes = overlayPanes = blockerFadeTimer = null;
         });
 
     } // class BaseView
 
-    // static methods ---------------------------------------------------------
-
-    /**
-     * Clears the current browser selection ranges.
-     */
-    BaseView.clearBrowserSelection = function () {
-        try {
-            // Bug 28515, bug 28711: IE fails to clear the selection (and to
-            // modify it afterwards), if it currently points to a DOM node that
-            // is not visible anymore. This happens e.g. after clicking on the
-            // 'Show/hide side panel' button shown by all OX Documents
-            // applications, which hide themselves together with the side pane
-            // or overlay pane after activation. Workaround is to move focus to
-            // an editable DOM node which will cause IE to update the browser
-            // selection object. Using another focusable node (e.g. the body
-            // element) is not sufficient. Use try/catch anyway to be notified
-            // about other problems with the selection.
-            if (_.browser.IE) { focusHelperNode.focus(); }
-            window.getSelection().removeAllRanges();
-        } catch (ex) {
-            Utils.exception(ex);
-        }
-    };
-
     // exports ================================================================
 
     // derive this class from class TriggerObject
