@@ -140,7 +140,7 @@ define('io.ox/office/baseframework/view/nodetracking',
             event = _.extend({ startX: startX, startY: startY }, data);
 
         // extend with specific tracking data
-        _(event).extend({
+        _.extend(event, {
             trackingType: trackingType,
             cursor: overlayNode.css('cursor'),
             startTime: startTime,
@@ -149,7 +149,7 @@ define('io.ox/office/baseframework/view/nodetracking',
 
         // extend with states of all modifier keys
         if (_.isObject(sourceEvent)) {
-            _(event).extend({
+            _.extend(event, {
                 originalEvent: sourceEvent.originalEvent,
                 shiftKey: sourceEvent.shiftKey,
                 altKey: sourceEvent.altKey,
@@ -437,6 +437,7 @@ define('io.ox/office/baseframework/view/nodetracking',
     function mouseDownHandler(event) {
         if (event.button === 0) {
             trackingStart(event, 'mouse', event.pageX, event.pageY);
+            if (_.browser.IE === 11) { event.preventDefault(); }  // Fix for 33987, avoid grabber in IE 11
         } else {
             cancelTracking();
         }
