@@ -62,7 +62,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
      *  The application instance containing this drawing collection.
      *
      * @param {Object} initOptions
-     *  A map with additional options. The following options are supported:
+     *  Optional parameters:
      *  @param {Function} initOptions.clientCreateModelHandler
      *      A factory function that constructs drawing model instances for the
      *      specified drawing type. Receives the type of the drawing model to
@@ -198,7 +198,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          *  position inside another drawing object.
          *
          * @param {Object} [options]
-         *  A map with additional options. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.deep=false]
          *      If set to true, searches for drawing objects embedded in other
          *      drawing objects.
@@ -242,7 +242,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          *  The drawing model whose position will be obtained.
          *
          * @param {Object} [options]
-         *  A map with additional options. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.deep=false]
          *      If set to true, also supports drawing objects embedded in other
          *      drawing objects, and resolves the complete logical position of
@@ -276,8 +276,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          *  immediately.
          *
          * @param {Object} [options]
-         *  A map with options controlling the behavior of this method. The
-         *  following options are supported:
+         *  Optional parameters:
          *  @param {Object} [options.context]
          *      If specified, the iterator will be called with this context
          *      (the symbol 'this' will be bound to the context inside the
@@ -292,7 +291,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
             var // the calling context for the iterator function
                 context = Utils.getOption(options, 'context');
 
-            return _(drawingModels).any(function (model) {
+            return _.any(drawingModels, function (model) {
                 return iterator.call(context, model) === Utils.BREAK;
             }) ? Utils.BREAK : undefined;
         };
@@ -436,7 +435,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            _(drawingModels).invoke('destroy');
+            _.invoke(drawingModels, 'destroy');
             drawingModels = null;
         });
 
