@@ -13,7 +13,7 @@
 
 define('io.ox/office/drawinglayer/model/drawingmodel',
     ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/format/attributedmodel'
+     'io.ox/office/editframework/model/attributedmodel'
     ], function (Utils, AttributedModel) {
 
     'use strict';
