@@ -14,9 +14,10 @@
 define('io.ox/office/drawinglayer/view/drawingframe',
     ['io.ox/core/date',
      'io.ox/office/tk/utils',
-     'io.ox/office/drawinglayer/lib/canvasjs.min',
-     'gettext!io.ox/office/drawing'
-    ], function (OXDate, Utils, CanvasJS, gt) {
+     'io.ox/office/tk/forms',
+     'io.ox/office/drawinglayer/utils/drawingutils',
+     'io.ox/office/drawinglayer/lib/canvasjs.min'
+    ], function (CoreDate, Utils, Forms, DrawingUtils, CanvasJS) {
 
     'use strict';
 
@@ -29,16 +30,6 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         // the CSS class name for active tracking mode
         TRACKING_CLASS = 'tracking-active',
 
-        // maps drawing object types to GUI names and icons
-        OBJECT_TYPE_SETTINGS = {
-            chart:    { type: gt('Chart'),   icon: 'fa-bar-chart-o' },
-            image:    { type: gt('Image'),   icon: 'fa-picture-o' },
-            shape:    { type: gt('Shape'),   icon: 'fa-picture-o' },
-            diagram:  { type: gt('Diagram'), icon: 'fa-sitemap' }
-        },
-
-        DEFAULT_TYPE_SETTINGS = { type: gt('Drawing'), icon: 'fa-picture-o' },
-
         CULTURE_INFO = null;
 
     // private global functions ===============================================
@@ -58,28 +49,6 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         return $(drawingFrame).first().children('.' + SELECTION_CLASS);
     }
 
-    /**
-     * Creates an image node in the passed content node with the specified
-     * source URL.
-     */
-    function createImageNodeFromUrl(contentNode, srcUrl) {
-        contentNode.empty().append($('<img>', { src: srcUrl }));
-    }
-
-    /**
-     * Creates an image node or an SVG image in the passed content node,
-     * according to the passed image data.
-     */
-    function createImageNodeFromData(contentNode, imageData) {
-        if (/^data:/.test(imageData)) {
-            createImageNodeFromUrl(contentNode, imageData);
-        } else if (/^<svg/.test(imageData)) {
-            contentNode[0].innerHTML = imageData;
-        } else {
-            Utils.warn('DrawingFrame.createImageNodeFromData() - unsupported source data for the image object');
-        }
-    }
-
     /**
      * Calculates the offset and size of the bitmap in an image frame for one
      * dimension (either horizontally or vertically), according to the passed
@@ -271,6 +240,20 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         return $(drawingFrame).first().children('.' + CONTENT_CLASS);
     };
 
+    /**
+     * Clears and returns the root content node of the specified drawing frame.
+     *
+     * @param {HTMLElement|jQuery} drawingFrame
+     *  The root node of the drawing frame. If this object is a jQuery
+     *  collection, uses the first DOM node it contains.
+     *
+     * @returns {jQuery}
+     *  The empty root content node of the specified drawing frame.
+     */
+    DrawingFrame.getAndClearContentNode = function (drawingFrame) {
+        return DrawingFrame.getContentNode(drawingFrame).empty().removeClass('placeholder').attr('style', '');
+    };
+
     /**
      * Inserts replacement layout nodes for unsupported drawing types. Inserts
      * the passed name and description of the drawing as text.
@@ -289,9 +272,8 @@ define('io.ox/office/drawinglayer/view/drawingframe',
 
         var // the type of the drawing frame
             type = DrawingFrame.getDrawingType(drawingFrame),
-            typeSettings = OBJECT_TYPE_SETTINGS[type] || DEFAULT_TYPE_SETTINGS,
-            // the content node
-            contentNode = DrawingFrame.getContentNode(drawingFrame),
+            // the empty content node
+            contentNode = DrawingFrame.getAndClearContentNode(drawingFrame),
             // the inner width and height available in the content node
             innerWidth = Math.max(0, drawingFrame.outerWidth() - 2),
             innerHeight = Math.max(0, drawingFrame.outerHeight() - 2),
@@ -304,7 +286,6 @@ define('io.ox/office/drawinglayer/view/drawingframe',
 
         // set border width at the content node, insert the picture icon
         contentNode
-            .empty()
             .addClass('placeholder')
             .css({
                 padding: Math.floor(verticalPadding) + 'px ' + Math.floor(verticalPadding * 2) + 'px',
@@ -314,13 +295,13 @@ define('io.ox/office/drawinglayer/view/drawingframe',
                 $('<div>')
                     .addClass('abs background-icon')
                     .css('line-height', innerHeight + 'px')
-                    .append(Utils.createIcon(typeSettings.icon).css('font-size', pictureIconSize + 'px')),
-                $('<p>').text(name || typeSettings.type)
+                    .append(Forms.createIconMarkup(DrawingUtils.getDrawingTypeIcon(type), { style: 'font-size:' + pictureIconSize + 'px' })),
+                $('<p>').text(_.noI18n(name) || DrawingUtils.getDrawingTypeLabel(type))
             );
 
         // insert description if there is a reasonable amount of space available
         if ((innerWidth >= 20) && (innerHeight >= 20)) {
-            contentNode.append($('<p>').text(description));
+            contentNode.append($('<p>').text(_.noI18n(description)));
         }
     };
 
@@ -353,7 +334,41 @@ define('io.ox/office/drawinglayer/view/drawingframe',
             // the content node inside the drawing frame
             contentNode = DrawingFrame.getContentNode(drawingFrame),
             // the replacement data for unsupported drawings
-            replacementData = Utils.getStringOption(drawingAttributes, 'replacementData', '');
+            replacementData = Utils.getStringOption(drawingAttributes, 'replacementData', ''),
+            // rendering result (false to fall-back to placeholder graphic)
+            rendered = false;
+
+        // creates an image node in the content node with the specified source URL
+        function createImageNodeFromUrl(srcUrl) {
+            return app.createImageNode(srcUrl, { timeout: 15000 }).done(function (imgNode) {
+                DrawingFrame.getAndClearContentNode(drawingFrame).append(imgNode);
+            });
+        }
+
+        // creates an image node or an SVG image according to the passed image data
+        function createImageNodeFromData(imageData) {
+            if (/^data:/.test(imageData)) {
+                return createImageNodeFromUrl(imageData);
+            }
+            if (/^<svg/.test(imageData)) {
+                DrawingFrame.getAndClearContentNode(drawingFrame)[0].innerHTML = imageData;
+                return $.when();
+            }
+            Utils.warn('DrawingFrame.createImageNodeFromData() - unsupported source data for the image object');
+            return $.Deferred().reject();
+        }
+
+        // updates all formatting for unsupported objects
+        function updateUnsupportedFormatting() {
+            if (replacementData.length === 0) {
+                // insert default replacement nodes, if no replacement data is available
+                DrawingFrame.insertReplacementNodes(drawingFrame, drawingAttributes.name, drawingAttributes.description);
+            } else if (!drawingFrame.data('has-replacement')) {
+                // replacement data MUST NOT change at runtime
+                createImageNodeFromData(replacementData);
+                drawingFrame.data('has-replacement', true);
+            }
+        }
 
         // updates all formatting for image objects
         function updateImageFormatting() {
@@ -373,96 +388,106 @@ define('io.ox/office/drawinglayer/view/drawingframe',
                 cropTop = swapVertCrop ? drawingAttributes.cropBottom : drawingAttributes.cropTop,
                 cropBottom = swapVertCrop ? drawingAttributes.cropTop : drawingAttributes.cropBottom;
 
-            // create the image or SVG node (considered to be constant at runtime)
-            if (contentNode.children().length === 0) {
-                if (drawingAttributes.imageData.length > 0) {
-                    createImageNodeFromData(contentNode, drawingAttributes.imageData);
-                } else if (drawingAttributes.imageUrl.length > 0) {
-                    // convert relative URL to an application-specific absolute URL
-                    createImageNodeFromUrl(contentNode, /:\/\//.test(drawingAttributes.imageUrl) ? drawingAttributes.imageUrl : app.getFilterModuleUrl({ action: 'getfile', get_filename: drawingAttributes.imageUrl }));
-                } else {
-                    Utils.warn('DrawingFrame.updateFormatting() - missing source data for the image object');
+            // updates attributes of the image node after loading
+            function updateImageAttributes() {
+                if ((drawingWidth > 0) && (drawingHeight > 0)) {
+                    horizontalSettings = calculateBitmapSettings(drawingWidth, cropLeft, cropRight);
+                    verticalSettings = calculateBitmapSettings(drawingHeight, cropTop, cropBottom);
+                    contentNode.find('>img').css({
+                        left: horizontalSettings.offset,
+                        top: verticalSettings.offset,
+                        width: horizontalSettings.size,
+                        height: verticalSettings.size
+                    });
                 }
             }
 
-            // set bitmap cropping
-            if ((drawingWidth > 0) && (drawingHeight > 0)) {
-                horizontalSettings = calculateBitmapSettings(drawingWidth, cropLeft, cropRight);
-                verticalSettings = calculateBitmapSettings(drawingHeight, cropTop, cropBottom);
-                contentNode.find('>img').css({
-                    left: horizontalSettings.offset,
-                    top: verticalSettings.offset,
-                    width: horizontalSettings.size,
-                    height: verticalSettings.size
-                });
+            // create the image or SVG node (considered to be constant at runtime)
+            if (contentNode.children().length === 0) {
+                // start with replacement graphics (image data will be loaded asynchronously)
+                updateUnsupportedFormatting();
+                if (drawingAttributes.imageData.length > 0) {
+                    createImageNodeFromData(drawingAttributes.imageData).done(updateImageAttributes);
+                } else if (drawingAttributes.imageUrl.length > 0) {
+                    // convert relative URL to an application-specific absolute URL
+                    createImageNodeFromUrl(/:\/\//.test(drawingAttributes.imageUrl) ? drawingAttributes.imageUrl : app.getFilterModuleUrl({ action: 'getfile', get_filename: drawingAttributes.imageUrl })).done(updateImageAttributes);
+                } else {
+                    Utils.warn('DrawingFrame.updateFormatting() - missing source data for the image object');
+                }
+            } else {
+                // update attributes of an existing image element
+                updateImageAttributes();
             }
+
+            // image successfully rendered
+            return true;
         }
 
         // updates chart formatting
         function updateChartFormatting() {
-            var chart = drawingFrame.data('chart'),
+
+            var // the drawing object model
                 model = DrawingFrame.getModel(drawingFrame),
+                // the chart rendering engine (lazy initialization, see below)
+                renderer = drawingFrame.data('chart-renderer'),
+                // unique DOM element identifier for the renderer
                 chartId = null,
                 drawingNode = null;
 
-            if (model && !chart) {
-                // create CanvasJS object
+            // drawing object model must exist for chart objects
+            if (!model) { return false; }
+
+            // lazy creation of the renderer
+            if (!renderer) {
                 chartId = 'io-ox-documents-chart-' + DrawingFrame.getUid(drawingFrame);
                 contentNode.addClass('chartholder');
-                if (model.getModelData) {
 
-                    //localization for CanvasJS
-                    if (!CULTURE_INFO) {
-                        CULTURE_INFO = {
-                            decimalSeparator: app.getDecimalSeparator(),
-                            digitGroupSeparator: app.getGroupSeparator(),
-                            shortDays: OXDate.locale.daysShort,
-                            days: OXDate.locale.days,
-                            months: OXDate.locale.months,
-                            shortMonths: OXDate.locale.monthsShort
-                        };
-                        CanvasJS.addCultureInfo('en', CULTURE_INFO);
-                    }
-
-                    drawingNode = $('<div>');
-                    drawingNode.addClass('chartnode');
-                    drawingNode.attr('id', chartId);
-                    contentNode.append(drawingNode);
-					chart = new CanvasJS.Chart(chartId, model.getModelData());
-                    drawingFrame.data('chart', chart);
-                    model.firstInit();
+                // localization for CanvasJS
+                if (!CULTURE_INFO) {
+                    CULTURE_INFO = {
+                        decimalSeparator: app.getDecimalSeparator(),
+                        digitGroupSeparator: app.getGroupSeparator(),
+                        shortDays: CoreDate.locale.daysShort,
+                        days: CoreDate.locale.days,
+                        months: CoreDate.locale.months,
+                        shortMonths: CoreDate.locale.monthsShort
+                    };
+                    CanvasJS.addCultureInfo('en', CULTURE_INFO);
                 }
+
+                drawingNode = $('<div>');
+                drawingNode.addClass('chartnode');
+                drawingNode.attr('id', chartId);
+                contentNode.append(drawingNode);
+
+                //realy important call, because of a bug in canvasJS
+                model.resetData();
+
+                renderer = new CanvasJS.Chart(chartId, model.getModelData());
+                drawingFrame.data('chart-renderer', renderer);
+                model.firstInit();
             }
 
-            if (chart) {
-                model.refresh();
-                try {
-                    chart.render();
-                } catch (e) {
-                    Utils.warn('chart rendering error', e);
-                }
-                //background color of model data is already prepared for css
-                contentNode.css('backgroundColor', model.getModelData().cssBackgroundColor);
+            // render the chart object
+            model.refresh();
+            try {
+                renderer.render();
+            } catch (ex) {
+                Utils.exception(ex, 'chart rendering error');
             }
-        }
 
-        // updates all formatting for unsupported objects
-        function updateUnsupportedFormatting() {
-            if (replacementData.length === 0) {
-                // insert default replacement nodes, if no replacement data is available
-                DrawingFrame.insertReplacementNodes(drawingFrame, drawingAttributes.name, drawingAttributes.description);
-            } else if (!drawingFrame.data('has-replacement')) {
-                // replacement data MUST NOT change at runtime
-                createImageNodeFromData(contentNode, replacementData);
-                drawingFrame.data('has-replacement', true);
-            }
+            // background color of model data is already prepared for css
+            contentNode.css('backgroundColor', model.getModelData().cssBackgroundColor);
+
+            // chart successfully rendered
+            return true;
         }
 
         // convert to single jQuery object
         drawingFrame = $(drawingFrame).first();
 
         // set text attribute to support tooltip for the drawing
-        Utils.setControlTooltip(drawingFrame, drawingAttributes.name);
+        Forms.setToolTip(drawingFrame, drawingAttributes.name);
 
         // vertically or horizontally flipped drawings
         if (drawingAttributes.flipH && drawingAttributes.flipV) {
@@ -476,16 +501,19 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         // always set the CSS attribute, to be able to remove existing transformations
         Utils.setCssAttributeWithPrefixes(contentNode, 'transform', transformations);
 
-        // update attributes specific to the drawing type
+        // update drawing frame specific to the drawing type
         switch (type) {
         case 'image':
         case 'ole':
-            updateImageFormatting();
+            rendered = updateImageFormatting();
             break;
         case 'chart':
-            updateChartFormatting();
+            rendered = updateChartFormatting();
             break;
-        default:
+        }
+
+        // add replacement in case the drawing type is not supported, or if rendering was not successful
+        if (!rendered) {
             updateUnsupportedFormatting();
         }
     };
@@ -503,7 +531,7 @@ define('io.ox/office/drawinglayer/view/drawingframe',
      *  Whether the specified drawing frame is selected.
      */
     DrawingFrame.isSelected = function (drawingFrame) {
-        return $(drawingFrame).hasClass(Utils.SELECTED_CLASS);
+        return Forms.isSelectedNode(drawingFrame);
     };
 
     /**
@@ -541,14 +569,14 @@ define('io.ox/office/drawinglayer/view/drawingframe',
 
             // create the border nodes
             markup += '<div class="borders">';
-            _(['t', 'b', 'l', 'r']).each(function (pos) {
+            _.each('tblr', function (pos) {
                 markup += '<div data-pos="' + pos + '"></div>';
             });
             markup += '</div>';
 
             // create resizer handles
             markup += '<div class="resizers">';
-            _(['t', 'b', 'l', 'r', 'tl', 'tr', 'bl', 'br']).each(function (pos) {
+            _.each(['t', 'b', 'l', 'r', 'tl', 'tr', 'bl', 'br'], function (pos) {
                 markup += '<div data-pos="' + pos + '"></div>';
             });
             markup += '</div>';
@@ -564,7 +592,7 @@ define('io.ox/office/drawinglayer/view/drawingframe',
 
         // update state classes at the drawing frame
         $(drawingFrame).first()
-            .addClass(Utils.SELECTED_CLASS)
+            .addClass(Forms.SELECTED_CLASS)
             .toggleClass('movable', Utils.getBooleanOption(options, 'movable', false))
             .toggleClass('resizable', Utils.getBooleanOption(options, 'resizable', false));
 
@@ -593,7 +621,7 @@ define('io.ox/office/drawinglayer/view/drawingframe',
      *  object.
      */
     DrawingFrame.clearSelection = function (drawingFrame) {
-        $(drawingFrame).first().removeClass(Utils.SELECTED_CLASS + ' movable resizable');
+        $(drawingFrame).first().removeClass(Forms.SELECTED_CLASS + ' movable resizable');
         getSelectionNode(drawingFrame).remove();
     };
 
