@@ -13,12 +13,18 @@
 
 define('io.ox/office/drawinglayer/view/imageutil',
     ['io.ox/core/config',
+     'io.ox/core/tk/folderviews',
+     'io.ox/core/folder/api',
+     'io.ox/core/cache',
+     'io.ox/core/tk/selection',
+     'io.ox/files/api',
      'io.ox/office/tk/utils',
      'io.ox/office/tk/io',
-     'io.ox/office/tk/dialogs',
-     'io.ox/office/editframework/view/editdialogmixin',
+     'io.ox/office/tk/keycodes',
+     'io.ox/office/editframework/view/editdialogs',
+     'settings!io.ox/office',
      'gettext!io.ox/office/main'
-    ], function (Config, Utils, IO, Dialogs, EditDialogMixin, gt) {
+    ], function (Config, FolderViews, FolderAPI, CoreCache, CoreSelection, FilesAPI, Utils, IO, KeyCodes, EditDialogs, Settings, gt) {
 
     'use strict';
 
@@ -91,7 +97,10 @@ define('io.ox/office/drawinglayer/view/imageutil',
      * @param {Number} offset
      *  Specifies offset to the beginning of base64 encoded data.
      *
-     * @returns {Number} crc32
+     * @returns {Number|Null} crc32
+     *  Null is returned, if no base64 encoded data could be found in the data string. Otherwise
+     *  the crc32 value is returned.
+     *
      */
     function getCRC32fromBase64(data, offset) {
 
@@ -99,7 +108,7 @@ define('io.ox/office/drawinglayer/view/imageutil',
 
         // data length must be at least two bytes to encode one character (if no pad bytes are used)
         if (dataLength < 2)
-            return;
+            return null;
 
         // taking care of padbytes
         if (data.charAt(offset + dataLength - 1) === '=')
@@ -148,13 +157,20 @@ define('io.ox/office/drawinglayer/view/imageutil',
 
         return IO.readClientFileAsDataUrl(file).then(function (dataUrl) {
 
-            var offset = dataUrl.indexOf('base64,') + 7,
-                crc32 = toHex32(getCRC32fromBase64(dataUrl, offset)),
-                extension = file.name.substring(file.name.lastIndexOf('.') + 1);
+            var offset = dataUrl && dataUrl.indexOf('base64,') + 7,
+                crc32 = null,
+                extension = null;
+
+            crc32 = getCRC32fromBase64(dataUrl, offset);
+
+            // if no crc32 value could be determined, the loading process must be stopped now.
+            if (!crc32) { return $.Deferred().reject(); }
+
+            crc32 = toHex32(crc32),
+            extension = file.name.substring(file.name.lastIndexOf('.') + 1);
 
 //            Utils.error('sendFilterRequest', crc32, file.name, extension, dataUrl);
 
-
             return app.sendFilterRequest({
                 method: 'POST',
                 params: {
@@ -232,6 +248,34 @@ define('io.ox/office/drawinglayer/view/imageutil',
         })).promise();
     }
 
+    /**
+     * Inserts a file from OX Drive.
+     *
+     * @param {ox.ui.App} app
+     *  The application object representing the edited document.
+     *
+     * @param {Number} fileDescriptor
+     *  The infostore descriptor of the file to be inserted
+     *
+     * @returns {jQuery.Promise}
+     *  a Promise which be resolved when the insert action is successful
+     */
+    function insertDriveImage(app, fileDescriptor) {
+
+        if (!fileDescriptor) { return $.Deferred().reject().promise(); }
+
+        return app.sendFilterRequest({
+            method: 'POST',
+            params: {
+                action: 'addfile',
+                add_ext: fileDescriptor.filename.substring(fileDescriptor.filename.lastIndexOf('.') + 1),
+                add_fileid: fileDescriptor.id,
+                add_folderid: fileDescriptor.folder_id,
+                alt_filename: fileDescriptor.filename
+            }
+        });
+    }
+
     // static class Image =====================================================
 
     /**
@@ -400,10 +444,34 @@ define('io.ox/office/drawinglayer/view/imageutil',
                 url: Utils.getStringOption(data, 'added_filename', ''),
                 name : Image.getFileNameWithoutExt(file.name)
             };
-        })
-        .promise();
+        }, function () {
+            app.getView().yell({ type: 'warning', message: gt('Image could not be loaded.') });
+        });
     };
 
+    /**
+     * Inserts the image file of the passed infostore descriptor into a document.
+     *
+     * @param {ox.ui.App} app
+     *  The application object representing the edited document.
+     *
+     * @param {Object} file
+     *  The infostore descriptor of the file being inserted.
+     *
+     * @returns {jQuery.Promise}
+     *  The Promise of a Deferred object that will be resolved if the image
+     *  has been uploaded to the server; or rejected, in case of an error.
+     */
+    Image.insertDriveImage = function (app, file) {
+        return insertDriveImage(app, file).then(function (data) {
+            return {
+                url: Utils.getStringOption(data, 'added_filename', ''),
+                name : Image.getFileNameWithoutExt(file.filename)
+            };
+        });
+    };
+
+
     /**
      * Inserts the image specified by a URL into a document.
      *
@@ -467,11 +535,15 @@ define('io.ox/office/drawinglayer/view/imageutil',
             dialog = null,
             filePanel = null,
             urlPanel = null,
+            drivePanel = null,
 
             // the file picker widget
             filePicker = filePickerWidget(),
             fileInput = filePicker.find('input[type="file"]'),
 
+            // the drive file picker
+            driveFilePicker = new DriveFilePicker(),
+
             // the 'Enter URL' widgets
             urlInput = $('<input>', {
                 tabindex: 0,
@@ -481,6 +553,7 @@ define('io.ox/office/drawinglayer/view/imageutil',
             // the file descriptor of the file currently selected
             uploadFile = null,
             uploadFileName = null,
+            selectedDriveFile = null,
 
             // the resulting promise
             promise = null;
@@ -523,8 +596,10 @@ define('io.ox/office/drawinglayer/view/imageutil',
             node.append(($('<div class="input-group">').append(buttonGroupFile, inputText)));
 
             node.on('keypress', function (e) {
-                if (e.which === 13 || e.which === 32) {
-                    inputFile.trigger('click');
+            	// press SPACE (ENTER isn't active, cause of the special 'send'-occupancy)
+                if (e.which === 32) {
+            		inputFile.focus(); // BUG #34034: FF needs to focus the input-element first
+                	inputFile.trigger('click');
                 }
             });
 
@@ -536,9 +611,161 @@ define('io.ox/office/drawinglayer/view/imageutil',
             return node;
         }
 
+        // create infostore (Drive) file system browser
+        function DriveFilePicker() {
+
+            var folderCache = new CoreCache.SimpleCache('folder-all'),
+                subFolderCache = new CoreCache.SimpleCache('subfolder-all'),
+                treeNode = $('<div>').addClass('drive-folder-list'),
+                filesPane = $('<ul>').addClass('io-ox-fileselection list-unstyled drive-image-list singleselection f6-target').attr({ tabindex: 0}),
+                filePickerNode = $('<div>').addClass('drive-image-picker').append(treeNode, filesPane),
+                folderTree = new FolderViews.ApplicationFolderTree(treeNode, {
+                    type: 'infostore',
+                    tabindex: 0,
+                    rootFolderId: '9',
+                    all: true,
+                    storage: {
+                        folderCache: folderCache,
+                        subFolderCache: subFolderCache
+                    },
+                    filter: function (folder) {
+                        return !FolderAPI.is('trash', folder);
+                    }
+                }),
+                self = this;
+
+            // extend filesPane with CoreSelection functionalities, plus keyboard selection support
+            CoreSelection.extend(this, filesPane, {});
+            self.selection.keyboard(filesPane, true);
+
+
+            // on foldertree change update file selection
+            folderTree.selection.on('select', function (e, folderId) {
+
+                filesPane.empty();
+
+                FilesAPI.getAll({ folder: folderId }, false).then(function (files) {
+
+                    // reset selected file after every folder load file action
+                    selectedDriveFile = null;
+
+                    // filter folder content for image files
+                    var imageFiles = _.filter(files, function (file) {
+                        return Image.hasFileImageExtension(file.filename);
+                    });
+
+                    // disable ok button and quit immediately if no image files is found in this folder
+                    if (imageFiles.length === 0) {
+                        dialog.enableOkButton(false);
+                        filesPane.append($('<p>').addClass('bg-info').text(gt('This folder has no images.')));
+                        return;
+                    }
+
+                    // prepare filespane node and insert into root filepickernode
+                    _.each(imageFiles, function (file) {
+                        var fileNameNode = $('<div>').addClass('name').text(file.filename || file.title),
+                            fileListItem = $('<li>').addClass('file selectable').attr('data-obj-id', _.cid(file));
+                        filesPane.append(fileListItem.append(fileNameNode));
+                    });
+                    filePickerNode.append(filesPane);
+
+                    // init the CoreSelection object of the filespane, select first item initially
+                    self.selection.clear();
+                    self.selection.init(imageFiles);
+
+                    // save initially selected image file to global variable of insertImageDialog
+                    var selectedList = self.selection.get();
+                    selectedDriveFile = selectedList.length > 0 ? selectedList[0] : null;
+
+                    // update selected file after change event on the filespane selection object
+                    self.selection.on('change', function (e, list) {
+                        selectedDriveFile = list.length > 0 ? list[0] : null;
+                        // activate/deactivate dialogs ok button according to selected file status and edit mode
+                        checkOkButton();
+                    });
+
+                });
+
+                // remember user's last insert image folder location, to be saved later while leaving dialog
+                filePickerNode.data('recent-folder', folderId);
+
+            });
+
+            // draw and init application folder tree
+            folderTree.paint().done(function () {
+                //preselect user's recent image folder if exists
+                var recentImageFolder = Settings.get('recentImageFolder');
+                if (recentImageFolder) {
+                    folderTree.select(recentImageFolder);
+                    folderTree.selection.select(recentImageFolder);
+                }
+                else {
+                    folderTree.selection.updateIndex().selectFirst();
+                }
+                treeNode.focus();
+            });
+
+			// grab "TAB" and "shift+TAB" keystrokes in FolderTree
+			// to prevent default handling an jump to prev/next Control
+			treeNode.on('keydown', function(event){
+                if (filesPane.find('.file').length === 0) { return; }
+				if (KeyCodes.matchKeyCode(event, 'TAB', { shift: false })) {
+
+					self.selection.selectFirst();
+					filesPane.focus();
+					$('li[data-obj-id="'+selectedDriveFile.folder_id+'.'+selectedDriveFile.id+'"]').first().focus();
+					checkOkButton();
+
+					event.preventDefault();
+					event.stopPropagation();
+					return false;
+
+				} else if (KeyCodes.matchKeyCode(event, 'TAB', { shift: true })) {
+					var arrFolders = folderTree.selection.getObservedItems(),
+						activeFolder = folderTree.selection.get()[0],
+						firstFolder = null;
+
+						_.each(arrFolders, function(i, ele){
+							if (i===0) { firstFolder = ele; }
+						});
+
+					if( arrFolders[activeFolder] !== 0 ){
+						$('div[data-obj-id="'+firstFolder+'"]').focus();
+					}
+				}
+			});
+
+			// grab "shift+TAB" in FilesPane
+			// to jump to prev Control respectivily to the "selected" Folder in FolderTree
+			filesPane.on('keydown', function(event){
+                if (filesPane.find('.file').length === 0) { return; }
+                if (KeyCodes.matchKeyCode(event, 'TAB', { shift: true })) {
+
+					self.selection.deselect(selectedDriveFile.folder_id+'.'+selectedDriveFile.id);
+
+					var folderId = folderTree.selection.get()[0];
+
+					// treeNode.focus();
+					$('div[data-obj-id="'+folderId+'"]').focus();
+
+					checkOkButton();
+
+					event.preventDefault();
+					event.stopPropagation();
+					return false;
+				}
+			});
+
+            return filePickerNode;
+        }
+
         // enable / disable OK button
         function checkOkButton() {
             switch (dialog.getActiveTabId()) {
+            case 'drive':
+                // enable Ok button when we have file descriptor ready
+                dialog.enableOkButton(model.getEditMode() && selectedDriveFile);
+                break;
             case 'file':
                 dialog.enableOkButton(model.getEditMode() && (uploadFile || uploadFileName));
                 break;
@@ -551,10 +778,14 @@ define('io.ox/office/drawinglayer/view/imageutil',
         }
 
         // create the dialog
-        dialog = new Dialogs.ModalTabDialog({ title: gt('Insert Image'), width: 430, async: async });
+        dialog = new EditDialogs.ModalTabDialog({ title: gt('Insert Image'), width: 600, async: async });
 
         // close dialog automatically after losing edit rights
-        EditDialogMixin.call(dialog, app);
+        app.getView().closeDialogOnReadOnlyMode(dialog);
+
+        // prepare the 'Drive' tab panel
+        drivePanel = dialog.addTabPanel('drive', { label: gt('Drive') });
+        drivePanel.append(driveFilePicker);
 
         // prepare the 'Upload File' tab panel
         filePanel = dialog.addTabPanel('file', { label: gt('Upload image') });
@@ -582,6 +813,15 @@ define('io.ox/office/drawinglayer/view/imageutil',
             case 'url':
                 urlInput.focus();
                 break;
+            case 'drive':
+            	var folderList = $(driveFilePicker).find('.drive-folder-list');
+            	if (folderList.length > 0) {
+            		var selectedItem = $(folderList[0]).find('.selected');
+            		if (selectedItem.length > 0) {
+            			selectedItem[0].focus();
+            		}
+            	}
+                break;
             }
             checkOkButton();
         });
@@ -601,10 +841,11 @@ define('io.ox/office/drawinglayer/view/imageutil',
         // register a handler at the url input field
         urlInput.on('input', checkOkButton);
 
-        // show the dialog, and activate the 'Upload File' tab panel
-        promise = dialog.show(function () {
-            dialog.activateTabPanel('file');
-        });
+        // activate the Drive image picker panel
+        dialog.activateTabPanel('drive');
+
+        // show the dialog
+        promise = dialog.show();
 
         // process the result after the OK button has been pressed
         promise = promise.then(function () {
@@ -612,12 +853,17 @@ define('io.ox/office/drawinglayer/view/imageutil',
             var file = null,
                 form = null,
                 isFileUpload = false,
+                isDriveFile = false,
                 def = $.Deferred();
 
             // show progress indicator
             dialog.getHeader().find('h4').first().busy();
 
             switch (dialog.getActiveTabId()) {
+            case 'drive':
+                isDriveFile = true;
+                file = selectedDriveFile;
+                break;
             case 'file':
                 if (uploadFile) {
                     file = uploadFile;
@@ -634,7 +880,7 @@ define('io.ox/office/drawinglayer/view/imageutil',
             }
 
             if (_.isObject(file) || _.isString(file) || _.isObject(form)) {
-                def.resolve(file, isFileUpload, form);
+                def.resolve(file, isDriveFile, isFileUpload, form);
             } else {
                 def.reject();
             }
@@ -642,16 +888,21 @@ define('io.ox/office/drawinglayer/view/imageutil',
             return def.promise();
         });
 
-        promise = promise.then(function (file, isFileUpload, form) {
+        promise = promise.then(function (file, isDriveFile, isFileUpload, form) {
+            if (isDriveFile) { return Image.insertDriveImage(app, file); }
             return isFileUpload ? Image.insertFile(app, file, form) : Image.insertURL(app, file.trim());
         });
 
         return promise.always(function () {
-            // close dialog manually in asynchronous mode
+            // save user's last selected folder
+            var userRecentFolder = driveFilePicker.data('recent-folder');
+            if (userRecentFolder) { Settings.set('recentImageFolder', userRecentFolder).save(); }
+            //close dialog manually in asynchronous mode
             dialog.close();
             // remove all event listeners
             fileInput.off();
             filePicker.off();
+            driveFilePicker.off();
 
             dialog = null;
         });
