@@ -13,8 +13,9 @@
 
 define('io.ox/office/editframework/app/editcontroller',
     ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/app/basecontroller'
-    ], function (Utils, BaseController) {
+     'io.ox/office/baseframework/app/basecontroller',
+     'io.ox/office/editframework/utils/editconfig'
+    ], function (Utils, BaseController, Config) {
 
     'use strict';
 
@@ -32,8 +33,8 @@ define('io.ox/office/editframework/app/editcontroller',
      *  The OX Documents application that has created this controller instance.
      *
      * @param {Object} [initOptions]
-     *  A map with options controlling the behavior of this controller.
-     *  Supports all options supported by the base class BaseController.
+     *  Optional parameters. Supports all options supported by the base class
+     *  BaseController.
      */
     function EditController(app, initOptions) {
 
@@ -53,26 +54,47 @@ define('io.ox/office/editframework/app/editcontroller',
 
                 // enabled unless the application is in 'internal error' state
                 'app/valid': {
+                    parent: 'app/imported',
                     enable: function () { return app.getState() !== 'error'; }
                 },
 
-                // toggle the main side pane
-                'view/sidepane': {
-                    parent: 'app/valid',
-                    get: function () { return app.getView().isSidePaneVisible(); },
-                    set: function (state) { app.getView().toggleSidePane(state); },
-                    shortcut: { keyCode: 'F3', ctrlOrMeta: true, value: function (state) { return !state; } }
+                // toggle visibility of the tool bars
+                'view/toolbars/show': {
+                    // always enabled (also in read-only mode, and in application error state)
+                    get: function () { return view.getToolPane().isVisible(); },
+                    set: function (state) { view.getToolPane().toggle(state); }
+                },
+
+                // the identifier of the active tool bar
+                'view/toolbars/tab': {
+                    // always enabled (also in read-only mode, and in application error state)
+                    get: function () { return view.getToolBarTabs().getActiveTabId(); },
+                    set: function (tabId) { view.getToolBarTabs().activateTab(tabId); },
+                    focusTarget: function (sourceType) {
+                        // when activating via ENTER key, move focus into tool pane
+                        return (sourceType === 'keyboard') ? view.getToolPane() : null;
+                    }
+                },
+
+                'view/searchpane': {
+                    enable: 'app/valid',
+                    get: function () { return view.getSearchPane().isVisible(); },
+                    set: function (state) { view.getSearchPane().toggle(state); },
+                    focusTarget: function () { return view.getSearchPane(); },
+                    // shortcuts always enable the search pane (no toggling)
+                    shortcut: [{ keyCode: 'F', ctrlOrMeta: true, value: true }, { keyCode: 'F3', value: true }]
                 },
 
                 // document ---------------------------------------------------
 
                 'document/acquireedit': {
                     parent: 'app/valid',
-                    enable: function () { return !app.isLocked() && (app.getState() !== 'offline') && !model.getEditMode(); },
+                    enable: function () { return app.isAcquireEditRightsEnabled(); },
                     set: function () { app.acquireEditRights(); }
                 },
 
                 'document/reload': {
+                    parent: 'app/imported',
                     enable: function () { return app.getState() === 'error'; },
                     set: function () { app.reloadDocument(); }
                 },
@@ -83,12 +105,21 @@ define('io.ox/office/editframework/app/editcontroller',
                     enable: function () { return model.getEditMode(); }
                 },
 
+                'document/fullname': {
+                    get: function () { return app.getFullFileName(); }
+                },
+
                 'document/rename': {
                     parent: 'document/editable',
                     get: function () { return app.getShortFileName(); },
                     set: function (fileName) { return app.rename(fileName); }
                 },
 
+                'document/rename/dialog': {
+                    parent: 'document/rename',
+                    set: function () { return view.showRenameDialog(); }
+                },
+
                 'document/undo': {
                     parent: 'document/editable',
                     enable: function () { return view.isUndoAvailable(); },
@@ -98,8 +129,7 @@ define('io.ox/office/editframework/app/editcontroller',
                        // native undo/redo and automatic item execution after losing focus
                         { keyCode: 'Z', ctrlOrMeta: true, selector: '.app-pane' },
                         { keyCode: 'BACKSPACE', alt: true, selector: '.app-pane' }
-                    ],
-                    busy: true
+                    ]
                 },
 
                 'document/redo': {
@@ -111,74 +141,37 @@ define('io.ox/office/editframework/app/editcontroller',
                         // native undo/redo and automatic item execution after losing focus
                         { keyCode: 'Y', ctrlOrMeta: true, selector: '.app-pane' },
                         { keyCode: 'BACKSPACE', shift: true, alt: true, selector: '.app-pane' }
-                    ],
-                    busy: true
+                    ]
                 },
 
-                'document/saveas': {
-                    parent: 'document/editable',
-                    enable: function () { return model.getEditMode(); },
-                    set: function () { return app.copyDocument(); },
-                    focus: 'wait' // wait for the dialog before returning focus to application
-                },
-
-                'document/saveastemplate': {
-                    parent: 'document/editable',
-                    enable: function () { return model.getEditMode(); },
-                    set: function () { return app.copyDocument({asTemplate: true}); },
-                    focus: 'wait' // wait for the dialog before returning focus to application
+                'document/saveas/dialog': {
+                    parent: 'app/valid',
+                    set: function (type) { return view.showSaveAsDialog(type); }
                 },
 
                 'document/autosave': {
-                    parent: 'document/editable',
-                    enable: function () { return model.getEditMode(); },
-                    set: function () { app.autoSave(); }
+                    parent: 'app/valid',
+                    get: function () { return app.isAutoSaveEnabled(); },
+                    set: function (state) { app.toggleAutoSave(state); }
+                },
+
+                'document/users': {
+                    parent: 'app/valid',
+                    enable : function () {
+                        var activeUsers = _.reject(app.getActiveUsers(), function (user) {
+                            return user.durationOfInactivity >= 60;
+                        });
+                        return activeUsers.length > 1;
+                    },
+                    get : function () { return view.getCollaboratorPopup().isVisible(); },
+                    set : function (state) { view.toggleCollaboratorPopup(state); }
                 },
 
                 // debug mode -------------------------------------------------
 
                 'debug/enabled': {
-                    enable: function () { return Utils.DEBUG; }
-                },
-
-                'debug/toggle': {
-                    parent: 'debug/enabled',
-                    get: function () { return view.isDebugPaneVisible(); },
-                    set: function (state) { view.toggleDebugPane(state); }
-                },
-
-                'debug/highlight': {
-                    parent: 'debug/enabled',
-                    get: function () { return view.isDebugHighlight(); },
-                    set: function (state) { view.toggleDebugHighlight(state); }
-                },
-
-                'debug/clipboard': {
-                    parent: 'debug/enabled',
-                    get: function () { return view.isClipboardPaneVisible(); },
-                    set: function (state) { view.toggleClipboardPane(state); }
-                },
-
-                'debug/realtime/trigger': {
-                    parent: 'debug/enabled',
-                    set: function (type) { app.debugTriggerRealtimeEvent(type); }
-                },
-
-                'debug/realtime/sendinvalidoperation': {
-                    parent: 'debug/enabled',
-                    enable: function () { return model.getEditMode(); },
-                    set: function () { app.debugSendInvalidOperation(); }
-                },
-
-                'debug/operations/apply': {
-                    parent: 'debug/enabled',
-                    enable: function () { return model.getEditMode(); },
-                    set: function (operation) { model.applyOperations(operation); }
-                },
-
-                'debug/quit': {
-                    parent: 'debug/enabled',
-                    set: function (type) { app.debugQuit(type); }
+                    parent: 'app/valid',
+                    enable: function () { return Config.DEBUG; }
                 }
             };
 
@@ -192,17 +185,23 @@ define('io.ox/office/editframework/app/editcontroller',
         this.registerDefinitions(items);
 
         // initialization after construction (model not available at construction time)
-        app.on('docs:init', function () {
+        app.onInit(function () {
             model = app.getModel();
             view = app.getView();
         });
 
-        // update GUI after operations, or changed state of edit mode
-        app.on('docs:import:after', function () {
+        // update controller after operations, or changed state of edit mode
+        app.onImport(function () {
+            app.on('docs:state', function () { self.update(); });
             model.on('operations:after change:editmode', function () { self.update(); });
             model.getUndoManager().on('change:count', function () { self.update(); });
         });
 
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            app = initOptions = self = model = view = items = null;
+        });
+
     } // class EditController
 
     // exports ================================================================
