@@ -13,16 +13,48 @@
 define('io.ox/office/editframework/app/fileactions',
     ['io.ox/core/extensions',
      'io.ox/core/extPatterns/links',
-     'io.ox/office/tk/utils',
-     'io.ox/office/baseframework/app/extensionregistry',
      'io.ox/core/api/folder',
+     'io.ox/office/tk/utils',
+     'io.ox/office/tk/io',
+     'io.ox/office/baseframework/app/extensionregistry',
      'gettext!io.ox/office/files'
-    ], function (ext, links, Utils, ExtensionRegistry, FolderAPI, gt) {
+    ], function (ext, links, FolderAPI, Utils, IO, ExtensionRegistry, gt) {
 
     'use strict';
 
     // static private functions ===============================================
 
+    /**
+     * Returns whether the files in the folder described by the passed baton
+     * object are considered to be editable. The trash folder for example will
+     * be treated as read-only folder.
+     *
+     * @param {Object} baton
+     *  The baton object providing access to a folder.
+     *
+     * @returns {Boolean|jQuery.Promise}
+     *  A Boolean value specifying whether the folder is considered editable,
+     *  or a Promise that will resolve with a Boolean value, if the folder data
+     *  in the baton object is not available yet.
+     */
+    function isEditableFolder(baton) {
+
+        // no baton.app, no trash folder (e.g. main portal)
+        if (!baton.app) { return true; }
+
+        // check that the file is not located in the trash folder
+        return baton.app.folder.getData().then(function (data) {
+            return !FolderAPI.is('trash', data);
+        });
+    }
+
+    /**
+     * Disables the passed extension point for invalid files (all files names
+     * with a specific error suffix in their extension).
+     *
+     * @param {String} actionId
+     *  The identifier of the action to be disabled.
+     */
     function disableActionForInvalidFiles(actionId) {
         new links.Action(actionId, {
             id: 'disable_action',
@@ -43,7 +75,7 @@ define('io.ox/office/editframework/app/fileactions',
     // methods ----------------------------------------------------------------
 
     /**
-     * Creates all required actions and links in the OX Files application for a
+     * Creates all required actions and links in the OX Drive application for a
      * specific OX Documents application.
      *
      * @param {String} appBaseName
@@ -51,20 +83,19 @@ define('io.ox/office/editframework/app/fileactions',
      *  directory).
      *
      * @param {Object} options
-     *  A map with options controlling the behavior of the created extensions
-     *  in the Files application. The following options are supported:
+     *  Optional parameters:
      *  @param {String} options.newDocumentLabel
      *      The string shown for the 'Create new document' action link inserted
-     *      into the Files tool bar.
+     *      into the tool bar of OX Drive.
      *  @param {Number} options.newDocumentIndex
      *      The index of the 'Create new document' action link that will be
-     *      inserted into the Files tool bar. The 'Create new document' action
-     *      links of the various OX Document applications will be ordered
-     *      increasing by this index.
+     *      inserted into the tool bar of OX Drive. The 'Create new document'
+     *      action links of the various OX Document applications will be
+     *      ordered increasing by this index.
      */
     FileActions.create = function (appBaseName, options) {
 
-        var // root for extension points in Files application
+        var // root for extension points in OX Drive application
             ACTION_POINT = 'io.ox/files/actions/office/' + appBaseName,
             // the module name of the edit application
             MODULE_NAME = 'io.ox/office/' + appBaseName;
@@ -78,37 +109,14 @@ define('io.ox/office/editframework/app/fileactions',
             ox.launch(MODULE_NAME + '/main', launchOptions);
         }
 
-
-        /**
-         * calls data of the folder in the baton,
-         * returns false if the folder is the trash folder
-         * if data is not loaded yet it return a deferred/promise with the correct check inside
-         */
-        function isBatonEditable(baton) {
-            function isBatonDataEditable(data) {
-                return !FolderAPI.is('trash', data);//no new files in trash folders
-            }
-
-            if (baton.data.length) {
-                return isBatonDataEditable(baton.data);
-            } else {
-                if (!baton.app) {
-                    Utils.warn('could not check, if baton is in Trash folder, because baton has no app entry!', baton);
-                    return true;
-                } else {
-                    return baton.app.folder.getData().then(isBatonDataEditable);
-                }
-            }
-        }
-
         // initialization -----------------------------------------------------
 
-        // Creates a new empty document in the current folder of the Files application.
+        // Creates a new empty document in the current folder of the OX Drive application.
         if (ExtensionRegistry.supportsEditMode(appBaseName)) {
             new links.Action(ACTION_POINT + '/new', {
                 requires: function (e) {
                 	if (e.collection.has('create')) { // bug 32367
-	                    return isBatonEditable(e.baton);
+	                    return isEditableFolder(e.baton);
 	                }
                 },
                 action: function (baton) {
@@ -117,14 +125,16 @@ define('io.ox/office/editframework/app/fileactions',
             });
         }
 
-        // Loads an existing document from the current file in the Files application.
+        // Loads an existing document from the current file in the OX Drive application.
         // If the document requires conversion to a supported file format, creates the
         // converted file and loads it.
         new links.Action(ACTION_POINT + '/edit', {
             requires: function (e) {
                 var filename = e.context.filename;
-                if (e.collection.has('one', 'read', 'modify') && ExtensionRegistry.isEditable(filename, MODULE_NAME) && !ExtensionRegistry.isTemplate(filename, MODULE_NAME)) {
-                    return isBatonEditable(e.baton);
+                if (e.collection.has('one', 'read', 'modify') && ExtensionRegistry.isEditable(filename, MODULE_NAME) && !ExtensionRegistry.isTemplate(filename, MODULE_NAME) && ExtensionRegistry.isNative(filename, MODULE_NAME)) {
+                    // prefetch application source code
+                    IO.prefetchModuleSource(MODULE_NAME);
+                    return isEditableFolder(e.baton);
                 }
             },
             filter: function (obj) {
@@ -139,11 +149,11 @@ define('io.ox/office/editframework/app/fileactions',
             }
         });
 
-        new links.Action(ACTION_POINT + '/newfromtemplate', {
+        new links.Action(ACTION_POINT + '/new-from-template', {
             requires: function (e) {
                 var filename = e.context.filename;
                 if (e.collection.has('one', 'read') && !ExtensionRegistry.isError(filename) && ExtensionRegistry.isTemplate(filename, MODULE_NAME)) {
-                    return isBatonEditable(e.baton);
+                    return isEditableFolder(e.baton);
                 }
             },
             filter: function (obj) {
@@ -154,13 +164,13 @@ define('io.ox/office/editframework/app/fileactions',
             }
         });
 
-        // Creates a new document as copy of the current file in the Files application.
+        // Creates a new document as copy of the current file in the OX Drive application.
         if (ExtensionRegistry.supportsEditMode(appBaseName)) {
-            new links.Action(ACTION_POINT + '/editasnew', {
+            new links.Action(ACTION_POINT + '/edit-as-new', {
                 requires: function (e) {
                     var filename = e.context.filename;
                     if (e.collection.has('one', 'read') && !ExtensionRegistry.isError(filename) && ExtensionRegistry.isEditable(filename, MODULE_NAME) && !ExtensionRegistry.isTemplate(filename, MODULE_NAME)) {
-                        return isBatonEditable(e.baton);
+                        return isEditableFolder(e.baton);
                     }
                 },
                 filter: function (obj) {
@@ -168,15 +178,15 @@ define('io.ox/office/editframework/app/fileactions',
                 },
                 action: function (baton) {
                     var convert = ExtensionRegistry.isConvertible(baton.data.filename);
-                    launchApplication({ action: convert ? 'convert' : 'new', folderId: baton.data.folder_id, templateFile: baton.data });
+                    launchApplication({ action: convert ? 'convert' : 'new', folderId: baton.data.folder_id, templateFile: baton.data, preserveFileName: convert ? true : false });
                 }
             });
 
-            new links.Action(ACTION_POINT + '/edittemplate', {
+            new links.Action(ACTION_POINT + '/edit-template', {
                 requires: function (e) {
                     var filename = e.context.filename;
                     if (e.collection.has('one', 'read', 'modify') && !ExtensionRegistry.isError(filename) && ExtensionRegistry.isTemplate(filename, MODULE_NAME) && !ExtensionRegistry.isTemplateLocked(filename, MODULE_NAME) && ExtensionRegistry.isEditable(filename, MODULE_NAME)) {
-                        return isBatonEditable(e.baton);
+                        return isEditableFolder(e.baton);
                     }
                 },
                 filter: function (obj) {
@@ -192,7 +202,7 @@ define('io.ox/office/editframework/app/fileactions',
             });
         }
 
-        // An action link in the main tool bar of the Files application used
+        // An action link in the main tool bar of the OX Drive application used
         // to create a new empty document.
         new links.ActionLink('io.ox/files/links/toolbar/default', {
             index: 200 + Utils.getIntegerOption(options, 'newDocumentIndex', 0),
@@ -221,7 +231,7 @@ define('io.ox/office/editframework/app/fileactions',
             index: 'first',
             section: 'edit',
             label: gt('Edit as new'),
-            ref: ACTION_POINT + '/editasnew'
+            ref: ACTION_POINT + '/edit-as-new'
         }));
 
         ext.point('io.ox/files/links/inline').extend(new links.Link({
@@ -230,7 +240,7 @@ define('io.ox/office/editframework/app/fileactions',
             mobile: 'hi',
             index: 150,
             label: gt('New from template'),
-            ref: ACTION_POINT + '/newfromtemplate'
+            ref: ACTION_POINT + '/new-from-template'
         }));
 
         ext.point('io.ox/files/links/inline').extend(new links.Link({
@@ -240,7 +250,7 @@ define('io.ox/office/editframework/app/fileactions',
             index: 'first',
             section: 'edit',
             label: gt('Edit template'),
-            ref: ACTION_POINT + '/edittemplate'
+            ref: ACTION_POINT + '/edit-template'
         }));
 
     };
