@@ -17,10 +17,10 @@ define('io.ox/office/editframework/app/rtconnection',
     ['io.ox/core/event',
      'io.ox/realtime/groups',
      'io.ox/office/tk/utils',
-     'io.ox/office/tk/config',
      'io.ox/office/tk/logger',
      'io.ox/office/tk/object/triggerobject',
-    ], function (Events, RTGroups, Utils, Config, Logger, TriggerObject) {
+     'io.ox/office/editframework/utils/editconfig'
+    ], function (Events, RTGroups, Utils, Logger, TriggerObject, Config) {
 
     'use strict';
 
@@ -56,8 +56,7 @@ define('io.ox/office/editframework/app/rtconnection',
      *  The application containing this operations queue.
      *
      * @param {Object} [initOptions]
-     *  A map of options to be called for certain states of the realtime
-     *  connection.
+     *  Optional parameters:
      *  @param {Number} [initOptions.realTimeDelay=700]
      *      The duration in milliseconds the realtime framework will collect
      *      messages before sending them to the server.
@@ -127,7 +126,7 @@ define('io.ox/office/editframework/app/rtconnection',
             Utils.log(output);
 
             // check that all array elements are objects
-            if (_(actions).all(_.isObject)) {
+            if (_.all(actions, _.isObject)) {
                 try {
                     Utils.log('Last remote action received by RT-Update: ' + JSON.stringify(actions[actions.length - 1]));
                 } catch (ex) {
@@ -144,9 +143,9 @@ define('io.ox/office/editframework/app/rtconnection',
          * The eventHandler will be called with the received payload data
          */
         function registerRTEvents(eventNames) {
-            _(eventNames.split(/\s+/)).each(function (eventName) {
+            _.each(eventNames.split(/\s+/), function (eventName) {
                 rtGroup.on('receive', function (event, stanza) {
-                    _(stanza.getAll('office', eventName)).each(function (payload) {
+                    _.each(stanza.getAll('office', eventName), function (payload) {
                         RTConnection.log('RTConnection.eventHub.trigger: office:' + eventName);
                         eventHub.trigger(eventName, payload.data);
                     });
@@ -174,7 +173,7 @@ define('io.ox/office/editframework/app/rtconnection',
             var // current time
                 now = new Date();
 
-            _(pendingSendAcks).each(function (entry) {
+            _.each(pendingSendAcks, function (entry) {
                 var timeSpan;
 
                 if (entry.deferred && entry.deferred.state() === 'pending') {
@@ -296,62 +295,6 @@ define('io.ox/office/editframework/app/rtconnection',
             return defer.promise();
         }
 
-        /**
-         * Send a message request to the connected RT object,
-         * returning a deferred object that will be resolved,
-         * if the result data to this request will have arrived.
-         * @deprecated use sendQuery instead
-         *
-         * @returns {jQuery.Promise}
-         *  The Promise of a Deferred object that will be resolved if the
-         *  result for this request has arrived, or rejected otherwise.
-         */
-        function sendRequest(action, element, data) {
-
-            var // the result Deferred object
-                def = $.Deferred();
-
-            data = Utils.extendOptions(data, { uniqueId: _.uniqueId() });
-
-            // resolves the Deferred object with the 'action' result
-            function requestHandler(event, resultData) {
-                // stop listening to 'requestHandler' events if we received exactly
-                // one event with the message id we sent with the initial request
-                if (_.isObject(resultData)) {
-                    RTConnection.log('RTConnection.sendRequest: Answer for sendRequest arrived: ' + action + ' , expected uniqueId: ' + data.uniqueId + ', received uniqueId:' + resultData.uniqueId);
-                    if (_.isString(resultData.uniqueId) && (resultData.uniqueId === data.uniqueId)) {
-                        stopListening();
-                        RTConnection.log('RTConnection.sendRequest: resolve deferred with resultData');
-                        def.resolve(resultData);
-                    }
-                } else {
-                    RTConnection.error('RTConnection.sendRequest: Unexpected answer - deferred rejected');
-                    def.reject();
-                }
-            }
-
-            // stops listening to action events
-            function stopListening() {
-                eventHub.off(action, requestHandler);
-            }
-
-            // resolves the Deferred object with the 'action' result
-            eventHub.on(action, requestHandler);
-
-            // connect and listen to the 'action' result
-            RTConnection.log('RTConnection.sendRequest: Sending request: ' + action + ', uniqueId: ' + data.uniqueId);
-            send(action, element, data).fail(function () {
-                if (def.state() === 'pending') {
-                    def.reject({cause: 'timeout'});
-                }
-            });
-            RTConnection.log('RTConnection.sendRequest: Waiting for answer to request ' + action + ' ,uniqueId: ' + data.uniqueId);
-
-            // return a Promise with an abort() extension that disconnects the
-            // event listener, leaving the Deferred in pending state
-            return _.extend(def.promise(), { abort: stopListening });
-        }
-
         /**
          * Handles the response from the RT low-level code and extract the
          * data part from, providing it as result via the deferred/promise.
@@ -453,8 +396,14 @@ define('io.ox/office/editframework/app/rtconnection',
          *  The Promise of a Deferred object that will be resolved with the
          *  getActions results.
          */
-        this.connect = function () {
-            var connectDeferred = null;
+        this.connect = function (connectData) {
+            var connectDeferred = null,
+                // optional connect data
+                connectOptions = {expectWelcomeMessage: true};
+
+            if (connectData) {
+                _.extend(connectOptions, {additionalPayloads: [{element: 'connectData', namespace: 'office', data: connectData, verbatim: true }]});
+            }
 
             if (!rtGroup.isRTWorking()) {
                 // the server doesn't support real-time communication, must be
@@ -463,7 +412,8 @@ define('io.ox/office/editframework/app/rtconnection',
             }
 
             RTConnection.log('RTConnection.connect called');
-            connectDeferred = handleResponse(rtGroup.join({expectWelcomeMessage: true}));
+
+            connectDeferred = handleResponse(rtGroup.join(connectOptions));
             // Remember *connect* deferred to be able to reject them if user
             // closes document while we are still connecting to the realtime group!
             return app.createAbortablePromise(connectDeferred);
@@ -499,20 +449,22 @@ define('io.ox/office/editframework/app/rtconnection',
         };
 
         /**
-         * Sends user data update to the server. This user update data will broadcasted
-         * to all collaborating clients.
+         * Sends user data update to the server. This user update data will
+         * broadcasted to all collaborating clients.
          *
          * @param {String} action
          *  Action name string to be handled by the server.
+         *
          * @param {Object} userData
          *  An object containing relevant additional user data on the document.
-         *  @poram userData.selection
+         *  @param {Object} userData.selection
          *      Contains a 'recent' user selection on a document. This will be
          *      used for showing user cursors of multiple collaborators while
          *      editing on documents.
+         *
          * @returns {jQuery.Promise}
          */
-        this.updateUserData = Config.showRemoteSelections() ? function (action, userData) {
+        this.updateUserData = Config.SHOW_REMOTE_SELECTIONS ? function (action, userData) {
             RTConnection.log('RTConnection.updateUserData called', userData);
             return send(action, 'state', userData);
         } : function () { return $.when(); };
@@ -550,30 +502,6 @@ define('io.ox/office/editframework/app/rtconnection',
             return handleGenericResponse(sendQuery('flushdocument'), 20000);
         };
 
-        /**
-         * Add the given resource id to the connected RT object
-         *
-         * @returns {jQuery.Promise}
-         *  The Promise of a Deferred object that will be resolved with the
-         *  result data of the request.
-         */
-        this.addResourceId = function (resourceId) {
-            RTConnection.log('RTConnection.addResourceId called');
-            return sendRequest('addresource', 'resource', { resourceid: resourceId });
-        };
-
-        /**
-         * Get the resource id of the connected RT object for the specified document
-         *
-         * @returns {jQuery.Promise}
-         *  The Promise of a Deferred object that will be resolved with the
-         *  result data of the request.
-         */
-        this.getDocumentId = function (data) {
-            RTConnection.log('RTConnection.getDocumentId called');
-            return sendRequest('getdocument', 'document', Utils.extendOptions(data, { returntype: 'resourceid' }));
-        };
-
         /**
          * Send our wish to acquire the edit rights to the connected RT object
          *
@@ -602,6 +530,19 @@ define('io.ox/office/editframework/app/rtconnection',
             return send('canloseeditrights', 'state', { osn: app.getModel().getOperationStateNumber() });
         };
 
+        /**
+         * Sends client log message to the server so it can be written to the
+         * server side log files for later debugging.
+         *
+         * @return {jQuery.Promise}
+         *  The Promise of a Deferred object that will be resolved when the
+         *  ACK of the internal send arrives.
+         */
+        this.sendLogMessage = function (data) {
+            RTConnection.log('RTConnection.sendLogMessage called');
+            return send('logMessage', 'state', { message: data });
+        };
+
         /**
          * Close the document at the connected RT object
          *
@@ -609,9 +550,14 @@ define('io.ox/office/editframework/app/rtconnection',
          *  The Promise of a Deferred object that will be resolved with the
          *  closing document status of the request.
          */
-        this.closeDocument = function () {
+        this.closeDocument = function (recentFileData) {
+            var leaveOptions = {expectSignOffMessage: true};
+
             RTConnection.log('RTConnection.closeDocument called');
-            return handleResponse(rtGroup.leave({ expectSignOffMessage: true }), 20000);
+            if (recentFileData) {
+                _.extend(leaveOptions, {additionalPayloads: [{element: 'recentfile', namespace: 'office', data: recentFileData, verbatim: true }]});
+            }
+            return handleResponse(rtGroup.leave(leaveOptions), 20000);
         };
 
         // initialization -----------------------------------------------------
@@ -623,7 +569,7 @@ define('io.ox/office/editframework/app/rtconnection',
 
         // read traceable actions from page URL
         if (_.url.hash('office:trace')) {
-            _(_.url.hash('office:trace').split(/\s*,\s*/)).each(function (action) {
+            _.each(_.url.hash('office:trace').split(/\s*,\s*/), function (action) {
                 traceActions[action.toLowerCase()] = true;
             });
         }
@@ -678,14 +624,19 @@ define('io.ox/office/editframework/app/rtconnection',
     // global initialization ==================================================
 
     // whether to show console output of this RTConnection class
-    RTConnection.debug = Config.isRealtimeDebug();
+    RTConnection.debug = Config.DEBUG && Config.getDebugState('realtimedebugging', 'office:log-realtime');
 
     // extend statically with logging methods
     Logger.extend(RTConnection, { enable: RTConnection.debug, prefix: 'RT' });
 
+    // add time stamps for RT debugging by default
+    if (RTConnection.debug && Config.getUrlFlag('office:log-timestamp', true)) {
+        Logger.LOG_TIMESTAMP = true;
+    }
+
     // enable DOM console for RT debugging by default
-    if (RTConnection.debug) {
-        Utils.enableDOMConsole();
+    if (RTConnection.debug && Config.getUrlFlag('office:debug-console', true)) {
+        Logger.enableDOMConsole();
     }
 
     // enable logging for real-time core
