@@ -13,10 +13,9 @@
 
 define('io.ox/office/editframework/model/editmodel',
     ['io.ox/office/tk/utils',
-     'io.ox/office/tk/config',
      'io.ox/office/baseframework/model/basemodel',
      'io.ox/office/editframework/model/operationsgenerator'
-    ], function (Utils, Config, BaseModel, OperationsGenerator) {
+    ], function (Utils, BaseModel, OperationsGenerator) {
 
     'use strict';
 
@@ -81,6 +80,10 @@ define('io.ox/office/editframework/model/editmodel',
             // maps all operation names to operation handler functions
             operationHandlers = {},
 
+            // Deferred object for current asynchronous operations, will remain
+            // in resolved state as long as no operations are applied
+            operationsDef = $.when(),
+
             // whether operations are currently applied (prevent recursive calls)
             processingOperations = false,
 
@@ -93,7 +96,10 @@ define('io.ox/office/editframework/model/editmodel',
             // a counter for the operations, that adds all single operations, taking care of merged
             // operations. So this number must be equal on every client, independent from merge
             // processes on client or server side.
-            operationStateNumber = -1;
+            operationStateNumber = -1,
+
+            // collecting the last two operations in cache
+            operationsCache = { previous: null, current: null };
 
         // base constructors --------------------------------------------------
 
@@ -233,7 +239,9 @@ define('io.ox/office/editframework/model/editmodel',
                 // the function that executes the operation
                 operationHandler = null,
                 // success value of operation
-                success = false;
+                success = false,
+                // a possible log message
+                message = null;
 
             // check operation
             if (!_.isObject(operation) || !_.isString(operation.name)) {
@@ -250,10 +258,16 @@ define('io.ox/office/editframework/model/editmodel',
             // get and check operation handler
             operationHandler = operationHandlers[operation.name];
             if (!_.isFunction(operationHandler)) {
-                Utils.error('EditModel.applyOperation(): invalid operation name "' + operation.name + '".');
+                message = 'EditModel.applyOperation(): invalid operation name "' + operation.name + '".';
+                Utils.error(message);
+                app.sendLogMessage(message);
                 return false;
             }
 
+            // Caching the last two operations
+            operationsCache.previous = operationsCache.current;
+            operationsCache.current = operation;
+
             // execute the operation handler (unless the operation scope is
             // 'filter' for operations intended to be executed in the filter
             // component only, or the filter callback function returns false)
@@ -266,7 +280,9 @@ define('io.ox/office/editframework/model/editmodel',
             if (success) {
                 totalOperations += 1;
             } else {
-                Utils.error('EditModel.applyOperation(): operation ' + (totalOperations + 1) + ' failed: ' + JSON.stringify(operation));
+                message = 'EditModel.applyOperation(): ' + (totalOperations + 1) + (external ? ' E ' : ' I ') + JSON.stringify(operation) + ' failed';
+                Utils.error(message);
+                app.sendLogMessage(message);
             }
             return success;
         }
@@ -304,7 +320,7 @@ define('io.ox/office/editframework/model/editmodel',
             if (beginProcessOperations(operations, external)) {
                 // apply all operations at once, group into an undo action
                 undoManager.enterUndoGroup(function () {
-                    success = _(operations).all(function (operation) {
+                    success = _.all(operations, function (operation) {
                         return applyOperation(operation, external, filter);
                     });
                 });
@@ -327,9 +343,6 @@ define('io.ox/office/editframework/model/editmodel',
          * @param {Boolean} external
          *  Whether the operation has been received from the server.
          *
-         * @param {Number} delay
-         *  The delay time between chunks of operations.
-         *
          * @param {Function} filter
          *  A predicate function that specifies whether to invoke the operation
          *  callback handler.
@@ -340,7 +353,7 @@ define('io.ox/office/editframework/model/editmodel',
          *  after applying an operation has failed. The Promise regularly sends
          *  progress updates while applying the operations.
          */
-        function applyOperationsAsync(operations, external, delay, filter) {
+        function applyOperationsAsync(operations, external, filter) {
 
             var // the resulting Promise
                 promise = null,
@@ -352,10 +365,12 @@ define('io.ox/office/editframework/model/editmodel',
 
             // enter operations mode (prevent recursive calls)
             if (beginProcessOperations(operations, external)) {
+
                 // apply all operations in a background task, group into an undo action
                 promise = undoManager.enterUndoGroup(function () {
+
                     // return the Deferred to enterUndoGroup() to defer the open undo group
-                    return app.processArrayDelayed(function (chunk) {
+                    return app.iterateArraySliced(operations, function (operation) {
 
                         // if the operation's origin is not an external client and
                         // we don't have edit rights then we skip the operations silently
@@ -364,14 +379,13 @@ define('io.ox/office/editframework/model/editmodel',
                             return Utils.BREAK;
                         }
 
-                        // apply the operations in the current chunk synchronously,
                         // break the loop and reject the promise if an operation has failed
-                        return _(chunk).all(function (operation) {
-                            return applyOperation(operation, external, filter);
-                        }) ? undefined : $.Deferred().reject();
+                        if (!applyOperation(operation, external, filter)) {
+                            return $.Deferred().reject();
+                        }
+                    });
+                });
 
-                    }, operations, { chunkLength: 50, delay: delay });
-                }, undefined, { async: true });
             } else {
                 // currently applying operations (called recursively)
                 promise = $.Deferred().reject();
@@ -447,7 +461,7 @@ define('io.ox/office/editframework/model/editmodel',
          *  Whether registering all operations for sending was successful.
          */
         this.sendOperations = function (operations) {
-            return applyOperationsSync(_(operations).getArray(), false, $.noop);
+            return applyOperationsSync(_.getArray(operations), false, $.noop);
         };
 
         /**
@@ -460,7 +474,7 @@ define('io.ox/office/editframework/model/editmodel',
          *  the document model.
          *
          * @param {Object} [options]
-         *  A map with additional options. The following options are supported:
+         *  Optional parameters:
          *  @param {Function} [options.filter]
          *      A predicate function that will be invoked for each operation
          *      contained in the passed actions. Receives the operation as
@@ -471,7 +485,22 @@ define('io.ox/office/editframework/model/editmodel',
          *  Whether applying all operations was successful.
          */
         this.applyOperations = function (operations, options) {
-            return applyOperationsSync(_(operations).getArray(), false, getFilterCallback(options));
+
+            var // whether the operations have been applied successfully
+                success = false;
+
+            // create an new Deferred object which remains in pending state as
+            // long as the operations will be applied
+            operationsDef = $.Deferred();
+
+            // apply the operations synchronously
+            success = applyOperationsSync(_.getArray(operations), false, getFilterCallback(options));
+
+            // resolve or reject the Deferred object, this will invoke all
+            // callbacks that have been registered inside the operation handlers
+            operationsDef[success ? 'resolve' : 'reject']();
+
+            return success;
         };
 
         /**
@@ -484,7 +513,7 @@ define('io.ox/office/editframework/model/editmodel',
          *  be applied at the document model.
          *
          * @param {Object} [options]
-         *  A map with additional options. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.external=false]
          *      If set to true, the actions have been received from an external
          *      sender. Otherwise, the actions have been generated by the
@@ -494,9 +523,6 @@ define('io.ox/office/editframework/model/editmodel',
          *      browser timeout loop. The Promise returned by this method will
          *      be resolved or rejected after applying all actions, and will be
          *      notified about the progress of the operation.
-         *  @param {Number} [options.delay=0]
-         *      In asynchronous mode, the delay time between actions and
-         *      between chunks of multiple operations in a single action.
          *  @param {Function} [options.filter]
          *      A predicate function that will be invoked for each operation
          *      contained in the passed actions. Receives the operation as
@@ -516,16 +542,12 @@ define('io.ox/office/editframework/model/editmodel',
          */
         this.applyActions = function (actions, options) {
 
-            var // the result Deferred object for asynchronous mode
-                def = null,
-                // the timer processing the array asynchronously
+            var // the timer processing the array asynchronously
                 timer = null,
                 // whether the actions have been received from the server
                 external = Utils.getBooleanOption(options, 'external', false),
                 // whether to execute asynchronously
                 async = Utils.getBooleanOption(options, 'async', false),
-                // delay time for asynchronous mode
-                delay = Utils.getIntegerOption(options, 'delay', 0, 0),
                 // invocation filter callback
                 filter = getFilterCallback(options),
                 // total number of operations
@@ -533,7 +555,11 @@ define('io.ox/office/editframework/model/editmodel',
                 // number of operations already applied
                 operationIndex = 0,
                 // the promise of the inner loop processing operations of an action
-                innerPromise = null;
+                innerPromise = null,
+                // resulting promise returned by this method
+                resultPromise = null,
+                // whether the operations have been applied successfully in synchronous mode
+                success = false;
 
             // convert parameter to an array, filter for actions containing
             // at least one operation, get total number of operations
@@ -544,53 +570,79 @@ define('io.ox/office/editframework/model/editmodel',
             }).value();
 
             // no operations: early exit
-            if (operationCount === 0) {
-                return async ? $.when() : true;
-            }
+            if (operationCount === 0) { return $.when(); }
 
-            // apply actions, get resulting Deferred object
+            // create an new Deferred object which remains in pending state as
+            // long as the operations will be applied
+            operationsDef = $.Deferred();
+
+            // apply actions, get resulting promise
             if (async) {
 
-                // create an own Deferred object, for custom progress calculation
-                def = $.Deferred();
-
                 // apply the actions in a browser timeout loop
-                timer = app.processArrayDelayed(function (action) {
-
-                    var // the operations contained in the action (action is passed as one-element array)
-                        operations = action[0].operations;
+                timer = app.iterateArraySliced(actions, function (action) {
 
                     // apply the operations and calculate the progress
-                    innerPromise = applyOperationsAsync(operations, external, delay, filter)
+                    innerPromise = applyOperationsAsync(action.operations, external, filter)
                         .progress(function (progress) {
-                            def.notify((operationIndex + operations.length * progress) / operationCount);
+                            operationsDef.notify((operationIndex + action.operations.length * progress) / operationCount);
                         })
                         .done(function () {
-                            operationIndex += operations.length;
+                            operationIndex += action.operations.length;
                         });
 
                     // clean up after finishing the loop
                     return innerPromise.always(function () { innerPromise = null; });
-
-                }, actions, { chunkLength: 1, delay: delay });
+                });
 
                 // forward the result of the timer to the own Deferred object
-                timer.done(function () { def.notify(1).resolve(); }).fail(_.bind(def.reject, def));
+                timer.done(function () { operationsDef.notify(1).resolve(); }).fail(_.bind(operationsDef.reject, operationsDef));
 
-                // extend the promise with an abort() method
-                return app.createAbortablePromise(def, function () {
+                // create an abortable promise from the Deferred object
+                resultPromise = app.createAbortablePromise(operationsDef, function () {
                     // abort the outer loop processing the actions
                     timer.abort();
                     // abort the inner loop processing the operations of an action
                     if (innerPromise) { innerPromise.abort(); }
                 });
+
+            } else {
+
+                // synchronous mode: apply all actions at once, but bail out on error
+                success = _.all(actions, function (action) {
+                    // applyOperationsSync() creates undo groups by itself
+                    return applyOperationsSync(action.operations, external, filter);
+                });
+
+                // resolve or reject the Deferred object, this will invoke all
+                // callbacks that have been registered inside the operation handlers
+                resultPromise = operationsDef[success ? 'resolve' : 'reject']().promise();
             }
 
-            // apply all actions at once, but bail out on error
-            return _(actions).all(function (action) {
-                // applyOperationsSync() creates undo groups by itself
-                return applyOperationsSync(action.operations, external, filter);
-            }) ? $.when() : $.Deferred().reject();
+            return resultPromise;
+        };
+
+        /**
+         * Returns the promise of a Deferred object representing the state of
+         * operations currently applied. If in pending state, this model
+         * currently applies operations (synchronously or asynchronously). If
+         * in resolved state, the last operations have been applied
+         * successfully. If in rejected state, the last operations have not
+         * been applied successfully.
+         *
+         * @attention
+         *  The promise MUST NOT be cached externally. Its solely purpose is to
+         *  register done/fail callbacks from operation handlers or other code
+         *  depending on operations. Whenever new operations will be applied, a
+         *  new promise will be created representing the new operations (the
+         *  old promises will remain in resolved/rejected state forever).
+         *
+         * @returns {jQuery.Promise}
+         *  The promise of a Deferred object representing the currently or last
+         *  applied operations.
+         */
+        this.getOperationsPromise = function () {
+            return operationsDef.promise();
         };
 
         /**
@@ -656,16 +708,16 @@ define('io.ox/office/editframework/model/editmodel',
         };
 
         /**
-         * Returns the style sheet container for the passed style family.
+         * Returns the style collection for the passed style family.
          *
          * @param {String} family
          *  The name of the attribute family.
          *
          * @returns {StyleSheets}
-         *  The style sheets container registered for the passed style family.
+         *  The style collection registered for the passed style family.
          */
-        this.getStyleSheets = function (styleFamily) {
-            return documentStyles.getStyleSheets(styleFamily);
+        this.getStyleCollection = function (styleFamily) {
+            return documentStyles.getStyleCollection(styleFamily);
         };
 
         /**
@@ -702,6 +754,38 @@ define('io.ox/office/editframework/model/editmodel',
             return operationStateNumber;
         };
 
+        /**
+         * Sets the operation state number of the document. Used when empty doc is fast displayed
+         * @param {Number} osn - new osn number
+         */
+        this.setOperationStateNumber = function (osn) {
+            operationStateNumber = osn;
+            this.trigger('change:osn', osn); // update osn value in operations pane
+        };
+
+        /**
+         * Returns the operation cache with the last two operations.
+         *
+         * @returns {Object}
+         *  An object containing the last two operations in the properties
+         *  last and current.
+         */
+        this.getOperationsCache = function () {
+            return operationsCache;
+        };
+
+        /**
+         * Returns the previous operation saved in the operation cache.
+         * This is required for performance reasons. So no copy is returned
+         * but the original operation object.
+         *
+         * @returns {Object}
+         *  An object containing the previous operation.
+         */
+        this.getPreviousOperation = function () {
+            return operationsCache.previous;
+        };
+
         // operation handlers -------------------------------------------------
 
         // setting default attributes is not undoable
@@ -712,7 +796,7 @@ define('io.ox/office/editframework/model/editmodel',
 
             // set the global document attributes
             if (_.isObject(attributes.document)) {
-                documentStyles.setAttributes(attributes.document);
+                documentStyles.setDocumentAttributes(attributes.document);
                 delete attributes.document;
             }
 
@@ -731,7 +815,7 @@ define('io.ox/office/editframework/model/editmodel',
         this.registerOperationHandler(OperationsGenerator.INSERT_STYLESHEET, function (operation) {
 
             var // target style sheet container
-                styleSheets = documentStyles.getStyleSheets(operation.type),
+                styleSheets = documentStyles.getStyleCollection(operation.type),
                 // passed attributes from operation
                 attributes = Utils.getObjectOption(operation, 'attrs', {});
 
@@ -753,7 +837,7 @@ define('io.ox/office/editframework/model/editmodel',
         this.registerOperationHandler(OperationsGenerator.DELETE_STYLESHEET, function (operation) {
 
             var // target style sheet container
-                styleSheets = documentStyles.getStyleSheets(operation.type);
+                styleSheets = documentStyles.getStyleCollection(operation.type);
 
             if (!_.isObject(styleSheets)) {
                 Utils.warn('EditModel.deleteStyleSheet(): invalid style family: "' + operation.type + '"');
