@@ -17,9 +17,8 @@ define('io.ox/office/portal/action/recentsactions',
     ['io.ox/core/extensions',
      'io.ox/core/extPatterns/links',
      'io.ox/core/extPatterns/actions',
-     'io.ox/office/portal/utils',
-     'io.ox/office/tk/keycodes'],
-function (Ext, Links, CoreActions, Utils, KeyCodes) {
+     'io.ox/office/portal/portalutils'],
+function (Ext, Links, CoreActions, PortalUtils) {
 
     'use strict';
 
@@ -33,7 +32,7 @@ function (Ext, Links, CoreActions, Utils, KeyCodes) {
         new Links.Action(recentDocumentExtension + '/open', {
 
             action: function (baton) {
-                var appModuleName = Utils.getModuleName(baton.data.filename);
+                var appModuleName = PortalUtils.getModuleName(baton.data.filename);
                 ox.launch(appModuleName + '/main', { action: 'load', file: baton.data });
             }
 
@@ -51,27 +50,23 @@ function (Ext, Links, CoreActions, Utils, KeyCodes) {
                         'data-placement': 'top',
                         'title': baton.data.filename,
                         'tabindex': 0
-                    })
-                    .tooltip(),
+                    }),
                     metadata = $('<div>').addClass('document-metadata'),
-                    name = $('<div>').addClass('document-name col-xs-8').html(baton.data.filename),
-                    date = $('<div>').addClass('document-date col-xs-4').html(Utils.formatDate(baton.data.last_modified ? baton.data.last_modified : baton.data.last_opened));
+                    name = $('<div>').addClass('document-name col-xs-8'),
+                    date = $('<div>').addClass('document-date col-xs-4')
+                        .text(PortalUtils.formatDate(baton.data.last_modified ? baton.data.last_modified : baton.data.last_opened));
 
-                link.append(
-                    metadata.append(
-                        name.prepend(Utils.createDocumentIcon(baton.data.filename)), date
-                    )
-                );
+                // enable Bootstrap tooltip except for touch devices, since it is buggy (see https://github.com/twbs/bootstrap/issues/6232)
+                if (!Modernizr.touch) { link.tooltip(); }
+
+                name.append(PortalUtils.createDocumentIcon(baton.data.filename), $('<div>').text(baton.data.filename));
+                link.append(metadata.append(name, date));
 
                 link.click(function () {
                     CoreActions.invoke(recentDocumentExtension + '/open', this, baton);
                 });
 
-                link.on('keydown', function (event) {
-                    if (KeyCodes.matchKeyCode(event, 'ENTER') || KeyCodes.matchKeyCode(event, 'SPACE')) { $(this).trigger('click'); }
-                    if (KeyCodes.matchKeyCode(event, 'LEFT_ARROW') || KeyCodes.matchKeyCode(event, 'UP_ARROW')) { Utils.moveFocus($(this), 'prev'); }
-                    if (KeyCodes.matchKeyCode(event, 'RIGHT_ARROW') || KeyCodes.matchKeyCode(event, 'DOWN_ARROW')) { Utils.moveFocus($(this), 'next'); }
-                });
+                link.on('keydown', PortalUtils.keydownHandler);
 
                 this.append(link);
             }
