@@ -13,12 +13,12 @@
 
 define('io.ox/office/portal/view/documentview',
     ['gettext!io.ox/office/portal',
-     'io.ox/office/portal/utils',
+     'io.ox/office/portal/portalutils',
      'io.ox/core/extensions',
      'io.ox/office/portal/action/recentsactions',
      'io.ox/core/capabilities'
     ],
-function (gt, Utils, Ext, RecentsActions) {
+function (gt, PortalUtils, Ext, RecentsActions) {
 
     'use strict';
 
@@ -28,12 +28,6 @@ function (gt, Utils, Ext, RecentsActions) {
 
         className: 'office-portal-document',
 
-        documentNode: null,
-
-        events: {
-
-        },
-
         initialize: function () {
 
             var // the document view object
@@ -41,9 +35,7 @@ function (gt, Utils, Ext, RecentsActions) {
                 // the handler for Window resize events
                 resizeHandler = _.debounce(function () { self.showOptimalCount(self, { isResize: true }); }, 250);
 
-            this.listenTo(this.collection, 'reset', this.render);
             this.listenTo(this.collection, 'destroy', this.remove);
-            this.render(Utils.getActiveApp(this.model));
 
             // Registering resize handler for the recent document list
             $(window).on('resize', resizeHandler);
@@ -65,8 +57,8 @@ function (gt, Utils, Ext, RecentsActions) {
          */
         render: function (appBaseName) {
 
-            var documentCollection = this.collection,
-                self = this,
+            var self = this,
+                documentCollection = self.collection,
                 emptyNotificationNode = $('<p>').addClass('office-portal-notification bg-info').text(gt('You have no recent documents.'));
 
             if (documentCollection.length === 0) {
@@ -79,17 +71,17 @@ function (gt, Utils, Ext, RecentsActions) {
 
             _.each(documentCollection.models, function (model) {
                 var // the application handling files with the specified name
-                    fileAppName = Utils.getAppBaseName(model.get('filename')),
-                    // the extension of files
+                    fileAppName = PortalUtils.getAppBaseName(model.get('filename')),
+                // the extension of files
                     errorSuffix = '_ox';
                 // quit if the current visible application is not the file application
                 // or if the file name ends with the error extension '_ox'
-                if ((appBaseName !== fileAppName) || Utils.stringEndsWith(model.get('filename'), errorSuffix)) { return; }
+                if ((appBaseName !== fileAppName) || PortalUtils.stringEndsWith(model.get('filename'), errorSuffix)) { return; }
                 // draw recent document items and bind actions  through extension invoke
                 RecentsActions.documentPoint.invoke('draw', self.$el, Ext.Baton({ data : model.attributes }));
             });
 
-            this.showOptimalCount(self);
+            self.showOptimalCount(self);
 
             // if no recent file item is drawn to the DOM after app type filter, show empty notification,
             // remove f6-target so that the recent file list is skipped while iterating with keyboard.
@@ -106,7 +98,7 @@ function (gt, Utils, Ext, RecentsActions) {
         showOptimalCount: function (self, options) {
 
             var // whether this list creation is triggered by a resize event
-                isResize = Utils.getBooleanOption(options, 'isResize', false),
+                isResize = PortalUtils.getBooleanOption(options, 'isResize', false),
                 // the node element with class 'office-portal-recents-list'
                 list = self.$el,
                 // the height in pixel of the complete recent list
@@ -118,7 +110,7 @@ function (gt, Utils, Ext, RecentsActions) {
                 // the length of the collection of recent documents
                 collectionLength = documents.length,
                 // whether the view is displayed in mode for small devices (width < 1024px)
-                isSmallDevice = (_.device('small') || _.device('medium')),
+                isSmallDevice = PortalUtils.COMPACT_DEVICE,
                 // the height in pixel of the header line
                 headlineHeight = list.parent().find('h2').outerHeight(),
                 // calculate optimal recent documents to show
