@@ -13,11 +13,11 @@
 
 define('io.ox/office/portal/view/toolbarview',
     ['gettext!io.ox/office/portal',
-        'io.ox/office/portal/utils',
+        'io.ox/office/portal/portalutils',
         'io.ox/office/portal/action/toolbaractionstext',
         'io.ox/office/portal/action/toolbaractionsspreadsheet',
         'io.ox/core/extensions'
-    ], function (gt, Utils, ToolbarActionsText, ToolbarActionsSpreadsheet, Ext) {
+    ], function (gt, PortalUtils, ToolbarActionsText, ToolbarActionsSpreadsheet, Ext) {
 
         'use strict';
 
@@ -33,7 +33,7 @@ define('io.ox/office/portal/view/toolbarview',
 
             initialize: function () {
 
-                this.render(Utils.getActiveApp(this.model));
+                this.render(PortalUtils.getActiveApp(this.model));
 
             },
 
@@ -44,7 +44,7 @@ define('io.ox/office/portal/view/toolbarview',
                 // create portal toolbar markup
                 var toolbar = $('<div>').addClass('office-portal-toolbar f6-target');
 
-                if (_.browser.Android && appBaseName === 'text') { toolbar.addClass('reduced-toolbar-functionality'); }
+                if ((_.browser.Android) && (appBaseName === 'text') && !PortalUtils.supportedChromeVersionOnAndroidForText()) { toolbar.addClass('reduced-toolbar-functionality'); }
 
                 // append to root element of this view
                 this.$el.prepend(toolbar);
