@@ -30,19 +30,25 @@ define('io.ox/office/presentation/app/application',
      *
      * @extends EditApplication
      *
-     * @param {Object} [appOptions]
-     *  A map of static application options, that have been passed to the
-     *  static method BaseApplication.createLauncher().
+     * @param {Object} launchOptions
+     *  All options passed to the core launcher (the ox.launch() method).
      *
-     * @param {Object} [launchOptions]
-     *  A map of options to control the properties of the application. Supports
-     *  all options supported by the base class EditApplication.
+     * @param {Object} [appOptions]
+     *  Static application options that have been passed to the static method
+     *  BaseApplication.createLauncher().
      */
-    var PresentationApplication = EditApplication.extend({ constructor: function (appOptions, launchOptions) {
+    var PresentationApplication = EditApplication.extend({ constructor: function (launchOptions, appOptions) {
 
         // base constructor ---------------------------------------------------
 
-        EditApplication.call(this, PresentationModel, PresentationView, PresentationController, appOptions, launchOptions);
+        EditApplication.call(this, PresentationModel, PresentationView, PresentationController, launchOptions, appOptions);
+
+        // initialization -----------------------------------------------------
+
+        // destroy all class members
+        this.on('docs:destroy', function () {
+            launchOptions = appOptions = null;
+        });
 
     }}); // class PresentationApplication
 
