@@ -28,8 +28,4 @@ define('io.ox/office/preview/app/fileactions', ['io.ox/office/preview/app/action
     // disable default 'Open' action defined by OX Files
     ActionsHelper.disableActionForViewable('io.ox/files/actions/open');
 
-    // exports ================================================================
-
-    return;
-
 });
