@@ -29,8 +29,4 @@ define('io.ox/office/preview/app/mailactions', ['io.ox/office/preview/app/action
     ActionsHelper.disableActionForViewable('io.ox/mail/actions/preview-attachment');
     ActionsHelper.disableActionForViewable('io.ox/mail/actions/open-attachment');
 
-    // exports ================================================================
-
-    return;
-
 });
