@@ -20,7 +20,7 @@ define('io.ox/office/preview/app/taskactions', ['io.ox/office/preview/app/action
     // global initialization ==================================================
 
     // create the action to view the file attachment in OX Viewer
-    ActionsHelper.createViewerAction(ACTION_ID, 'task');
+    ActionsHelper.createViewerAction(ACTION_ID, 'module');
 
     // create a link in the attachment drop-down menu
     ActionsHelper.createViewerLink('io.ox/tasks/attachment/links', ACTION_ID, { index: 250 });
@@ -29,8 +29,4 @@ define('io.ox/office/preview/app/taskactions', ['io.ox/office/preview/app/action
     ActionsHelper.disableActionForViewable('io.ox/tasks/actions/preview-attachment');
     ActionsHelper.disableActionForViewable('io.ox/tasks/actions/open-attachment');
 
-    // exports ================================================================
-
-    return;
-
 });
