@@ -13,25 +13,23 @@
 
 define('io.ox/office/preview/view/view',
     ['io.ox/office/tk/utils',
+     'io.ox/office/baseframework/app/extensionregistry',
      'io.ox/office/baseframework/view/baseview',
-     'io.ox/office/baseframework/view/basecontrols',
-     'io.ox/office/baseframework/view/pane',
      'io.ox/office/baseframework/view/sidepane',
      'io.ox/office/baseframework/view/component',
      'io.ox/office/baseframework/view/toolbox',
+     'io.ox/office/preview/view/toppane',
      'io.ox/office/preview/view/controls',
      'io.ox/office/preview/view/pagegroup',
      'io.ox/office/preview/view/pageloader',
+     'io.ox/office/preview/view/nodetouch',
      'gettext!io.ox/office/preview',
      'less!io.ox/office/preview/view/style'
-    ], function (Utils, BaseView, BaseControls, Pane, SidePane, Component, ToolBox, Controls, PageGroup, PageLoader, gt) {
+    ], function (Utils, ExtensionRegistry, BaseView, SidePane, Component, ToolBox, TopPane, Controls, PageGroup, PageLoader, NodeTouch, gt) {
 
     'use strict';
 
-    var // class name shortcuts
-        Button = BaseControls.Button,
-
-        // predefined zoom factors
+    var // predefined zoom factors
         ZOOM_FACTORS = [25, 35, 50, 75, 100, 150, 200, 300, 400, 600, 800],
 
         // the speed of scroll animations
@@ -61,21 +59,15 @@ define('io.ox/office/preview/view/view',
             // the scrollable document content node
             contentRootNode = null,
 
+            // the root DOM node of the search pane provided by the core framework
+            searchPaneNode = null,
+
+            // the top-level view pane containing global controls and the tool bar tabs
+            topPane = null,
+
             // the main side pane containing the thumbnails
             sidePane = null,
 
-            // tool pane floating over the top of the application pane
-            topOverlayPane = null,
-
-            // tool pane floating over the bottom of the application pane
-            bottomOverlayPane = null,
-
-            // the tool box containing page and zoom control groups
-            bottomToolBox = null,
-
-            // the status label for the page number and zoom factor
-            statusLabel = new BaseControls.StatusLabel(app, { type: 'info' }),
-
             // the page preview control
             pageGroup = null,
 
@@ -113,7 +105,7 @@ define('io.ox/office/preview/view/view',
 
         BaseView.call(this, app, {
             initHandler: initHandler,
-            deferredInitHandler: deferredInitHandler,
+            initGuiHandler: initGuiHandler,
             contentFocusable: true,
             contentScrollable: true,
             contentMargin: 30,
@@ -122,35 +114,6 @@ define('io.ox/office/preview/view/view',
 
         // private methods ----------------------------------------------------
 
-        /**
-         * Updates the status label with the current page number.
-         */
-        function updatePageStatus() {
-            statusLabel.setValue(
-                //#. A label showing the current page index in the OX Documents preview application
-                //#. %1$d is the current page index
-                //#. %2$d is the total number of pages
-                //#, c-format
-                gt('Page %1$d of %2$d', _.noI18n(selectedPage), _.noI18n(model.getPageCount()))
-            );
-        }
-
-        /**
-         * Shows the current zoom factor in the status label for a short time,
-         * then switches back to the display of the current page number.
-         *
-         * @param {Number} [scale=1]
-         *  If specified, a scaling factor relative to the current zoom factor.
-         */
-        var updateZoomStatus = app.createDebouncedMethod(function (scale) {
-            scale = _.isNumber(scale) ? scale : 1;
-            statusLabel.setValue(
-                //#. %1$d is the current zoom factor, in percent
-                //#, c-format
-                gt('Zoom: %1$d%', _.noI18n(Math.round(zoomFactor * scale)))
-            );
-        }, updatePageStatus, { delay: 1000 });
-
         /**
          * Selects the specified page. Updates the status label, the thumbnail
          * preview, and optionally scrolls the view to the specified page.
@@ -162,7 +125,6 @@ define('io.ox/office/preview/view/view',
             if ((1 <= page) && (page <= model.getPageCount()) && (page !== selectedPage)) {
                 selectedPage = page;
                 pageGroup.selectAndShowPage(page);
-                updatePageStatus();
                 app.getController().update();
             }
         }
@@ -264,13 +226,13 @@ define('io.ox/office/preview/view/view',
         function loadVisiblePages() {
 
             var // find the first page that is visible at the top border of the visible area
-                beginPage = _(pageNodes).sortedIndex(contentRootNode.scrollTop(), function (value) {
+                beginPage = _.sortedIndex(pageNodes, contentRootNode.scrollTop(), function (value) {
                     if (_.isNumber(value)) { return value; }
                     var pagePosition = Utils.getChildNodePositionInNode(contentRootNode, value);
                     return pagePosition.top + pagePosition.height - 1;
                 }) + 1,
                 // find the first page that is not visible anymore at the bottom border of the visible area
-                endPage = _(pageNodes).sortedIndex(contentRootNode.scrollTop() + contentRootNode[0].clientHeight, function (value) {
+                endPage = _.sortedIndex(pageNodes, contentRootNode.scrollTop() + contentRootNode[0].clientHeight, function (value) {
                     if (_.isNumber(value)) { return value; }
                     return Utils.getChildNodePositionInNode(contentRootNode, value).top;
                 }) + 1,
@@ -306,7 +268,7 @@ define('io.ox/office/preview/view/view',
             }, { delay: 500 });
 
             // clear all other pages
-            _(loadedPageNodes).each(function (pageNode, page) {
+            _.each(loadedPageNodes, function (pageNode, page) {
                 if ((page < beginPageLowerPrio) || (page >= endPageLowerPrio)) {
                     app.destroyImageNodes(pageNode);
                     pageNode.empty();
@@ -357,7 +319,7 @@ define('io.ox/office/preview/view/view',
             var // half of the content root node height
                 contentCenter = contentRootNode.height() / 2,
                 // find the page covering the vertical center of the visible area
-                centerPage = _(pageNodes).sortedIndex(contentRootNode.scrollTop() + contentCenter, function (value) {
+                centerPage = _.sortedIndex(pageNodes, contentRootNode.scrollTop() + contentCenter, function (value) {
                     if (_.isNumber(value)) { return value; }
                     var pagePosition = Utils.getChildNodePositionInNode(contentRootNode, value);
                     return pagePosition.top + pagePosition.height - 1;
@@ -419,32 +381,61 @@ define('io.ox/office/preview/view/view',
         }
 
         /**
-         * Returns a save point containing all settings needed to restore the
-         * current view settings.
+         * Enables or disables a smaller version of the search pane.
          */
-        function failSaveHandler() {
-            return { page: selectedPage, zoom: zoomType };
+        function toggleSmallSearchPane(state) {
+            searchPaneNode.find('.search-query-container').css({ width: state ? 180 : 300 });
+            searchPaneNode.find('.replace-query-container').css({ width: state ? 330 : 400 });
         }
 
         /**
-         * Initializes the view settings after the document has been opened.
-         *
-         * @param {Object} point
-         *  A save point that may have been created before by this application.
+         * Toggles the small search pane mode according to the current display
+         * orientation.
          */
-        function importSuccessHandler(point) {
-
-            // restore zoom type
-            zoomType = Utils.getOption(point, 'zoom');
-            if (_.isNumber(zoomType)) {
-                zoomType = Math.round(zoomType);
-            } else if (!_.isString(zoomType) || (zoomType.length === 0)) {
-                zoomType = 'page';
+        function updateSmallSearchPane() {
+            if (Modernizr.touch) {
+                if (window.matchMedia('(orientation: landscape)').matches) {
+                    toggleSmallSearchPane(false);
+                } else if (window.matchMedia('(orientation: portrait)').matches) {
+                    toggleSmallSearchPane(true);
+                } else {
+                    Utils.warn('BaseView.updateSmallSearchPane(): unknown display orientation');
+                }
             }
+        }
 
-            // first restore zoom, then the selected page (to keep it at the top screen border)
-            refreshLayout();
-            self.showPage(Utils.getIntegerOption(point, 'page', 1, 1, model.getPageCount()));
+        /**
+         * Updates the view after the global search bar has been opened.
+         */
+        function searchOpenHandler() {
+            updateSmallSearchPane();
+            // event triggers before the DOM element becomes visible
+            _.defer(function () { self.refreshPaneLayout(); });
+        }
+
+        /**
+         * Updates the view after the global search bar has been closed.
+         */
+        function searchCloseHandler() {
+            // event triggers before the DOM element becomes visible
+            _.defer(function () { self.refreshPaneLayout(); });
+        }
+
+        /**
+         * Handles 'orientationchange' events from mobile devices.
+         */
+        function orientationChangeHandler(event) {
+            // use event.orientation and media queries for the orientation detection.
+            // 'window.orientation' depends on where the device defines it's top
+            // and therefore says nothing reliable about the orientation.
+            if (event && (event.orientation === 'landscape')) {
+                toggleSmallSearchPane(false);
+            } else if (event && (event.orientation === 'portrait')) {
+                toggleSmallSearchPane(true);
+            } else {
+                // set search pane size according to a media query
+                updateSmallSearchPane();
+            }
         }
 
         /**
@@ -479,7 +470,7 @@ define('io.ox/office/preview/view/view',
             function trailingAnimation() {
 
                 var now = updateLastMoves(),
-                    move = _(lastMoves).reduce(function (memo, entry) {
+                    move = _.reduce(lastMoves, function (memo, entry) {
                         var factor = (now - entry.now) / COLLECT_DURATION;
                         memo.x -= entry.x * factor;
                         memo.y -= entry.y * factor;
@@ -517,59 +508,223 @@ define('io.ox/office/preview/view/view',
         }()); // end of trackingHandler() local scope
 
         /**
-         * Handles gesture events and changes the current zoom.
+         * Handles pinch events.
+         *
+         * @param {String} phase
+         * The current pinch phase ('start', 'move', 'end' or 'cancel')
          *
          * @param {jQuery.Event} event
-         *  The jQuery gesture event.
+         *  The jQuery tracking event.
+         *
+         * @param {Number} distance
+         * The current distance in px between the two fingers
+         *
+         * @param {Point} midPoint
+         * The current center position between the two fingers
+         *
          */
-        var gestureHandler = (function () {
+        var pinchHandler = (function() {
 
-            var // the minimum/maximum scale according to current zoom factor
-                minScale = 1, maxScale = 1;
+            var minScale, maxScale, scale,
+                scrollTop, scrollLeft, scrollWidth, scrollHeight,
+                startDistance,
+                contentWidth, contentHeight,
+                windowPosition,
+                contentRootScaleOrigin = {
+                    x : 0,
+                    y : 0
+                },
+                // the initial left and top position of the pageContainer in relation to the contentRootNode
+                initialPageContainer = {
+                    left   : 0,
+                    top    : 0,
+                    width  : 0,
+                    height : 0
+                },
+                newPageContainer = {
+                    left   : 0,
+                    top    : 0,
+                    width  : 0,
+                    height : 0
+                },
+                translateX, translateY;
 
-            // get the current relative scale factor from the passed event
-            function getScale(event) {
-                var scale = event.originalEvent.scale;
-                // if current zoom type is 'zoom-to-something', do not change it
-                // while scaling factor from event is between 0.95 and 1.05
-                return (_.isNumber(zoomType) || (scale <= 0.95) || (scale >= 1.05)) ? Utils.minMax(scale, minScale, maxScale) : 1;
-            }
+            function pinchHandler(phase, event, distance, midPoint) {
 
-            // changes the temporary page scaling while zooming
-            function changePageScale(scale) {
-                var translate = (1 - scale) * ((contentRootNode.height() - pageContainerNode.height()) / 2 + contentRootNode.scrollTop());
-                Utils.setCssAttributeWithPrefixes(pageContainerNode, 'transform', 'translateY(' + Math.round(translate) + 'px) scale(' + scale + ')');
-                updateZoomStatus(scale);
-            }
+                switch (phase) {
+                    case 'start':
+                        minScale = self.getMinZoomFactor() / zoomFactor;
+                        maxScale = self.getMaxZoomFactor() / zoomFactor;
+                        scrollLeft = contentRootNode.scrollLeft();
+                        scrollTop = contentRootNode.scrollTop();
+                        scrollWidth = contentRootNode[0].scrollWidth;
+                        scrollHeight = contentRootNode[0].scrollHeight;
+                        startDistance = distance;
+                        initialPageContainer.width = pageContainerNode.width();
+                        initialPageContainer.height = pageContainerNode.height();
+                        contentWidth = contentRootNode.width();
+                        contentHeight = contentRootNode.height();
+                        windowPosition = Utils.getNodePositionInPage(contentRootNode);
 
-            // updates the zoom according to the current scale factor
-            function changeZoom(scale) {
-                Utils.setCssAttributeWithPrefixes(pageContainerNode, 'transform', '');
-                if (scale !== 1) {
-                    updateZoomStatus(scale);
-                    self.setZoomType(zoomFactor * scale);
+                        // determining the topleft position of our page container (taking care of the scrollBar position)
+                        if(scrollLeft>0) {
+                            initialPageContainer.left = -scrollLeft;
+                        }
+                        else if(initialPageContainer.width<contentWidth) {
+                            initialPageContainer.left = (contentWidth-initialPageContainer.width)/2;
+                        }
+                        else {
+                            initialPageContainer.left = 0;
+                        }
+                        if(scrollTop>0) {
+                            initialPageContainer.top = -scrollTop;
+                        }
+                        else if(initialPageContainer.height<contentHeight) {
+                            initialPageContainer.top = (contentHeight-initialPageContainer.height)/2;
+                        }
+                        else {
+                            initialPageContainer.top = 0;
+                        }
+                        /* falls through */
+                    case 'move' : {
+
+                        contentRootScaleOrigin.x = midPoint.x - windowPosition.left;
+                        contentRootScaleOrigin.y = midPoint.y - windowPosition.top;
+
+
+                        // setting the proper scale origin
+                        Utils.setCssAttributeWithPrefixes(pageContainerNode, 'transform-origin', Math.round(contentRootScaleOrigin.x+scrollLeft) + 'px' + ' ' + Math.round(contentRootScaleOrigin.y+scrollTop) + 'px');
+
+                        // setting the scale factor for the transformation
+                        scale = Utils.minMax(distance / startDistance, minScale, maxScale);
+                        var transform = 'scale(' + scale + ')';
+
+                        if(initialPageContainer.width<contentWidth) {
+                            contentRootScaleOrigin.x += (contentWidth - initialPageContainer.width)/2;
+                        }
+                        if(initialPageContainer.height<contentHeight) {
+                            contentRootScaleOrigin.y += (contentHeight - initialPageContainer.height)/2;
+                        }
+                        newPageContainer.left = ((initialPageContainer.left - contentRootScaleOrigin.x)*scale)+contentRootScaleOrigin.x;
+                        newPageContainer.top  = ((initialPageContainer.top - contentRootScaleOrigin.y)*scale)+contentRootScaleOrigin.y;
+
+                        newPageContainer.width  = initialPageContainer.width*scale;
+                        newPageContainer.height = initialPageContainer.height*scale;
+
+                        translateX = 0;
+                        translateY = 0;
+
+                        // check left & right border, translation might be necessary
+                        if(newPageContainer.left>0) {
+                            translateX = -newPageContainer.left;                                // translation for page container to be starting at left border
+                            if(newPageContainer.width<contentWidth) {
+                                translateX += (contentWidth-newPageContainer.width)/2;          // centering if needed
+                            }
+                        }
+                        else if((newPageContainer.left+newPageContainer.width)<contentWidth) {
+                            translateX = contentWidth-(newPageContainer.left+newPageContainer.width);   // translation for page container to end at the right border
+                            if(newPageContainer.width<contentWidth) {
+                                translateX -= (contentWidth-newPageContainer.width)/2;          // centering if needed
+                            }
+                        }
+
+                        // check top & bottom border, translation might be necessary
+                        if(newPageContainer.top>0) {
+                            translateY = -newPageContainer.top;                                 // translation for page container to be starting at top border
+                            if(newPageContainer.height<contentHeight) {
+                                translateY += (contentHeight-newPageContainer.height)/2;        // centering if needed
+                            }
+                        }
+                        else if((newPageContainer.top+newPageContainer.height)<contentHeight) {
+                            translateY = contentHeight-(newPageContainer.top+newPageContainer.height);  // translation for page container to end at bottom border
+                            if(newPageContainer.height<contentHeight) {
+                                translateY -= (contentHeight-newPageContainer.height)/2;        // centering
+                            }
+                        }
+                        if(translateX!==0||translateY!==0) {
+                            transform += ' translate(' + Math.round(translateX/scale) + 'px,' + Math.round(translateY/scale) + 'px)';
+                        }
+                        Utils.setCssAttributeWithPrefixes(pageContainerNode, 'transform', transform);
+                        break;
+                    }
+
+                    case 'end': {
+                        Utils.setCssAttributeWithPrefixes(pageContainerNode, 'transform', '');
+                        Utils.setCssAttributeWithPrefixes(pageContainerNode, 'transform-origin', '');
+                        if (scale !== 1) {
+                            self.setZoomType(scale * zoomFactor);
+                        }
+                        app.getController().update();
+                        var newScrollLeft = -(newPageContainer.left+translateX),
+                            newScrollTop  = -(newPageContainer.top+translateY);
+                        if(newScrollLeft<0) {
+                            newScrollLeft = 0;
+                        }
+                        if(newScrollTop<0) {
+                            newScrollTop = 0;
+                        }
+                        contentRootNode.scrollLeft(newScrollLeft);
+                        contentRootNode.scrollTop(newScrollTop);
+                        break;
+                    }
+
+                    case 'cancel': {
+                        break;
+                    }
                 }
+            }
+            return pinchHandler;
+        }()); // end of pinchHandler() local scope
+
+
+        /**
+         * Handles tap events.
+         *
+         * @param {jQuery.Event} event
+         *  The jQuery tracking event.
+         *
+         * @param {Number} taps
+         * The number of detected taps
+         *
+         */
+        function tapHandler(event, taps) {
+            if(taps===2) {
+                self.setZoomType(self.getZoomType()==='page'?'width':'page');
+                app.getController().update();
+                if(self.getZoomType()==='page') {
+                    // the zoom type was switched to page via double-tap, we will determine the touched page and scroll to the beginning
+                    var tappedVertPagePosition = (event.data.tapPosY1-Utils.getNodePositionInPage(contentRootNode).top) + contentRootNode.scrollTop(),
+                        page = _.sortedIndex(pageNodes, tappedVertPagePosition, function (value) {
+                        if (_.isNumber(value)) { return value; }
+                        var pagePosition = Utils.getChildNodePositionInNode(contentRootNode, value);
+                        return pagePosition.top + pagePosition.height - 1;
+                    }) + 1;
+                    selectPage(page);
+                    scrollToPage(page);
+                }
+            }
+        }
+
+        /**
+         * Handles swipe events.
+         *
+         * @param {String} phase
+         * The current swipe phase (swipeStrictMode is true, so we only get the 'end' phase)
+         *
+         * @param {jQuery.Event} event
+         *  The jQuery tracking event.
+         *
+         * @param {Number} distance
+         * The swipe distance in pixel, the sign determines the swipe direction (left to right or right to left)
+         *
+         */
+        function horSwipeHandler(phase, event, distance) {
+            // swiping only if no horizontal scrolling is possible
+            if(pageContainerNode.width()<=contentRootNode.width()) {
+                self.showPage(distance<0?'next':'previous');
                 app.getController().update();
             }
-
-            function gestureHandler(event) {
-                switch (event.type) {
-                case 'gesturestart':
-                    minScale = self.getMinZoomFactor() / zoomFactor;
-                    maxScale = self.getMaxZoomFactor() / zoomFactor;
-                    break;
-                case 'gesturechange':
-                    changePageScale(getScale(event));
-                    break;
-                case 'gestureend':
-                    changeZoom(getScale(event));
-                    break;
-                }
-                return false;
-            }
-
-            return gestureHandler;
-        }()); // end of gestureHandler() local scope
+        }
 
         /**
          * Handles scroll events and updates the visible pages.
@@ -589,9 +744,12 @@ define('io.ox/office/preview/view/view',
 
             model = app.getModel();
             contentRootNode = self.getContentRootNode();
+            searchPaneNode = app.getWindow().nodes.search;
+
+            self.addPane(topPane = new TopPane(app));
 
             // create the side pane
-            self.addPane(sidePane = new SidePane(app, 'sidepane', {
+            self.addPane(sidePane = new SidePane(app, {
                 position: 'right',
                 size: Modernizr.touch ? 152 : SidePane.DEFAULT_WIDTH,
                 resizable: !Modernizr.touch,
@@ -604,81 +762,47 @@ define('io.ox/office/preview/view/view',
 
             // initialize the side pane
             sidePane
-                .addViewComponent(new ToolBox(app, 'main', { fixed: 'top' })
-                    .addGroup('view/sidepane', new Button(BaseControls.HIDE_SIDEPANE_OPTIONS))
-                    .addRightTab()
-                    .addGroup('document/edit', new Controls.EditDocumentButton(app))
-                    .addGap()
-                    .addGroup('app/quit', new Button(BaseControls.QUIT_OPTIONS))
-                )
-                .addViewComponent(new Component(app, 'thumbs')
+                .addViewComponent(new Component(app)
                     .addGroup('pages/current', pageGroup)
                 );
 
-            // create the top overlay pane
-            self.addPane(topOverlayPane = new Pane(app, 'overlaytop', { position: 'top', classes: 'inline right', overlay: true, transparent: true, hoverEffect: true }));
-
-            // create the bottom overlay pane
-            self.addPane(bottomOverlayPane = new Pane(app, 'overlaybottom', { position: 'bottom', classes: 'inline right', overlay: true, transparent: true, hoverEffect: true })
-                .addViewComponent(bottomToolBox = new ToolBox(app, 'overlaypages'))
-            );
-
             // initially, hide the side pane, and show the overlay tool bars
             self.toggleSidePane(false);
 
             // Bug 25924: sometimes, Firefox selects the entire page
             if (_.browser.Firefox) {
                 contentRootNode.on('focus', function () {
-                    BaseView.clearBrowserSelection();
+                    Utils.clearBrowserSelection();
                 });
             }
+
+            // listen to orientation events when the application window is visible
+            if (Modernizr.touch) {
+                app.registerGlobalEventHandler(window, 'orientationchange', orientationChangeHandler);
+            }
+
+            // refresh layout when the search pane has been toggled
+            this.listenTo(app.getWindow(), 'search:open', searchOpenHandler);
+            this.listenTo(app.getWindow(), 'search:close', searchCloseHandler);
+
+            // enabling the text input field and setting the proper validator when page count is known after import
+            this.listenTo(app.getImportPromise(), 'done', function () {
+                topPane.updatePageCount(model.getPageCount());
+            });
+
+            // search pane is a focus landmark (TODO: move to core code)
+            searchPaneNode.addClass('f6-target');
         }
 
-        function deferredInitHandler() {
+        function initGuiHandler() {
 
-            var // the bottom tool box in the side pane
-                sidePaneToolBox = null,
-                // the tool box in the top overlay pane
-                overlayPaneToolBox = new ToolBox(app, 'overlaymain'),
-                // the number of pages in the document
+            var // the number of pages in the document
                 pageCount = model.getPageCount(),
                 // the HTML mark-up for the empty page nodes
                 pageMarkup = '';
 
             if (pageCount >= 1) {
 
-                // initialize side pane depending on page count
-                sidePane.addViewComponent(sidePaneToolBox = new ToolBox(app, 'pages', { fixed: 'bottom' }));
-                if (pageCount > 1) {
-                    sidePaneToolBox
-                        .addGroup('pages/previous', new Button(Controls.PREV_OPTIONS))
-                        .addGroup('pages/current',  new Controls.PagePicker(app, { position: 'top', align: 'center' }))
-                        .addGroup('pages/next',     new Button(Controls.NEXT_OPTIONS));
-                    if (Modernizr.touch) { sidePaneToolBox.newLine(); }
-                }
-                if (!Modernizr.touch) { sidePaneToolBox.addRightTab(); }
-                sidePaneToolBox
-                    .addGroup('zoom/dec',  new Button(BaseControls.ZOOMOUT_OPTIONS))
-                    .addGroup('zoom/type', new Controls.ZoomTypePicker({ position: 'top', align: 'center' }))
-                    .addGroup('zoom/inc',  new Button(BaseControls.ZOOMIN_OPTIONS));
-
-                // create the status overlay pane
-                self.addPane(new Pane(app, 'statuslabel', { position: 'bottom', classes: 'inline right', overlay: true, transparent: true })
-                    .addViewComponent(new ToolBox(app, 'statuslabel').addPrivateGroup(statusLabel))
-                );
-
-                // initialize bottom overlay pane depending on page count
-                if (pageCount > 1) {
-                    bottomToolBox.newLine().addRightTab()
-                        .addGroup('pages/previous', new Button(Controls.PREV_OPTIONS))
-                        .addGroup('pages/current',  new Controls.PagePicker(app))
-                        .addGroup('pages/next',     new Button(Controls.NEXT_OPTIONS));
-                }
-                bottomToolBox.newLine().addRightTab()
-                    .addGroup('zoom/dec',  new Button(BaseControls.ZOOMOUT_OPTIONS))
-                    .addGroup('zoom/type', new Controls.ZoomTypePicker())
-                    .addGroup('zoom/inc',  new Button(BaseControls.ZOOMIN_OPTIONS));
-
                 // save the scroll position while view is hidden
                 app.getWindow().on({ beforehide: windowHideHandler, show: windowShowHandler });
 
@@ -687,30 +811,26 @@ define('io.ox/office/preview/view/view',
 
                 // initialize touch-like tracking with mouse, attach the scroll event handler
                 contentRootNode
-                    .enableTracking({ selector: '.page', sourceEvents: 'mouse' })
+                    .enableTracking({selector: '.page', sourceEvents: 'mouse'})
+                    .enableTouch({selector: '.page', pinchHandler: pinchHandler, tapHandler: tapHandler, horSwipeHandler: horSwipeHandler})
                     .on('tracking:start tracking:move tracking:end tracking:cancel', trackingHandler)
-                    .on('gesturestart gesturechange gestureend', gestureHandler)
                     .on('scroll', scrollHandler);
 
-                // initialize all page nodes
-                _(pageCount).times(function (index) {
+                // create mark-up for all page nodes
+                _.times(pageCount, function (index) {
                     pageMarkup += '<div class="page" data-page="' + (index + 1) + '"></div>';
                 });
-                self.insertContentNode(pageContainerNode.html(pageMarkup));
+
+                // insert the mark-up into the container node and initialize the
+                // variable 'pageNodes' BEFORE inserting everything into the living
+                // DOM (variable is used when refreshing the layout)
+                pageContainerNode[0].innerHTML = pageMarkup;
                 pageNodes = pageContainerNode.children();
 
-                // add controls to the top overlay pane
-                overlayPaneToolBox
-                    .addGroup('view/sidepane', new Button(BaseControls.SHOW_SIDEPANE_OPTIONS))
-                    .addGap()
-                    .addGroup('document/edit', new Controls.EditDocumentButton(app))
-                    .addGap();
+                // insert page container node into the view
+                self.insertContentNode(pageContainerNode);
             }
 
-            // always add the quit button
-            overlayPaneToolBox.addGroup('app/quit', new Button(BaseControls.QUIT_OPTIONS));
-            topOverlayPane.addViewComponent(overlayPaneToolBox);
-
             // set focus to application pane, and update the view
             self.grabFocus();
             app.getController().update();
@@ -742,8 +862,6 @@ define('io.ox/office/preview/view/view',
         this.toggleSidePane = function (state) {
             this.lockPaneLayout(function () {
                 sidePane.toggle(state);
-                topOverlayPane.toggle(!state);
-                bottomOverlayPane.toggle(!state);
             });
             return this;
         };
@@ -848,7 +966,6 @@ define('io.ox/office/preview/view/view',
             if (zoomType !== newZoomType) {
                 zoomType = newZoomType;
                 refreshLayout();
-                updateZoomStatus();
             }
             return this;
         };
@@ -880,17 +997,46 @@ define('io.ox/office/preview/view/view',
         this.increaseZoomLevel = function () {
 
             var // find first entry in ZOOM_FACTORS with a factor greater than current zoom
-                nextZoomFactor = _(ZOOM_FACTORS).find(function (factor) { return factor > zoomFactor; });
+                nextZoomFactor = _.find(ZOOM_FACTORS, function (factor) { return factor > zoomFactor; });
 
             return this.setZoomType(_.isNumber(nextZoomFactor) ? nextZoomFactor : this.getMaxZoomFactor());
         };
 
         // initialization -----------------------------------------------------
 
-        // fail-save handler returns data needed to restore the application after browser refresh
-        app.registerFailSaveHandler(failSaveHandler);
-        // initialize the view after import with the fail-save information
-        app.on('docs:import:success', importSuccessHandler);
+        // initialize the view after import
+        app.onImportSuccess(function () {
+
+            var // file type configuration (by file extension)
+                extSettings = ExtensionRegistry.getExtensionSettings(app.getFullFileName());
+
+            // determine zoom type by application type (full width for text documents)
+            zoomType = 'page';
+            if (extSettings) {
+                switch (extSettings.type) {
+                case 'text':
+                    zoomType = 'width';
+                    break;
+                case 'spreadsheet':
+                case 'presentation':
+                    zoomType = 'page';
+                    break;
+                }
+            }
+
+            // first set zoom, then the first page (to keep it at the top screen border)
+            refreshLayout();
+            self.showPage(1);
+        });
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            pageLoader.destroy();
+            app = self = model = null;
+            contentRootNode = searchPaneNode = sidePane = null;
+            pageGroup = pageContainerNode = pageNodes = pageLoader = loadedPageNodes = null;
+            loadMorePagesTimer = scrollAnimation = null;
+        });
 
     } // class PreviewView
 
