@@ -13,13 +13,14 @@
 
 define('io.ox/office/spreadsheet/app/controller',
     ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/format/border',
-     'io.ox/office/editframework/model/format/mixedborder',
+     'io.ox/office/editframework/utils/border',
+     'io.ox/office/editframework/utils/mixedborder',
      'io.ox/office/editframework/app/editcontroller',
+     'io.ox/office/spreadsheet/utils/config',
      'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/app/config',
+     'io.ox/office/spreadsheet/utils/paneutils',
      'io.ox/office/spreadsheet/controller/drawingcontroller'
-    ], function (Utils, Border, MixedBorder, EditController, SheetUtils, Config, DrawingController) {
+    ], function (Utils, Border, MixedBorder, EditController, Config, SheetUtils, PaneUtils, DrawingController) {
 
     'use strict';
 
@@ -51,46 +52,33 @@ define('io.ox/office/spreadsheet/app/controller',
                 'document/search/search': {
                     // enabled in read-only mode
                     set: function (searchQuery) { view.search(searchQuery); },
-                    focus: 'never'
+                    preserveFocus: true // do not return focus to document
                 },
 
                 'document/search/nextResult': {
                     // enabled in read-only mode
-                    set: function () {
-                        if (app.getWindow().search.active) {
-                            view.searchIterate('next');
-                        }
-                    },
+                    set: function () { view.searchIterate('next'); },
                     shortcut: [{ keyCode: 'G', ctrlOrMeta: true }, { keyCode: 'F3' }],
-                    focus: 'never' // do not grab focus after setter
+                    preserveFocus: true // do not return focus to document
                 },
 
                 'document/search/previousResult': {
                     // enabled in read-only mode
-                    set: function () {
-                        if (app.getWindow().search.active) {
-                            view.searchIterate('prev');
-                        }
-                    },
+                    set: function () { view.searchIterate('prev'); },
                     shortcut: [{ keyCode: 'G', ctrlOrMeta: true, shift: true }, { keyCode: 'F3', shift: true }],
-                    focus: 'never'
+                    preserveFocus: true // do not return focus to document
                 },
 
                 'document/search/replaceAll': {
                     parent: 'document/editable',
-                    set: function () { view.searchReplace({all: true}); },
-                    focus: 'never' // do not grab focus after setter
+                    set: function () { view.searchReplace({ all: true }); },
+                    preserveFocus: true // do not return focus to document
                 },
 
                 'document/search/replaceSelected': {
                     parent: 'document/editable',
                     set: function () { view.searchReplace(); },
-                    focus: 'never'
-                },
-
-                'document/search/close': {
-                    // enabled in read-only mode
-                    set: function () { view.searchClear({clearInputs: true}); }
+                    preserveFocus: true // do not return focus to document
                 },
 
                 // selection mode ---------------------------------------------
@@ -109,18 +97,27 @@ define('io.ox/office/spreadsheet/app/controller',
                     enable: function () { return !view.isCellEditMode(); }
                 },
 
+                'view/statuspane/show': {
+                    parent: 'app/valid',
+                    get: function () { return view.getStatusPane().isVisible(); },
+                    set: function (state) { view.getStatusPane().toggle(state); }
+                },
+
                 'view/sheet/active': {
+                    parent: 'app/valid',
                     get: function () { return view.getActiveSheet({ visible: true }); },
                     set: function (index) { view.setActiveSheet(index, { visible: true }); }
                 },
 
                 'view/sheet/prev': {
+                    parent: 'app/valid',
                     enable: function () { return view.getActiveSheet({ visible: true }) > 0; },
                     set: function () { view.activatePreviousSheet(); },
                     shortcut: { keyCode: 'PAGE_UP', ctrlOrMeta: true, alt: true }
                 },
 
                 'view/sheet/next': {
+                    parent: 'app/valid',
                     enable: function () { return view.getActiveSheet({ visible: true }) + 1 < view.getVisibleSheetCount(); },
                     set: function () { view.activateNextSheet(); },
                     shortcut: { keyCode: 'PAGE_DOWN', ctrlOrMeta: true, alt: true }
@@ -128,14 +125,30 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'view/subtotals/type': {
                     parent: 'app/valid',
+                    enable: function () {
+                        var subtotals = view.getSubtotals();
+                        return (subtotals.cells > 1) && (this.getValue() in subtotals);
+                    },
                     get: function () { return app.getUserSettingsValue('subtotalType', 'sum'); },
                     set: function (subtotalType) { app.setUserSettingsValue('subtotalType', subtotalType); }
                 },
 
+                'view/status': {
+                    parent: 'app/valid',
+                    enable: function () { return _.isString(view.getStatusLabel()); },
+                    get: function () { return view.getStatusLabel(); }
+                },
+
+                'view/formula': {
+                    parent: 'view/cell/editmode/off',
+                    enable: function () { return view.isSingleCellSelection() && _.isString(this.getValue()); },
+                    get: function () { return view.getCellContents().formula; }
+                },
+
                 'view/grid/show': {
                     parent: 'app/valid',
-                    get: function () { return view.getActiveSheetModel().getViewAttribute('showGrid'); },
-                    set: function (state) { view.getActiveSheetModel().setViewAttribute('showGrid', state); }
+                    get: function () { return view.getSheetViewAttribute('showGrid'); },
+                    set: function (state) { view.setSheetViewAttribute('showGrid', state); }
                 },
 
                 'view/split/dynamic': {
@@ -147,26 +160,29 @@ define('io.ox/office/spreadsheet/app/controller',
                 'view/split/frozen': {
                     parent: 'view/cell/editmode/off',
                     get: function () { return view.hasFrozenSplit(); },
-                    set: function (value) {
-                        if (_.isBoolean(value)) {
-                            view.setFrozenSplit(value);
-                        } else if (value === 'toggle') {
-                            view.setFrozenSplit(!this.getValue());
-                        } else if (_.isObject(value)) {
-                            view.setFixedFrozenSplit(value.cols, value.rows);
-                        }
-                    }
+                    set: function (value) { view.setFrozenSplit(value); }
+                },
+
+                'view/split/frozen/fixed': {
+                    parent: 'view/cell/editmode/off',
+                    set: function (value) { view.setFixedFrozenSplit(value.cols, value.rows); }
+                },
+
+                'view/zoom': {
+                    parent: 'app/valid',
+                    get: function () { return view.getZoom(); },
+                    set: function (zoom) { view.setZoom(zoom); }
                 },
 
                 'view/zoom/dec': {
-                    parent: 'app/valid',
-                    enable: function () { return view.getZoom() > SheetUtils.MIN_ZOOM; },
+                    parent: 'view/zoom',
+                    enable: function (zoom) { return zoom > SheetUtils.MIN_ZOOM; },
                     set: function () { view.decZoom(); }
                 },
 
                 'view/zoom/inc': {
-                    parent: 'app/valid',
-                    enable: function () { return view.getZoom() < SheetUtils.MAX_ZOOM; },
+                    parent: 'view/zoom',
+                    enable: function (zoom) { return zoom < SheetUtils.MAX_ZOOM; },
                     set: function () { view.incZoom(); }
                 },
 
@@ -174,7 +190,7 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'sheet/unlocked': {
                     parent: 'document/editable',
-                    enable: function () { return !view.getActiveSheetModel().isLocked(); }
+                    enable: function () { return !view.isSheetLocked(); }
                 },
 
                 'sheet/attributes': {
@@ -197,7 +213,7 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'sheet/insert': {
                     parent: 'document/editable',
-                    enable: function () { return model.getSheetCount() < Config.getMaxSheetCount(); },
+                    enable: function () { return model.getSheetCount() < Config.MAX_SHEET_COUNT; },
                     set: function () { view.insertSheet(); }
                 },
 
@@ -215,21 +231,18 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'sheet/rename/dialog': {
                     parent: 'sheet/name',
-                    set: function () { return view.showRenameSheetDialog(); },
-                    focus: 'wait' // wait for the dialog before returning focus to application
+                    set: function () { return view.showRenameSheetDialog(); }
                 },
 
                 'sheet/copy/dialog': {
                     parent: 'sheet/insert',
-                    set: function () { return view.showCopySheetDialog(); },
-                    focus: 'wait' // wait for the dialog before returning focus to application
+                    set: function () { return view.showCopySheetDialog(); }
                 },
 
                 'sheet/reorder/dialog': {
                     parent: 'document/editable',
                     enable: function () { return view.getVisibleSheetCount() > 1; },
-                    set: function () { return view.showReorderSheetsDialog(); },
-                    focus: 'wait'
+                    set: function () { return view.showReorderSheetsDialog(); }
                 },
 
                 // column operations ------------------------------------------
@@ -240,41 +253,37 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'column/insert': {
                     parent: 'column/operation',
-                    enable: function () { return view.columnsInsertable(); },
+                    enable: function () { return view.canInsertColumns(); },
                     set: function () { view.insertColumns(); }
                 },
 
                 'column/delete': {
                     parent: 'column/operation',
-                    enable: function () { return view.columnsDeleteable(); },
+                    enable: function () { return view.canDeleteColumns(); },
                     set: function () { view.deleteColumns(); }
                 },
 
+                // returns the mixed column attributes
                 'column/attributes': {
                     parent: 'column/operation',
                     get: function () { return view.getColumnAttributes(); }
                 },
 
-                'column/visible': {
-                    parent: 'column/attributes',
-                    get: function (attributes) { return attributes.column.visible; },
-                    set: function (state) { view.setColumnAttributes({ column: { visible: state } }); }
-                },
-
                 'column/hide': {
-                    parent: 'column/visible',
-                    enable: function () { return this.getValue() !== false; },
-                    set: function () { view.setColumnAttributes({ column: { visible: false } }); }
+                    parent: 'column/attributes',
+                    // enabled when any visible columns are selected
+                    enable: function (attributes) { return attributes.visible !== false; },
+                    set: function () { view.setColumnAttributes({ visible: false }); }
                 },
 
                 'column/show': {
-                    parent: 'column/visible',
-                    enable: function () { return this.getValue() !== true; },
-                    set: function () { view.setColumnAttributes({ column: { visible: true } }); }
+                    parent: 'column/operation',
+                    enable: function () { return view.canShowColumns(); },
+                    set: function () { view.showColumns(); }
                 },
 
                 'column/width/optimal': {
-                    parent: 'column/hide', // only enabled when visible columns are selected
+                    parent: 'column/hide', // only enabled when any visible columns are selected
                     set: function () { view.setOptimalColumnWidth(); }
                 },
 
@@ -297,41 +306,37 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'row/insert': {
                     parent: 'row/operation',
-                    enable: function () { return view.rowsInsertable(); },
+                    enable: function () { return view.canInsertRows(); },
                     set: function () { view.insertRows(); }
                 },
 
                 'row/delete': {
                     parent: 'row/operation',
-                    enable: function () { return view.rowsDeleteable(); },
+                    enable: function () { return view.canDeleteRows(); },
                     set: function () { view.deleteRows(); }
                 },
 
+                // returns the mixed row attributes
                 'row/attributes': {
                     parent: 'row/operation',
                     get: function () { return view.getRowAttributes(); }
                 },
 
-                'row/visible': {
-                    parent: 'row/attributes',
-                    get: function (attributes) { return attributes.row.visible; },
-                    set: function (state) { view.setRowAttributes({ row: { visible: state } }); }
-                },
-
                 'row/hide': {
-                    parent: 'row/visible',
-                    enable: function () { return this.getValue() !== false; },
-                    set: function () { view.setRowAttributes({ row: { visible: false } }); }
+                    parent: 'row/attributes',
+                    // enabled when any visible rows are selected
+                    enable: function (attributes) { return attributes.visible !== false; },
+                    set: function () { view.setRowAttributes({ visible: false }); }
                 },
 
                 'row/show': {
-                    parent: 'row/visible',
-                    enable: function () { return this.getValue() !== true; },
-                    set: function () { view.setRowAttributes({ row: { visible: true } }); }
+                    parent: 'row/operation',
+                    enable: function () { return view.canShowRows(); },
+                    set: function () { view.showRows(); }
                 },
 
                 'row/height/optimal': {
-                    parent: 'row/hide', // only enabled when visible rows are selected
+                    parent: 'row/hide', // only enabled when any visible rows are selected
                     set: function () { view.setOptimalRowHeight(); }
                 },
 
@@ -361,13 +366,14 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'cell/autoformula': {
                     parent: 'cell/operation',
-                    set: function (type) { view.insertAutoSum(type); }
+                    enable: function () { return view.getSelectedRanges().length === 1; },
+                    set: function (funcName) { view.insertAutoFormula(funcName); }
                 },
 
                 'cell/painter': {
                     parent: 'cell/operation',
-                    get: function () { return view.applyPainterState(); },
-                    set: function (state) { view.applyPainter(state); }
+                    get: function () { return view.isFormatPainterActive(); },
+                    set: function (state) { view.activateFormatPainter(state); }
                 },
 
                 // cell attributes --------------------------------------------
@@ -435,13 +441,13 @@ define('io.ox/office/spreadsheet/app/controller',
                 // enabled if at least one border in the selection is visible
                 'cell/border/visible': {
                     parent: 'cell/border/attributes',
-                    enable: function (borderAttributes) { return _(borderAttributes).any(MixedBorder.isVisibleBorder); }
+                    enable: function (borderAttributes) { return _.any(borderAttributes, MixedBorder.isVisibleBorder); }
                 },
 
                 // a single mixed border for all borders in the selection
                 'cell/border/value': {
                     parent: ['cell/border/visible', 'view/cell/editmode/off'],
-                    get: function (borderAttributes) { return MixedBorder.mixBorders(_(borderAttributes).values()); }
+                    get: function (borderAttributes) { return MixedBorder.mixBorders(_.values(borderAttributes)); }
                 },
 
                 'cell/border/style': {
@@ -453,7 +459,7 @@ define('io.ox/office/spreadsheet/app/controller',
                 // width of the border lines, in points (rounded to 1/2 points)
                 'cell/border/width': {
                     parent: 'cell/border/value',
-                    get: function (border) { return _.isNumber(border.width) ? Utils.convertHmmToLength(border.width, 'pt', 0.5) : null; },
+                    get: function (border) { return !MixedBorder.isVisibleBorder(border) ? 0 : _.isNumber(border.width) ? Utils.convertHmmToLength(border.width, 'pt', 0.5) : null; },
                     set: function (value) { view.changeVisibleBorders({ width: Utils.convertLengthToHmm(value, 'pt') }); }
                 },
 
@@ -463,6 +469,13 @@ define('io.ox/office/spreadsheet/app/controller',
                     set: function (value) { view.changeVisibleBorders({ color: value }); }
                 },
 
+                // combined style and width as enumeration as used by Excel
+                'cell/border/style/preset': {
+                    parent: 'cell/border/value',
+                    get: function (border) { return PaneUtils.getPresetStyleForBorder(border); },
+                    set: function (style) { view.changeVisibleBorders(PaneUtils.getBorderForPresetStyle(style)); }
+                },
+
                 'cell/numberformat/category': {
                     parent: ['document/editable/cell', 'view/cell/editmode/off'],
                     get: function () { return view.getNumberFormatCategory(); },
@@ -479,9 +492,9 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'cell/numberformat/code': {
                     parent: 'cell/numberformat/category',
-                    enable: function (category) { return !_(['standard', 'custom']).contains(category); },
+                    enable: function (category) { return !/^(standard|custom)$/.test(category); },
                     get: function () { return view.getNumberFormatCode(); },
-                    set: function (formatCode) { view.setCellAttribute('numberFormat', { code: formatCode }); }
+                    set: function (formatCode) { view.setNumberFormatCode(formatCode); }
                 },
 
                 // cell attributes (family 'character') -----------------------
@@ -545,30 +558,14 @@ define('io.ox/office/spreadsheet/app/controller',
 
                 'character/hyperlink/dialog': {
                     parent: ['character/url', 'view/cell/editmode/off'],
-                    set: function () { return view.editHyperlink(); },
-                    focus: 'wait' // wait for the dialog before returning focus to application
+                    set: function () { return view.editHyperlink(); }
                 },
 
                 'character/hyperlink/remove': {
                     parent: ['character/url', 'view/cell/editmode/off'],
                     set: function () { view.removeHyperlink(); }
-                },
-
-                // drawing ---------------------------------------------------
-                // drawing comes now from the DrawingController
-
-                // debug ------------------------------------------------------
-
-                'debug/view/update': {
-                    parent: 'debug/enabled',
-                    set: function (type) { view.debugRequestUpdate(type); }
-                },
-
-                'debug/insert/drawing': {
-                    parent: 'debug/enabled',
-                    enable: function () { return model.getEditMode(); },
-                    set: function (type) { view.debugInsertDrawing(type); }
                 }
+
             };
 
         // base constructor ---------------------------------------------------
@@ -582,7 +579,7 @@ define('io.ox/office/spreadsheet/app/controller',
         this.registerDefinitions(items);
 
         // initialization after construction
-        app.on('docs:init', function () {
+        app.onInit(function () {
 
             // model and view are not available at construction time
             model = app.getModel();
