@@ -15,8 +15,8 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
     ['io.ox/office/tk/utils',
      'io.ox/office/baseframework/model/modelobject',
      'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/cellmodel'
-    ], function (Utils, ModelObject, SheetUtils, CellModel) {
+     'io.ox/office/spreadsheet/model/cellattributesmodel'
+    ], function (Utils, ModelObject, SheetUtils, CellAttributesModel) {
 
     'use strict';
 
@@ -72,7 +72,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             entries = [],
 
             // the largest valid column/row index
-            maxIndex = columns ? model.getMaxCol() : model.getMaxRow(),
+            maxIndex = model.getMaxIndex(columns),
 
             // the current zoom factor of the document
             zoom = sheetModel.getEffectiveZoom();
@@ -83,7 +83,14 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
 
         // private class EntryModel -------------------------------------------
 
-        var EntryModel = CellModel.extend({ constructor: function (first, last, attributes) {
+        /**
+         * Represents the collection entries of a ColRowCollection instance.
+         *
+         * @constructor
+         *
+         * @extends CellAttributesModel
+         */
+        var EntryModel = CellAttributesModel.extend({ constructor: function (first, last, attributes) {
 
             // column/row interval covered by this entry
             this.first = first;
@@ -100,7 +107,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             // base constructor - - - - - - - - - - - - - - - - - - - - - - - -
 
             // do not trigger any events (collection entries are potential mass objects)
-            CellModel.call(this, app, attributes, {
+            CellAttributesModel.call(this, app, attributes, {
                 additionalFamilies: FAMILY_NAME,
                 parentModel: sheetModel
             });
@@ -133,8 +140,8 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          */
         EntryModel.prototype.updateSize = function () {
             var attributes = this.getMergedAttributes()[FAMILY_NAME];
-            this.sizeHmm = attributes.visible ? Math.max(0, attributes[SIZE_ATTR_NAME]) : 0;
-            this.size = (this.sizeHmm === 0) ? 0 : Math.max(5, convertHmmToPixels(this.sizeHmm));
+            this.sizeHmm = attributes.visible ? Math.max(0, Math.round(attributes[SIZE_ATTR_NAME])) : 0;
+            this.size = (this.sizeHmm === 0) ? 0 : Math.max(SheetUtils.MIN_CELL_SIZE, convertHmmToPixels(this.sizeHmm));
         };
 
         /**
@@ -208,7 +215,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  existing collection entry.
          */
         function findEntryIndex(index) {
-            return _(entries).sortedIndex({ last: index }, 'last');
+            return _.sortedIndex(entries, { last: index }, 'last');
         }
 
         /**
@@ -371,7 +378,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          */
         function deleteEntries(arrayIndex, count) {
             // call destructor for all deleted entries
-            _(entries.splice(arrayIndex, count)).invoke('destroy');
+            _.invoke(entries.splice(arrayIndex, count), 'destroy');
         }
 
         /**
@@ -419,8 +426,8 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  (no predecessor or successor exists in the collection).
          *
          * @param {Object} [options]
-         *  A map with options specifying which entries will be merged with the
-         *  specified entry. The following options are supported:
+         *  Optional parameters specifying which entries will be merged with
+         *  the specified entry:
          *  @param {Boolean} [options.prev=false]
          *      If set to true, the entry and its predecessor will be tried to
          *      merge.
@@ -488,8 +495,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  process will be stopped immediately.
          *
          * @param {Object} [options]
-         *  A map with additional options for this method. The following
-         *  options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.reverse=false]
          *      If set to true, visits the covered collection entries from last
          *      to first. Must not be used together with 'options.modify'.
@@ -645,8 +651,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  to the option 'pixel' (see below).
          *
          * @param {Object} [options]
-         *  A map with additional options controlling the behavior of this
-         *  method. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.pixel=false]
          *      If set to true, the parameter 'offset' is interpreted as length
          *      in pixels according to the current sheet zoom factor.
@@ -679,7 +684,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                 // the passed offset, restricted to the valid sheet dimension
                 currOffset = Utils.minMax(offset, 0, totalSize - 1),
                 // the index of the first collection entry after the offset
-                arrayIndex = _(entries).sortedIndex(Utils.makeSimpleObject(OFFSET_NAME, currOffset + 1), OFFSET_NAME),
+                arrayIndex = _.sortedIndex(entries, Utils.makeSimpleObject(OFFSET_NAME, currOffset + 1), OFFSET_NAME),
                 // the current collection entry
                 entry = entries[arrayIndex - 1],
                 // the end offset of the previous entry
@@ -734,7 +739,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
         function changeViewAttributesHandler(event, attributes) {
             if ('zoom' in attributes) {
                 zoom = sheetModel.getEffectiveZoom();
-                _(entries).invoke('updateSize');
+                _.invoke(entries, 'updateSize');
                 defaultEntry.updateSize();
                 updateEntryOffsets(0);
             }
@@ -862,8 +867,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  process will be stopped immediately.
          *
          * @param {Object} [options]
-         *  A map with additional options controlling the behavior of this
-         *  method. The following options are supported:
+         *  Optional parameters:
          *  @param {Object} [options.context]
          *      If specified, the iterator will be called with this context
          *      (the symbol 'this' will be bound to the context inside the
@@ -988,12 +992,12 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                     }
 
                     // mix the style identifier of the visited interval
-                    _(attributes).each(function (values, family) {
+                    _.each(attributes, function (values, family) {
                         if (family === 'styleId') {
                             mixAttribute(mixedAttributes, 'styleId', values);
                         } else {
                             var mixedValues = mixedAttributes[family];
-                            _(values).each(function (value, name) {
+                            _.each(values, function (value, name) {
                                 mixAttribute(mixedValues, name, value);
                             });
                         }
@@ -1007,73 +1011,6 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             return mixedAttributes;
         };
 
-        /**
-         * Returns the visible column/row intervals contained in the passed
-         * interval.
-         *
-         * @param {Object} interval
-         *  The column/row interval to be processed, with the zero-based index
-         *  properties 'first' and 'last'.
-         *
-         * @param {Object} [options]
-         *  The following options are supported:
-         *  @param {Number} [options.lastHidden=0]
-         *      If set to a number greater than 0, the start positions of the
-         *      returned intervals will be decreased by the specified number,
-         *      so that they include that number of hidden columns/rows
-         *      preceding each interval of visible columns/rows.
-         *
-         * @returns {Array}
-         *  An array with all visible column/row intervals contained in the
-         *  passed interval. If the passed interval is completely hidden, the
-         *  returned array will be empty.
-         */
-        this.getVisibleIntervals = function (interval, options) {
-
-            var // number of preceding hidden column/row in each interval
-                lastHidden = Utils.getIntegerOption(options, 'lastHidden', 0, 0),
-                // the resulting visible intervals
-                visibleIntervals = [],
-                // the last inserted result interval
-                lastInterval = null;
-
-            iterateEntries(interval, function (entry, first, last) {
-
-                if (entry.size === 0) { return; }
-
-                // expand start point of the interval
-                first = Math.max(interval.first, first - lastHidden);
-
-                // try to expand last existing interval in the result
-                if (lastInterval && (lastInterval.last + 1 >= first)) {
-                    lastInterval.last = last;
-                } else {
-                    visibleIntervals.push(lastInterval = { first: first, last: last });
-                }
-            });
-
-            return visibleIntervals;
-        };
-
-        /**
-         * Returns the indexes of all visible collection entries in the passed
-         * interval as plain array.
-         *
-         * @param {Object} interval
-         *  The column/row interval to be processed, with the zero-based index
-         *  properties 'first' and 'last'.
-         *
-         * @returns {Number[]}
-         *  The indexes of all visible collection entries, in ascending order.
-         */
-        this.getVisibleEntryIndexes = function (interval) {
-            var indexes = [];
-            this.iterateEntries(interval, function (entry) {
-                indexes.push(entry.index);
-            });
-            return indexes;
-        };
-
         /**
          * Returns whether the column/row with the specified index is visible.
          *
@@ -1087,49 +1024,6 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             return this.getEntry(index).size > 0;
         };
 
-        /**
-         * Returns the absolute sheet offset of a specific position inside a
-         * column/row in 1/100 mm, independent from the current sheet zoom
-         * factor.
-         *
-         * @param {Number} index
-         *  The zero-based index of the column/row.
-         *
-         * @param {Number} offsetHmm
-         *  The relative offset inside the column/row, in 1/100 mm. If this
-         *  offset is larger than the total size of the column/row, the
-         *  resulting absolute sheet offset will be located at the end position
-         *  of the column/row.
-         *
-         * @returns {Number}
-         *  The absolute position of the specified entry offset, in 1/100 mm.
-         */
-        this.getEntryOffsetHmm = function (index, offsetHmm) {
-            var entryData = this.getEntry(index);
-            return entryData.offsetHmm + Utils.minMax(offsetHmm, 0, entryData.sizeHmm);
-        };
-
-        /**
-         * Returns the absolute sheet offset of a specific position inside a
-         * column/row in pixels, according to the current sheet zoom factor.
-         *
-         * @param {Number} index
-         *  The zero-based index of the column/row.
-         *
-         * @param {Number} offsetHmm
-         *  The relative offset inside the column/row, in 1/100 mm. If this
-         *  offset is larger than the total size of the column/row, the
-         *  resulting absolute sheet offset will be located at the end position
-         *  of the column/row.
-         *
-         * @returns {Number}
-         *  The absolute position of the specified entry offset, in pixels.
-         */
-        this.getEntryOffset = function (index, offsetHmm) {
-            var entryData = this.getEntry(index);
-            return entryData.offset + Utils.minMax(convertHmmToPixels(offsetHmm), 0, entryData.size);
-        };
-
         /**
          * Returns information about the column/row with the passed index, if
          * it is visible; otherwise about the first visible column/row that
@@ -1138,18 +1032,23 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          * @param {Number} index
          *  The zero-based index of a column/row.
          *
+         * @param {Number} [lastIndex]
+         *  If specified, searching for a visible entry will stop after the
+         *  column/row with this index. The index MUST be equal or greater than
+         *  the passed index. If omitted, searches to the end of the sheet.
+         *
          * @returns {Object|Null}
          *  A descriptor object with all properties described for the return
          *  value of the method ColRowCollection.getEntry(); or null, if no
          *  more visible columns/rows are available.
          */
-        this.getNextVisibleEntry = function (index) {
+        this.getNextVisibleEntry = function (index, lastIndex) {
 
             var // the resulting entry data
                 entryData = null;
 
             // use iterator to visit the first visible entry following the passed index
-            this.iterateEntries({ first: index, last: maxIndex }, function (entry) {
+            this.iterateEntries({ first: index, last: _.isNumber(lastIndex) ? lastIndex : maxIndex }, function (entry) {
                 entryData = entry;
                 return Utils.BREAK;
             });
@@ -1165,18 +1064,24 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          * @param {Number} index
          *  The zero-based index of a column/row.
          *
+         * @param {Number} [firstIndex]
+         *  If specified, searching for a visible entry will stop before the
+         *  column/row with this index. The index MUST be equal or less than
+         *  the passed index. If omitted, searches to the beginning of the
+         *  sheet.
+         *
          * @returns {Object|Null}
          *  A descriptor object with all properties described for the return
          *  value of the method ColRowCollection.getEntry(); or null, if no
          *  more visible columns/rows are available.
          */
-        this.getPrevVisibleEntry = function (index) {
+        this.getPrevVisibleEntry = function (index, firstIndex) {
 
             var // the resulting entry data
                 entryData = null;
 
             // use iterator to visit the first visible entry following the passed index
-            this.iterateEntries({ first: 0, last: index }, function (entry) {
+            this.iterateEntries({ first: _.isNumber(firstIndex) ? firstIndex : 0, last: index }, function (entry) {
                 entryData = entry;
                 return Utils.BREAK;
             }, { reverse: true });
@@ -1261,10 +1166,171 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          * @returns {Boolean}
          *  Whether all columns/rows in the passed interval are hidden.
          */
-        this.isHiddenInterval = function (interval) {
+        this.isIntervalHidden = function (interval) {
             return this.getIntervalPosition(interval).size === 0;
         };
 
+        /**
+         * Returns a copy of the passed column/row interval. If the first entry
+         * of the passed interval is hidden, the returned interval will start
+         * at the first visible entry contained in the interval. Same applies
+         * to the last entry of the passed interval. If the entire interval is
+         * hidden, null will be returned.
+         *
+         * @param {Object} interval
+         *  The column/row interval to be processed, with the zero-based index
+         *  properties 'first' and 'last'.
+         *
+         * @returns {Object|Null}
+         *  The visible part of the passed interval (but may contain inner
+         *  hidden columns/rows); or null, if the entire interval is hidden.
+         */
+        this.getVisibleInterval = function (interval) {
+
+            var // first visible collection entry
+                firstEntry = this.getNextVisibleEntry(interval.first, interval.last),
+                // last visible collection entry
+                lastEntry = firstEntry ? this.getPrevVisibleEntry(interval.last, interval.first) : null;
+
+            // return the resulting interval
+            return (firstEntry && lastEntry) ? { first: firstEntry.index, last: lastEntry.index } : null;
+        };
+
+        /**
+         * Returns a copy of the passed column/row interval, which will be
+         * expanded to include the next available visible entry in both
+         * directions.
+         *
+         * @param {Object} interval
+         *  The column/row interval to be expanded, with the zero-based index
+         *  properties 'first' and 'last'.
+         *
+         * @returns {Object}
+         *  The expanded interval. If no visible entry has been found before or
+         *  after the passed interval, the respective index of the passed
+         *  original interval will be returned.
+         */
+        this.expandVisibleInterval = function (interval) {
+
+            var // preceding visible collection entry
+                prevEntry = this.getPrevVisibleEntry(interval.first - 1),
+                // following visible collection entry
+                nextEntry = this.getNextVisibleEntry(interval.last + 1);
+
+            // return the resulting interval
+            return { first: prevEntry ? prevEntry.index : interval.first, last: nextEntry ? nextEntry.index : interval.last };
+        };
+
+        /**
+         * Returns the visible column/row intervals contained in the passed
+         * interval.
+         *
+         * @param {Object} interval
+         *  The column/row interval to be processed, with the zero-based index
+         *  properties 'first' and 'last'.
+         *
+         * @param {Object} [options]
+         *  The following options are supported:
+         *  @param {Number} [options.lastHidden=0]
+         *      If set to a number greater than 0, the start positions of the
+         *      returned intervals will be decreased by the specified number,
+         *      so that they include that number of hidden columns/rows
+         *      preceding each interval of visible columns/rows.
+         *
+         * @returns {Array}
+         *  An array with all visible column/row intervals contained in the
+         *  passed interval. If the passed interval is completely hidden, the
+         *  returned array will be empty.
+         */
+        this.getVisibleIntervals = function (interval, options) {
+
+            var // number of preceding hidden column/row in each interval
+                lastHidden = Utils.getIntegerOption(options, 'lastHidden', 0, 0),
+                // the resulting visible intervals
+                visibleIntervals = [],
+                // the last inserted result interval
+                lastInterval = null;
+
+            iterateEntries(interval, function (entry, first, last) {
+
+                if (entry.size === 0) { return; }
+
+                // expand start point of the interval
+                first = Math.max(interval.first, first - lastHidden);
+
+                // try to expand last existing interval in the result
+                if (lastInterval && (lastInterval.last + 1 >= first)) {
+                    lastInterval.last = last;
+                } else {
+                    visibleIntervals.push(lastInterval = { first: first, last: last });
+                }
+            });
+
+            return visibleIntervals;
+        };
+
+        /**
+         * Returns the indexes of all visible collection entries in the passed
+         * interval as plain array.
+         *
+         * @param {Object} interval
+         *  The column/row interval to be processed, with the zero-based index
+         *  properties 'first' and 'last'.
+         *
+         * @returns {Number[]}
+         *  The indexes of all visible collection entries, in ascending order.
+         */
+        this.getVisibleEntryIndexes = function (interval) {
+            var indexes = [];
+            this.iterateEntries(interval, function (entry) {
+                indexes.push(entry.index);
+            });
+            return indexes;
+        };
+
+        /**
+         * Returns the absolute sheet offset of a specific position inside a
+         * column/row in 1/100 mm, independent from the current sheet zoom
+         * factor.
+         *
+         * @param {Number} index
+         *  The zero-based index of the column/row.
+         *
+         * @param {Number} offsetHmm
+         *  The relative offset inside the column/row, in 1/100 mm. If this
+         *  offset is larger than the total size of the column/row, the
+         *  resulting absolute sheet offset will be located at the end position
+         *  of the column/row.
+         *
+         * @returns {Number}
+         *  The absolute position of the specified entry offset, in 1/100 mm.
+         */
+        this.getEntryOffsetHmm = function (index, offsetHmm) {
+            var entryData = this.getEntry(index);
+            return entryData.offsetHmm + Utils.minMax(offsetHmm, 0, entryData.sizeHmm);
+        };
+
+        /**
+         * Returns the absolute sheet offset of a specific position inside a
+         * column/row in pixels, according to the current sheet zoom factor.
+         *
+         * @param {Number} index
+         *  The zero-based index of the column/row.
+         *
+         * @param {Number} offsetHmm
+         *  The relative offset inside the column/row, in 1/100 mm. If this
+         *  offset is larger than the total size of the column/row, the
+         *  resulting absolute sheet offset will be located at the end position
+         *  of the column/row.
+         *
+         * @returns {Number}
+         *  The absolute position of the specified entry offset, in pixels.
+         */
+        this.getEntryOffset = function (index, offsetHmm) {
+            var entryData = this.getEntry(index);
+            return entryData.offset + Utils.minMax(convertHmmToPixels(offsetHmm), 0, entryData.size);
+        };
+
         /**
          * Returns information about the column/row covering the passed offset
          * in the sheet in pixels according to the current sheet zoom factor,
@@ -1275,8 +1341,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  millimeters, according to the 'pixel' option (see below).
          *
          * @param {Object} [options]
-         *  A map with additional options controlling the behavior of this
-         *  method. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.pixel=false]
          *      If set to true, the parameter 'offset' is interpreted as length
          *      in pixels according to the current sheet zoom factor.
@@ -1312,8 +1377,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  millimeters, according to the 'pixel' option (see below).
          *
          * @param {Object} [options]
-         *  A map with additional options controlling the behavior of this
-         *  method. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.pixel=false]
          *      If set to true, the parameter 'offset' is interpreted as length
          *      in pixels according to the current sheet zoom factor.
@@ -1692,8 +1756,8 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
 
         // update entry sizes after the document has been loaded
         if (!app.isImportFinished()) {
-            // use listenTo(), the collection may be destroyed before import finishes
-            this.listenTo(app, 'docs:import:success', updateDefaultSize);
+            // the collection may be destroyed before import finishes (operation 'deleteSheet')
+            this.listenTo(app.getImportPromise(), 'done', updateDefaultSize);
         }
 
         // constructor called from BaseObject.clone()
@@ -1704,12 +1768,12 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
 
         // clone collection entries passed as hidden argument to the c'tor
         if (_.isArray(arguments[ColRowCollection.length])) {
-            entries = _(arguments[ColRowCollection.length]).map(EntryModel.cloneEntry);
+            entries = _.map(arguments[ColRowCollection.length], EntryModel.cloneEntry);
         }
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            _(entries).invoke('destroy');
+            _.invoke(entries, 'destroy');
             defaultEntry.destroy();
             model = sheetModel = entries = defaultEntry = null;
         });
