@@ -12,22 +12,22 @@
  */
 
 define('io.ox/office/spreadsheet/model/drawing/axistitlemodel',
-    ['io.ox/office/editframework/model/format/attributedmodel'
+    ['io.ox/office/editframework/model/attributedmodel'
     ], function (AttributedModel) {
 
     'use strict';
 
-    // class AxisTitleModel =====================================================
+    // class AxisTitleModel ===================================================
 
     /**
-     * visualisation model for chart axes,
+     * visualization model for chart axes,
      * it defines colors and textual colors
      */
-    function AxisTitleModel(chart, attrs) {
+    function AxisTitleModel(chart) {
 
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, chart.getApp(), attrs, { additionalFamilies: ['text', 'character'] });
+        AttributedModel.call(this, chart.getApp(), {}, { additionalFamilies: ['text', 'character'] });
 
     } // class AxisTitleModel
 
