@@ -14,20 +14,20 @@
 
 define('io.ox/office/spreadsheet/model/drawing/chartmodel',
         ['io.ox/office/tk/utils',
-         'io.ox/office/editframework/model/format/color',
+         'io.ox/office/editframework/utils/color',
          'io.ox/office/drawinglayer/model/drawingmodel',
+         'io.ox/office/drawinglayer/view/chartstyleutil',
+         'io.ox/office/drawinglayer/view/chartformatter',
          'io.ox/office/spreadsheet/utils/sheetutils',
          'io.ox/office/spreadsheet/model/operations',
-         'io.ox/office/spreadsheet/model/tokenarray',
+         'io.ox/office/spreadsheet/model/formula/tokenarray',
          'io.ox/office/spreadsheet/model/drawing/drawingmodelmixin',
          'io.ox/office/spreadsheet/model/drawing/manualcolorchart',
          'io.ox/office/spreadsheet/model/drawing/axismodel',
          'io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
          'io.ox/office/spreadsheet/model/drawing/legendmodel',
-         'io.ox/office/spreadsheet/model/drawing/axistitlemodel',
-         'io.ox/office/drawinglayer/view/chartstyleutil',
-         'gettext!io.ox/office/spreadsheet'
-        ], function (Utils, Color, DrawingModel, SheetUtils, Operations, TokenArray, DrawingModelMixin, ManualColor, AxisModel, DataSeriesModel, LegendModel, AxisTitleModel, ChartStyleUtil, gt) {
+         'io.ox/office/spreadsheet/model/drawing/axistitlemodel'
+        ], function (Utils, Color, DrawingModel, ChartStyleUtil, ChartFormatter, SheetUtils, Operations, TokenArray, DrawingModelMixin, ManualColor, AxisModel, DataSeriesModel, LegendModel, AxisTitleModel) {
 
     'use strict';
 
@@ -68,11 +68,6 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
     };
     var MARKER_LIST = ['circle', 'square', 'triangle', 'cross'];
 
-
-    //'io.ox/office/spreadsheet/model/numberformatter'
-    // null date (corresponding to cell value zero) TODO: use null date from document settings
-    var nullDate = new Date(1899, 11, 30).getTime();
-
     // class SheetChartModel ==================================================
 
     /**
@@ -115,21 +110,12 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
 //            zoomEnabled: true,
             bg: 'white',
             backgroundColor: 'transparent',
-            axisX: {},
+            axisX: {stripLines: []},
             axisY: {stripLines: []},
+            axisZ: {stripLines: []},
             creditHref: '',
             creditText: '',
-            title: {
-                text: ''
-                    /*
-                    ,
-                text: 'invisible',
-                verticalAlign: 'center',
-                horizontalAlign: 'right',
-                fontColor: 'transparent',
-                fontSize: 10
-                */
-            },
+            title: { text: ''},
             legend: {},
             data: []
         };
@@ -138,7 +124,8 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
         data.axisY.stripLines[0] = {
             value: 0,
             color: 'black',
-            label : ''
+            label : '',
+            zero: true
         };
 
         var indexLabelAttrs = {
@@ -148,6 +135,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
         };
 
         var colorHandler = new ManualColor(app, self, data);
+        var chartFormatter = new ChartFormatter(self);
 
         // base constructors --------------------------------------------------
 
@@ -159,14 +147,21 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             return app;
         };
 
+        /**
+         * this is only called by the Drawingframe to initialize the REAL CanvasJS-chart
+         */
+        this.getModelData = function () {
+            return data;
+        };
+
         var axes = {
-            x : new AxisModel(this, {}, data.axisX, new AxisTitleModel(this, {}, data.axisX), data.axisY.stripLines),
-            y : new AxisModel(this, {}, data.axisY, new AxisTitleModel(this, {}, data.axisY), data.axisX.stripLines),
-            z : new AxisModel(this, {})
+            x : new AxisModel(self, 'x', 'y'),
+            y : new AxisModel(self, 'y', 'x'),
+            z : new AxisModel(self, 'z', 'x')
         };
 
         var mainName = null;
-        var mainTitleModel = new AxisTitleModel(this, {}, null);
+        var mainTitleModel = new AxisTitleModel(self);
 
         var legendModel = new LegendModel(this, {}, data.legend);
 
@@ -228,6 +223,47 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             if (changed) { self.refresh(); }
         }
 
+        /**
+         * Handles 'change:cells' events triggered by the document model (from
+         * any sheet), and refreshes this chart model debounced, if it refers
+         * to any of the changed cells.
+         */
+        var changeCellsHandler = (function () {
+
+            var // whether any data series overlaps with the changed ranges
+                changed = false;
+
+            function eventHandler(event, sheet, ranges) {
+
+                // chart is already dirty, no need to check the ranges
+                if (changed) { return; }
+
+                // insert sheet index into the ranges (required by series model)
+                ranges = _.map(ranges, function (range) {
+                    return _.extend({ sheet: sheet }, range);
+                });
+
+                // check passed ranges if they overlap with the source ranges of the data series
+                _.each(data.data, function (dataSeries) {
+                    if (dataSeries.needUpdate) {
+                        changed = true;
+                    } else if (dataSeries.model.rangesOverlap(ranges)) {
+                        dataSeries.needUpdate = changed = true;
+                    }
+                });
+            }
+
+            function refreshChart() {
+                if (changed && self && !self.destroyed) {
+                    changed = false;
+                    self.refresh();
+                }
+            }
+
+            // wait quite a long time before refreshing the chart, cell ranges
+            // may be queried in a background loop
+            return app.createDebouncedMethod(eventHandler, refreshChart, { delay: 500 });
+        }());
 
         function changeAttributes(evt, mergedAttributes, oldMergedAttributes) {
             var mc = mergedAttributes.chart;
@@ -253,14 +289,12 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             var lAttrs = legendModel.getMergedAttributes();
             var attributes = self.getMergedAttributes();
 
-            var xyChart = isXYType();
             var cjst = getCJSType();
 
-            var biggestY = 0;
-            var smallestY = 9999999999999;
-            //collect data to keep to small values visible
-
             _(data.data).each(function (dataSeries, index) {
+                if(dataSeries.needUpdate){
+                    return;
+                }
                 if (subtype === 'series' || !index) {
                     colorHandler.handleColor(dataSeries.model.getMergedAttributes(), index);
                 }
@@ -287,63 +321,29 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
                 if (attributes.chart.dataLabel) {
                     dataSeries.indexLabel = '{name}';
                     dataSeries.indexLabelLineColor = null;
+                    dataSeries.indexLabelPlacement = 'inside';
                 } else {
                     dataSeries.indexLabel = ' ';
                     dataSeries.indexLabelLineColor = 'transparent';
+                    dataSeries.indexLabelPlacement = null;
                 }
 
-                var unclean2d = false;
-
-
                 if (cjst === 'line' || cjst === 'spline') {
                     dataSeries.legendMarkerType = dataSeries.marker;
                 } else {
                     dataSeries.legendMarkerType = 'square';
                 }
 
-                _.each(dataSeries.dataPoints, function (dataPoint, n) {
+                _.each(dataSeries.dataPoints, function (dataPoint) {
                     if (cjst === 'line' || cjst === 'spline') {
                         dataPoint.markerType = dataSeries.marker;
                     } else {
                         dataPoint.markerType = null;
                     }
-
-                    //bubble & scatter are 2d-coordinate oriented, they want real numbers,b ut we dont always have real numbers!
-                    if (xyChart) {
-                        dataPoint.x = dataPoint.realX;
-                        if (dataPoint.x === undefined || dataPoint.x === null || (!_.isDate(dataPoint.x) && isNaN(dataPoint.x))) {
-                            unclean2d = true;
-                        }
-                    } else {
-                        dataPoint.x = n + 1;
-                        biggestY = Math.max(Math.abs(dataPoint.realY), biggestY);
-                        smallestY = Math.min(Math.abs(dataPoint.realY), smallestY);
-                    }
                 });
-
-                if (unclean2d) {
-                    _.each(dataSeries.dataPoints, function (dataPoint, n) {
-                        dataPoint.x = n + 1;
-                    });
-                }
             });
 
-            var thresholdY = 0.005;
-            //workaround that too small values stay visible
-            if (!xyChart && (smallestY / biggestY) < thresholdY) {
-                _(data.data).each(function (dataSeries) {
-                    _.each(dataSeries.dataPoints, function (dataPoint) {
-                        var rel = dataPoint.realY / biggestY;
-                        if (Math.abs(rel)<thresholdY) {
-                            if (rel<0) {
-                                dataPoint.y = -thresholdY * biggestY;
-                            } else {
-                                dataPoint.y = thresholdY * biggestY;
-                            }
-                        }
-                    });
-                });
-            }
+            chartFormatter.format();
 
             var mnt = mainTitleModel.getMergedAttributes();
             data.title.text = mnt.text.link[0] || '';
@@ -382,32 +382,12 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             self.trigger('change:drawing');
         }
 
-
-        function optCellString(cell, def) {
-            if (cell && cell.display) {
-                var trim = cell.display.trim();
-                if (trim.length) {
-                    return trim;
-                }
-            }
-            return def;
-        }
-
-        /**
-         * checks for assigned numberformat,
-         * if there is only a code it looks in the clipboard-api for the code's numberformat
-         *
-         * if there is a 'm' in the numberformat, the code returns true because minutes or months are in Dates
-         */
-        function isDate(values) {
-            if (!values || !values.length) {
-                return;
-            }
-            return ChartStyleUtil.isDate(values[0]);
-        }
-
         // public methods -----------------------------------------------------
 
+        this.setMainName = function(name) {
+            mainName = name;
+        };
+
         /**
          * is called by the DrawingController for a selected chart
          */
@@ -422,7 +402,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
                 type = type + ' ' + stacking;
             }
             var curved = attributes.chart.curved;
-            if (curved && (type === 'line' || type === 'scatter')) {
+            if (curved && /(line|scatter)/.test(type)) {
                 type = type + ' curved';
             }
             return type;
@@ -433,13 +413,6 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             return chartType.replace('clustered', 'standard');
         };
 
-        /**
-         * this is only called by the Drawingframe to initialize the REAL CanvasJS-chart
-         */
-        this.getModelData = function () {
-            return data;
-        };
-
         /**
          * @param operation
          */
@@ -456,6 +429,10 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
          */
         this.refresh = app.createDebouncedMethod($.noop, function () {
 
+            if (!app.isImportFinished()) {
+                return;
+            }
+
             var // all ranges to be querried from server, as array of arrays of range addresses
                 sourceRanges = null,
                 // maps series indexes and source link types to array elements in 'sourceRanges'
@@ -495,17 +472,12 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             if (sourceRanges.length > 0) {
 
                 var // query cell contents for the data series
-                    request = app.queryCellResults(sourceRanges, {attributes: true});
+                    request = docModel.queryCellContents(sourceRanges, { attributes: true });
 
-                request.done(function (allContents) {
+                // do not call the handler function, if the chart has been deleted in the meantime
+                self.listenTo(request, 'done', function (allContents) {
 
-                    // asynchronous server request: chart may have been deleted in the meantime
-                    if (!self || self.destroyed) { return; }
-
-                    _(collectInfo).each(function (info, seriesIndex) {
-
-                        var seriesIndex = info.series,
-                            dataSeries = data.data[seriesIndex];
+                    _(collectInfo).each(function (info) {
 
                         // returns the cell contents for the specified source link type
                         function getCellContents(key) {
@@ -521,77 +493,8 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
                         var namesSource = getCellContents('names');
                         var bubblesSource = getCellContents('bubbles');
 
+                        chartFormatter.update(info.series, valuesSource, titleSource, namesSource, bubblesSource);
 
-                        var nameIsDate = isDate(namesSource);
-
-                        // build series title
-                        if (titleSource.length > 0) {
-                            // TODO: join title originating from multiple cells
-                            dataSeries.name = optCellString(titleSource[0]);
-                            if (!seriesIndex) {
-                                mainName = dataSeries.name;
-                            }
-                        } else {
-                            if (!seriesIndex) {
-                                mainName = null;
-                            }
-                        }
-                        if (!dataSeries.name) {
-                            dataSeries.name =
-                            //#. A data series in a chart object (a sequence of multiple data points with a title)
-                            //#. %1$d is the numeric index of the series (e.g. "Series 1", "Series 2", etc.)
-                            //#. This label should match the default title for data series in other spreadsheet
-                            //#. applications (OpenOffice/LibreOffice Calc, Microsoft Excel).
-                            //#, c-format
-                            gt('Series %1$d', _.noI18n(seriesIndex + 1));
-                        }
-
-                        dataSeries.dataPoints = [];
-                        _.each(valuesSource, function (valueCell, n) {
-                            var dataPoint = {};
-                            dataSeries.dataPoints.push(dataPoint);
-
-                            // add value of data point (Y value)
-
-                            dataPoint.y = isNaN(valueCell.result) ? 0 : valueCell.result;
-
-                            // add title of data point (category name), add X value of data point (in scatter charts)
-                            var nameCell = namesSource[n];
-                            dataPoint.label = optCellString(nameCell, String(n + 1));
-                            if (dataPoint.label.length > 23) {
-                                dataPoint.label = dataPoint.label.substring(0, 20) + '...';
-                            }
-                            dataPoint.legendText = dataPoint.label;
-
-                            var display = optCellString(valueCell, String(n + 1));
-                            dataPoint.name = display;
-                            if (dataPoint.name && dataPoint.name.length > 23) {
-                                dataPoint.name = dataPoint.name.substring(0, 20) + '...';
-                            }
-
-
-                            dataPoint.toolTipContent = '{label}: {name}';
-
-
-                            dataPoint.realX = (nameCell && !isNaN(nameCell.result)) ? nameCell.result : null;
-                            if (nameIsDate && !isNaN(dataPoint.realX)) {
-                                dataPoint.realX = new Date(nullDate + Math.floor(dataPoint.realX * 86400000));
-                            }
-
-                            if (dataPoint.y === undefined || dataPoint.y === null || isNaN(dataPoint.y)) {
-                                dataPoint.y = 0;
-                            }
-
-                            dataPoint.realY = dataPoint.y;
-
-                            // add bubble sizes (bubble charts only)
-                            var sizeCell = bubblesSource[n];
-                            if (sizeCell) {
-                                dataPoint.z = sizeCell.result;
-                            } else {
-                                dataPoint.z = 1;
-                            }
-                        });
                     });
                     refreshAxis();
                     self.trigger('change:drawing', 'series');
@@ -602,7 +505,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
                 });
                 self.trigger('change:drawing', 'series');
             }
-        }, { delay: 50, maxDelay: 250 });
+        }, { delay: 50, mayDelay: 250 });
 
         /**
          * saves the Operation-Series in the model, but fetches the correct Data only if its visible
@@ -740,178 +643,12 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             controllerToken.parseFormula(form);
         }
 
-        function makeMin(min, max) {
-            if (Math.abs(max - min) < 0.1 || (min / max < 5 / 6)) {
-                return 0;
-            } else {
-                return min - ((max - min) / 2);
-            }
-        }
-
-        function makeMax(min, max) {
-            var res = max + 0.2 * (max - min);
-            if (res < max) {
-                res = max;
-            }
-            return res;
-        }
-
         function refreshAxis() {
-            refreshAxisX();
-            refreshAxisY();
-        }
-
-        function refreshAxisX() {
             if (!data.data.length) {
                 return;
             }
             axes.x.refreshInfo();
-
-            if (isXYType() || self.getMergedAttributes().chart.type.indexOf('area') === 0) {
-                var xMin = +9999999999999;
-                var xMax = -9999999999999;
-
-                _.each(data.data, function (dd) {
-                    _.each(dd.dataPoints, function (dataPoint) {
-                        var x = dataPoint.x;
-                        if (_.isDate(x)) {
-                            x = x.getTime();
-                        }
-                        xMin = Math.min(xMin, x);
-                        xMax = Math.max(xMax, x);
-                    });
-                });
-
-				//TODO: the random is only here, because canvasjs ignores the values otherwise
-                if (isXYType()) {
-                    data.axisX.minimum = xMin - 1.5 - Math.random() * 0.001;
-                    data.axisX.maximum = xMax + 1.5 + Math.random() * 0.001;
-                } else {
-                    data.axisX.minimum = xMin - 0.25 - Math.random() * 0.001;
-                    data.axisX.maximum = xMax + 0.25 + Math.random() * 0.001;
-                }
-            } else {
-                data.axisX.minimum = null;
-                data.axisX.maximum = null;
-            }
-        }
-
-        function refreshAxisY() {
-            if (!data.data.length) {
-                return;
-            }
-
-            var type = getCJSType();
-
-            var givenMin = null;
-            var givenMax = null;
-
-            var axisInfo = axes.y.getMergedAttributes();
-            if (axisInfo) {
-                if (axisInfo.min !== 'auto') {
-                    givenMin = axes.y.min;
-                }
-                if (axisInfo.max !== 'auto') {
-                    givenMax = axes.y.max;
-                }
-            }
-
             axes.y.refreshInfo();
-
-            var yMin = 9999999999999;
-            var yMax = -9999999999999;
-
-            if (type.indexOf('100') > 0) {
-                var first = data.data[0];
-                _.times(first.dataPoints.length, function (j) {
-                    var y = 0;
-                    _.each(data.data, function (dd) {
-                        y += dd.dataPoints[j].y;
-                    });
-                    yMin = Math.min(yMin, y);
-                    yMax = Math.max(yMax, y);
-                });
-            } else {
-                _.each(data.data, function (dd) {
-                    _.each(dd.dataPoints, function (dataPoint) {
-                        var y = dataPoint.y;
-                        yMin = Math.min(yMin, y);
-                        yMax = Math.max(yMax, y);
-                    });
-                });
-            }
-
-            var intervalCount = 8;
-
-            if (isXYType()) {
-                var diff = yMax - yMin;
-                yMax += 0.5 + diff / intervalCount;
-                yMin -= 0.5 + diff / intervalCount;
-            } else if (type.indexOf('100') > 0) {
-                //Trial & error...
-                if (yMax >= 0 && yMin >= 0) {
-                    //Szenario 1
-                    yMin = givenMin || 0;
-                    yMax = givenMax || 100;
-                } else if (yMax <= 0 && yMin <= 0) {
-                    //Szenario 2
-                    yMax = givenMax || 0;
-                    yMin = givenMin || -100;
-                } else {
-                    //Szenario 3
-                    yMax =  givenMax || 100;
-                    yMin =  givenMin || -100;
-                }
-            } else {
-                if (yMax >= 0 && yMin >= 0) {
-                    //Szenario 1
-                    yMin = givenMin || makeMin(yMin, yMax);
-                    yMax = givenMax || makeMax(yMin, yMax);
-                } else if (yMax <= 0 && yMin <= 0) {
-                    //Szenario 2
-                    yMax = givenMax || -makeMin(-yMax, -yMin);
-                    yMin = givenMin || -makeMax(-yMax, -yMin);
-                } else {
-                    //Szenario 3
-                    var ma = yMax;
-                    var mi = yMin;
-                    yMax =  givenMax || makeMax(mi, ma);
-                    yMin =  givenMin || -makeMax(-ma, -mi);
-                }
-            }
-
-            yMin = Math.round(yMin.toPrecision(2));
-            yMax = Math.round(yMax.toPrecision(2));
-
-            if (yMin === yMax) {
-                yMax = yMin + 1;
-            }
-
-            if (yMin < yMax) {
-                //TODO: the random is only here, because canvasjs ignores the values otherwise
-                var interval = (yMax - yMin) / intervalCount;
-                interval = Math.round(interval.toPrecision(1));
-
-                if (interval === 0) {
-                    data.axisY.minimum = yMin - Math.random() * 0.001;
-                    data.axisY.maximum = yMax + Math.random() * 0.001;
-                    data.axisY.interval = null;
-                } else {
-                    var intCount = Math.round((yMax - yMin) / interval);
-
-                    if (Math.abs(yMax) < Math.abs(yMin)) {
-                        data.axisY.minimum = yMax - (interval * intCount) + Math.random() * 0.001;
-                        data.axisY.maximum = yMax + Math.random() * 0.001;
-                        data.axisY.interval = interval;
-                    } else {
-                        data.axisY.minimum = yMin + Math.random() * 0.001;
-                        data.axisY.maximum = yMin + (interval * intCount) + Math.random() * 0.001;
-                        data.axisY.interval = interval;
-                    }
-
-
-                }
-            }
         }
 
         function getStyleId() {
@@ -921,8 +658,8 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
         function clearSeriesColor(generator, position) {
             _(data.data).each(function (dataSeries, index) {
                 var at = dataSeries.model.getMergedAttributes();
-                if (at.fill.color.type !== 'auto') {
-                    generator.generateDrawingOperation(Operations.SET_CHART_DATASERIES_ATTRIBUTES, position[0], position[1], { series: index, attrs: { fill: { color: Color.AUTO } } });
+                if (at.fill.color.type !== 'auto' || at.line.color.type !== 'auto') {
+                    generator.generateDrawingOperation(Operations.SET_CHART_DATASERIES_ATTRIBUTES, position[0], position[1], { series: index, attrs: { fill: { color: Color.AUTO }, line: { color: Color.AUTO } } });
                 }
             });
         }
@@ -937,6 +674,8 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             return drAttrs;
         }
 
+        this.isXYType = isXYType;
+
         /**
          * return true if the chart has only one series or is type of bubble or donut (because there you see only the first series)
          * @returns {Boolean}
@@ -1068,6 +807,14 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             updateChartType();
         };
 
+        /**
+         * must be called for having correct behavior mit "new CanvasJS.Chart" is called, because of a crazy bug inside there
+         */
+        this.resetData = function() {
+            data.axisX._oldOptions = null;
+            data.axisY._oldOptions = null;
+        };
+
         /**
          * iterates all sources and calculates min and max positions,
          * direction of the series-values and the sheetindex
@@ -1092,35 +839,40 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
 
             _.each(data.data, function (dataSeries) {
                 dataSeries.model.iterateTokens(function (token, name) {
-                    var range = token.resolveRangeList()[0];
-                    if (range) {
-                        if (sheet === null) {
-                            sheet = range.sheet;
-                        } else if (sheet !== range.sheet) {
-                            warn = 'sheets';
-                        }
-                        if (name === 'values') {
-                            var axis = null;
-                            var col = SheetUtils.getColCount(range);
-                            var row = SheetUtils.getRowCount(range);
-                            if (col > row) {
-                                axis = 0;
-                            } else if (row > 1) {
-                                axis = 1;
-                            }
-                            if (_.isNumber(axis)) {
-                                if (mainAxis === null) {
-                                    mainAxis = axis;
-                                } else if (mainAxis !== axis) {
-                                    warn = 'directions';
-                                }
-                            }
-                        }
-                        _.times(2, function (ax) {
-                            start[ax] = Math.min(start[ax], range.start[ax]);
-                            end[ax] = Math.max(end[ax], range.end[ax]);
-                        });
+                    var rangeList = token.resolveRangeList();
+                    if (!rangeList) {
+                        return;
                     }
+                    var range = rangeList[0];
+                    if (!range) {
+                        return;
+                    }
+                    if (sheet === null) {
+                        sheet = range.sheet;
+                    } else if (sheet !== range.sheet) {
+                        warn = 'sheets';
+                    }
+                    if (name === 'values') {
+                        var axis = null;
+                        var col = SheetUtils.getColCount(range);
+                        var row = SheetUtils.getRowCount(range);
+                        if (col > row) {
+                            axis = 0;
+                        } else if (row > 1) {
+                            axis = 1;
+                        }
+                        if (_.isNumber(axis)) {
+                            if (mainAxis === null) {
+                                mainAxis = axis;
+                            } else if (mainAxis !== axis) {
+                                warn = 'directions';
+                            }
+                        }
+                    }
+                    _.times(2, function (ax) {
+                        start[ax] = Math.min(start[ax], range.start[ax]);
+                        end[ax] = Math.max(end[ax], range.end[ax]);
+                    });
                 });
             });
             if (warn) {
@@ -1138,6 +890,19 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
         this.on('change:drawing', changeDrawing);
         this.listenTo(app, 'docs:update:cells', updateNotificationHandler);
         this.listenTo(docModel, 'insert:columns delete:columns insert:rows delete:rows', colRowChangeHandler);
+        this.listenTo(docModel, 'change:cells', changeCellsHandler);
+
+		this.listenTo(app.getImportPromise(), 'done', function() {
+		    app.executeDelayed(function() {
+		        //small hack for fastcalc-loading
+		        var position = self.getPosition();
+		        var view = app.getView();
+
+		        if (position[0] === view.getActiveSheet()) {
+		            self.refresh();
+		        }
+            });
+        });
 
         // constructor called from BaseObject.clone()
         this.registerCloneConstructor(function (newSheetModel, newSheet) {
@@ -1172,7 +937,10 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
                     if (!self || self.destroyed) { return; }
                     _.each(cloneData.axes, function (axisModel, axisType) {
                         self.setAxisAttributes({ axis: axisType, attrs: axisModel.getExplicitAttributes() });
-                        self.setGridlineAttributes({ axis: axisType, attrs: axisModel.getGrid().getExplicitAttributes() });
+                        var axGrid = axisModel.getGrid();
+                        if(axGrid){
+                            self.setGridlineAttributes({ axis: axisType, attrs: axGrid.getExplicitAttributes() });
+                        }
                         var axTitle = axisModel.getTitle();
                         if (axTitle) {
                             self.setTitleAttributes({ axis: axisType, attrs: axTitle.getExplicitAttributes() });
