@@ -144,7 +144,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
          *  part of this drawing collection, otherwise null.
          */
         function resolveDrawingPosition(drawingModel) {
-            var index = _(drawingModels).indexOf(drawingModel);
+            var index = _.indexOf(drawingModels, drawingModel);
             return (index >= 0) ? [index] : null;
         }
 
@@ -319,8 +319,8 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
             }
 
             // whether the drawing is located entirely in hidden columns or rows
-            position.hiddenCols = colCollection.isHiddenInterval({ first: position.startCol, last: position.endCol });
-            position.hiddenRows = rowCollection.isHiddenInterval({ first: position.startRow, last: position.endRow });
+            position.hiddenCols = colCollection.isIntervalHidden({ first: position.startCol, last: position.endCol });
+            position.hiddenRows = rowCollection.isIntervalHidden({ first: position.startRow, last: position.endRow });
 
             return position;
         }
@@ -333,8 +333,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
          *  The values of the drawing anchor attributes, and the anchor type.
          *
          * @param {Object} [options]
-         *  A map with options to control the behavior of this method. The
-         *  following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.displayAnchor=false]
          *      If set to true, uses the runtime behavior defined by the
          *      'anchorType' formatting attribute as anchor mode, instead of
@@ -396,8 +395,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
          *  The drawing model whose anchor attributes will be updated.
          *
          * @param {Object} [options]
-         *  A map with options to control the behavior of this method. The
-         *  following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.displayAnchor=false]
          *      If set to true, uses the runtime behavior defined by the
          *      'anchorType' formatting attribute as anchor mode, instead of
@@ -455,16 +453,14 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
             var // the attribute names according to the passed direction
                 ATTR_NAMES = columns ? HORIZONTAL_ATTRIBUTE_NAMES : VERTICAL_ATTRIBUTE_NAMES,
                 // the maximum column/row
-                maxIndex = columns ? model.getMaxCol() : model.getMaxRow(),
+                maxIndex = model.getMaxIndex(columns),
                 // the number of columns/rows to be inserted
-                move = SheetUtils.getIntervalSize(interval),
-                // the helper function to convert the drawing range to an interval
-                getIntervalFunc = columns ? SheetUtils.getColInterval : SheetUtils.getRowInterval;
+                move = SheetUtils.getIntervalSize(interval);
 
-            _(drawingModels).each(function (drawingModel) {
+            _.each(drawingModels, function (drawingModel) {
 
                 var // the column/row interval covered by the drawing model
-                    drawingInterval = getIntervalFunc(drawingModel.getRange()),
+                    drawingInterval = SheetUtils.getInterval(drawingModel.getRange(), columns),
                     // the maximum number of columns/rows the drawing can be moved
                     maxMove = Math.min(move, maxIndex - drawingInterval.last),
                     // the new anchor attributes
@@ -555,13 +551,10 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
          */
         function changeInterval(interval, columns) {
 
-            var // the helper function to convert the drawing range to an interval
-                getIntervalFunc = columns ? SheetUtils.getColInterval : SheetUtils.getRowInterval;
-
-            _(drawingModels).each(function (drawingModel) {
+            _.each(drawingModels, function (drawingModel) {
 
                 var // the column/row interval covered by the drawing model
-                    drawingInterval = getIntervalFunc(drawingModel.getRange());
+                    drawingInterval = SheetUtils.getInterval(drawingModel.getRange(), columns);
 
                 if (interval.first <= drawingInterval.last) {
                     updateInactiveAnchorAttributes(drawingModel, { displayAnchor: true, notify: true });
@@ -583,8 +576,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
          *  object, the iteration process will be stopped immediately.
          *
          * @param {Object} [options]
-         *  A map with options controlling the behavior of this method. The
-         *  following options are supported:
+         *  Optional parameters:
          *  @param {Object} [options.context]
          *      If specified, the iterator will be called with this context
          *      (the symbol 'this' will be bound to the context inside the
@@ -699,12 +691,12 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
         this.addGenericAnchorAttributes = function (attributes) {
 
             var // the merged attribute set (with default values for all missing attributes)
-                mergedAttributes = model.getStyleSheets('drawing').getMergedAttributes(attributes),
+                mergedAttributes = model.getStyleCollection('drawing').getMergedAttributes(attributes),
                 // the inactive anchor attributes
                 anchorAttributes = getInactiveAnchorAttributes(mergedAttributes.drawing);
 
             // add the generic anchor attributes to the passed attribute set
-            _(['left', 'top', 'width', 'height']).each(function (attrName) {
+            _.each(['left', 'top', 'width', 'height'], function (attrName) {
                 if (attrName in anchorAttributes) {
                     (attributes.drawing || (attributes.drawing = {}))[attrName] = anchorAttributes[attrName];
                 }
@@ -743,7 +735,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
         (function (args) {
             var cloneData = args[SheetDrawingCollection.length];
             if (_.isObject(cloneData)) {
-                _(cloneData.entries).each(function (drawingModel, index) {
+                _.each(cloneData.entries, function (drawingModel, index) {
                     self.insertModelInstance([index], drawingModel.clone(sheetModel, cloneData.newSheet));
                 });
             }
