@@ -35,7 +35,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
      *      (2) {Array} ranges
      *          An array with the addresses of all new merged ranges.
      *      (3) {Object} [options]
-     *          A map with additional options.
+     *          Optional parameters:
      *          - {Boolean} [options.implicit=false]
      *              If set to true, the merged ranges have been inserted
      *              implicitly, for example while rows, columns, or cell ranges
@@ -48,7 +48,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
      *      (2) {Array} ranges
      *          An array with the addresses of all deleted merged ranges.
      *      (3) {Object} [options]
-     *          A map with additional options.
+     *          Optional parameters:
      *          - {Boolean} [options.implicit=false]
      *              If set to true, the merged ranges have been deleted
      *              implicitly, for example while rows, columns, or cell ranges
@@ -101,14 +101,14 @@ define('io.ox/office/spreadsheet/model/mergecollection',
 
             // the number of merged ranges contained by the passed range must be equal to the
             // column/row count of the range, otherwise there is no need to iterate at all
-            if (containedRanges.length !== (columns ? SheetUtils.getColCount(range) : SheetUtils.getRowCount(range))) {
+            if (containedRanges.length !== SheetUtils.getIndexCount(range, columns)) {
                 return false;
             }
 
             // check size and position of the merged ranges (if all merged ranges start and
             // end at the boundaries of the passed range, and are single-sized in the other
             // direction, the entire range is filled by single-sized merged ranges)
-            return _(containedRanges).all(function (mergedRange) {
+            return _.all(containedRanges, function (mergedRange) {
                 return (mergedRange.start[addrIndex] === mergedRange.end[addrIndex]) &&
                     (mergedRange.start[1 - addrIndex] === range.start[1 - addrIndex]) &&
                     (mergedRange.end[1 - addrIndex] === range.end[1 - addrIndex]);
@@ -139,11 +139,11 @@ define('io.ox/office/spreadsheet/model/mergecollection',
             // visit all merged ranges in reversed order, to be able to delete in-place
             Utils.iterateArray(mergedRanges, function (mergedRange, index) {
 
-                var // the transformed range
-                    newRange = model.transformRange(mergedRange, interval, insert, columns);
+                var // get the transformed range (will return null for resulting 1x1 ranges)
+                    newRange = self.transformRange(mergedRange, interval, insert, columns);
 
-                // handle deleted and changed merged ranges (if the resulting range consists of a single cell, remove it too)
-                if (!newRange || (SheetUtils.getCellCount(newRange) === 1)) {
+                // handle deleted and changed merged ranges
+                if (!newRange) {
                     deletedRanges.push(mergedRange);
                     mergedRanges.splice(index, 1);
                 } else if (!_.isEqual(mergedRange, newRange)) {
@@ -214,9 +214,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
          *  Whether the specified range is contained in this collection.
          */
         this.isMergedRange = function (range) {
-            return _(mergedRanges).any(function (mergedRange) {
-                return _.isEqual(mergedRange, range);
-            });
+            return _.any(mergedRanges, function (mergedRange) { return _.isEqual(range, mergedRange); });
         };
 
         /**
@@ -315,18 +313,25 @@ define('io.ox/office/spreadsheet/model/mergecollection',
                 resultRanges = [];
 
             // process all passed ranges
-            _.chain(ranges).getArray().any(function (range) {
+            _.any(_.getArray(ranges), function (range) {
 
                 // iterate backwards through pendingRanges to be able to remove array elements
                 Utils.iterateArray(pendingRanges, function (mergedRange, index) {
 
                     // move matching merged range from pendingRanges to resultRanges
-                    if (SheetUtils.rangesOverlap(range, mergedRange)) {
-                        if ((type === 'reference') && SheetUtils.rangeContainsCell(range, mergedRange.start)) {
-                            resultRanges.push(mergedRange);
-                        } else if ((type === 'contain') && SheetUtils.rangeContainsRange(range, mergedRange)) {
-                            resultRanges.push(mergedRange);
-                        } else {
+                    if (SheetUtils.rangeOverlapsRange(range, mergedRange)) {
+                        switch (type) {
+                        case 'reference':
+                            if (SheetUtils.rangeContainsCell(range, mergedRange.start)) {
+                                resultRanges.push(mergedRange);
+                            }
+                            break;
+                        case 'contain':
+                            if (SheetUtils.rangeContainsRange(range, mergedRange)) {
+                                resultRanges.push(mergedRange);
+                            }
+                            break;
+                        default:
                             resultRanges.push(mergedRange);
                         }
                         // remove ALL overlapping ranges from pendingRanges (regardless of passed type)
@@ -342,35 +347,36 @@ define('io.ox/office/spreadsheet/model/mergecollection',
         };
 
         /**
-         * Returns whether the passed cell range contains or overlaps with at
-         * least one merged cell range.
+         * Returns the merged range that completely contains or is equal to the
+         * passed cell range. Note the different to the method
+         * getMergedRanges() which returns all merged ranges that are contained
+         * IN the passed ranges.
          *
          * @param {Object} range
          *  A range address.
          *
-         * @returns {Boolean}
-         *  Whether the passed cell range contains or overlaps with at least
-         *  one merged cell range.
+         * @returns {Object|Null}
+         *  The address of a merged range that contains the passed range if
+         *  existing, otherwise null
          */
-        this.rangeOverlapsMergedRange = function (range) {
-            return _(mergedRanges).any(function (mergedRange) {
-                return SheetUtils.rangesOverlap(range, mergedRange);
-            });
+        this.getBoundingMergedRange = function (range) {
+            var boundingRange = _.find(mergedRanges, function (mergedRange) { return SheetUtils.rangeContainsRange(mergedRange, range); });
+            return boundingRange ? boundingRange : null;
         };
 
         /**
-         * Returns whether any of the passed cell ranges contains or overlaps
-         * with at least one merged cell range.
+         * Returns whether any of the passed cell ranges overlaps with at least
+         * one merged cell range.
          *
-         * @param {Array} ranges
-         *  Array of range addresses.
+         * @param {Object|Array} ranges
+         *  Address of a single cell range, or an array of range addresses.
          *
          * @returns {Boolean}
          *  Whether any of the passed cell ranges contains or overlaps with at
          *  least one merged cell range.
          */
         this.rangesOverlapAnyMergedRange = function (ranges) {
-            return _(ranges).any(_.bind(this.rangeOverlapsMergedRange, this));
+            return SheetUtils.rangesOverlapRanges(ranges, mergedRanges);
         };
 
         /**
@@ -394,7 +400,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
             // loop iterator function must be defined outside the do-while loop
             function expandToMergedRange(mergedRange, index) {
                 // process all merged ranges that overlap the current range
-                if (SheetUtils.rangesOverlap(range, mergedRange)) {
+                if (SheetUtils.rangeOverlapsRange(range, mergedRange)) {
                     range = SheetUtils.getBoundingRange(range, mergedRange);
                     pendingRanges.splice(index, 1);
                 }
@@ -449,7 +455,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
                 // remove the merged range from the pending array
                 pendingRanges.splice(index, 1);
                 // nothing to do, if merged range is completely outside
-                if (!SheetUtils.rangesOverlap(range, mergedRange)) { return; }
+                if (!SheetUtils.rangeOverlapsRange(range, mergedRange)) { return; }
 
                 // shrink leading border of the range
                 if ((mergedRange.start[addressIndex] < range.start[addressIndex]) && (range.start[addressIndex] <= mergedRange.end[addressIndex])) {
@@ -506,7 +512,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
             function insertMergedRange(mergedRange) {
 
                 var // the array index of the passed range in the 'deletedRanges' array
-                    deletedIndex = Utils.findFirstIndex(deletedRanges, _.bind(_.isEqual, _, mergedRange));
+                    deletedIndex = Utils.findFirstIndex(deletedRanges, _.partial(_.isEqual, mergedRange));
 
                 // if the range has been deleted, remove it from the array, otherwise
                 // insert it into the 'insertedRanges' array (this prevents to notify
@@ -523,8 +529,8 @@ define('io.ox/office/spreadsheet/model/mergecollection',
 
             // always remove all merged ranges that overlap with the passed range,
             // regardless of passed type (needed to merge over merged ranges)
-            mergedRanges = _(mergedRanges).reject(function (mergedRange) {
-                var overlaps = SheetUtils.rangesOverlap(range, mergedRange);
+            mergedRanges = _.reject(mergedRanges, function (mergedRange) {
+                var overlaps = SheetUtils.rangeOverlapsRange(range, mergedRange);
                 if (overlaps) { deletedRanges.push(mergedRange); }
                 return overlaps;
             });
@@ -564,6 +570,40 @@ define('io.ox/office/spreadsheet/model/mergecollection',
             return this;
         };
 
+        /**
+         * Transforms the passed merged range according to the specified column
+         * or row operation. Does NOT modify any merged range in this merged
+         * ranges collection.
+         *
+         * @param {Object} mergedRange
+         *  The address of the merged range to be transformed.
+         *
+         * @param {Object} operationInterval
+         *  The column/row interval of the operation, with the index properties
+         *  'first' and 'last'.
+         *
+         * @param {Boolean} insert
+         *  Whether the specified column/row interval has been inserted into
+         *  the sheet (true), or deleted from the sheet (false).
+         *
+         * @param {Boolean} columns
+         *  Whether the specified interval is a column interval (true), or a
+         *  row interval (false).
+         *
+         * @returns {Object|Null}
+         *  The address of the transformed merged range. If the range has been
+         *  deleted completely while deleting columns/rows, or has been shifted
+         *  outside the sheet completely while inserting columns/rows, or if
+         *  the resulting merged range covers a single cell only, null will be
+         *  returned instead.
+         */
+        this.transformRange = function (mergedRange, operationInterval, insert, columns) {
+            // transform the passed range without expanding the end of the range while inserting
+            mergedRange = model.transformRange(mergedRange, operationInterval, insert, columns);
+            // a transformed merged range covering a single cell is considered deleted
+            return (mergedRange && (SheetUtils.getCellCount(mergedRange) > 1)) ? mergedRange : null;
+        };
+
         // initialization -----------------------------------------------------
 
         // update merged ranges after inserting/deleting columns or rows in the sheet
