@@ -14,9 +14,9 @@
 define('io.ox/office/spreadsheet/model/validationcollection',
     ['io.ox/office/tk/utils',
      'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/editframework/model/format/attributedmodel',
+     'io.ox/office/editframework/model/attributedmodel',
      'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/tokenarray'
+     'io.ox/office/spreadsheet/model/formula/tokenarray'
     ], function (Utils, ModelObject, AttributedModel, SheetUtils, TokenArray) {
 
     'use strict';
@@ -143,8 +143,8 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          */
         this.queryListValues = function (targetAddress) {
 
-            var // the original server query for 'source' validation
-                serverQuery = null,
+            var // the original query for 'source' validation
+                sourceQuery = null,
                 // the resulting promise
                 resultPromise = null;
 
@@ -159,9 +159,9 @@ define('io.ox/office/spreadsheet/model/validationcollection',
                 var refAddress = self.getRefAddress(),
                     sourceRanges = tokenArray1.resolveRangeList({ refAddress: refAddress, targetAddress: targetAddress });
                 if (!_.isArray(sourceRanges)) { return $.Deferred().reject(); }
-                serverQuery = app.queryCellResults([sourceRanges]);
-                return serverQuery.then(function (resultContents) {
-                    return _(resultContents[0]).pluck('display');
+                sourceQuery = model.queryCellContents([sourceRanges], { maxCount: 1000 });
+                return sourceQuery.then(function (resultContents) {
+                    return _.pluck(resultContents[0], 'display');
                 });
             }
 
@@ -179,9 +179,9 @@ define('io.ox/office/spreadsheet/model/validationcollection',
             }
 
             // add an 'abort()' method to the promise that aborts the server query
-            return _(resultPromise).extend({
+            return _.extend(resultPromise, {
                 abort: function () {
-                    if (serverQuery) { serverQuery.abort(); }
+                    if (sourceQuery) { sourceQuery.abort(); }
                     return this;
                 }
             });
@@ -227,10 +227,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          *  the validation collection.
          */
         this.transformRanges = function (interval, insert, columns) {
-            ranges = _.chain(ranges)
-                .map(function (range) { return model.transformRange(range, interval, insert, columns, true); })
-                .filter(_.isObject)
-                .value();
+            ranges = model.transformRanges(ranges, interval, insert, columns, 'expandEnd');
             return ranges.length > 0;
         };
 
@@ -333,7 +330,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          *  validated at all, returns null.
          */
         function getEntry(address) {
-            var entry = _(entries).find(function (entry2) {
+            var entry = _.find(entries, function (entry2) {
                 return entry2.containsCell(address);
             });
             return entry ? entry : null;
@@ -558,7 +555,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
         // destroy all class members
         this.registerDestructor(function () {
             model.unregisterTransformationHandler(transformValues);
-            _(entries).invoke('destroy');
+            _.invoke(entries, 'destroy');
             model = entries = null;
         });
 
