@@ -87,8 +87,7 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', ['io.ox/office/tk/uti
          * 'change:viewattributes' event.
          *
          * @param {Object} attributes
-         *  A map with all view attributes to be changed. The values must
-         *  neither be undefined nor null.
+         *  A map with all view attributes to be changed.
          *
          * @returns {ViewSettingsMixin}
          *  A reference to this instance.
@@ -98,7 +97,7 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', ['io.ox/office/tk/uti
             var // all changed attributes, for listeners
                 changedAttributes = {};
 
-            _(attributes).each(function (value, name) {
+            _.each(attributes, function (value, name) {
                 if (name in definitions) {
 
                     // execute validator
@@ -143,7 +142,7 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', ['io.ox/office/tk/uti
 
         // initialization -----------------------------------------------------
 
-        _(definitions).each(function (definition, name) {
+        _.each(definitions, function (definition, name) {
             viewAttributes[name] = _.copy(definition.def, true);
         });
 
