@@ -12,11 +12,11 @@
  */
 define('io.ox/office/spreadsheet/utils/clipboard',
         ['io.ox/office/tk/utils',
+         'io.ox/office/editframework/utils/color',
+         'io.ox/office/editframework/view/hyperlinkutil',
          'io.ox/office/spreadsheet/utils/sheetutils',
-         'io.ox/office/spreadsheet/model/tokenarray',
-         'io.ox/office/editframework/model/format/color',
-         'io.ox/office/editframework/view/hyperlinkutil'
-        ], function (Utils, SheetUtils, TokenArray, Color, HyperlinkUtil) {
+         'io.ox/office/spreadsheet/model/formula/tokenarray'
+        ], function (Utils, Color, HyperlinkUtil, SheetUtils, TokenArray) {
 
     'use strict';
 
@@ -43,6 +43,9 @@ define('io.ox/office/spreadsheet/utils/clipboard',
             'Estandar'
         ],
 
+        // regex to check for error codes
+        ERROR_CODES_REGEX = /#(DIV\/0|N\/A|NAME|NULL|NUM|REF|VALUE)(\!|\?)/i,
+
         // predefined number formats
         PREDEFINED_NUMBER_FORMATS = (function () {
 
@@ -62,15 +65,15 @@ define('io.ox/office/spreadsheet/utils/clipboard',
                 id_11: '0.00E+00',
                 id_12: '# ?/?',
                 id_13: '# ??/??',
-                id_14: 'mm-dd-yy',
-                id_15: 'd-mmm-yy',
-                id_16: 'd-mmm',
-                id_17: 'mmm-yy',
+                id_14: 'MM-DD-YY',
+                id_15: 'D-MMM-YY',
+                id_16: 'D-MMM',
+                id_17: 'MMM-YY',
                 id_18: 'h:mm AM/PM',
                 id_19: 'h:mm:ss AM/PM',
                 id_20: 'h:mm',
                 id_21: 'h:mm:ss',
-                id_22: 'm/d/yy h:mm',
+                id_22: 'M/D/YY h:mm',
                 id_37: '#,##0 ;(#,##0)',
                 id_38: '#,##0 ;[Red](#,##0)',
                 id_39: '#,##0.00;(#,##0.00)',
@@ -426,6 +429,19 @@ define('io.ox/office/spreadsheet/utils/clipboard',
         }
     };
 
+    /**
+     * Returns whether the passed value is an error code.
+     *
+     * @param {String} value
+     *  Any literal value that can be used in formulas.
+     *
+     * @returns {Boolean}
+     *  Whether the passed value is an error code.
+     */
+    Clipboard.isErrorCode = function (value) {
+        return ERROR_CODES_REGEX.test(value);
+    };
+
     /**
      * Creates a HTML table from the active range of the current selection.
      *
@@ -528,7 +544,9 @@ define('io.ox/office/spreadsheet/utils/clipboard',
                     formula = tokenArray.getRelocatedFormula(range.start, [0, 0]);
                     Utils.log('clipboard - formula - range: ' + JSON.stringify(range) + ', original: ' + cellData.formula + ', relocated: ' + formula);
 
-                    table += ' x:fmla="' + Utils.escapeHTML(formula) + '"';
+                    if (!Clipboard.isErrorCode(formula)) {
+                        table += ' x:fmla="' + Utils.escapeHTML(formula) + '"';
+                    }
                 }
 
                 // add HTML attributes
@@ -541,7 +559,7 @@ define('io.ox/office/spreadsheet/utils/clipboard',
 
                 table += '</td>';
             }
-        }, { merged: true });
+        }, { merged: true, ordered: true });
 
         tokenArray.destroy();
         table += '</tr></tbody></table></body></html>';
@@ -577,7 +595,7 @@ define('io.ox/office/spreadsheet/utils/clipboard',
                 result += '\r\n';
             }
             result += cellData.display;
-        });
+        }, { ordered: true });
 
         return result;
     };
@@ -607,7 +625,7 @@ define('io.ox/office/spreadsheet/utils/clipboard',
 
         if (!charAttributes) { return ''; }
 
-        if (charAttributes.fontName) { characterStyle += 'font-family:' + documentStyles.getCssFontFamily(charAttributes.fontName).replace(/"/g, '\'') + ';'; }
+        if (charAttributes.fontName) { characterStyle += 'font-family:' + documentStyles.getCssFontFamily(charAttributes.fontName) + ';'; }
         if (charAttributes.fontSize) { characterStyle += 'font-size:' + charAttributes.fontSize + 'pt;'; }
 
         characterStyle += 'font-weight:' + ((charAttributes.bold) ? 'bold;' : 'normal;');
@@ -987,12 +1005,12 @@ define('io.ox/office/spreadsheet/utils/clipboard',
         // look for the cell separator
         sep = (text.indexOf('\t') > -1) ? '\t' : ';';
 
-        _(rows).each(function (row) {
+        _.each(rows, function (row) {
             var rowData = [],
                 cells;
 
             cells = row.split(sep);
-            _(cells).each(function (cell) {
+            _.each(cells, function (cell) {
                 rowData.push({ value: Utils.cleanString(cell) });
             });
             contents.push(rowData);
