@@ -42,6 +42,46 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         // regular expression object matching a range name
         RE_RANGE_NAME_A1 = new RegExp('^(' + COL_PATTERN + ROW_PATTERN + '):(' + COL_PATTERN + ROW_PATTERN + ')$', 'i');
 
+    // class ErrorCode ========================================================
+
+    /**
+     * Instances of this class represent literal error codes, as used in cells,
+     * in formula expressions, and as the result of formulas. Implemented as
+     * named class to be able to use the 'instanceof' operator to distinguish
+     * error codes from other cell values.
+     *
+     * @constructor
+     *
+     * @param {String} code
+     *  The native string representation of the error code.
+     *
+     * @property {String} code
+     *  The native string representation of the error code, as passed to the
+     *  constructor.
+     */
+    function ErrorCode(code) { this.code = code; }
+
+    // prototype methods ------------------------------------------------------
+
+    /**
+     * Returns whether the passed value is an error code object (an instance of
+     * the class ErrorCode) of the same type as represented by this error code
+     * object.
+     *
+     * @param {Any} value
+     *  Any literal value that can be used in formulas.
+     *
+     * @returns {Boolean}
+     *  Whether the passed value is an error code object with the same type as
+     *  this error code.
+     */
+    ErrorCode.prototype.equals = function (value) {
+        return (value instanceof ErrorCode) && (value.code === this.code);
+    };
+
+    // string representation for debug logging
+    ErrorCode.prototype.toString = function () { return this.code; };
+
     // static class SheetUtils ================================================
 
     var SheetUtils = {};
@@ -65,11 +105,20 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
     SheetUtils.MAX_ROW_HEIGHT = 20000;
 
     /**
-     * The maximum number of cells to be filled with one range operation.
+     * The maximum number of cells to be filled with one range operation, or
+     * auto-fill operation.
      *
      * @constant
      */
-    SheetUtils.MAX_FILL_CELL_COUNT = 1000;
+    SheetUtils.MAX_FILL_CELL_COUNT = 2000;
+
+    /**
+     * The maximum number of entire columns/rows to be filled with one
+     * auto-fill operation.
+     *
+     * @constant
+     */
+    SheetUtils.MAX_AUTOFILL_COL_ROW_COUNT = 10;
 
     /**
      * Maximum number of merged ranges to be created with a single operation.
@@ -78,6 +127,27 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.MAX_MERGED_RANGES_COUNT = 1000;
 
+    /**
+     * The maximum number of cells to be filled when unmerging a range.
+     *
+     * @constant
+     */
+    SheetUtils.MAX_UNMERGE_CELL_COUNT = 5000;
+
+    /**
+     * The maximum number of columns to be changed with one row operation.
+     *
+     * @constant
+     */
+    SheetUtils.MAX_CHANGE_COLS_COUNT = 2000;
+
+    /**
+     * The maximum number of rows to be changed with one row operation.
+     *
+     * @constant
+     */
+    SheetUtils.MAX_CHANGE_ROWS_COUNT = 2000;
+
     /**
      * Maximum number of cells contained in the selection for local update of
      * selection settings (prevent busy JS if selection is too large).
@@ -102,6 +172,15 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.MAX_LENGTH_STANDARD_EDIT = 21;
 
+    /**
+     * Minimum effective size of a cell, in pixels. Also used as maximum width
+     * of cell border lines, to prevent that the left/right borders or the top/
+     * bottom borders of a cell overlay each other.
+     *
+     * @constant
+     */
+    SheetUtils.MIN_CELL_SIZE = 5;
+
     /**
      * Minimum allowed zoom factor in the spreadsheet view.
      *
@@ -123,7 +202,53 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.MULTI_SELECTION = !Modernizr.touch;
 
-    // methods ----------------------------------------------------------------
+
+    /**
+     * Predefined native error codes, as used in operations.
+     *
+     * @constant
+     */
+    SheetUtils.ErrorCodes = {
+        DIV0:  new ErrorCode('#DIV/0!'),
+        NA:    new ErrorCode('#N/A'),
+        NAME:  new ErrorCode('#NAME?'),
+        NULL:  new ErrorCode('#NULL!'),
+        NUM:   new ErrorCode('#NUM!'),
+        REF:   new ErrorCode('#REF!'),
+        VALUE: new ErrorCode('#VALUE!')
+    };
+
+    // public methods ---------------------------------------------------------
+
+    /**
+     * Returns whether the passed value is an error code object (an instance of
+     * the internal class ErrorCode).
+     *
+     * @param {Any} value
+     *  Any literal value that can be used in formulas.
+     *
+     * @returns {Boolean}
+     *  Whether the passed value is an error code object.
+     */
+    SheetUtils.isErrorCode = function (value) {
+        return value instanceof ErrorCode;
+    };
+
+    /**
+     * Creates an error code object for the passed string representation of an
+     * error code.
+     *
+     * @param {String} code
+     *  The native string representation of an error code.
+     *
+     * @returns {ErrorCode}
+     *  The error code object for the passed error code.
+     */
+    SheetUtils.makeErrorCode = function (code) {
+        return new ErrorCode(code);
+    };
+
+    // cell and range addresses -----------------------------------------------
 
     /**
      * Checks the column and row indexes contained in the passed range, and
@@ -186,6 +311,23 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         return range.end[1] - range.start[1] + 1;
     };
 
+    /**
+     * Returns the number of columns or rows covered by the passed cell range.
+     *
+     * @param {Object} range
+     *  The address of the range.
+     *
+     * @param {Boolean} columns
+     *  Whether to return the number of columns (true), or the number of rows
+     *  (false) in the range.
+     *
+     * @returns {Number}
+     *  The number of columns or rows in the passed range.
+     */
+    SheetUtils.getIndexCount = function (range, columns) {
+        return (columns ? SheetUtils.getColCount : SheetUtils.getRowCount)(range);
+    };
+
     /**
      * Returns the number of cells covered by the passed cell range.
      *
@@ -231,6 +373,25 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         return (range.start[1] <= row) && (row <= range.end[1]);
     };
 
+    /**
+     * Returns whether the passed range contains the specified column index.
+     *
+     * @param {Object} range
+     *  The address of the range.
+     *
+     * @param {Number} index
+     *  The zero-based column or row index to be checked.
+     *
+     * @param {Boolean} columns
+     *  Whether to check the column index (true), or the row index (false).
+     *
+     * @returns {Boolean}
+     *  Whether the passed range contains the column or row index.
+     */
+    SheetUtils.rangeContainsIndex = function (range, index, columns) {
+        return (columns ? SheetUtils.rangeContainsCol : SheetUtils.rangeContainsRow)(range, index);
+    };
+
     /**
      * Returns whether the passed bounding cell range contains the specified
      * cell.
@@ -298,7 +459,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  Whether any of the passed cell ranges contains the column index.
      */
     SheetUtils.rangesContainCol = function (ranges, col) {
-        return _(ranges).any(function (range) { return SheetUtils.rangeContainsCol(range, col); });
+        return _.any(ranges, function (range) { return SheetUtils.rangeContainsCol(range, col); });
     };
 
     /**
@@ -315,7 +476,27 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  Whether any of the passed cell ranges contains the row index.
      */
     SheetUtils.rangesContainRow = function (ranges, row) {
-        return _(ranges).any(function (range) { return SheetUtils.rangeContainsRow(range, row); });
+        return _.any(ranges, function (range) { return SheetUtils.rangeContainsRow(range, row); });
+    };
+
+    /**
+     * Returns whether any of the cell ranges in the passed array contains the
+     * specified column or row index.
+     *
+     * @param {Array} ranges
+     *  The addresses of the cell ranges to be checked.
+     *
+     * @param {Number} index
+     *  The zero-based column or row index to be checked.
+     *
+     * @param {Boolean} columns
+     *  Whether to check the column index (true), or the row index (false).
+     *
+     * @returns {Boolean}
+     *  Whether any of the passed cell ranges contains the column or row index.
+     */
+    SheetUtils.rangesContainIndex = function (ranges, index, columns) {
+        return (columns ? SheetUtils.rangesContainCol : SheetUtils.rangesContainRow)(ranges, index);
     };
 
     /**
@@ -332,7 +513,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  Whether any of the passed cell ranges contains the cell address.
      */
     SheetUtils.rangesContainCell = function (ranges, address) {
-        return _(ranges).any(function (range) { return SheetUtils.rangeContainsCell(range, address); });
+        return _.any(ranges, function (range) { return SheetUtils.rangeContainsCell(range, address); });
     };
 
     /**
@@ -349,7 +530,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  cell, or null if no range could be found.
      */
     SheetUtils.findFirstRange = function (ranges, address) {
-        var result = _(ranges).find(function (range) { return SheetUtils.rangeContainsCell(range, address); });
+        var result = _.find(ranges, function (range) { return SheetUtils.rangeContainsCell(range, address); });
         return result ? result : null;
     };
 
@@ -381,7 +562,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  The address of the resulting unique cell ranges.
      */
     SheetUtils.getUniqueRanges = function (ranges) {
-        return _.chain(ranges).getArray().unique(SheetUtils.getRangeName).value();
+        return _.unique(_.getArray(ranges), SheetUtils.getRangeName);
     };
 
     /**
@@ -396,10 +577,34 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      * @returns {Boolean}
      *  Whether the passed ranges are overlapping.
      */
-    SheetUtils.rangesOverlap = function (range1, range2) {
+    SheetUtils.rangeOverlapsRange = function (range1, range2) {
         return (range1.start[0] <= range2.end[0]) && (range2.start[0] <= range1.end[0]) && (range1.start[1] <= range2.end[1]) && (range2.start[1] <= range1.end[1]);
     };
 
+    /**
+     * Returns whether at least one range in the first range list overlaps with
+     * at least one range in the second range list.
+     *
+     * @param {Object|Array} ranges1
+     *  The address of a single cell range, or an array with range addresses.
+     *
+     * @param {Object|Array} ranges2
+     *  The address of a single cell range, or an array with range addresses.
+     *
+     * @returns {Boolean}
+     *  Whether any ranges in the first list overlaps with any range in the
+     *  second list.
+     */
+    SheetUtils.rangesOverlapRanges = function (ranges1, ranges2) {
+        ranges1 = SheetUtils.getUniqueRanges(ranges1);
+        ranges2 = SheetUtils.getUniqueRanges(ranges2);
+        return _.any(ranges1, function (range1) {
+            return _.any(ranges2, function (range2) {
+                return SheetUtils.rangeOverlapsRange(range1, range2);
+            });
+        });
+     };
+
     /**
      * Returns whether any two ranges in the passed array of cell range
      * addresses overlap each other.
@@ -412,10 +617,8 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.anyRangesOverlap = function (ranges) {
         // TODO: this has O(n^2) complexity, better algorithm?
-        return _(ranges).any(function (range1, index) {
-            return _(ranges.slice(index + 1)).any(function (range2) {
-                return SheetUtils.rangesOverlap(range1, range2);
-            });
+        return _.any(ranges, function (range, index) {
+            return SheetUtils.rangesOverlapRanges(ranges.slice(index + 1), range);
         });
     };
 
@@ -433,7 +636,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  existing; otherwise null.
      */
     SheetUtils.getIntersectionRange = function (range1, range2) {
-        return SheetUtils.rangesOverlap(range1, range2) ? {
+        return SheetUtils.rangeOverlapsRange(range1, range2) ? {
             start: [Math.max(range1.start[0], range2.start[0]), Math.max(range1.start[1], range2.start[1])],
             end: [Math.min(range1.end[0], range2.end[0]), Math.min(range1.end[1], range2.end[1])]
         } : null;
@@ -456,7 +659,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.getIntersectionRanges = function (ranges, boundRange) {
         var result = [];
-        _.chain(ranges).getArray().each(function (range) {
+        _.each(_.getArray(ranges), function (range) {
             var intersectRange = SheetUtils.getIntersectionRange(range, boundRange);
             if (intersectRange) { result.push(intersectRange); }
         });
@@ -487,8 +690,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  process will be stopped immediately.
      *
      * @param {Object} [options]
-     *  A map with options controlling the behavior of this method. The
-     *  following options are supported:
+     *  Optional parameters:
      *  @param {Object} [options.context]
      *      If specified, the iterator will be called with this context (the
      *      symbol 'this' will be bound to the context inside the iterator
@@ -533,8 +735,8 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.getBoundingRange = function () {
         var boundRange = null;
-        _(arguments).each(function (ranges) {
-            _.chain(ranges).getArray().each(function (range) {
+        _.each(arguments, function (ranges) {
+            _.each(_.getArray(ranges), function (range) {
                 if (!boundRange) {
                     boundRange = _.copy(range, true);
                 } else {
@@ -613,10 +815,24 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      * @returns {Boolean}
      *  Whether the passed intervals are overlapping.
      */
-    SheetUtils.intervalsOverlap = function (interval1, interval2) {
+    SheetUtils.intervalOverlapsInterval = function (interval1, interval2) {
         return (interval1.first <= interval2.last) && (interval2.first <= interval1.last);
     };
 
+    /**
+     * Returns the unique intervals form the passed index intervals. Does NOT
+     * join overlapping or adjacent intervals.
+     *
+     * @param {Object|Array} intervals
+     *  A single index interval, or an array with index intervals.
+     *
+     * @returns {Array}
+     *  The resulting unique index intervals.
+     */
+    SheetUtils.getUniqueIntervals = function (intervals) {
+        return _.unique(_.getArray(intervals), function (interval) { return interval.first + ':' + interval.last; });
+    };
+
     /**
      * Returns the intersection of the passed index intervals.
      *
@@ -655,13 +871,40 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.getIntersectionIntervals = function (intervals, boundInterval) {
         var result = [];
-        _.chain(intervals).getArray().each(function (interval) {
+        _.each(_.getArray(intervals), function (interval) {
             var intersectInterval = SheetUtils.getIntersectionInterval(interval, boundInterval);
             if (intersectInterval) { result.push(intersectInterval); }
         });
         return result;
     };
 
+    /**
+     * Returns the bounding interval of the passed index intervals (the
+     * smallest interval that contains all passed intervals).
+     *
+     * @param {Object|Array} [...]
+     *  A single index interval, or an array with index intervals. The number
+     *  of parameters that can be passed to this method is not limited.
+     *
+     * @returns {Object|Null}
+     *  The bounding interval containing all passed index intervals; or null,
+     *  if no interval has been passed.
+     */
+    SheetUtils.getBoundingInterval = function () {
+        var boundInterval = null;
+        _.each(arguments, function (intervals) {
+            _.each(_.getArray(intervals), function (interval) {
+                if (!boundInterval) {
+                    boundInterval = _.clone(interval);
+                } else {
+                    boundInterval.first = Math.min(boundInterval.first, interval.first);
+                    boundInterval.last = Math.max(boundInterval.last, interval.last);
+                }
+            });
+        });
+        return boundInterval;
+    };
+
     /**
      * Merges the passed column or row intervals.
      *
@@ -682,7 +925,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         intervals.sort(function (i1, i2) { return i1.first - i2.first; });
 
         // merge overlapping intervals
-        _(intervals).each(function (interval, index) {
+        _.each(intervals, function (interval, index) {
             var nextInterval = null;
             // extend current interval, as long as adjacent or overlapping with next interval
             while ((nextInterval = intervals[index + 1]) && (interval.last + 1 >= nextInterval.first)) {
@@ -718,11 +961,12 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      * @returns {Array}
      *  The column intervals covering the passed cell ranges. Each interval
      *  object contains the zero-based index properties 'first' and 'last'. The
-     *  intervals are sorted, and intervals covering overlapping or adjacent
-     *  column ranges have been merged into a single interval object.
+     *  intervals are sorted, and overlapping or adjacent intervals have been
+     *  merged into a single interval object.
      */
     SheetUtils.getColIntervals = function (ranges) {
-        return SheetUtils.getUnifiedIntervals(_.chain(ranges).getArray().map(SheetUtils.getColInterval).value());
+        var intervals = _.map(_.getArray(ranges), SheetUtils.getColInterval);
+        return SheetUtils.getUnifiedIntervals(intervals);
     };
 
     /**
@@ -749,11 +993,51 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      * @returns {Array}
      *  The row intervals covering the passed cell ranges. Each interval object
      *  contains the zero-based index properties 'first' and 'last'. The
-     *  intervals are sorted, and intervals covering overlapping or adjacent
-     *  row ranges have been merged into a single interval object.
+     *  intervals are sorted, and overlapping or adjacent intervals have been
+     *  merged into a single interval object.
      */
     SheetUtils.getRowIntervals = function (ranges) {
-        return SheetUtils.getUnifiedIntervals(_.chain(ranges).getArray().map(SheetUtils.getRowInterval).value());
+        var intervals = _.map(_.getArray(ranges), SheetUtils.getRowInterval);
+        return SheetUtils.getUnifiedIntervals(intervals);
+    };
+
+    /**
+     * Returns the column or row interval covered by the passed cell range.
+     *
+     * @param {Object} range
+     *  The address of the cell range.
+     *
+     * @param {Boolean} columns
+     *  Whether to return the column interval (true), or the row interval
+     *  (false) of the range.
+     *
+     * @returns {Object}
+     *  The column or row interval covering the passed cell range, containing
+     *  the zero-based index properties 'first' and 'last'.
+     */
+    SheetUtils.getInterval = function (range, columns) {
+        return (columns ? SheetUtils.getColInterval : SheetUtils.getRowInterval)(range);
+    };
+
+    /**
+     * Returns the sorted and merged column or row intervals covering the
+     * passed range addresses.
+     *
+     * @param {Object|Array} ranges
+     *  The address of a single cell range, or an array with range addresses.
+     *
+     * @param {Boolean} columns
+     *  Whether to return the merged column intervals (true), or the merged row
+     *  intervals (false) of the ranges.
+     *
+     * @returns {Array}
+     *  The column or row intervals covering the passed cell ranges. Each
+     *  interval object contains the zero-based index properties 'first' and
+     *  'last'. The intervals are sorted, and overlapping or adjacent intervals
+     *  have been merged into a single interval object.
+     */
+    SheetUtils.getIntervals = function (ranges, columns) {
+        return (columns ? SheetUtils.getColIntervals : SheetUtils.getRowIntervals)(ranges);
     };
 
     /**
@@ -777,6 +1061,40 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         };
     };
 
+    /**
+     * Builds an array of range addresses from the passed lists of column and
+     * row intervals.
+     *
+     * @param {Object|Number|Array} colIntervals
+     *  A column interval, in the zero-based column index properties 'first'
+     *  and 'last', or a single zero-based column index, or an array of column
+     *  intervals (with interval objects, and/or zero-based column indexes).
+     *
+     * @param {Object|Number|Array} rowIntervals
+     *  A row interval, in the zero-based row index properties 'first' and
+     *  'last', or a single zero-based row index, or an array of row intervals
+     *  (with interval objects, and/or zero-based row indexes).
+     *
+     * @returns {Array}
+     *  The range addresses built from the passed intervals.
+     */
+    SheetUtils.makeRangesFromIntervals = function (colIntervals, rowIntervals) {
+
+        var ranges = [];
+
+        colIntervals = _.getArray(colIntervals);
+        rowIntervals = _.getArray(rowIntervals);
+
+        if ((colIntervals.length > 0) && (rowIntervals.length > 0)) {
+            _.each(colIntervals, function (colInterval) {
+                _.each(rowIntervals, function (rowInterval) {
+                    ranges.push(SheetUtils.makeRangeFromIntervals(colInterval, rowInterval));
+                });
+            });
+        }
+        return ranges;
+    };
+
     // unified ranges ---------------------------------------------------------
 
     /**
@@ -810,7 +1128,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         ranges.sort(function (r1, r2) { return r1.start[1] - r2.start[1]; });
 
         // build the row bands array
-        _(ranges).each(function (range) {
+        _.each(ranges, function (range) {
 
             var // the start and end row index of the range
                 firstRow = range.start[1], lastRow = range.end[1],
@@ -851,7 +1169,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         });
 
         // create the column intervals in all row bands
-        _(rowBands).each(function (rowBand) {
+        _.each(rowBands, function (rowBand) {
             rowBand.intervals = SheetUtils.getColIntervals(rowBand.ranges);
             delete rowBand.ranges;
         });
@@ -876,7 +1194,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         }
 
         // build ranges from the row bands and their column intervals
-        _(rowBands).each(function (rowBand, bandIndex) {
+        _.each(rowBands, function (rowBand, bandIndex) {
 
             var // the preceding row band
                 prevRowBand = (bandIndex > 0) ? rowBands[bandIndex - 1] : null,
@@ -887,7 +1205,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
             if (prevRowBand && (prevRowBand.last + 1 === rowBand.first)) {
 
                 // process all intervals of the current row band, extend ranges or move them to the result array
-                _(rowBand.intervals).each(function (interval) {
+                _.each(rowBand.intervals, function (interval) {
 
                     // move all ranges from bandRanges to resultRanges that cannot be merged with the current interval anymore
                     // (taking advantage of the fact that all ranges in bandRanges are distinct and ordered by column)
@@ -912,12 +1230,9 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
 
                 // store all old band ranges in the result array
                 resultRanges = resultRanges.concat(bandRanges);
-                bandRanges = [];
 
                 // create new ranges for this row band
-                _(rowBand.intervals).each(function (interval) {
-                    bandRanges.push(SheetUtils.makeRangeFromIntervals(interval, rowBand));
-                });
+                bandRanges = SheetUtils.makeRangesFromIntervals(rowBand.intervals, rowBand);
             }
         });
 
@@ -965,10 +1280,10 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         // sort and unify the cell addresses
         addresses = _.clone(addresses);
         addresses.sort(SheetUtils.compareCells);
-        addresses = _(addresses).unique(true, SheetUtils.getCellName);
+        addresses = _.unique(addresses, true, SheetUtils.getCellName);
 
         // create row bands with all rows that contains any cells
-        _(addresses).each(function (address) {
+        _.each(addresses, function (address) {
 
             var // the last row band in the array of all row bands
                 lastRowBand = _.last(rowBands),
@@ -1026,10 +1341,10 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
             resultRanges = [];
 
             // process each range in the source range list
-            _(sourceRanges).each(function (range1) {
+            _.each(sourceRanges, function (range1) {
 
                 // check if the ranges cover each other at all
-                if (!SheetUtils.rangesOverlap(range1, range2)) {
+                if (!SheetUtils.rangeOverlapsRange(range1, range2)) {
                     resultRanges.push(range1);
                     return;
                 }
@@ -1079,6 +1394,76 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         return SheetUtils.getUnifiedRanges(resultRanges);
     };
 
+    /**
+     * Shortens the passed cell ranges so that the resulting ranges do not
+     * contain more cells than specified. If a range in the list does not fit
+     * completely, it will be shortened to the appropriate number of upper rows
+     * of the range. If the range cannot be shortened exactly to entire rows,
+     * it will be split into two ranges (the latter range being the leading
+     * part of an inner row). Overlapping ranges will NOT be merged. Example:
+     * Shortening the ranges A1:C3,D1:F3 to 14 cells results in the ranges
+     * A1:C3,D1:F1,D2:E2.
+     *
+     * @param {Object|Array} ranges
+     *  The address of a single range, or an array of range addresses.
+     *
+     * @param {Number} maxCount
+     *  The maximum number of cells allowed in the resulting ranges.
+     *
+     * @returns {Array}
+     *  The shortened range list.
+     */
+    SheetUtils.shortenRangesByCellCount = function (ranges, maxCount) {
+
+        // create deep copy of the passed ranges
+        ranges = _.chain(ranges).getArray().copy(true).value();
+
+        Utils.iterateArray(ranges, function (range, index) {
+
+            // no more cells left (e.g., previous range has occupied the last cell)
+            if (maxCount <= 0) {
+                ranges.splice(index);
+                return Utils.BREAK;
+            }
+
+            var // number of cells in the current range
+                count = SheetUtils.getCellCount(range);
+
+            // skip entirely fitting ranges
+            if (count <= maxCount) {
+                maxCount -= count;
+                return;
+            }
+
+            var // number of columns in the range
+                cols = SheetUtils.getColCount(range),
+                // number of entire rows fitting in the result
+                rows = Math.floor(maxCount / cols);
+
+            // shorten range to entire rows
+            range.end[1] = range.start[1] + rows - 1;
+            if (rows > 0) {
+                maxCount -= (cols * rows);
+                ranges.splice(index + 1);
+            } else {
+                ranges.splice(index);
+            }
+
+            // add part of next row if needed
+            if (maxCount > 0) {
+                var nextRange = _.copy(range, true);
+                nextRange.start[1] = nextRange.end[1] = range.end[1] + 1;
+                nextRange.end[0] = nextRange.start[0] + maxCount - 1;
+                ranges.push(nextRange);
+            }
+
+            // stop iteration
+            return Utils.BREAK;
+        });
+
+        return ranges;
+    };
+
     // unique keys for maps of cells/ranges -----------------------------------
 
     /**
@@ -1107,7 +1492,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  The cell address parsed from the passed unique cell key.
      */
     SheetUtils.parseCellKey = function (key) {
-        return _(key.split(',')).map(function (index) { return parseInt(index, 10); });
+        return _.map(key.split(','), function (index) { return parseInt(index, 10); });
     };
 
     /**
@@ -1189,7 +1574,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  The upper-case string representation of the column intervals.
      */
     SheetUtils.getColIntervalsName = function (intervals, separator) {
-        return _.chain(intervals).getArray().map(SheetUtils.getColIntervalName).value().join(separator);
+        return _.map(_.getArray(intervals), SheetUtils.getColIntervalName).join(separator);
     };
 
     /**
@@ -1233,7 +1618,62 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  The one-based string representation of the row intervals.
      */
     SheetUtils.getRowIntervalsName = function (intervals, separator) {
-        return _.chain(intervals).getArray().map(SheetUtils.getRowIntervalName).value().join(separator);
+        return _.map(_.getArray(intervals), SheetUtils.getRowIntervalName).join(separator);
+    };
+
+    /**
+     * Returns the string representation of the passed column or row index.
+     *
+     * @param {Number} index
+     *  The zero-based column or row index.
+     *
+     * @param {Boolean} columns
+     *  Whether to return a column name (true), or a row name (false).
+     *
+     * @returns {String}
+     *  The string representation of the column or row index.
+     */
+    SheetUtils.getIndexName = function (index, columns) {
+        return (columns ? SheetUtils.getColName : SheetUtils.getRowName)(index);
+    };
+
+    /**
+     * Returns the string representation of the passed column or row interval.
+     *
+     * @param {Object|Number} interval
+     *  The column or row interval, in the zero-based index properties 'first'
+     *  and 'last', or a single zero-based column/row index.
+     *
+     * @param {Boolean} columns
+     *  Whether to return the name of a column interval (true), or a row
+     *  interval (false).
+     *
+     * @returns {String}
+     *  The string representation of the interval.
+     */
+    SheetUtils.getIntervalName = function (interval, columns) {
+        return (columns ? SheetUtils.getColIntervalName : SheetUtils.getRowIntervalName)(interval);
+    };
+
+    /**
+     * Returns the string representation of the passed column or row intervals.
+     *
+     * @param {Number|Object|Array} intervals
+     *  A single column/row index, a single column/row interval, or an array of
+     *  column/row indexes and/or intervals.
+     *
+     * @param {Boolean} columns
+     *  Whether to return the name of column intervals (true), or row intervals
+     *  (false).
+     *
+     * @param {String} [separator=',']
+     *  The separator text inserted between the interval names.
+     *
+     * @returns {String}
+     *  The string representation of the column/row intervals.
+     */
+    SheetUtils.getIntervalsName = function (intervals, columns, separator) {
+        return (columns ? SheetUtils.getColIntervalsName : SheetUtils.getRowIntervalsName)(intervals, separator);
     };
 
     /**
@@ -1275,7 +1715,7 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      *  The string representation of the cell ranges, in A1:A1 notation.
      */
     SheetUtils.getRangesName = function (ranges, separator) {
-        return _.chain(ranges).getArray().map(SheetUtils.getRangeName).value().join(separator);
+        return _.map(_.getArray(ranges), SheetUtils.getRangeName).join(separator);
     };
 
     /**
