@@ -15,14 +15,18 @@ define('io.ox/office/spreadsheet/view/chartcreator',
     ['io.ox/office/tk/utils',
      'io.ox/office/spreadsheet/utils/sheetutils',
      'io.ox/office/spreadsheet/model/operations',
-     'io.ox/office/spreadsheet/model/tokenarray',
+     'io.ox/office/spreadsheet/model/formula/tokenarray',
      'io.ox/office/drawinglayer/view/chartstyleutil',
      'gettext!io.ox/office/spreadsheet'
     ], function (Utils, SheetUtils, Operations, TokenArray, ChartStyleUtil, gt) {
 
     'use strict';
 
-    // class ChartCreator =================================================
+    var // default width of new chart objects
+        CHART_DEFAULT_WIDTH = Utils.convertHmmToLength(15000, 'px', 1),
+
+        // default height of new chart objects
+        CHART_DEFAULT_HEIGHT = Utils.convertHmmToLength(10000, 'px', 1);
 
     var DEFAULT_TEXTCOLOR = {type: 'auto'};
     var DEFAULT_LINECOLOR = {transformations: [{type: 'lumMod', value: 15000}, {type: 'lumOff',value: 85000}],type: 'scheme', value: 'text1'};
@@ -31,7 +35,6 @@ define('io.ox/office/spreadsheet/view/chartcreator',
     var STANDARD_CHAR = {color: DEFAULT_TEXTCOLOR};
     var HEAD_CHAR = {color: DEFAULT_TEXTCOLOR, fontSize: 16};
 
-
     var INSERT_DEFAULTS = (function () {
 
         var AXIS_ENABLED =  { axis: {label: true},  line: NONE_SHAPE, character: STANDARD_CHAR};
@@ -80,7 +83,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
         return formula;
     }
 
-    function addDataSeries(app, generator, sheet, position, series, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray) {
+    function addDataSeries(app, generator, sheet, chartPos, series, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray) {
         var insert = {
             values: getFormula(app, sheet, valueFrom, valueTo)
         };
@@ -103,14 +106,15 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                 fill: {type: 'solid'},
             }
         };
-        generator.generateDrawingOperation(Operations.INSERT_CHART_DATASERIES, sheet, position, options);
+        generator.generateDrawingOperation(Operations.INSERT_CHART_DATASERIES, chartPos[0], chartPos[1], options);
     }
 
-    function generateSeriesOperations(app, range, sheet, generator, position, cells, axis, chart) {
+    function generateSeriesOperations(app, range, sheet, generator, chartPos, cells, axis, chart) {
         if (_.isUndefined(chart.varyColors)) {
             chart.varyColors = true;
         }
-        generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, sheet, position, {attrs: { chart: chart}});
+
+        generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, chartPos[0], chartPos[1], {attrs: { chart: chart}});
 
         var start = range.start;
         var end = range.end;
@@ -136,9 +140,9 @@ define('io.ox/office/spreadsheet/view/chartcreator',
 
         var seriesCount;
         if (chart.type && chart.type.indexOf('bubble') === 0) {
-            seriesCount = generateBubbleSeriesOperations(app, range, sheet, generator, position, cells, useAxis);
+            seriesCount = generateBubbleSeriesOperations(app, range, sheet, generator, chartPos, cells, useAxis);
         } else {
-            seriesCount =  generateStandardSeriesOperations(app, range, sheet, generator, position, cells, useAxis);
+            seriesCount =  generateStandardSeriesOperations(app, range, sheet, generator, chartPos, cells, useAxis);
         }
         if ((seriesCount === 1 && !chart.varyColors) || (chart.type === 'scatter2d' || chart.type === 'line2d')) {
             var send = {};
@@ -147,7 +151,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             }
             //CURVED is a workaround, filter has not all the information it needs!
             send.curved = chart.curved;
-            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, sheet, position, {attrs: { chart: send}});
+            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, chartPos[0], chartPos[1], {attrs: { chart: send}});
         }
         return seriesCount;
     }
@@ -171,7 +175,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      *                         second is a Boolean for the cell right of the first cell
      *  @param activeAxis [1 or 0] column or row for the direction of any series data
      */
-    function generateStandardSeriesOperations(app, range, sheet, generator, position, cells, activeAxis) {
+    function generateStandardSeriesOperations(app, range, sheet, generator, chartPos, cells, activeAxis) {
         var start = range.start;
         var end = range.end;
 
@@ -208,7 +212,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             }
             var valueTo = makeAddress(activeAxis, otherAxis, activeIndex, end[otherAxis]);
 
-            addDataSeries(app, generator, sheet, position, i, keyFrom, keyTo, title, valueFrom, valueTo);
+            addDataSeries(app, generator, sheet, chartPos, i, keyFrom, keyTo, title, valueFrom, valueTo);
         }
         return max;
     }
@@ -225,7 +229,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      *                         second is a Boolean for the cell right of the first cell
      *  @param activeAxis [1 or 0] column or row for the direction of any series data
      */
-    function generateBubbleSeriesOperations(app, range, sheet, generator, position, cells, activeAxis) {
+    function generateBubbleSeriesOperations(app, range, sheet, generator, chartPos, cells, activeAxis) {
         var start = range.start;
         var end = range.end;
 
@@ -284,7 +288,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                 bubbleTo = makeAddress(activeAxis, otherAxis, valueTo[activeAxis] + 1, valueTo[otherAxis]);
             }
 
-            addDataSeries(app, generator, sheet, position, size, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray);
+            addDataSeries(app, generator, sheet, chartPos, size, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray);
             size++;
         }
         return size;
@@ -300,27 +304,36 @@ define('io.ox/office/spreadsheet/view/chartcreator',
         var height = end[1] - start[1];
 
         if (width > 50 || height > 50) {
-            view.yell('warning', gt('It is not possible to create a chart out of more than 50 input cells.'));
+            view.yell({ type: 'warning', message: gt('It is not possible to create a chart out of more than 50 input cells.') });
             if (def) {
                 def.reject();
             }
         } else {
-			var bottAddress = [start[0], start[1] + 1];
+            var bottAddress = [start[0], start[1] + 1];
             var rightAddress = [start[0] + 1, start[1]];
-			var sData = [
-                [{ sheet: sheet, start: bottAddress }],
-                [{ sheet: sheet, start: rightAddress }],
-                [{ sheet: sheet, start: start }]
-            ];
+            var rangesArray = [[
+                    { sheet: sheet, start: bottAddress, end: bottAddress },
+                    { sheet: sheet, start: rightAddress, end: rightAddress },
+                    { sheet: sheet, start: start, end: start }
+                ]];
+
+            app.getModel().queryCellContents(rangesArray, { attributes: true }).done(function (res) {
 
-			var request = app.queryCellResults(sData, {attributes: true});
-            request.done(function (res) {
                 var bottom = false;
                 var right = false;
                 var axis = null;
 
+                // first array element of the result is an array with the contents of the three cells
+                var bottomCell = res[0][0];
+                var rightCell = res[0][1];
+                var startCell = res[0][2];
+
+                function isDate(cellContents) {
+                    return (/^(date|time|datetime)$/).test(cellContents.format.cat);
+                }
+
                 if (width === 0 || height === 0) {
-                    var firstString = (res[2][0] && !_.isNumber(res[2][0].result));
+                    var firstString = !_.isNumber(startCell.result);
                     if (width === 0) {
                         axis = 0;
                         if (firstString) {
@@ -333,23 +346,16 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                         }
                     }
                 } else {
-                    if (!res[2][0] || !res[2][0].display || !res[2][0].display.length) {
-                        bottom = true;
-                        right = true;
-                    } else {
-                        if (res[0][0]) {
-                            bottom = ChartStyleUtil.isDate(res[0][0]) || (!_.isNumber(res[0][0].result));
-                        }
-                        if (res[1][0]) {
-                            right = ChartStyleUtil.isDate(res[1][0]) || (!_.isNumber(res[1][0].result));
-                        }
-                    }
+                    bottom = (startCell.display.length === 0) || isDate(bottomCell) || !_.isNumber(bottomCell.result);
+                    right = (startCell.display.length === 0) || isDate(rightCell) || !_.isNumber(rightCell.result);
                 }
                 follow([bottom, right], axis);
             });
         }
     }
 
+    // class ChartCreator =====================================================
+
     /**
      * ChartCreator makes an insert-Chart-Op and Insert-Chart-DataSeries chosen by current Selection
      * and changes the Selection to a Multiselection by title, names and values
@@ -361,32 +367,30 @@ define('io.ox/office/spreadsheet/view/chartcreator',
         var def = $.Deferred();
 
         var view = app.getView();
-        var sheetModel = view.getActiveSheetModel();
-        var drawingCollection = sheetModel.getDrawingCollection();
+        var drawingCollection = view.getDrawingCollection();
         var sheet = view.getActiveSheet();
         var selection = view.getSelection();
 
         var range = selection.ranges[selection.activeRange];
 
-
         getContentForRange(app, range, sheet, def, function (cells, axis) {
+
             var rect = gridPane.getVisibleRectangle();
+
             var clip = {
-                left: Math.round(rect.left + rect.width / 3),
-                top: Math.round(rect.top + rect.height / 4),
-                width: Math.round(rect.width / 3),
-                height: Math.round(rect.height / 2)
+                left: Math.max(0, Math.round(rect.left + (rect.width - CHART_DEFAULT_WIDTH) / 2)),
+                top: Math.max(0, Math.round(rect.top + (rect.height - CHART_DEFAULT_HEIGHT) / 2)),
+                width: CHART_DEFAULT_WIDTH,
+                height: CHART_DEFAULT_HEIGHT
             };
+
             var attrs = drawingCollection.getAttributesForRectangle(clip);
 
             view.insertDrawing('chart', attrs, function (generator, sheet, position) {
-                if (!chart.chartStyleId) {
-                    chart.chartStyleId = 2;
-                }
-                var seriesCount = generateSeriesOperations(app, range, sheet, generator, position, cells, axis, chart);
-
                 var start = [sheet, position[0]];
 
+                var seriesCount = generateSeriesOperations(app, range, sheet, generator, start, cells, axis, chart);
+
                 var legendPos;
                 if (seriesCount === 1) {
                     //TODO: title
@@ -412,6 +416,11 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                     break;
                 }
 
+                if (!chart.chartStyleId) {
+                    chart.chartStyleId = 2;
+                    generator.generateOperation(Operations.SET_DRAWING_ATTRIBUTES, {start: start, attrs:{chart:{chartStyleId:2},fill:{color:{type:'scheme',value:'light1',fallbackValue:'#ffffff'}}}});
+                }
+
                 generator.generateOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, {axis: 'x', start: start, attrs: defaults.xAxis});
                 generator.generateOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, {axis: 'y', start: start, attrs: defaults.yAxis});
                 generator.generateOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, {axis: 'x', start: start, attrs: defaults.xGrid});
@@ -443,7 +452,6 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      */
     ChartCreator.generateOperations = function (app, range, sheet, chartModel, axis, chartAttrs) {
         var pos = chartModel.getPosition();
-        var position = pos[1];
         var chart;
         if (chartAttrs) {
             chart = chartAttrs;
@@ -473,7 +481,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             });
 
 
-            generateSeriesOperations(app, range, sheet, generator, position, cells, axis, chart);
+            generateSeriesOperations(app, range, sheet, generator, pos, cells, axis, chart);
             model.applyOperations(generator.getOperations());
         });
 
