@@ -14,12 +14,12 @@
 
 define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
     ['io.ox/office/tk/utils',
-     'io.ox/office/tk/dialogs',
      'io.ox/office/tk/keycodes',
-     'io.ox/office/editframework/view/editdialogmixin',
+     'io.ox/office/tk/forms',
+     'io.ox/office/editframework/view/editdialogs',
      'io.ox/office/drawinglayer/view/imageutil',
      'gettext!io.ox/office/spreadsheet'
-    ], function (Utils, Dialogs, KeyCodes, EditDialogMixin, ImageUtil, gt) {
+    ], function (Utils, KeyCodes, Forms, EditDialogs, ImageUtil, gt) {
 
     'use strict';
 
@@ -44,6 +44,20 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
 
         // private methods ----------------------------------------------------
 
+        /**
+         * If the cell in-place edit mode is active, the current edit text will
+         * be validated and committed to the document.
+         *
+         * @returns {jQuery.Promise}
+         *  The promise of a Deferred object that has been resolved, if the
+         *  cell edit mode was not active, or has been left successfully; or
+         *  that has been rejected, if the cell edit mode is still active (e.g.
+         *  if the current formula expression is invalid).
+         */
+        function leaveCellEditMode() {
+            return self.leaveCellEditMode('auto', { validate: true }) ? $.when() : $.Deferred().reject();
+        }
+
         /**
          * Shows a dialog that allows to enter a sheet name, and will invoke an
          * arbitrary callback function for the sheet name. The dialog will be
@@ -57,8 +71,7 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
          *  the dialog will be kept open.
          *
          * @param {Object} [options]
-         *  A map with additional options for the dialog. The following options
-         *  are supported:
+         *  Optional parameters:
          *  @param {String} [options.okLabel]
          *      An alternative label text for the OK button.
          *  @param {String} [options.initialName]
@@ -76,7 +89,7 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
             function showDialog(initialName) {
 
                 var // the dialog box
-                    dialog = new Dialogs.ModalInputDialog({
+                    dialog = new EditDialogs.ModalInputDialog({
                         title: title,
                         okLabel: Utils.getStringOption(options, 'okLabel'),
                         value: initialName,
@@ -86,7 +99,7 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
                     promise = null;
 
                 // close dialog automatically after losing edit rights
-                EditDialogMixin.call(dialog, app);
+                self.closeDialogOnReadOnlyMode(dialog);
 
                 // show the dialog
                 promise = dialog.show();
@@ -99,7 +112,9 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
             }
 
             // show the dialog, return the piped Deferred
-            return showDialog(Utils.getStringOption(options, 'initialName', self.getSheetName()));
+            return leaveCellEditMode().then(function () {
+                return showDialog(Utils.getStringOption(options, 'initialName', self.getSheetName()));
+            });
         }
 
         // methods ------------------------------------------------------------
@@ -116,8 +131,6 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
          *  or if the document has switched to read-only mode.
          */
         this.showCopySheetDialog = function () {
-            // leave cell edit mode before showing the dialog
-            this.leaveCellEditMode('cell');
             // the callback function will be called in the context of this view instance
             return showSheetNameDialog(gt('Copy Sheet'), this.copySheet, { okLabel: gt('Copy'), initialName: app.getModel().generateUnusedSheetName() });
         };
@@ -134,8 +147,6 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
          *  if the document has switched to read-only mode.
          */
         this.showRenameSheetDialog = function () {
-            // leave cell edit mode before showing the dialog
-            this.leaveCellEditMode('cell');
             // the callback function will be called in the context of this view instance
             return showSheetNameDialog(gt('Rename Sheet'), this.setSheetName, { okLabel: gt('Rename') });
         };
@@ -151,11 +162,13 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
          */
         this.showInsertImageDialog = function () {
 
-            // leave cell edit mode before showing the dialog
-            this.leaveCellEditMode('cell');
+            // shows the generic 'Insert Image' dialog
+            function showInsertImageDialog() {
+                return ImageUtil.showInsertImageDialog(app);
+            }
 
-            // show the generic 'Insert Image' dialog
-            return ImageUtil.showInsertImageDialog(app).then(function (imageHolder) {
+            // leave cell edit mode, show the dialog, handle the dialog result
+            return leaveCellEditMode().then(showInsertImageDialog).then(function (imageHolder) {
 
                 var // create the image DOM node, wait for it to load
                     createImagePromise = app.createImageNode(ImageUtil.getFileUrl(app, imageHolder.url));
@@ -215,186 +228,188 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
          */
         this.showReorderSheetsDialog = function () {
 
-            var model = app.getModel(),
-                view = app.getView(),
+            // leave cell edit mode, show the dialog
+            return leaveCellEditMode().then(function () {
 
-                names = model.getSheetNames(),
-                currentSheet = view.getActiveSheet(),
-                currentChosen = null,
+                var model = app.getModel(),
 
-                collectOps = [],
+                    names = model.getSheetNames(),
+                    currentSheet = self.getActiveSheet(),
+                    currentChosen = null,
 
-                dHolder = $('<div>').addClass('dialog-ordersheets-holder'),
+                    collectOps = [],
 
-                list = $('<div>', { role: 'tablist' }).addClass('dialog-ordersheets-list'),
+                    dHolder = $('<div>').addClass('dialog-ordersheets-holder'),
 
-                upButton = $.button({ click: upButtonClickHandler })
-                        .addClass('btn')
-                        .attr('tabindex', 0)
-                        .css({ marginTop: '-3px', marginBottom: '5px', display: 'block' })
-                        .append(Utils.createIcon('fa-arrow-up')),
+                    list = $('<div>', { role: 'tablist' }).addClass('dialog-ordersheets-list'),
 
-                downButton = $.button({ data: { dir: 1 }, click: downButtonClickHandler })
-                        .addClass('btn')
-                        .attr('tabindex', 0)
-                        .css('display', 'block')
-                        .append(Utils.createIcon('fa-arrow-down')),
+                    upButton = $.button({ click: upButtonClickHandler })
+                            .addClass('btn')
+                            .attr('tabindex', 1)
+                            .css({ marginTop: '-3px', marginBottom: '5px', display: 'block' })
+                            .append(Forms.createIconMarkup('fa-arrow-up')),
 
-                control = $('<div>').addClass('dialog-ordersheets-control').append(upButton, downButton),
+                    downButton = $.button({ data: { dir: 1 }, click: downButtonClickHandler })
+                            .addClass('btn')
+                            .attr('tabindex', 1)
+                            .css('display', 'block')
+                            .append(Forms.createIconMarkup('fa-arrow-down')),
 
-                dialog = new Dialogs.ModalDialog({ width: 300, title: gt('Reorder sheets') });
+                    control = $('<div>').addClass('dialog-ordersheets-control').append(upButton, downButton),
 
-            function upButtonClickHandler() {
-                var index = currentChosen.attr('data-index') | 0;
-                var other = currentChosen.prev('*');
-                if (other.length > 0) {
-                    var otherIndex = other.attr('data-index') | 0;
-                    currentChosen.insertBefore(other);
-                    collectOps.push({
-                        from: index,
-                        to: otherIndex
-                    });
-                    currentChosen.attr('data-index', otherIndex);
-                    other.attr('data-index', index);
+                    dialog = new EditDialogs.ModalDialog({ width: 300, title: gt('Reorder sheets') });
+
+                function upButtonClickHandler() {
+                    var index = currentChosen.attr('data-index') | 0;
+                    var other = currentChosen.prev('*');
+                    if (other.length > 0) {
+                        var otherIndex = other.attr('data-index') | 0;
+                        currentChosen.insertBefore(other);
+                        collectOps.push({
+                            from: index,
+                            to: otherIndex
+                        });
+                        currentChosen.attr('data-index', otherIndex);
+                        other.attr('data-index', index);
+                        checkButtonsActive();
+                    }
+                    checkScrollOutside();
+                }
+
+                function downButtonClickHandler() {
+                    var index = currentChosen.attr('data-index') | 0;
+                    var other = currentChosen.next('*');
+                    if (other.length > 0) {
+                        var otherIndex = other.attr('data-index') | 0;
+                        currentChosen.insertAfter(other);
+                        collectOps.push({
+                            from: index,
+                            to: otherIndex
+                        });
+                        currentChosen.attr('data-index', otherIndex);
+                        other.attr('data-index', index);
+                        checkButtonsActive();
+                    }
+                    checkScrollOutside();
+                }
+
+                function checkScrollOutside() {
+                    var pHeight = dHolder.height();
+                    var cHeight = currentChosen.height();
+                    var pos = currentChosen.position();
+                    if (pos.top < 0) {
+                        Utils.scrollToChildNode(dHolder, currentChosen);
+                    } else if (pos.top + cHeight > pHeight) {
+                        Utils.scrollToChildNode(dHolder, currentChosen);
+                    }
+                }
+
+                function checkButtonsActive() {
+                    if (currentChosen.prev('*').length > 0) {
+                        upButton.removeClass('disabled');
+                        upButton.css({ pointerEvents: 'auto' });
+                        upButton.attr('tabindex', 1);
+                    } else {
+                        upButton.addClass('disabled');
+                        upButton.css({ pointerEvents: 'none'});
+                        upButton.removeAttr('tabindex');
+                    }
+
+                    if (currentChosen.next('*').length > 0) {
+                        downButton.removeClass('disabled');
+                        downButton.css({ pointerEvents: 'auto' });
+                        downButton.attr('tabindex', 1);
+                    } else {
+                        downButton.addClass('disabled');
+                        downButton.css({ pointerEvents: 'none'});
+                        downButton.removeAttr('tabindex');
+                    }
+                }
+
+                function selectButton(evt) {
+                    currentChosen.removeClass('btn-primary');
+                    currentChosen = $(evt.currentTarget);
+                    currentChosen.addClass('btn-primary');
                     checkButtonsActive();
                 }
-                checkScrollOutside();
-            }
 
-            function downButtonClickHandler() {
-                var index = currentChosen.attr('data-index') | 0;
-                var other = currentChosen.next('*');
-                if (other.length > 0) {
-                    var otherIndex = other.attr('data-index') | 0;
-                    currentChosen.insertAfter(other);
-                    collectOps.push({
-                        from: index,
-                        to: otherIndex
-                    });
-                    currentChosen.attr('data-index', otherIndex);
-                    other.attr('data-index', index);
-                    checkButtonsActive();
-                }
-                checkScrollOutside();
-            }
+                // close dialog automatically after losing edit rights
+                self.closeDialogOnReadOnlyMode(dialog);
 
-            function checkScrollOutside() {
-                var pHeight = dHolder.height();
-                var cHeight = currentChosen.height();
-                var pos = currentChosen.position();
-                if (pos.top < 0) {
-                    Utils.scrollToChildNode(dHolder, currentChosen);
-                } else if (pos.top + cHeight > pHeight) {
-                    Utils.scrollToChildNode(dHolder, currentChosen);
-                }
-            }
+                _.each(names, function (name, index) {
+                    var sheetModel = model.getSheetModel(index);
 
-            function checkButtonsActive() {
-                if (currentChosen.prev('*').length > 0) {
-                    upButton.removeClass('disabled');
-                    upButton.css({ pointerEvents: 'auto' });
-                    upButton.attr('tabindex', 0);
-                } else {
-                    upButton.addClass('disabled');
-                    upButton.css({ pointerEvents: 'none'});
-                    upButton.removeAttr('tabindex');
+                    if (!sheetModel.getMergedAttributes().sheet.visible) {
+                        return;
+                    }
+
+                    // check the sheet type
+                    if (!self.isSheetTypeSupported(sheetModel.getType())) { return; }
+
+                    name = _.noI18n(Utils.escapeHTML(name));
+                    var button = $('<button class="btn" data-index="' + index + '" title="' + name + '" tabindex="1" role="button">' + name + '</button>');
+
+                    button.on('click', selectButton);
+                    if (index === currentSheet) {
+                        button.addClass('btn-primary');
+                        currentChosen = button;
+                    }
+                    list.append(button);
+                });
+
+                var cnt = dialog.getBody();
+                cnt.css({
+                    paddingRight: '2px',
+                    paddingTop: 0,
+                    paddingBottom: 0
+                });
+
+                function handleUpDown(upDown, ctrlKey) {
+                    var button;
+                    if (ctrlKey) {
+                        button = upDown ? upButton : downButton;
+                    } else {
+                        button = upDown ? currentChosen.prev('*') : currentChosen.next('*');
+                    }
+                    button.click();
+                    currentChosen.focus();
                 }
 
-                if (currentChosen.next('*').length > 0) {
-                    downButton.removeClass('disabled');
-                    downButton.css({ pointerEvents: 'auto' });
-                    downButton.attr('tabindex', 0);
-                } else {
-                    downButton.addClass('disabled');
-                    downButton.css({ pointerEvents: 'none'});
-                    downButton.removeAttr('tabindex');
+                /**
+                 * Handles 'keydown' events on the dialog node
+                 * up/down changes the selected sheet in the dialog
+                 * ctrl + up/down 'moves' the selected sheet inside the dialog
+                 * enter click on the ok button
+                 */
+                function keyDownHandler(evt) {
+                    switch (evt.keyCode) {
+                    case KeyCodes.UP_ARROW:
+                        handleUpDown(true, evt.ctrlKey);
+                        return true;
+                    case KeyCodes.DOWN_ARROW:
+                        handleUpDown(false, evt.ctrlKey);
+                        return true;
+                    }
                 }
-            }
 
-            function selectButton(evt) {
-                currentChosen.removeClass('btn-primary');
-                currentChosen = $(evt.currentTarget);
-                currentChosen.addClass('btn-primary');
+                cnt.parent().on('keydown', keyDownHandler);
+
+                dHolder.append(list);
+
+                dialog.append(dHolder);
+                dialog.append(control);
+
                 checkButtonsActive();
-            }
 
-            // leave cell edit mode before showing the dialog
-            this.leaveCellEditMode('cell');
+                var promise = dialog.show();
 
-            // close dialog automatically after losing edit rights
-            EditDialogMixin.call(dialog, app);
+                checkScrollOutside();
 
-            _.each(names, function (name, index) {
-                var sheetModel = model.getSheetModel(index);
-                if (!sheetModel.getMergedAttributes().sheet.visible) {
-                    return;
-                }
-
-                var button = $('<button>', { 'data-index': index, title: name, tabindex: 0, role: 'button' })
-                    .addClass('btn')
-                    .text(name);
-
-                button.on('click', selectButton);
-                if (index === currentSheet) {
-                    button.addClass('btn-primary');
-                    currentChosen = button;
-                }
-                list.append(button);
-
-            });
-
-            var cnt = dialog.getBody();
-            cnt.css({
-                paddingRight: '2px',
-                paddingTop: 0,
-                paddingBottom: 0
-            });
-
-            function handleUpDown(upDown, ctrlKey) {
-                var button;
-                if (ctrlKey) {
-                    button = upDown ? upButton : downButton;
-                } else {
-                    button = upDown ? currentChosen.prev('*') : currentChosen.next('*');
-                }
-                button.click();
-                currentChosen.focus();
-            }
-
-            /**
-             * Handles 'keydown' events on the dialog node
-             * up/down changes the selected sheet in the dialog
-             * ctrl + up/down 'moves' the selected sheet inside the dialog
-             * enter click on the ok button
-             */
-            function keyDownHandler(evt) {
-                switch (evt.keyCode) {
-                case KeyCodes.UP_ARROW:
-                    handleUpDown(true, evt.ctrlKey);
-                    return true;
-                case KeyCodes.DOWN_ARROW:
-                    handleUpDown(false, evt.ctrlKey);
-                    return true;
-                }
-            }
-
-            cnt.parent().on('keydown', keyDownHandler);
-
-            dHolder.append(list);
-
-            dialog.append(dHolder);
-            dialog.append(control);
-
-            checkButtonsActive();
-
-            var promise = dialog.show();
-
-            checkScrollOutside();
-
-            return promise.done(function () {
-                model.getUndoManager().enterUndoGroup(function () {
-                    _(collectOps).each(function (op) {
-                        model.moveSheet(op.from, op.to);
+                return promise.done(function () {
+                    model.getUndoManager().enterUndoGroup(function () {
+                        _.each(collectOps, function (op) {
+                            model.moveSheet(op.from, op.to);
+                        });
                     });
                 });
             });
