@@ -14,7 +14,7 @@
 
 define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
     ['io.ox/office/tk/utils',
-     'io.ox/office/drawinglayer/model/drawingutils',
+     'io.ox/office/drawinglayer/utils/drawingutils',
      'io.ox/office/drawinglayer/view/drawingframe',
      'io.ox/office/spreadsheet/utils/sheetutils',
      'io.ox/office/spreadsheet/utils/paneutils'
@@ -49,12 +49,6 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
             model = null,
             view = null,
 
-            // the model and collections of the active sheet
-            sheetModel = null,
-            colCollection = null,
-            rowCollection = null,
-            drawingCollection = null,
-
             // the type of the current tracking, and the last finished tracking
             trackingType = null,
             lastTrackingType = null,
@@ -76,16 +70,6 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
             }
         }
 
-        /**
-         * Initializes this grid pane after the active sheet has been changed.
-         */
-        function changeActiveSheetHandler(event, activeSheet, activeSheetModel) {
-            sheetModel = activeSheetModel;
-            colCollection = sheetModel.getColCollection();
-            rowCollection = sheetModel.getRowCollection();
-            drawingCollection = sheetModel.getDrawingCollection();
-        }
-
         /**
          * Returns all important data for the passed tracking event.
          *
@@ -123,8 +107,8 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
             // add the column and row collection entry
             options = Utils.extendOptions(options, { pixel: true });
-            result.colEntry = colCollection.getEntryByOffset(result.left, options);
-            result.rowEntry = rowCollection.getEntryByOffset(result.top, options);
+            result.colEntry = view.getColCollection().getEntryByOffset(result.left, options);
+            result.rowEntry = view.getRowCollection().getEntryByOffset(result.top, options);
 
             // add the cell address
             result.address = [result.colEntry.index, result.rowEntry.index];
@@ -214,7 +198,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 var // the array index in cell addresses
                     index = columns ? 0 : 1,
                     // the maximum column/row index
-                    maxIndex = columns ? model.getMaxCol() : model.getMaxRow(),
+                    maxIndex = model.getMaxIndex(columns),
                     // the column/row descriptor
                     entryData = columns ? trackingData.colEntry : trackingData.rowEntry,
                     // whether the leading border is tracked
@@ -332,7 +316,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
                 if (event.duration <= 750) {
                     // short-tap: select a single cell, or start in-place edit mode
-                    if (!view.hasDrawingSelection() && _.isEqual(selection.activeCell, startAddress)) {
+                    if (!view.hasDrawingSelection() && (_.isEqual(selection.activeCell, startAddress) || SheetUtils.rangesContainCell(selection.ranges, startAddress))) {
                         view.enterCellEditMode();
                     } else {
                         self.trigger('select:start', startAddress, 'select');
@@ -470,7 +454,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     // the root node of the highlighted range that will be tracked
                     rangeNode = trackerNode.closest('.range'),
                     // the highlighted ranges
-                    highlightRanges = sheetModel.getViewAttribute('highlightRanges'),
+                    highlightRanges = view.getSheetViewAttribute('highlightRanges'),
                     // the array index of the tracked range
                     highlightIndex = Utils.getElementAttributeAsInteger(rangeNode, 'data-index', -1);
 
@@ -489,7 +473,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 anchorCell = getAnchorCell(event, origRange, trackerPos, resize);
 
                 // render tracked range with special highlighting
-                sheetModel.setViewAttribute('highlightIndex', highlightIndex);
+                view.setSheetViewAttribute('highlightIndex', highlightIndex);
             }
 
             // updates the highlighted range according to the passed tracking event
@@ -508,7 +492,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
             // finalizes the highlighted range
             function finalizeTracking() {
-                sheetModel.setViewAttribute('highlightIndex', null);
+                view.setSheetViewAttribute('highlightIndex', null);
             }
 
             // return the actual highlightTrackingHandler() method
@@ -545,7 +529,9 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
          */
         var autoFillTrackingHandler = (function () {
 
-            var // the initial selection range
+            var // the sheet range without leading/trailing hidden columns/rows
+                visibleRange = null,
+                // the initial selection range
                 origRange = null,
                 // the sheet rectangle covered by the initial selection range
                 origRectangle = null,
@@ -553,23 +539,53 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 startPos = null,
                 // whether to restrict tracking direction (column/row ranges)
                 fixedColumns = null,
+                // resulting maximum number of columns that can be filled at the leading/trailing border
+                maxLeadingCols = 0,
+                maxTrailingCols = 0,
+                // resulting maximum number of rows that can be filled at the leading/trailing border
+                maxLeadingRows = 0,
+                maxTrailingRows = 0,
                 // the current value for the 'autoFillData' attribute
                 autoFillData = null;
 
             // initializes the auto fill handling according to the passed tracking event
             function initializeTracking() {
 
+                // cancel custom selection when starting auto-fill
+                view.leaveCustomSelectionMode();
+
+                // do not expand auto-fill range into leading/trailing hidden columns/rows
+                visibleRange = view.getVisibleRange(model.getSheetRange());
+                if (!visibleRange) { $.cancelTracking(); return; }
+
                 // store address and position of the original selection range
                 origRange = view.getSelectedRanges()[0];
-                origRectangle = sheetModel.getRangeRectangle(origRange);
+                origRectangle = view.getRangeRectangle(origRange);
                 startPos = { left: origRectangle.left + origRectangle.width, top: origRectangle.top + origRectangle.height };
 
                 // special behavior for column/row ranges
                 fixedColumns = model.isColRange(origRange) ? true : model.isRowRange(origRange) ? false : null;
 
+                // restrict number of columns/rows according to selection mode
+                var maxCols = 0, maxRows = 0;
+                if (fixedColumns === true) {
+                    maxCols = SheetUtils.MAX_AUTOFILL_COL_ROW_COUNT;
+                } else if (fixedColumns === false) {
+                    maxRows = SheetUtils.MAX_AUTOFILL_COL_ROW_COUNT;
+                } else {
+                    maxCols = Math.floor(SheetUtils.MAX_FILL_CELL_COUNT / SheetUtils.getRowCount(origRange));
+                    maxRows = Math.floor(SheetUtils.MAX_FILL_CELL_COUNT / SheetUtils.getColCount(origRange));
+                }
+
+                // calculate maximum number of columns/rows that can be filled at once
+                maxLeadingCols = Math.min(maxCols, Math.max(0, origRange.start[0] - visibleRange.start[0]));
+                maxTrailingCols = Math.min(maxCols, Math.max(0, visibleRange.end[0] - origRange.end[0]));
+                maxLeadingRows = Math.min(maxRows, Math.max(0, origRange.start[1] - visibleRange.start[1]));
+                maxTrailingRows = Math.min(maxRows, Math.max(0, visibleRange.end[1] - origRange.end[1]));
+
                 // render selected range with modified auto-fill style
                 autoFillData = { border: 'bottom', count: 0 };
-                sheetModel.setViewAttribute('autoFillData', autoFillData);
+                view.setSheetViewAttribute('autoFillData', autoFillData);
             }
 
             // updates the auto fill area according to the passed tracking event
@@ -601,13 +617,13 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 } else if (leading) {
                     // expand leading range border
                     autoFillData.border = columns ? 'left' : 'top';
-                    autoFillData.count = origRange.start[addrIndex] - range.start[addrIndex];
+                    autoFillData.count = Math.min(origRange.start[addrIndex] - range.start[addrIndex], columns ? maxLeadingCols : maxLeadingRows);
                 } else {
                     // expand trailing range border
                     autoFillData.border = columns ? 'right' : 'bottom';
-                    autoFillData.count = range.end[addrIndex] - origRange.end[addrIndex];
+                    autoFillData.count = Math.min(range.end[addrIndex] - origRange.end[addrIndex], columns ? maxTrailingCols : maxTrailingRows);
                 }
-                sheetModel.setViewAttribute('autoFillData', autoFillData);
+                view.setSheetViewAttribute('autoFillData', autoFillData);
 
                 // set correct mouse pointer according to direction
                 event.cursor = (_.isBoolean(fixedColumns) || (autoFillData.count === 0)) ? null : columns ? 'ew-resize' : 'ns-resize';
@@ -615,7 +631,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
             // finalizes the selection tracking
             function finalizeTracking() {
-                sheetModel.setViewAttribute('autoFillData', null);
+                view.setSheetViewAttribute('autoFillData', null);
             }
 
             // return the actual autoFillTrackingHandler() method
@@ -690,7 +706,12 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
             // initializes the drawing according to the passed tracking event
             function initializeTracking(event) {
 
-                var // the grid pane layer rectangle
+                var // the column/row collections of the active sheet
+                    colCollection = view.getColCollection(),
+                    rowCollection = view.getRowCollection(),
+                    // the drawing collection of the active sheet
+                    drawingCollection = view.getDrawingCollection(),
+                    // the grid pane layer rectangle
                     layerRectangle = self.getLayerRectangle(),
                     // whether to toggle the selection of the clicked drawing frame
                     toggle = event.shiftKey || event.ctrlKey || event.metaKey,
@@ -727,7 +748,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 }
 
                 // check edit mode and sheet protection after selecting the drawing
-                if (!model.getEditMode() || view.getActiveSheetModel().isLocked()) {
+                if (!model.getEditMode() || view.isSheetLocked()) {
                     $.cancelTracking();
                     return;
                 }
@@ -762,7 +783,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     allStartWidths = [];
                     allStartHeights = [];
 
-                    _(drawings).each(function (position) {
+                    _.each(drawings, function (position) {
 
                         var localDrawingFrame = self.getDrawingFrame(drawingCollection.findModel(position).getUid()).first(),
                             localMoveBox = localDrawingFrame.find('.tracker');
@@ -778,7 +799,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
                 // setting correct size of moveBox (required after undo of drawing attributes on selected drawing
                 if (isMultiSelection) {
-                    _(allDrawingMoveBoxes).each(function (localMoveBox, index) {
+                    _.each(allDrawingMoveBoxes, function (localMoveBox, index) {
                         localMoveBox.css({
                             left: 0,
                             top: 0,
@@ -807,7 +828,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     topResize = /[t]/.test(pos);
 
                     if (isMultiSelection) {
-                        _(allDrawingFrames).each(function (oneDrawingFrame) {
+                        _.each(allDrawingFrames, function (oneDrawingFrame) {
                             allStartWidths.push(oneDrawingFrame.width());
                             allStartHeights.push(oneDrawingFrame.height());
                         });
@@ -834,7 +855,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
                 // the maximum allowed change to the left, top, right and bottom
                 if (isMultiSelection) {
-                    _(allDrawingFrames).each(function (oneDrawingFrame, index) {
+                    _.each(allDrawingFrames, function (oneDrawingFrame, index) {
                         if (doResize) {
                             // receiving explicit attributes of the drawing
                             oneDrawingAttributes = DrawingFrame.getModel(oneDrawingFrame).getExplicitAttributes().drawing;
@@ -1060,7 +1081,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                         // make move box visible when tracking position has moved
 
                         if (isMultiSelection) {
-                            _(allDrawingFrames).each(function (drawingFrame) {
+                            _.each(allDrawingFrames, function (drawingFrame) {
                                 DrawingFrame.toggleTracking(drawingFrame, true);
                             });
                         } else {
@@ -1071,7 +1092,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                             if (! doResize) {
                                 // moving the drawing(s)
                                 if (isMultiSelection) {
-                                    _(allDrawingMoveBoxes).each(function (localMoveBox) {
+                                    _.each(allDrawingMoveBoxes, function (localMoveBox) {
                                         localMoveBox.css({ left: shiftX, top: shiftY });
                                     });
                                 } else {
@@ -1080,7 +1101,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                             } else {
                                 // resizing the drawing(s)
                                 if (isMultiSelection) {
-                                    _(allDrawingMoveBoxes).each(function (localMoveBox, index) {
+                                    _.each(allDrawingMoveBoxes, function (localMoveBox, index) {
                                         localMoveBox.css(getCurrentDrawingResizeAttributes(localMoveBox, allStartWidths[index], allStartHeights[index]));
                                     });
                                 } else {
@@ -1130,7 +1151,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 }
 
                 // modifying attributes
-                attributes = drawingCollection.getAttributesForRectangle({ left: newLeft, top: newTop, width: newWidth, height: newHeight});
+                attributes = view.getDrawingCollection().getAttributesForRectangle({ left: newLeft, top: newTop, width: newWidth, height: newHeight});
 
                 // setting moveBox back into the drawing frame
                 localMoveBox.css({ left: 0, top: 0 });
@@ -1149,7 +1170,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 // generate an operation only, if the drawing was really moved
                 if (mouseMoved && moveBox && applyPosition) {
                     if (isMultiSelection) {
-                        _(allDrawingMoveBoxes).each(function (oneMoveBox, index) {
+                        _.each(allDrawingMoveBoxes, function (oneMoveBox, index) {
                             // index can be used for drawingFrames and drawingPositions, too
                             setDrawingAttributes(oneMoveBox, allDrawingFrames[index], index);
                         });
@@ -1160,7 +1181,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
                 // Clarify: setting moveBox to null is important for further selections of drawing
                 if (isMultiSelection) {
-                    _(allDrawingFrames).each(function (drawingFrame) {
+                    _.each(allDrawingFrames, function (drawingFrame) {
                         // make move box invisible after tracking has ended
                         DrawingFrame.toggleTracking(drawingFrame, false);
                     });
@@ -1269,7 +1290,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
         // initialization -----------------------------------------------------
 
         // initialize class members
-        app.on('docs:init', function () {
+        app.onInit(function () {
 
             // the spreadsheet model and view instances
             model = app.getModel();
@@ -1278,9 +1299,6 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
             // leave in-place edit mode or cancel tracking, if document goes into read-only mode
             model.on('change:editmode', editModeHandler);
 
-            // listen to view events
-            view.on('change:activesheet', changeActiveSheetHandler);
-
             // tracking for cell selection with mouse/touch
             layerRootNode.on({
                 'tracking:start': trackingStartHandler,
@@ -1291,7 +1309,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            model = view = sheetModel = colCollection = rowCollection = drawingCollection = null;
+            app = layerRootNode = self = model = view = null;
         });
 
     } // class GridTrackingMixin
