@@ -15,11 +15,13 @@
 
 define('io.ox/office/text/app/application',
     ['io.ox/office/tk/utils',
+     'io.ox/office/tk/errorcode',
      'io.ox/office/editframework/app/editapplication',
      'io.ox/office/text/editor',
      'io.ox/office/text/view/view',
-     'io.ox/office/text/app/controller'
-    ], function (Utils, EditApplication, TextModel, TextView, TextController) {
+     'io.ox/office/text/app/controller',
+     'gettext!io.ox/office/text'
+    ], function (Utils, ErrorCode, EditApplication, TextModel, TextView, TextController, gt) {
 
     'use strict';
 
@@ -32,27 +34,28 @@ define('io.ox/office/text/app/application',
      *
      * @extends EditApplication
      *
-     * @param {Object} [appOptions]
-     *  A map of static application options, that have been passed to the
-     *  static method BaseApplication.createLauncher().
+     * @param {Object} launchOptions
+     *  All options passed to the core launcher (the ox.launch() method).
      *
-     * @param {Object} [launchOptions]
-     *  A map of options to control the properties of the application. Supports
-     *  all options supported by the base class EditApplication.
+     * @param {Object} [appOptions]
+     *  Static application options that have been passed to the static method
+     *  BaseApplication.createLauncher().
      */
-    var TextApplication = EditApplication.extend({ constructor: function (appOptions, launchOptions) {
+    var TextApplication = EditApplication.extend({ constructor: function (launchOptions, appOptions) {
 
         var // self reference
             self = this;
 
         // base constructor ---------------------------------------------------
 
-        EditApplication.call(this, TextModel, TextView, TextController, appOptions, launchOptions, {
+        EditApplication.call(this, TextModel, TextView, TextController, launchOptions, appOptions, {
+            applyActionsDetached: true,
             postProcessHandler: postProcessDocument,
             postProcessHandlerStorage: postProcessHandlerStorage,
-            postProcessProgressSize: 0.7,
+            fastEmptyLoadHandler: fastEmptyLoadHandler,
             optimizeOperationsHandler: optimizeOperations,
             operationFilter: operationFilter,
+            importFailedHandler: importFailedHandler,
             sendActionsDelay: 1000,
             localStorageApp: true
         });
@@ -85,6 +88,22 @@ define('io.ox/office/text/app/application',
             return self.getModel().updateDocumentFormattingStorage();
         }
 
+        /**
+         * Handler will be called by base class if importing the document
+         * failed. The functions handles Text specific errors.
+         */
+        function importFailedHandler(response) {
+
+            var // specific error code sent by the server
+                error = new ErrorCode(response);
+
+            switch (error.getCodeAsConstant()) {
+            case 'LOADDOCUMENT_COMPLEXITY_TOO_HIGH_ERROR':
+                response.message = gt('This document exceeds the complexity limits.');
+                break;
+            }
+        }
+
         /**
          * Optimizing the actions (operations) before sending them to the server.
          *
@@ -113,6 +132,30 @@ define('io.ox/office/text/app/application',
             return self.getModel().operationFilter(operation);
         }
 
+        /**
+         * Handler for fast load of empty documents
+         *
+         * @param {String} markup
+         *  The HTML mark-up to be shown as initial document contents.
+         *
+         * @param {Array} actions
+         *  The operation actions to be applied to finalize the fast import.
+         *
+         * @returns {jQuery.Promise}
+         *  The promise of a Deferred object that will be resolved when the
+         *  actions have been applied.
+         */
+        function fastEmptyLoadHandler(markup, actions) {
+            return self.getModel().fastEmptyLoadHandler(markup, actions);
+        }
+
+        // initialization -----------------------------------------------------
+
+        // destroy all class members
+        this.on('docs:destroy', function () {
+            launchOptions = appOptions = null;
+        });
+
     }}); // class TextApplication
 
     // static methods ---------------------------------------------------------
@@ -122,7 +165,7 @@ define('io.ox/office/text/app/application',
      * without parameters, to launch text applications.
      */
     TextApplication.createLauncher = function () {
-        return EditApplication.createLauncher('io.ox/office/text', TextApplication, { icon: 'fa-align-left', search: true });
+        return EditApplication.createLauncher('io.ox/office/text', TextApplication, { icon: 'fa-align-left' });
     };
 
     // exports ================================================================
