@@ -13,18 +13,18 @@
 
 define('io.ox/office/text/app/controller',
     ['io.ox/office/tk/utils',
+     'io.ox/office/editframework/utils/border',
+     'io.ox/office/editframework/utils/mixedborder',
+     'io.ox/office/editframework/utils/lineheight',
      'io.ox/office/editframework/app/editcontroller',
-     'io.ox/office/editframework/model/format/border',
-     'io.ox/office/editframework/model/format/mixedborder',
-     'io.ox/office/editframework/model/format/lineheight',
-     'io.ox/office/text/app/config',
      'io.ox/office/drawinglayer/view/imageutil',
+     'io.ox/office/text/utils/config',
      'io.ox/office/text/hyperlink',
      'io.ox/office/text/format/listcollection',
      'io.ox/office/text/format/paragraphstyles',
      'io.ox/office/text/format/tablestyles',
      'io.ox/office/text/format/drawingstyles'
-    ], function (Utils, EditController, Border, MixedBorder, LineHeight, TextConfig, Image, Hyperlink, ListCollection, ParagraphStyles, TableStyles, DrawingStyles) {
+    ], function (Utils, Border, MixedBorder, LineHeight, EditController, Image, Config, Hyperlink, ListCollection, ParagraphStyles, TableStyles, DrawingStyles) {
 
     'use strict';
 
@@ -51,6 +51,32 @@ define('io.ox/office/text/app/controller',
             // all the little controller items
             items = {
 
+                // view -------------------------------------------------------
+
+                'view/zoom': {
+                    parent: 'app/valid',
+                    get: function () { return view.getZoomFactor() / 100; },
+                    set: function (zoom) { view.setZoomType(zoom * 100); }
+                },
+
+                'view/zoom/dec': {
+                    parent: 'app/valid',
+                    enable: function () { return view.getZoomFactor() > view.getMinZoomFactor(); },
+                    set: function () { view.decreaseZoomLevel(); }
+                },
+
+                'view/zoom/inc': {
+                    parent: 'app/valid',
+                    enable: function () { return view.getZoomFactor() < view.getMaxZoomFactor(); },
+                    set: function () { view.increaseZoomLevel(); }
+                },
+
+                'view/zoom/type': {
+                    parent: 'app/valid',
+                    get: function () { return view.getZoomType(); },
+                    set: function (zoomType) { view.setZoomType(zoomType); }
+                },
+
                 'document/editable/text': {
                     parent: 'document/editable',
                     enable: function () { return model.isTextSelected(); }
@@ -59,9 +85,9 @@ define('io.ox/office/text/app/controller',
                 'document/selectall': {
                     // enabled in read-only mode
                     set: function () { model.selectAll(); },
-                    focus: 'never', // do not grab focus after setter (32002)
                     // restrict keyboard shortcut to application pane
-                    shortcut: { keyCode: 'A', ctrlOrMeta: true, selector: '.app-pane' }
+                    shortcut: { keyCode: 'A', ctrlOrMeta: true, selector: '.app-pane' },
+                    preserveFocus: true // do not return focus to document
                 },
 
                 // document contents
@@ -69,27 +95,19 @@ define('io.ox/office/text/app/controller',
                 'document/search/highlight': {
                     // enabled in read-only mode
                     set: function (query) { model.quickSearch(query, 0); },
-                    focus: 'never' // do not grab focus after setter
+                    preserveFocus: true // do not return focus to document
                 },
                 'document/search/nextResult': {
                     // enabled in read-only mode
-                    set: function () {
-                        if (app.getWindow().search.active) {
-                            model.selectNextSearchResult();
-                        }
-                    },
+                    set: function () { model.selectNextSearchResult(); },
                     shortcut: [{ keyCode: 'G', ctrlOrMeta: true }, { keyCode: 'F3' }],
-                    focus: 'never' // do not grab focus after setter
+                    preserveFocus: true // do not return focus to document
                 },
                 'document/search/previousResult': {
                     // enabled in read-only mode
-                    set: function () {
-                        if (app.getWindow().search.active) {
-                            model.selectPreviousSearchResult();
-                        }
-                    },
+                    set: function () { model.selectPreviousSearchResult(); },
                     shortcut: [{ keyCode: 'G', ctrlOrMeta: true, shift: true }, { keyCode: 'F3', shift: true }],
-                    focus: 'never' // do not grab focus after setter
+                    preserveFocus: true // do not return focus to document
                 },
                 'document/search/replaceAll': {
                     parent: 'document/editable',
@@ -99,10 +117,6 @@ define('io.ox/office/text/app/controller',
                     parent: 'document/editable',
                     set: function () { model.replaceSelectedSearchResult(app.getWindow().search.replacement); }
                 },
-                'document/search/close': {
-                    // enabled in read-only mode
-                    set: function () { model.removeHighlighting(); }
-                },
 
                 'document/cut': {
                     parent: 'document/editable',
@@ -122,26 +136,30 @@ define('io.ox/office/text/app/controller',
 
                 // spelling
 
-                'document/spelling/enabled': {
-                    parent: 'document/editable/text',
-                    enable: function () { return TextConfig.isSpellingEnabled(); }
+                'document/spelling/available': {
+                    parent: 'document/editable',
+                    enable: function () { return Config.SPELLING_ENABLED; }
                 },
                 'document/onlinespelling': {
-                    parent: 'document/spelling/enabled',
+                    parent: 'document/spelling/available',
                     get: function () { return model.isOnlineSpelling(); },
                     set: function (state) { model.setOnlineSpelling(state); }
                 },
 
                 // paragraphs
 
-                'paragraph/stylesheet': {
+                'paragraph/group': {
                     parent: 'document/editable/text',
-                    get: function () { return model.getAttributes('paragraph').styleId; },
+                    get: function () { return model.getAttributes('paragraph', {maxIterations: 10}) || {}; }
+                },
+                'paragraph/stylesheet': {
+                    parent: 'paragraph/group',
+                    get: function (paragraph) { return paragraph.styleId; },
                     set: function (styleId) { model.setAttributes('paragraph', { styleId: styleId }, { clear: true }); }
                 },
                 'paragraph/attributes': {
-                    parent: 'document/editable/text',
-                    get: function () { return model.getAttributes('paragraph').paragraph || {}; }
+                    parent: 'paragraph/group',
+                    get: function (paragraph) { return paragraph.paragraph || {}; }
                 },
                 'paragraph/alignment': {
                     parent: 'paragraph/attributes',
@@ -173,23 +191,24 @@ define('io.ox/office/text/app/controller',
                 },
                 'paragraph/borders': {
                     parent: 'paragraph/attributes',
-                    get: function (attributes) { return ParagraphStyles.getBorderModeFromAttributes(attributes); },
-                    set: function (borderMode) { model.setAttributes('paragraph', { paragraph: ParagraphStyles.getAttributesFromBorderMode(borderMode, model.getAttributes('paragraph').paragraph) }); }
+                    get: function (attributes) { return MixedBorder.getBorderMode(attributes, { paragraph: true }); },
+                    set: function (borderMode) {
+                        var borderAttrs = MixedBorder.getBorderAttributes(borderMode, this.getParentValue(), ParagraphStyles.SINGLE_BORDER, { paragraph: true });
+                        model.setAttributes('paragraph', { paragraph: borderAttrs });
+                    }
                 },
 
                 // toggle default bullet list, or select bullet type
                 'paragraph/list/bullet': {
                     parent: 'paragraph/attributes',
-                    get: getListStyleId,
-                    set: setListStyleId,
-                    userData: 'bullet'
+                    get: _.partial(getListStyleId, 'bullet'),
+                    set: _.partial(setListStyleId, 'bullet')
                 },
                 // toggle default numbered list, or select numbering type
                 'paragraph/list/numbered': {
                     parent: 'paragraph/attributes',
-                    get: getListStyleId,
-                    set: setListStyleId,
-                    userData: 'numbering'
+                    get: _.partial(getListStyleId, 'numbering'),
+                    set: _.partial(setListStyleId, 'numbering')
                 },
 
                 // parent for controller items only enabled inside lists
@@ -231,14 +250,18 @@ define('io.ox/office/text/app/controller',
 
                 // characters
 
-                'character/stylesheet': {
+                'character/group': {
                     parent: 'document/editable/text',
-                    get: function () { return model.getAttributes('character').styleId; },
+                    get: function () { return model.getAttributes('character', {maxIterations: 10}) || {}; }
+                },
+                'character/stylesheet': {
+                    parent: 'character/group',
+                    get: function (character) { return character.styleId; },
                     set: function (styleId) { model.setAttributes('character', { styleId: styleId }, { clear: true }); }
                 },
                 'character/attributes': {
-                    parent: 'document/editable/text',
-                    get: function () { return model.getAttributes('character').character || {}; }
+                    parent: 'character/group',
+                    get: function (character) { return character.character || {}; }
                 },
                 'character/fontname': {
                     parent: 'character/attributes',
@@ -302,41 +325,50 @@ define('io.ox/office/text/app/controller',
                 'character/hyperlink/dialog': {
                     parent: 'character/hyperlink',
                     get: function () { return Hyperlink.hasPopup(model); },
-                    set: function () { return model.insertHyperlinkDialog(); },
+                    set: function () { return model.insertHyperlinkDialog(); }
                     // removed for the 7.2.0 due to clashes with browser shortcuts
-//                    shortcut: { keyCode: 'K', ctrlOrMeta: true },
-                    focus: 'wait' // wait for the dialog before returning focus to application
+//                    shortcut: { keyCode: 'K', ctrlOrMeta: true }
                 },
                 'character/hyperlink/remove': {
                     parent: 'character/hyperlink',
                     set: function () { return model.removeHyperlink(); }
                 },
 
-                'character/resetAttributes': {
+                'character/reset': {
                     parent: 'document/editable/text',
                     set: function () { model.resetAttributes(); },
                     shortcut: { keyCode: 'SPACE', ctrl: true }
                 },
-                'character/insertLineBreak': {
+                'character/insert/break': {
                     parent: 'document/editable/text',
-                    set: function () { return model.insertHardBreak(); },
-                    focus: 'wait'
+                    set: function () { return model.insertHardBreak(); }
                 },
-                'character/insertTab': {
+                'character/insert/tab': {
                     parent: 'document/editable/text',
-                    set: function () { return model.insertTab(); },
-                    focus: 'wait'
+                    set: function () { return model.insertTab(); }
+                },
+                'character/insert/pagebreak': {
+                    parent: 'document/editable/text',
+                    set: function () { return model.insertManualPageBreak(); }
                 },
 
                 // tables
 
-                'table/insert': {
+                'table/group': {
                     parent: 'document/editable/text',
-                    set: function (size) { model.insertTable(size); }
+                    get: function () { return model.getAttributes('table') || {}; }
+                },
+                'table/insert/available': {
+                    enable: function () { return Config.MAX_TABLE_CELLS > 0; }
+                },
+                'table/insert': {
+                    parent: ['document/editable/text', 'table/insert/available'],
+                    set: function (size) { model.insertTable(size); },
+                    preserveFocus: true // do not return focus to document
                 },
 
                 'document/editable/table': {
-                    parent: 'document/editable/text',
+                    parent: 'table/group',
                     enable: function () { return model.isPositionInTable(); }
                 },
                 'table/insert/row': {
@@ -349,23 +381,21 @@ define('io.ox/office/text/app/controller',
                 },
                 'table/delete/row': {
                     parent: 'document/editable/table',
-                    set: function () { return model.deleteRows(); },
-                    focus: 'wait'
+                    set: function () { return model.deleteRows(); }
                 },
                 'table/delete/column': {
                     parent: 'document/editable/table',
-                    set: function () { return model.deleteColumns(); },
-                    focus: 'wait'
+                    set: function () { return model.deleteColumns(); }
                 },
 
                 'table/stylesheet': {
                     parent: 'document/editable/table',
-                    get: function () { return model.getAttributes('table').styleId; },
+                    get: function (table) { return table.styleId; },
                     set: function (styleId) { model.setAttributes('table', { styleId: styleId }, { clear: true }); }
                 },
                 'table/attributes': {
                     parent: 'document/editable/table',
-                    get: function () { return model.getAttributes('table').table || {}; }
+                    get: function (table) { return table.table || {}; }
                 },
                 'table/cellattributes': {
                     parent: 'document/editable/table',
@@ -374,18 +404,29 @@ define('io.ox/office/text/app/controller',
                 'table/cellborder': {
                     parent: 'table/attributes',
                     get: function (attributes) { return MixedBorder.getBorderMode(attributes); },
-                    set: function (borderMode) { model.setAttributes('table', { table: MixedBorder.getBorderAttributes(borderMode, model.getAttributes('table').table) }, { cellSpecificTableAttribute: true }); }
+                    set: function (borderMode) {
+                        var borderAttrs = MixedBorder.getBorderAttributes(borderMode, this.getParentValue(), ParagraphStyles.SINGLE_BORDER);
+                        model.setAttributes('table', { table: borderAttrs }, { cellSpecificTableAttribute: true });
+                    }
                 },
                 'table/borderwidth': {
                     parent: 'table/cellattributes',
                     get: function (attributes) { return TableStyles.getBorderStyleFromAttributes(attributes); },
-                    set: function (borderWidth) { model.setAttributes('table', { table: TableStyles.getAttributesFromBorderStyle(borderWidth, model.getAttributes('table').table) }, { onlyVisibleBorders: true }); }
+                    set: function (borderWidth) {
+                        model.setAttributes('table', {
+                            table: TableStyles.getAttributesFromBorderStyle(borderWidth, model.getAttributes('table').table)
+                        }, { onlyVisibleBorders: true });
+                    }
                 },
                 'table/fillcolor': {
                     parent: 'table/cellattributes',
                     get: function (attributes) { return attributes.fillColor; },
                     set: function (color) { model.setAttribute('cell', 'fillColor', color); }
                 },
+                'table/split': {
+                    parent: 'document/editable/table',
+                    set: function () { model.splitTable(); }
+                },
 
                 // drawing
 
@@ -395,8 +436,7 @@ define('io.ox/office/text/app/controller',
                 },
                 'drawing/delete': {
                     parent: 'document/editable/drawing',
-                    set: function () { return model.deleteSelected(); },
-                    focus: 'wait' // wait for the dialog before returning focus to application
+                    set: function () { return model.deleteSelected(); }
                 },
 
                 'drawing/attributes': {
@@ -406,15 +446,82 @@ define('io.ox/office/text/app/controller',
                 'drawing/position': {
                     parent: 'drawing/attributes',
                     get: function (attributes) { return DrawingStyles.getPositionFromAttributes(attributes); },
-                    set: function (position) { model.setAttributes('drawing', { drawing: DrawingStyles.getAttributesFromPosition(position, model.getAttributes('drawing').drawing) }); }
+                    set: function (position) { model.setAttributes('drawing', { drawing: DrawingStyles.getAttributesFromPosition(position, model.getAttributes('drawing').drawing) }, { ctSupportsDrawingPosition: model.getChangeTrack().ctSupportsDrawingPosition() }); }
                 },
 
                 // images
 
                 'image/insert/dialog': {
                     parent: 'document/editable/text',
-                    set: function () { return model.showInsertImageDialog(); },
-                    focus: 'wait' // wait for the dialog before returning focus to application
+                    set: function () { return model.showInsertImageDialog(); }
+                },
+
+                // change tracking
+
+                'checkCurrentChangeTracking' : {
+                    parent: 'document/editable',
+                    enable: function(){ return model.getChangeTrack().isOnChangeTrackNode(); }
+                },
+                'toggleChangeTracking' : {
+                    parent: 'document/editable',
+                    get: function () { return model.getChangeTrack().isActiveChangeTracking(); },
+                    set: function (state) { model.getChangeTrack().setActiveChangeTracking(state); }
+                },
+                'acceptChangeTracking' : {
+                    parent: 'document/editable',
+                    set: function () {
+                        model.getChangeTrack().resolveChangeTracking(null, { accepted: true, all: true });
+                        view.getChangeTrackPopup().hide();
+                    }
+                },
+                'rejectChangeTracking' : {
+                    parent: 'document/editable',
+                    set: function () {
+                        model.getChangeTrack().resolveChangeTracking(null, { accepted: false, all: true });
+                        view.getChangeTrackPopup().hide();
+                    }
+                },
+                'acceptSelectedChangeTracking' : {
+                    parent: 'checkCurrentChangeTracking',
+                    set: function () {
+                        model.getChangeTrack().resolveChangeTracking(null, { accepted: true, all: false });
+                        view.getChangeTrackPopup().hide();
+                    }
+                },
+                'acceptMoveSelectedChangeTracking' : {
+                    parent: 'document/editable',
+                    set: function () {
+                        model.getChangeTrack().resolveChangeTracking(null, { accepted: true, all: false });
+                        view.getChangeTrackPopup().hide();
+                        model.getChangeTrack().getNextChangeTrack();
+                    }
+                },
+                'rejectSelectedChangeTracking' : {
+                    parent: 'checkCurrentChangeTracking',
+                    set: function () {
+                        model.getChangeTrack().resolveChangeTracking(null, { accepted: false, all: false });
+                        view.getChangeTrackPopup().hide();
+                    }
+                },
+                'rejectMoveSelectedChangeTracking' : {
+                    parent: 'document/editable',
+                    set: function () {
+                        model.getChangeTrack().resolveChangeTracking(null, { accepted: false, all: false });
+                        view.getChangeTrackPopup().hide();
+                        model.getChangeTrack().getNextChangeTrack();
+                    }
+                },
+                'selectNextChangeTracking' : {
+                    parent: 'document/editable',
+                    set: function () { model.getChangeTrack().getNextChangeTrack({ next: true }); }
+                },
+                'selectPrevChangeTracking' : {
+                    parent: 'document/editable',
+                    set: function () { model.getChangeTrack().getNextChangeTrack({ next: false }); }
+                },
+                'changetrackPopupShow' : {
+                    parent: 'document/editable/text',
+                    set: function () { view.changeTrackPopupSelect(); }
                 },
 
                 // debug
@@ -429,6 +536,14 @@ define('io.ox/office/text/app/controller',
                     enable: function () { return model.hasSelectedRange(); },
                     set: function (mail) { model.accessRightsExport(mail); }
                 },
+                'debug/uselocalstorage' : {
+                    get: function () { return app.getUserSettingsValue('debugUseLocalStorage', Config.USE_LOCAL_STORAGE); },
+                    set: function (state) { app.setUserSettingsValue('debugUseLocalStorage', state); }
+                },
+                'debug/useFastLoad' : {
+                    get: function () { return app.getUserSettingsValue('documentFastLoad', true); },
+                    set: function (state) { app.setUserSettingsValue('documentFastLoad', state); }
+                },
                 'debug/recordoperations' : {
                     get: function () { return model.isRecordingOperations(); },
                     set: function (state) { model.setRecordingOperations(state); }
@@ -452,26 +567,19 @@ define('io.ox/office/text/app/controller',
                 'debug/pagehtmlcode' : {
                     parent: 'debug/enabled'
                 },
-                'debug/togglepagebreaks' : {
-                    set: function () { view.setPageBreaks(); }
+                'debug/pagebreaks/toggle' : {
+                    parent: 'debug/enabled',
+                    enable: function () { return !model.isDraftMode() && !Utils.SMALL_DEVICE; },
+                    get: function () { return model.isPageBreakMode(); },
+                    set: function (state) { model.togglePageBreakMode(state); }
                 },
-
-                //zoom
-
-                'zoom/dec': {
-                    enable: function () { return view.getZoomFactor() > view.getMinZoomFactor(); },
-                    set: function () { view.decreaseZoomLevel(); }
-                },
-
-                'zoom/inc': {
-                    enable: function () { return view.getZoomFactor() < view.getMaxZoomFactor(); },
-                    set: function () { view.increaseZoomLevel(); }
-                },
-
-                'zoom/type': {
-                    get: function () { return view.getZoomType(); },
-                    set: function (zoomType) { view.setZoomType(zoomType); }
+                'debug/draftmode/toggle' : {
+                    parent: 'debug/enabled',
+                    enable: function () { return !Utils.SMALL_DEVICE; },
+                    get: function () { return model.isDraftMode(); },
+                    set: function (state) { model.toggleDraftMode(state); }
                 }
+
             };
 
         // base constructor ---------------------------------------------------
@@ -484,6 +592,9 @@ define('io.ox/office/text/app/controller',
          * Item getter function for a list style identifier. Will be called in
          * the context of a controller item instance.
          *
+         * @param {String} listType
+         *  The type of the list styles: either 'bullet' or 'numbering'.
+         *
          * @param {Object} attributes
          *  The merged attribute set of the selected paragraphs.
          *
@@ -493,18 +604,17 @@ define('io.ox/office/text/app/controller',
          *  type specified in the item user data ('bullets' or 'numbering'),
          *  otherwise an empty string. If ambiguous, returns the value null.
          */
-        function getListStyleId(attributes) {
+        function getListStyleId(listType, attributes) {
 
             var listStyleId = attributes.listStyleId,
                 listLevel = attributes.listLevel,
-                itemListType = this.getUserData(),
                 isListType = false;
 
             // ambiguous attributes
             if (_.isNull(listStyleId) || _.isNull(listLevel)) { return null; }
 
             if (listLevel > -1) {
-                switch (itemListType) {
+                switch (listType) {
                 case 'bullet':
                     isListType = model.getListCollection().isBulletsList(listStyleId, listLevel);
                     break;
@@ -522,18 +632,19 @@ define('io.ox/office/text/app/controller',
          * Item setter function for a list style identifier. Will be called in
          * the context of a controller item instance.
          *
+         * @param {String} listType
+         *  The type of the list styles: either 'bullet' or 'numbering'.
+         *
          * @param {String} listStyleId
          *  The new list style identifier. The special value
          *  Lists.DEFAULT_VALUE can be used to toggle the default list style.
          */
-        function setListStyleId(listStyleId) {
-
-            var itemListType = this.getUserData();
+        function setListStyleId(listType, listStyleId) {
 
             // simulate toggle behavior for default list style
             if (listStyleId === ListCollection.DEFAULT_VALUE) {
                 if (this.getValue() === '') {
-                    model.createDefaultList(itemListType);
+                    model.createDefaultList(listType);
                 } else {
                     model.removeListAttributes();
                 }
@@ -545,11 +656,23 @@ define('io.ox/office/text/app/controller',
 
         // initialization -----------------------------------------------------
 
+        if(_.browser.Android){
+            var originalUpdate = self.update;
+
+            self.update = function() {
+                if (app.getModel().isAndroidTyping()) {
+                    app.executeDelayed(self.update, {delay: 500});
+                } else {
+                    originalUpdate();
+                }
+            };
+        }
+
         // register item definitions
         this.registerDefinitions(items);
 
         // initialization after construction
-        app.on('docs:init', function () {
+        app.onInit(function () {
             // model and view may not be available at construction time
             model = app.getModel();
             view = app.getView();
