@@ -14,8 +14,10 @@
 define('io.ox/office/text/dom',
     ['io.ox/office/tk/utils',
      'io.ox/office/tk/keycodes',
-     'io.ox/office/drawinglayer/view/drawingframe'
-    ], function (Utils, KeyCodes, DrawingFrame) {
+     'io.ox/office/drawinglayer/view/drawingframe',
+     'io.ox/office/tk/forms',
+     'gettext!io.ox/office/text'
+    ], function (Utils, KeyCodes, DrawingFrame, Forms, gt) {
 
     'use strict';
 
@@ -519,6 +521,25 @@ define('io.ox/office/text/dom',
         return $(pageNode).children(DOM.PAGECONTENT_NODE_SELECTOR);
     };
 
+    /**
+     * A jQuery selector that matches elements representing a page break node.
+     */
+    DOM.PAGEBREAK_NODE_SELECTOR = 'div.page-break';
+
+    /**
+     * Returns whether the passed node is a page break node.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a page break node.
+     */
+    DOM.isPageBreakNode = function (node) {
+        return $(node).is(DOM.PAGEBREAK_NODE_SELECTOR);
+    };
+
     // paragraphs and tables ==================================================
 
     /**
@@ -678,6 +699,94 @@ define('io.ox/office/text/dom',
         return DOM.isTableNode(node) && $(node).hasClass('size-exceeded');
     };
 
+    /**
+     * Makes a exceeded size table placeholder from a provided empty table node.
+     *
+     * @paaram {Node|jQuery|Null} [tableNode]
+     */
+    DOM.makeExceededSizeTable = function (tableNode, overflowElement, value, maxValue) {
+        var placeHolder,
+            cell = $('<td>').append(
+                       placeHolder = $('<div>').addClass('placeholder').append(
+                        $('<div>').addClass('abs background-icon').append(Forms.createIconMarkup('fa-table')),
+                        $('<p>').text(gt('Table')),
+                        $('<p>').text(gt('This table is too large to be displayed here.'))
+                    )),
+            labels = DOM.getTableSizeExceededTextLabels(overflowElement, value, maxValue);
+
+        if (labels.countLabel && labels.limitLabel) {
+            placeHolder.append($('<p>').text(labels.countLabel + ' ' + labels.limitLabel));
+        }
+
+        $(tableNode).addClass('size-exceeded')
+            .attr('contenteditable', false)
+            .append($('<tr>').append(cell));
+    };
+
+    /**
+    * Provides the correct localized labels for a table exceeds size case.
+    *
+    * @param {String} [type]
+    *  Specifies which part of a table exceeded the size. Currently the parts
+    *  "rows", "cols", "cells".
+    *
+    * @param {Integer} [value]
+    *  Specifies the current value of the table part which exceeds the limit.
+    *
+    * @param {Integer} [maxValue]
+    *  Specifies the maximal allowed value of the table part which exceeds the
+    *  limit.
+    */
+   DOM.getTableSizeExceededTextLabels = function (type, value, maxValue) {
+       var result = {countLabel: null, limitLabel: null};
+
+       switch (type) {
+       case 'rows':
+           result.countLabel = gt.format(
+                   //#. %1$d is the number of rows in an oversized text table
+                   //#, c-format
+                   gt.ngettext('The table contains %1$d row.', 'This table contains %1$d rows.', value),
+                   _.noI18n(value)
+           );
+           result.limitLabel = gt.format(
+                   //#. %1$d is the maximum allowed number of rows in a text table
+                   //#, c-format
+                   gt.ngettext('Tables are limited to %1$d row.', 'Tables are limited to %1$d rows.', maxValue),
+                   _.noI18n(maxValue)
+           );
+           break;
+       case 'cols':
+           result.countLabel = gt.format(
+                   //#. %1$d is the number of columns in an oversized text table
+                   //#, c-format
+                   gt.ngettext('The table contains %1$d column.', 'This table contains %1$d columns.', value),
+                   _.noI18n(value)
+           );
+           result.limitLabel = gt.format(
+                   //#. %1$d is the maximum allowed number of columns in a text table
+                   //#, c-format
+                   gt.ngettext('Tables are limited to %1$d column.', 'Tables are limited to %1$d columns.', maxValue),
+                   _.noI18n(maxValue)
+           );
+           break;
+       case 'cells':
+           result.countLabel = gt.format(
+                   //#. %1$d is the number of cells in an oversized text table
+                   //#, c-format
+                   gt.ngettext('The table contains %1$d cell.', 'This table contains %1$d cells.', value),
+                   _.noI18n(value)
+           );
+           result.limitLabel = gt.format(
+                   //#. %1$d is the maximum allowed number of cells in a text table
+                   //#, c-format
+                   gt.ngettext('Tables are limited to %1$d cell.', 'Tables are limited to %1$d cells.', maxValue),
+                   _.noI18n(maxValue)
+           );
+       }
+
+       return result;
+   };
+
     /**
      * Returns whether the passed node is a table node containing the splitting
      * class.
@@ -740,6 +849,40 @@ define('io.ox/office/text/dom',
         return $(node).is(DOM.TABLE_ROWNODE_SELECTOR);
     };
 
+    /**
+     * A jQuery selector that matches elements representing a table page break row.
+     */
+    DOM.TABLE_PAGEBREAK_ROWNODE_SELECTOR = 'tr.pb-row';
+
+    /**
+     * Returns whether the passed node is a table page break row element.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a table page break row element.
+     */
+    DOM.isTablePageBreakRowNode = function (node) {
+        return $(node).is(DOM.TABLE_PAGEBREAK_ROWNODE_SELECTOR);
+    };
+
+    /**
+     * Returns whether the passed node is a table node containing a page break
+     * row element.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a table node containing a page break row element.
+     */
+    DOM.isTableWithPageBreakRowNode = function (node) {
+        return DOM.isTableNode(node) && ($(node).find(DOM.TABLE_PAGEBREAK_ROWNODE_SELECTOR).length > 0);
+    };
+
     /**
      * Returns the collection of table rows from the passed table element.
      *
@@ -781,8 +924,7 @@ define('io.ox/office/text/dom',
      *  table cell.
      *
      * @param {Object} [options]
-     *  A map of options controlling the properties of the returned cell node.
-     *  The following options are supported:
+     *  Optional parameters:
      *  @param {Boolean} [options.plain=false]
      *      If set to true, no resize nodes will be inserted into the cell, and
      *      other special formatting will be omitted (for example, the
@@ -796,7 +938,9 @@ define('io.ox/office/text/dom',
         var cellContentNode = $('<div>').addClass('cellcontent'),
             cellChildNode = $('<div>').addClass('cell'),
             cellNode = $('<td>'),
-            plain = Utils.getBooleanOption(options, 'plain', false);
+            plain = Utils.getBooleanOption(options, 'plain', false),
+            // contenteditable must be false in IE and true for all other browsers
+            contentEditable = !_.browser.IE;
 
         if (!plain) {
             cellChildNode.append($('<div>').addClass('resize bottom'), $('<div>').addClass('resize right'));
@@ -804,8 +948,11 @@ define('io.ox/office/text/dom',
         }
         cellNode.append(cellChildNode.append(cellContentNode.append(paragraph)));
 
-        if (!plain && _.browser.IE) {  // -> setting attribute only for IE. this leads to a yellow border around div.p in Chrome
-            cellChildNode.attr('contenteditable', false);
+        if (!plain) {
+            // -> setting attribute only for IE.
+            //    this leads to a yellow border around div.p in Chrome
+            // -> Chrome Bug is fixed with: "outline: none !important;"
+            cellChildNode.attr('contenteditable', contentEditable);
             cellContentNode.attr({ contenteditable: true, 'data-focus-role': 'cell' });
         }
 
@@ -963,17 +1110,19 @@ define('io.ox/office/text/dom',
     };
 
     /**
-     * Returns whether the passed node is a <span> element containing a single
-     * text node which just contains a <br> element.
+     * Returns whether the passed node is a <span> whose parent is a
+     * div with class 'hardbreak'
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
      *
-     * @param node
      * @returns {Boolean}
-     *  Whether the passed node is a span element with a break node;
+     *  Whether the passed node is a hard break span element.
      */
     DOM.isHardBreakSpan = function (node) {
-        var contents = $(node).contents();
-        return ($(node).is('span') && ((contents.length === 1) && ($(contents[0]).is('br')) && (!DOM.isDummyTextNode(contents[0])) ||
-                (contents.length === 2) && ($(contents[0]).is('br')) && (!DOM.isDummyTextNode(contents[0])) && $(contents[1]).text().length === 0)); //IE 9 case (we have an empty text node after the 'br')
+        var domNode = node && Utils.getDomNode(node);
+        return $(domNode).is('span') && domNode.parentNode && DOM.isHardBreakNode(domNode.parentNode);
     };
 
     /**
@@ -1018,10 +1167,8 @@ define('io.ox/office/text/dom',
      *  Whether the passed node is a span element without a text node.
      */
     DOM.isTextSpanWithoutTextNode = function (node) {
-        var contents = null;
-        node = node && Utils.getDomNode(node);
-        contents = $(node).contents();
-        return $(node).is('span') && (contents.length === 0) && DOM.isParagraphNode(node.parentNode);
+        var domNode = node && Utils.getDomNode(node);
+        return $(domNode).is('span') && ($(domNode).contents().length === 0) && DOM.isParagraphNode(domNode.parentNode);
     };
 
     /**
@@ -1036,8 +1183,8 @@ define('io.ox/office/text/dom',
      *  Whether the passed node is a text node in a portion span element.
      */
     DOM.isTextNodeInPortionSpan = function (node) {
-        node = node && Utils.getDomNode(node);
-        return node && (node.nodeType === 3) && DOM.isPortionSpan(node.parentNode);
+        var domNode = node && Utils.getDomNode(node);
+        return domNode && (domNode.nodeType === 3) && DOM.isPortionSpan(domNode.parentNode);
     };
 
     /**
@@ -1070,6 +1217,167 @@ define('io.ox/office/text/dom',
         return Utils.findDescendantNode(paragraph, function () { return DOM.isTextSpan(this); }, { children: true, reverse: true });
     };
 
+    // change tracking ------------------------------------------
+
+    /**
+     * A jQuery selector that matches elements representing a node with
+     * a changes attribute.
+     */
+    DOM.CHANGETRACK_NODE_SELECTOR = '[data-change-track-inserted="true"], [data-change-track-removed="true"], [data-change-track-modified="true"]';
+
+    /**
+     * A jQuery selector that matches elements representing a node with
+     * the changes attribute 'inserted'.
+     */
+    DOM.CHANGETRACK_INSERTED_NODE_SELECTOR = '[data-change-track-inserted="true"]';
+
+    /**
+     * A jQuery selector that matches elements representing a node with
+     * the changes attribute 'removed'.
+     */
+    DOM.CHANGETRACK_REMOVED_NODE_SELECTOR = '[data-change-track-removed="true"]';
+
+    /**
+     * A jQuery selector that matches elements representing a node with
+     * the changes attribute 'modified'.
+     */
+    DOM.CHANGETRACK_MODIFIED_NODE_SELECTOR = '[data-change-track-modified="true"]';
+
+    /**
+     * A jQuery selector that matches table rows with the changes attribute 'inserted'.
+     */
+    DOM.CHANGETRACK_INSERTED_ROW_SELECTOR = 'tr[data-change-track-inserted="true"]';
+
+    /**
+     * A jQuery selector that matches table rows with the changes attribute 'removed'.
+     */
+    DOM.CHANGETRACK_REMOVED_ROW_SELECTOR = 'tr[data-change-track-removed="true"]';
+
+    /**
+     * Returns whether the passed node is a node with a changes
+     * attribute.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a node with change tracking attribute.
+     */
+    DOM.isChangeTrackNode = function (node) {
+        return $(node).is(DOM.CHANGETRACK_NODE_SELECTOR) || DOM.isChangeTrackInlineNode(node);
+    };
+
+    /**
+     * Returns whether the passed node is a node with changes 'inserted'
+     * attribute.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a node with changes 'inserted'
+     *  attribute.
+     */
+    DOM.isChangeTrackInsertNode = function (node) {
+        return $(node).is(DOM.CHANGETRACK_INSERTED_NODE_SELECTOR) || DOM.isChangeTrackInsertInlineNode(node);
+    };
+
+    /**
+     * Returns whether the passed node is a node with changes 'removed'
+     * attribute.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a node  with changes 'removed'
+     *  attribute.
+     */
+    DOM.isChangeTrackRemoveNode = function (node) {
+        return $(node).is(DOM.CHANGETRACK_REMOVED_NODE_SELECTOR) || DOM.isChangeTrackRemoveInlineNode(node);
+    };
+
+    /**
+     * Returns whether the passed node is a node with changes 'modified'
+     * attribute.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a node with changes 'modified' attribute.
+     */
+    DOM.isChangeTrackModifyNode = function (node) {
+        return $(node).is(DOM.CHANGETRACK_MODIFIED_NODE_SELECTOR) || DOM.isChangeTrackModifyInlineNode(node);
+    };
+
+    /**
+     * Returns whether the passed node is an inline node with changes attribute
+     * at its span (tab, hard break, field, ...).
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is an inline node with changes attribute set to its span.
+     */
+    DOM.isChangeTrackInlineNode = function (node) {
+        return DOM.isInlineComponentNode(node) && $(Utils.getDomNode(node).firstChild).is(DOM.CHANGETRACK_NODE_SELECTOR);
+    };
+
+    /**
+     * Returns whether the passed node is an inline node with changes 'inserted'
+     * attribute at its span (tab, hard break, field, ...).
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is an inline node with changes 'inserted' attribute
+     *  set to its span.
+     */
+    DOM.isChangeTrackInsertInlineNode = function (node) {
+        return DOM.isInlineComponentNode(node) && $(Utils.getDomNode(node).firstChild).is(DOM.CHANGETRACK_INSERTED_NODE_SELECTOR);
+    };
+
+    /**
+     * Returns whether the passed node is an inline node with changes 'removed'
+     * attribute at its span (tab, hard break, field, ...).
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is an inline node with changes 'removed' attribute
+     *  set to its span.
+     */
+    DOM.isChangeTrackRemoveInlineNode = function (node) {
+        return DOM.isInlineComponentNode(node) && $(Utils.getDomNode(node).firstChild).is(DOM.CHANGETRACK_REMOVED_NODE_SELECTOR);
+    };
+
+    /**
+     * Returns whether the passed node is an inline node with changes 'modified'
+     * attribute at its span (tab, hard break, field, ...).
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is an inline node with changes 'modified' attribute
+     *  set to its span.
+     */
+    DOM.isChangeTrackModifyInlineNode = function (node) {
+        return DOM.isInlineComponentNode(node) && $(Utils.getDomNode(node).firstChild).is(DOM.CHANGETRACK_MODIFIED_NODE_SELECTOR);
+    };
+
     // text components: fields, tabs ------------------------------------------
 
     /**
@@ -1111,25 +1419,9 @@ define('io.ox/office/text/dom',
      *  text component in a paragraph.
      */
     DOM.isTextComponentNode = function (node) {
-        return DOM.isInlineComponentNode(node) &&
-               (DOM.isTextSpan(Utils.getDomNode(node).firstChild) ||
-                DOM.isHardBreakSpan(Utils.getDomNode(node).firstChild) ||
-                (_.browser.IE && DOM.isSpanWithoutChild(Utils.getDomNode(node).firstChild))); // needed for IE9+
-    };
-
-    /**
-     * Returns whether the passed node is a span which contains no child
-     * node.
-     *
-     * @param {Node|jQuery|Null} [node]
-     *  The DOM node to be checked. If this object is a jQuery collection, uses
-     *  the first DOM node it contains. If missing or null, return false.
-     *
-     * @returns {Boolean}
-     *  Whether the passed node is a <span> element without any child node.
-     */
-    DOM.isSpanWithoutChild = function (node) {
-        return ($(node).is('span') && ($(node).contents().length === 0));
+        // text component nodes are all inline nodes, except drawings.
+        // This includes tabs, fields and hard breaks
+        return DOM.isInlineComponentNode(node) && !DOM.isDrawingFrame(node);
     };
 
     /**
@@ -1209,6 +1501,22 @@ define('io.ox/office/text/dom',
         return $('<div>', { contenteditable: false}).addClass('inline tab');
     };
 
+    /**
+     * Returns whether the passed node is a <span> whose parent is a
+     * div with class 'tab'
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a tab span element.
+     */
+    DOM.isTabSpan = function (node) {
+        var domNode = node && Utils.getDomNode(node);
+        return $(domNode).is('span') && domNode.parentNode && DOM.isTabNode(domNode.parentNode);
+    };
+
     /**
      * A jQuery selector that matches elements representing a hard-break.
      */
@@ -1231,11 +1539,15 @@ define('io.ox/office/text/dom',
     /**
      * Returns a new hard-break element.
      *
+     * @param {String} type
+     *  The type of the hard-break element. Supported values are
+     *  'page', 'column' or 'textWrapping'.
+     *
      * @returns {jQuery}
      *  A new hard-break element, as jQuery object.
      */
-    DOM.createHardBreakNode = function () {
-        return $('<div>', { contenteditable: false }).addClass('inline hardbreak');
+    DOM.createHardBreakNode = function (type) {
+        return $('<div>', { contenteditable: false }).addClass('inline hardbreak').data('type', type);
     };
 
     // paragraph helper nodes -------------------------------------------------
@@ -1472,6 +1784,8 @@ define('io.ox/office/text/dom',
         var img = $(node).is('img') ? $(node) : $(node).find('img'),
             src = img.attr('src');
 
+        if (!src) { return false; }  // replacement data is shown
+
         // check for base64 data-URL first
         if (src.substring(0, 10) === 'data:image') {
             return false;
@@ -1616,8 +1930,7 @@ define('io.ox/office/text/dom',
      *  span will be inserted.
      *
      * @param {Object} [options]
-     *  A map of options to control the split operation. Supports the following
-     *  options:
+     *  Optional parameters:
      *  @param {Boolean} [options.append=false]
      *      If set to true, the right part of the text will be inserted after
      *      the passed text span; otherwise the left part of the text will be
@@ -1757,22 +2070,144 @@ define('io.ox/office/text/dom',
         return $();
     };
 
+    /**
+     * Finding the allowed neighboring node taking into account all nodes
+     * that are used for calculating the logical positions. For example as
+     * top level nodes only paragraphs and tables are allowed, page-break
+     * nodes need to be skipped.
+     *
+     * @param {Node|jQuery} node
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains.
+     *
+     * @param {Object} options
+     *  A map with additional options controlling the search of the neighbor.
+     *  The following options are supported:
+     *  @param {Boolean} [options.next=true]
+     *      The search direction. If not specified, the next neighboring node
+     *      will be searched.
+     *
+     * @returns {Node|null}
+     *  The search element of the specified node. If it cannot be determined
+     *  null is returned.
+     */
+    DOM.getAllowedNeighboringNode = function (node, options) {
+
+        var // determining the direction for searching the neighbor
+            next = Utils.getBooleanOption(options, 'next', true),
+            // the dom node of the specified node
+            domNode = Utils.getDomNode(node),
+            // the iterator function
+            iterator = next ? 'nextSibling' : 'previousSibling',
+            // the neighboring node
+            neighbor = domNode[iterator] || null;
+
+         while (neighbor && DOM.isInvalidNode(neighbor)) {
+             neighbor = neighbor[iterator];
+         }
+
+         return neighbor;
+    };
+
+    /**
+     * Checking, whether a specified node is part of the calculation of
+     * logical positions.
+     *
+     * @param {Node|jQuery} node
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains.
+     *
+     * @returns {Boolean}
+     *  Whether the specified node is part of the calculation of
+     *  logical positions.
+     */
+    DOM.isInvalidNode = function (node) {
+        return DOM.isPageBreakNode(node) || DOM.isListLabelNode(node);  // TODO: List needs to be expanded
+    };
+
     /**
      * Creates a white list of allowed DOM nodes (paragraphs and tables currently),
      * and ignores pagebreaks and clearfloating divs
      *
      * @param {HTMLElement|jQuery} childNodes
+     *  List of nodes that we want to filter, to contain only allowed ones, such as top level nodes paragraphs and tables
      *
-     * @returns {HTMLElement|jQuery} filteredNodes
+     * @returns {HTMLElement|jQuery} filteredNodes - list of nodes after filtering
      */
     DOM.allowedChildNodes = function (childNodes) {
-    //        if (!(childNodes instanceof jQuery)) {
-    //            childNodes = $(childNodes);
-    //        }
-
         return $(childNodes).filter(DOM.PARAGRAPH_NODE_SELECTOR + ',' + DOM.TABLE_NODE_SELECTOR);
     };
 
+    // page break nodes ========================================================
+
+    /**
+     * A jQuery selector that matches page break div, used to separate pages in page layout.
+     */
+    DOM.PAGE_BREAK_SELECTOR = 'div.page-break';
+
+    /**
+     * A jQuery selector that matches manualy inserted page break by user
+     */
+    DOM.MANUAL_PAGE_BREAK_SELECTOR = '.manual-page-break';
+
+    /**
+     * Returns whether the passed node is a page break element.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a page break element.
+     */
+    DOM.isPageBreakNode = function (node) {
+        return $(node).is(DOM.PAGE_BREAK_SELECTOR);
+    };
+
+    /**
+     * Returns whether the passed node is a page break element inserted by user.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a manual page break element.
+     */
+    DOM.isManualPageBreakNode = function (node) {
+        return $(node).is(DOM.MANUAL_PAGE_BREAK_SELECTOR);
+    };
+
+    /**
+     * Returns whether the passed node is a table element that contains a page break
+     * in its first paragraph in the upper left cell.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a table node with manual page break element.
+     */
+    DOM.isTableWithManualPageBreak = function (node) {
+        return DOM.isTableNode(node) && $(node).find('div.p').first().is(DOM.MANUAL_PAGE_BREAK_SELECTOR);
+    };
+
+    /**
+     * Returns whether node contains manual page break after element attribute,
+     * set as a class name.
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a table node with manual page break after element.
+     * */
+    DOM.isNodeWithPageBreakAfterAttribute = function (node) {
+        return $(node).hasClass('manual-pb-after');
+    };
+
     // exports ================================================================
 
     return DOM;
