@@ -13,11 +13,12 @@
 
 define('io.ox/office/text/drawingResize',
     ['io.ox/office/tk/utils',
+     'io.ox/office/editframework/utils/attributeutils',
      'io.ox/office/drawinglayer/view/drawingframe',
      'io.ox/office/text/dom',
      'io.ox/office/text/operations',
      'io.ox/office/text/position'
-    ], function (Utils, DrawingFrame, DOM, Operations, Position) {
+    ], function (Utils, AttributeUtils, DrawingFrame, DOM, Operations, Position) {
 
     'use strict';
 
@@ -221,6 +222,10 @@ define('io.ox/office/text/drawingResize',
                     upperDrawings = 0,
                     // whether the drawing is moved inside its paragraph
                     sameParagraph = false,
+                    // attributes object for operations
+                    allDrawingAttrs = null,
+                    // the old drawing attributes
+                    oldDrawingAttrs = null,
                     // value of margin top of drawing node
                     drawingMarginTop = 0,
                     // maximum allowed height of page without page margins, in 1/100 mm
@@ -250,6 +255,10 @@ define('io.ox/office/text/drawingResize',
 
                     if (! (node instanceof $)) { node = $(node); }
 
+                    if (node.closest(DOM.TABLE_CELLNODE_SELECTOR).length > 0) { // #33245 moving floating drawing inside table cell
+                        node = node.closest(DOM.TABLE_CELLNODE_SELECTOR);
+                    }
+
                     return ((Math.round(node.offset().left / zoomFactor) <= posX) && (posX <= Math.round((node.offset().left / zoomFactor) + node.outerWidth())) &&
                             (Math.round(node.offset().top  / zoomFactor) <= posY) && (posY <= Math.round((node.offset().top / zoomFactor) + node.outerHeight())));
                 }
@@ -307,7 +316,7 @@ define('io.ox/office/text/drawingResize',
                             tableCell = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.TABLE_CELLNODE_SELECTOR, DrawingFrame.NODE_SELECTOR, {'reverse': true});
                         }
 
-                        if ((tableCell) && (DOM.isTableCellNode(tableCell))) {
+                        if ((tableCell) && (DOM.isTableCellNode(tableCell)) && (! DOM.isExceededSizeTableNode($(tableCell).closest(DOM.TABLE_NODE_SELECTOR)))) { // check if table is exceeded size for #33275
                             paragraph = DOM.getCellContentNode(tableCell)[0].lastChild;  // the last paragraph of the cell content
                         }
                     }
@@ -317,6 +326,18 @@ define('io.ox/office/text/drawingResize',
                     return paragraph;
                 }
 
+                // helper function to receive a reasonable value for inserting
+                // a moved drawing with activated change tracking.
+                function getDrawingInsertPosition() {
+
+                    var // the best possible position to insert the drawing
+                        insertPosition = _.clone(updatePosition);  // shifting drawing inside the same paragraph
+
+                    insertPosition[insertPosition.length - 1] = Position.getLeadingFloatingDrawingCount(paragraph, anchorVertOffset, drawingNode); //#33245 passing optional drawing node, to exclude itself from count
+
+                    return insertPosition;
+                }
+
                 // begin of stopMoveDrawing
 
                 if (! mouseMoved) {
@@ -357,7 +378,7 @@ define('io.ox/office/text/drawingResize',
                     drawingHeight = Utils.convertLengthToHmm(drawingNode.height(), 'px');
                     paraWidth = Utils.convertLengthToHmm(paragraph.width(), 'px');
                     // evaluating attributes
-                    drawingNodeAttrs = editor.getStyleSheets('drawing').getElementAttributes(drawingNode).drawing;
+                    drawingNodeAttrs = editor.getStyleCollection('drawing').getElementAttributes(drawingNode).drawing;
                     oldAnchorHorOffset = drawingNodeAttrs.anchorHorOffset;
                     oldAnchorVertOffset = drawingNodeAttrs.anchorVertOffset ? drawingNodeAttrs.anchorVertOffset : 0;
                     anchorHorBase = drawingNodeAttrs.anchorHorBase;
@@ -424,7 +445,7 @@ define('io.ox/office/text/drawingResize',
                             // the drawing must be moved to the beginning of the paragraph behind other floated drawings.
                             if ((moveY < 0) && ((- moveY) > offsetNodeHeight)) {
                                 destPosition = _.clone(updatePosition);
-                                destPosition[destPosition.length - 1] = Position.getLeadingFloatingDrawingCount(paragraph, drawingTopY - (paragraph.offset().top / zoomFactor));
+                                destPosition[destPosition.length - 1] = Position.getLeadingFloatingDrawingCount(paragraph, drawingTopY - (paragraph.offset().top / zoomFactor), drawingNode); //#33245 passing optional drawing node, to exclude itself from count
                                 moveImage = true;
                             }
 
@@ -476,46 +497,113 @@ define('io.ox/office/text/drawingResize',
                         }
                     }
 
-                    // switching from inline to floated mode, if required, using default values for floated mode
-                    if (DOM.isInlineDrawingNode(drawingNode) && ((anchorHorOffset !== oldAnchorHorOffset) || (anchorVertOffset !== oldAnchorVertOffset))) {
-                        generator.generateOperation(Operations.SET_ATTRIBUTES, {
-                            attrs: { drawing: { inline: false, anchorHorBase: 'column', anchorHorAlign: 'right', anchorHorOffset: 0, textWrapMode: 'topAndBottom', textWrapSide: null } },
-                            start: updatePosition
-                        });
-                    }
-
-                    if ((moveImage) && (! _.isEqual(updatePosition, destPosition))) {
-
-                        // check, if destPosition is located in a non-implicit paragraph -> otherwise create paragraph
-                        // similar to editor.handleImplicitParagraph
-                        localPosition = _.clone(destPosition);
-
-                        if (localPosition.pop() === 0) {  // is this an empty paragraph?
-                            localParagraph = Position.getParagraphElement(rootNode, localPosition);
-                            if ((DOM.isImplicitParagraphNode(localParagraph)) && (Position.getParagraphNodeLength(localParagraph) === 0)) {
-                                // removing implicit paragraph node
-                                $(localParagraph).remove();
-                                // creating new paragraph explicitly
-                                generator.generateOperation(Operations.PARA_INSERT, {start: localPosition});
-                            }
-                        }
-
-                        generator.generateOperation(Operations.MOVE, {
-                            start: updatePosition,
-                            end: updatePosition,
-                            to: destPosition
-                        });
-
-                        updatePosition = _.clone(destPosition); // for setting attributes required
-                        if (moveDownwards) {
-                            updatePosition[updatePosition.length - 1] -= 1;  // moving from [0,0] to [0,2], so that it ends at [0,1]
-                        }
-
-                        imageMoved = true;
-                    }
+                    // Generating operations. If change tracking is activated, different operations need
+                    // to be created, because the drawing at the old position stays on its position but
+                    // needs to be marked as 'deleted'. At the new position a new drawing needs to be
+                    // inserted, that is marked as inserted.
+                    // If change tracking is NOT active, it is first necessary to switch from in-line mode
+                    // to floated mode. Then a move operation might be necessary and finally a further
+                    // setAttributes operation, that sets the attributes for the position of the drawing
+                    // at its new location.
 
+                    // 1. Switching from in-line to floated mode, if required, using default values for floated mode
+                    // -> in the case of activated change tracking this operation is not required. Instead it is
+                    // necessary to mark the drawing as 'deleted'.
                     if ((anchorHorOffset !== oldAnchorHorOffset) || (anchorVertOffset !== oldAnchorVertOffset)) {
 
+                        if (editor.getChangeTrack().isActiveChangeTracking() && !editor.getChangeTrack().isInsertNodeByCurrentAuthor(drawingNode)) {
+                            // if change tracking is active, the old drawing position must be marked as 'deleted' ...
+                            // -> this is necessary for in-line and non-in-line drawings
+                            generator.generateOperation(Operations.SET_ATTRIBUTES, {
+                                attrs: { changes: { 'removed': editor.getChangeTrack().getChangeTrackInfo() }},
+                                start: updatePosition
+                            });
+                        } else if (DOM.isInlineDrawingNode(drawingNode)) {
+                            // switching from in-line to floated mode
+                            generator.generateOperation(Operations.SET_ATTRIBUTES, {
+                                attrs: { drawing: { inline: false, anchorHorBase: 'column', anchorHorAlign: 'right', anchorHorOffset: 0, textWrapMode: 'topAndBottom', textWrapSide: null } },
+                                start: updatePosition
+                            });
+                        }
+                    }
+
+                    // 2. Moving the image, if this is necessary
+                    // -> In case of activated change tracking, this is only necessary, if the drawing was inserted
+                    //    before by the current author.
+                    if ((moveImage) && (! _.isEqual(updatePosition, destPosition))) {
+
+                        // moving the drawing, if change tracking is NOT active or it is active
+                        // and the current author has inserted the drawing before
+                        if (!editor.getChangeTrack().isActiveChangeTracking() || (editor.getChangeTrack().isActiveChangeTracking() && editor.getChangeTrack().isInsertNodeByCurrentAuthor(drawingNode))) {
+
+                            // check, if destPosition is located in a non-implicit paragraph -> otherwise create paragraph
+                            // similar to editor.handleImplicitParagraph
+                            localPosition = _.clone(destPosition);
+
+                            if (localPosition.pop() === 0) {  // is this an empty paragraph?
+                                localParagraph = Position.getParagraphElement(rootNode, localPosition);
+                                if ((DOM.isImplicitParagraphNode(localParagraph)) && (Position.getParagraphNodeLength(localParagraph) === 0)) {
+                                    // removing implicit paragraph node
+                                    $(localParagraph).remove();
+                                    // creating new paragraph explicitly
+                                    generator.generateOperation(Operations.PARA_INSERT, {start: localPosition});
+                                }
+                            }
+
+                            // move operation
+                            generator.generateOperation(Operations.MOVE, {
+                                start: updatePosition,
+                                end: updatePosition,
+                                to: destPosition
+                            });
+
+                            updatePosition = _.clone(destPosition); // for setting attributes required
+                            if (moveDownwards) {
+                                updatePosition[updatePosition.length - 1] -= 1;  // moving from [0,0] to [0,2], so that it ends at [0,1]
+                            }
+
+                            imageMoved = true;
+                        }
+                    }
+
+                    // 3. Setting attributes to the drawing at its new location (for the correct positioning)
+                    if ((anchorHorOffset !== oldAnchorHorOffset) || (anchorVertOffset !== oldAnchorVertOffset)) {
+
+                        // Inserting a new drawing if change tracking is active and the drawing was not inserted
+                        // by the current author before. This is always necessary, if the offset was modified. Independent
+                        // whether the drawing was moved before or not.
+                        if (editor.getChangeTrack().isActiveChangeTracking() && !editor.getChangeTrack().isInsertNodeByCurrentAuthor(drawingNode)) {
+
+                            // If change tracking is active and the drawing was not inserted by the current author before, then:
+                            // - the drawing needs to be inserted at its new position with an insertDrawing operation.
+                            // - if a move operation was required (without change tracking) the new logical position
+                            //   is already determined and saved in 'destPosition'.
+                            // - if no move operation was required (moving inside the same paragraph), the destPosition needs
+                            //   to be determined now using helper function 'getDrawingInsertPosition'.
+                            if (!destPosition) { destPosition = getDrawingInsertPosition(); }
+
+                            oldDrawingAttrs = AttributeUtils.getExplicitAttributes(drawingNode);
+                            allDrawingAttrs = oldDrawingAttrs;
+
+                            // setting default floated value for the inserted drawing
+                            if (DOM.isInlineDrawingNode(drawingNode)) {
+                                allDrawingAttrs.drawing.inline = false;  // the drawing is never in-line in new position
+                                allDrawingAttrs.drawing.textWrapMode = 'topAndBottom';
+                                allDrawingAttrs.drawing.textWrapSide = null;
+                            }
+
+                            allDrawingAttrs.changes = { 'inserted': editor.getChangeTrack().getChangeTrackInfo(), 'removed': null };
+
+                            generator.generateOperation(Operations.DRAWING_INSERT, {
+                                                            start: destPosition,
+                                                            type: DrawingFrame.getDrawingType(drawingNode),
+                                                            attrs: allDrawingAttrs
+                                                        });
+
+                            updatePosition = _.clone(destPosition); // for setting attributes required
+                        }
+
+                        // and finally setting the new position attributes via a setAttributes operation
                         generator.generateOperation(Operations.SET_ATTRIBUTES, {
                             attrs: { drawing: { anchorHorOffset: anchorHorOffset, anchorVertOffset: anchorVertOffset, anchorHorAlign: anchorHorAlign, anchorVertAlign: anchorVertAlign, anchorHorBase: anchorHorBase, anchorVertBase: anchorVertBase } },
                             start: updatePosition
@@ -638,7 +726,7 @@ define('io.ox/office/text/drawingResize',
                 paragraph = drawingNode.parent();
                 maxDrawingWidth = paragraph.width();  // using only the paragraph width, no padding or margin (task 28867)
 
-                pageAttributes = editor.getDocumentStyles().getStyleSheets('page').getElementAttributes(rootNode);
+                pageAttributes = editor.getDocumentStyles().getStyleCollection('page').getElementAttributes(rootNode);
                 // reading page attributes, they are always available -> no need to check existence
                 maxDrawingHeight = pageAttributes.page.height - pageAttributes.page.marginTop - pageAttributes.page.marginBottom;
                 maxDrawingHeight = Utils.convertHmmToLength(maxDrawingHeight, 'px', 1);
@@ -717,6 +805,8 @@ define('io.ox/office/text/drawingResize',
             function stopResizeDrawing() {
 
                 var generator = editor.createOperationsGenerator(),
+                    // all attributes for the operation
+                    allAttrs = {},
                     // drawing attributes for the operation
                     attributes = {},
                     // the final scaling of the drawing width and height
@@ -750,7 +840,9 @@ define('io.ox/office/text/drawingResize',
                     // the height of the offset node, in 1/100 mm
                     offsetNodeHeight = 0,
                     // the number of floated upper drawings in a paragraph
-                    upperDrawings = 0;
+                    upperDrawings = 0,
+                    // the old drawing attributes
+                    oldAttrs = null;
 
                 // Calculating new width and height
 
@@ -772,7 +864,7 @@ define('io.ox/office/text/drawingResize',
 
                 // Check, if the drawing needs to be moved upwards
                 // -> recalculating offsets and sometimes position
-                if ((topResize) && (moveY !== 0) && (options.movable)) {
+                if ((topResize) && (moveY !== 0) && (options.movable) && !DOM.isInlineDrawingNode(drawingNode)) {
 
                     // moveY > 0 -> Drawing was increased -> Moving drawing upwards
                     // moveY < 0 -> Drawing was decreased -> Moving drawing downwards
@@ -786,7 +878,7 @@ define('io.ox/office/text/drawingResize',
                     if (paragraphDeltaY < 0) { paragraphDeltaY = 0; }
 
                     // evaluating attributes
-                    drawingNodeAttrs = editor.getStyleSheets('drawing').getElementAttributes(drawingNode).drawing;
+                    drawingNodeAttrs = editor.getStyleCollection('drawing').getElementAttributes(drawingNode).drawing;
                     oldAnchorVertOffset = drawingNodeAttrs.anchorVertOffset ? drawingNodeAttrs.anchorVertOffset : 0;
                     anchorVertBase = drawingNodeAttrs.anchorVertBase;
                     anchorVertAlign = drawingNodeAttrs.anchorVertAlign;
@@ -852,11 +944,27 @@ define('io.ox/office/text/drawingResize',
                     }
                 }
 
-                // Generating the setAttribute operation
+                // Generating the setAttribute operation, if necessary
                 if (!_.isEmpty(attributes)) {
+
+                    allAttrs = { drawing: attributes };
+
+                    // Resize of drawing is not supported yet in OOXML file format
+                    // -> this code needs to be activated, if OOXML supports change tracking of resizing of drawings
+                    // -> in this case ctSupportsDrawingResizing() has to return 'true'
+                    if (editor.getChangeTrack().isActiveChangeTracking() && editor.getChangeTrack().ctSupportsDrawingResizing()) {
+                        // Expanding operation for change tracking with old explicit attributes
+                        oldAttrs = editor.getChangeTrack().getOldNodeAttributes(drawingNode);
+                        // adding the old attributes, author and date for change tracking
+                        if (oldAttrs) {
+                            oldAttrs = _.extend(oldAttrs, editor.getChangeTrack().getChangeTrackInfo());
+                            allAttrs.changes = { 'modified': oldAttrs };
+                        }
+                    }
+
                     generator.generateOperation(Operations.SET_ATTRIBUTES, {
                         start: updatePosition,
-                        attrs: { drawing: attributes }
+                        attrs: allAttrs
                     });
                 }
 
