@@ -14,12 +14,12 @@
 
 define('io.ox/office/text/format/drawingstyles',
     ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/format/border',
-     'io.ox/office/drawinglayer/model/drawingstyles',
+     'io.ox/office/editframework/utils/border',
+     'io.ox/office/drawinglayer/model/drawingstylecollection',
      'io.ox/office/drawinglayer/view/drawingframe',
      'io.ox/office/text/dom',
      'io.ox/office/text/drawingResize'
-    ], function (Utils, Border, DrawingStyles, DrawingFrame, DOM, DrawingResize) {
+    ], function (Utils, Border, DrawingStyleCollection, DrawingFrame, DOM, DrawingResize) {
 
     'use strict';
 
@@ -180,7 +180,7 @@ define('io.ox/office/text/format/drawingstyles',
      *
      * @constructor
      *
-     * @extends DrawingStyles
+     * @extends DrawingStyleCollection
      *
      * @param {TextApplication} app
      *  The root application instance.
@@ -192,7 +192,7 @@ define('io.ox/office/text/format/drawingstyles',
 
         // base constructor ---------------------------------------------------
 
-        DrawingStyles.call(this, app, documentStyles, { formatHandler: updateDrawingFormatting });
+        DrawingStyleCollection.call(this, app, documentStyles, { additionalFamilies: ['changes'], formatHandler: updateDrawingFormatting });
 
         // private methods ----------------------------------------------------
 
@@ -425,9 +425,11 @@ define('io.ox/office/text/format/drawingstyles',
                     default:
                         leftOffset = 0;
                     }
-                } else if (drawingAttributes.anchorHorBase === 'page') {
+                } else if ((drawingAttributes.anchorHorBase === 'page') || (drawingAttributes.anchorHorBase === 'character')) {
                     // 'page' looks better if it is handled like column, not like the page node 'div.page'
                     // pageWidth = Utils.convertLengthToHmm(drawing.closest(DOM.PAGE_NODE_SELECTOR).width(), 'px');
+                    // Also 'character' looks better, if it is handled like 'column' (33276). The value for 'anchorHorOffset'
+                    // is not relative to the character, but to the paragraph.
                     switch (drawingAttributes.anchorHorAlign) {
                     case 'center':
                         leftOffset = (paraWidth - drawingWidth) / 2;
@@ -520,6 +522,9 @@ define('io.ox/office/text/format/drawingstyles',
                 });
             }
 
+            // change track attribute handling
+            app.getModel().getChangeTrack().updateChangeTrackAttributes(drawing, mergedAttributes);
+
             // set generic formatting to the drawing frame
             DrawingFrame.updateFormatting(app, drawing, mergedAttributes);
         }
@@ -622,7 +627,7 @@ define('io.ox/office/text/format/drawingstyles',
 
     // exports ================================================================
 
-    // derive this class from class DrawingStyles
-    return DrawingStyles.extend({ constructor: TextDrawingStyles });
+    // derive this class from class DrawingStyleCollection
+    return DrawingStyleCollection.extend({ constructor: TextDrawingStyles });
 
 });
