@@ -370,7 +370,7 @@ define('io.ox/office/text/format/listcollection',
 
             defaultBulletNumId,
 
-            characterStyles = documentStyles.getStyleSheets('character'),
+            characterStyles = documentStyles.getStyleCollection('character'),
 
             odfFormat = false;
 
@@ -801,6 +801,61 @@ define('io.ox/office/text/format/listcollection',
             return newOperation;
         };
 
+        /**
+         * Generates an Insert List operation from a MS list defintion (generated by the clipboard parser).
+         *
+         * @param {Object} definition
+         *          {Object} level
+         *              {String} numberFormat
+         *                  The number format of the list level
+         *              {String} levelText
+         *                  The text for the list level
+         *              {String} fontFamily
+         *                  The level text font
+         *              {Number} indentFirstLine
+         *                  The first line text indent in 1/100 of millimeters.
+         *              {Number} indentLeft
+         *                  The left indent in 1/100 of millimeters.
+         *              {String} numberPosition
+         *                  The number position
+         *
+         * @returns {Object}
+         *  The operation that creates the requested list.
+         */
+        this.getListOperationFromListDefinition = function (definition) {
+
+            var sFreeId = this.getFreeListId(),
+                newOperation = { name: Operations.INSERT_LIST, listStyleId: sFreeId },
+                type = definition && definition[0] && definition[0].numberFormat;
+
+            // set list definition for either bullet or numbering
+            newOperation.listDefinition = _.copy((type === 'bullet') ? DEFAULT_BULLET_LIST_DEFINITION : DEFAULT_NUMBERING_LIST_DEFINITION, true);
+
+            _.each(definition, function (listItem, listLevel) {
+
+                if (listItem.numberFormat) {
+                    newOperation.listDefinition['listLevel' + listLevel].numberFormat = listItem.numberFormat;
+                }
+                if (listItem.levelText) {
+                    newOperation.listDefinition['listLevel' + listLevel].levelText = listItem.levelText;
+                }
+                if (listItem.fontFamily) {
+                    newOperation.listDefinition['listLevel' + listLevel].fontName = listItem.fontFamily;
+                }
+                if (listItem.indentFirstLine) {
+                    newOperation.listDefinition['listLevel' + listLevel].indentFirstLine = listItem.indentFirstLine;
+                }
+                if (listItem.indentLeft) {
+                    newOperation.listDefinition['listLevel' + listLevel].indentLeft = listItem.indentLeft;
+                }
+                if (listItem.numberPosition) {
+                    newOperation.listDefinition['listLevel' + listLevel].textAlign = listItem.numberPosition;
+                }
+            });
+
+            return newOperation;
+        };
+
         /**
          * Generates an Insert List operation from a list style id.
          *
@@ -1085,8 +1140,8 @@ define('io.ox/office/text/format/listcollection',
 
         // initialization -----------------------------------------------------
 
-        app.on('docs:import:success', function () {
-            odfFormat = documentStyles.getAttributes().fileFormat === 'odf';
+        app.onImportSuccess(function () {
+            odfFormat = documentStyles.getDocumentAttributes().fileFormat === 'odf';
         });
 
     } // class ListCollection
