@@ -13,12 +13,12 @@
 
 define('io.ox/office/text/format/paragraphstyles',
     ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/format/color',
-     'io.ox/office/editframework/model/format/border',
-     'io.ox/office/editframework/model/format/lineheight',
-     'io.ox/office/editframework/model/format/stylesheets',
+     'io.ox/office/editframework/utils/color',
+     'io.ox/office/editframework/utils/border',
+     'io.ox/office/editframework/utils/lineheight',
+     'io.ox/office/editframework/model/stylecollection',
      'io.ox/office/text/dom'
-    ], function (Utils, Color, Border, LineHeight, StyleSheets, DOM) {
+    ], function (Utils, Color, Border, LineHeight, StyleCollection, DOM) {
 
     'use strict';
 
@@ -123,8 +123,30 @@ define('io.ox/office/text/format/paragraphstyles',
 
             marginBottom: { def: 0 },
 
-            contextualSpacing: { def: false }
+            contextualSpacing: { def: false },
 
+            pageBreakBefore: {
+                def: false,
+                format: function (element, value) {
+                    if (value) {
+                        if (element.closest('tr').length > 0) {
+                            element.parents('table').last().addClass('manual-page-break');
+                        }
+                        element.addClass('manual-page-break');
+                    }
+                },
+            },
+            pageBreakAfter: {
+                def: false,
+                format: function (element, value) {
+                    if (value) {
+                        if (element.closest('tr').length > 0) {
+                            element.parents('table').last().addClass('manual-pb-after');
+                        }
+                        element.addClass('manual-pb-after');
+                    }
+                },
+            }
         },
 
         // parent families with parent element resolver functions
@@ -133,10 +155,7 @@ define('io.ox/office/text/format/paragraphstyles',
         },
 
         // maps fill character attribute values to fill characters
-        TAB_FILL_CHARS = { dot: '.', hyphen: '-', underscore: '_' },
-
-        // the names of all border properties in paragraphs, mapped by border mode properties
-        PARAGRAPH_BORDER_ATTRIBUTES = { left: 'borderLeft', right: 'borderRight', top: 'borderTop', bottom: 'borderBottom', insideh: 'borderInside' };
+        TAB_FILL_CHARS = { dot: '.', hyphen: '-', underscore: '_' };
 
     // private static functions ===============================================
 
@@ -188,6 +207,15 @@ define('io.ox/office/text/format/paragraphstyles',
             spanNode.contents().remove().end().text(shortFillString);
             if (! shortFillString) { DOM.ensureExistingTextNode(spanNode); }
         }
+
+        // workaround for iOS & Safari mobile - needs a text node directly within
+        // the tab div to enable an external keyboard to jump through a tab chains
+        // with empty spans
+        if (_.device('ios') && _.device('safari')) {
+            if (!spanNode[0].nextSibling) {
+                spanNode.after(document.createTextNode('\u00a0'));
+            }
+        }
     }
 
     // class ParagraphStyles ==================================================
@@ -198,7 +226,7 @@ define('io.ox/office/text/format/paragraphstyles',
      *
      * @constructor
      *
-     * @extends StyleSheets
+     * @extends StyleCollection
      *
      * @param {TextApplication} app
      *  The root application instance.
@@ -215,8 +243,8 @@ define('io.ox/office/text/format/paragraphstyles',
 
         // base constructor ---------------------------------------------------
 
-        StyleSheets.call(this, app, documentStyles, 'paragraph', {
-            additionalFamilies: 'character',
+        StyleCollection.call(this, app, documentStyles, 'paragraph', {
+            additionalFamilies: ['character', 'changes'],
             parentResolvers: PARENT_RESOLVERS,
             formatHandler: updateParagraphFormatting,
             previewHandler: setParagraphPreviewFormatting
@@ -244,7 +272,7 @@ define('io.ox/office/text/format/paragraphstyles',
         function updateTabStops(paragraph, mergedAttributes) {
 
             var // default tab stop width from document settings
-                defaultTabStop = documentStyles.getAttributes().defaultTabStop,
+                defaultTabStop = documentStyles.getDocumentAttributes().defaultTabStop,
                 // paragraph tab stop definitions
                 paraTabStops = mergedAttributes.paragraph.tabStops,
                 // effective left margin of the paragraph
@@ -499,7 +527,7 @@ define('io.ox/office/text/format/paragraphstyles',
             var // the paragraph attributes of the passed attribute map
                 paragraphAttributes = mergedAttributes.paragraph,
                 // the character styles/formatter
-                characterStyles = documentStyles.getStyleSheets('character'),
+                characterStyles = documentStyles.getStyleCollection('character'),
 
                 leftMargin = 0,
                 rightMargin = 0,
@@ -523,6 +551,13 @@ define('io.ox/office/text/format/paragraphstyles',
             // depends on the paragraph fill color. Also visit all helper nodes
             // containing text spans, e.g. numbering labels.
             Utils.iterateDescendantNodes(paragraph, function (node) {
+
+                // visiting the span inside a hard break node
+                // -> this is necessary for change tracking attributes
+                if (DOM.isHardBreakNode(node)) {
+                    characterStyles.updateElementFormatting(node.firstChild);
+                }
+
                 DOM.iterateTextSpans(node, function (span) {
                     characterStyles.updateElementFormatting(span, { baseAttributes: mergedAttributes });
                 });
@@ -563,7 +598,7 @@ define('io.ox/office/text/format/paragraphstyles',
             var listStyleId = paragraphAttributes.listStyleId;
             if (listStyleId.length) {
                 var listLevel = paragraphAttributes.listLevel,
-                     lists = documentStyles.getContainer('lists');
+                     lists = documentStyles.getCustomCollection('lists');
                 if (listLevel < 0) {
                     // is a numbering level assigned to the current paragraph style?
                     listLevel = lists.findIlvl(listStyleId, mergedAttributes.styleId);
@@ -608,6 +643,11 @@ define('io.ox/office/text/format/paragraphstyles',
                 'text-indent': (textIndent / 100) + 'mm'
             });
 
+            // Overwrite of margin left for lists in draft mode (conversion from fixed unit mm to %)
+            if (Utils.SMALL_DEVICE && paragraph.data('draftRatio')) {
+                $(paragraph).css('margin-left',  (parseInt($(paragraph).css('margin-left')) * paragraph.data('draftRatio')) + '%');
+            }
+
             //no distance between paragraph using the same style if contextualSpacing is set
             var noDistanceToPrev = prevAttributes.paragraph.contextualSpacing && (mergedAttributes.styleId === prevAttributes.styleId),
                 noDistanceToNext = paragraphAttributes.contextualSpacing && (mergedAttributes.styleId === nextAttributes.styleId);
@@ -636,6 +676,9 @@ define('io.ox/office/text/format/paragraphstyles',
             if (paragraph.find(DOM.TAB_NODE_SELECTOR).length > 0) {
                 updateTabStops(paragraph, mergedAttributes);
             }
+
+            // change track attribute handling
+            app.getModel().getChangeTrack().updateChangeTrackAttributes(paragraph, mergedAttributes);
         }
 
         /**
@@ -652,7 +695,7 @@ define('io.ox/office/text/format/paragraphstyles',
         function setParagraphPreviewFormatting(paragraph, mergedAttributes) {
 
             var // the character styles/formatter
-                characterStyles = documentStyles.getStyleSheets('character');
+                characterStyles = documentStyles.getStyleCollection('character');
 
             // format the text spans contained in the paragraph element
             paragraph.children().each(function () {
@@ -685,78 +728,24 @@ define('io.ox/office/text/format/paragraphstyles',
         documentStyles.registerAttributeDefinitions('paragraph', DEFINITIONS);
 
         //initialization after construction
-        app.on('docs:init', function () {
+        app.onInit(function () {
             view = app.getView();
         });
 
     } // class ParagraphStyles
 
-    // static methods ---------------------------------------------------------
+    // constants --------------------------------------------------------------
 
     /**
-     * Converts the passed paragraph attributes to an object describing the
-     * visibility states of all border attributes.
+     * Default solid border line for paragraphs and tables.
      *
-     * @param {Object} attributes
-     *  The paragraph attributes.
-     *
-     * @returns {Object}
-     *  An object with position keys ('left', 'top', etc.), and Boolean values
-     *  or Null values specifying whether the respective borders are visible.
-     *  If a border attribute in the passed attribute map is ambiguous (value
-     *  null or missing), the map value will be null.
+     * @constant
      */
-    ParagraphStyles.getBorderModeFromAttributes = function (attributes) {
-
-        var // result mapping border attribute names to boolean values
-            borderMode = {};
-
-        _(PARAGRAPH_BORDER_ATTRIBUTES).each(function (attrName, propName) {
-            borderMode[propName] = _.isObject(attributes[attrName]) ? Border.isVisibleBorder(attributes[attrName]) : null;
-        });
-
-        return borderMode;
-    };
-
-    /**
-     * Converts the passed object describing the visibility states of paragraph
-     * border attributes to a paragraph attribute map containing all border
-     * attributes.
-     *
-     * @param {Object} borderMode
-     *  An object with position keys ('left', 'top', etc.), and Boolean values
-     *  specifying whether the respective borders are visible. Attributes not
-     *  contained in this object will not occur in the returned attribute set.
-     *  If the passed object contains a single property set to true, the
-     *  current visibility state of the respective border attribute will be
-     *  toggled.
-     *
-     * @param {Object} [origAttributes]
-     *  The original attributes of the target paragraphs.
-     *
-     * @returns {Object}
-     *  The paragraph attributes containing all border attributes that are
-     *  mentioned in the passed object.
-     */
-    ParagraphStyles.getAttributesFromBorderMode = function (borderMode, origAttributes) {
-
-        var // the resulting paragraph attributes
-            attributes = {},
-            // whether to toggle the visibility of a single border
-            toggle = _.size(borderMode) === 1;
-
-        _(borderMode).each(function (visible, propName) {
-            var borderAttrName = PARAGRAPH_BORDER_ATTRIBUTES[propName];
-            visible = toggle ? !Border.isVisibleBorder(origAttributes[borderAttrName]) : visible;
-            attributes[borderAttrName] = visible ? Border.SINGLE : Border.NONE;
-        });
-
-        return attributes;
-    };
+    ParagraphStyles.SINGLE_BORDER = { style: 'single', width: 17, space: 140, color: Color.AUTO };
 
     // exports ================================================================
 
-    // derive this class from class StyleSheets
-    return StyleSheets.extend({ constructor: ParagraphStyles });
+    // derive this class from class StyleCollection
+    return StyleCollection.extend({ constructor: ParagraphStyles });
 
 });
