@@ -13,13 +13,14 @@
 
 define('io.ox/office/text/format/tablestyles',
     ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/format/color',
-     'io.ox/office/editframework/model/format/border',
-     'io.ox/office/editframework/model/format/lineheight',
-     'io.ox/office/editframework/model/format/stylesheets',
+     'io.ox/office/editframework/utils/color',
+     'io.ox/office/editframework/utils/border',
+     'io.ox/office/editframework/utils/lineheight',
+     'io.ox/office/editframework/utils/attributeutils',
+     'io.ox/office/editframework/model/stylecollection',
      'io.ox/office/text/dom',
      'io.ox/office/text/table'
-    ], function (Utils, Color, Border, LineHeight, StyleSheets, DOM, Table) {
+    ], function (Utils, Color, Border, LineHeight, AttributeUtils, StyleCollection, DOM, Table) {
 
     'use strict';
 
@@ -138,7 +139,7 @@ define('io.ox/office/text/format/tablestyles',
      *
      * @constructor
      *
-     * @extends StyleSheets
+     * @extends StyleCollection
      *
      * @param {TextApplication} app
      *  The root application instance.
@@ -150,8 +151,8 @@ define('io.ox/office/text/format/tablestyles',
 
         // base constructor ---------------------------------------------------
 
-        StyleSheets.call(this, app, documentStyles, 'table', {
-            additionalFamilies: ['cell', 'paragraph', 'character'],
+        StyleCollection.call(this, app, documentStyles, 'table', {
+            additionalFamilies: ['cell', 'paragraph', 'character', 'changes'],
             styleAttributesResolver: resolveTableStyleAttributes,
             elementAttributesResolver: resolveTableElementAttributes,
             formatHandler: updateTableFormatting,
@@ -198,7 +199,7 @@ define('io.ox/office/text/format/tablestyles',
                 isLastCol = colRange && (colRange.end === lastTableCol),
 
                 // the excluded conditional keys
-                excludedConditionalKeys = StyleSheets.getExplicitAttributes(tableNode, 'table').exclude,
+                excludedConditionalKeys = AttributeUtils.getExplicitAttributes(tableNode, { family: 'table', direct: true }).exclude,
                 firstRowIncluded = isConditionalKeyIncluded('firstRow'),
                 lastRowIncluded = isConditionalKeyIncluded('lastRow'),
                 firstColIncluded = isConditionalKeyIncluded('firstCol'),
@@ -382,38 +383,32 @@ define('io.ox/office/text/format/tablestyles',
         function updateTableFormatting(table, mergedAttributes, async) {
 
             var // the table styles/formatter
-                tableStyles = documentStyles.getStyleSheets('table'),
+                tableStyles = documentStyles.getStyleCollection('table'),
                 // the table row styles/formatter
-                tableRowStyles = documentStyles.getStyleSheets('row'),
+                tableRowStyles = documentStyles.getStyleCollection('row'),
                 // the table cell styles/formatter
-                tableCellStyles = documentStyles.getStyleSheets('cell'),
+                tableCellStyles = documentStyles.getStyleCollection('cell'),
                 // the paragraph styles/formatter
-                paragraphStyles = documentStyles.getStyleSheets('paragraph'),
+                paragraphStyles = documentStyles.getStyleCollection('paragraph'),
                 // the column widths
                 tableGrid = mergedAttributes.table.tableGrid,
                 // the explicit table attributes
-                tableAttributes = StyleSheets.getExplicitAttributes(table).table,
+                tableAttributes = AttributeUtils.getExplicitAttributes(table, { family: 'table' }),
                 // all row nodes in the table
                 rowNodes = DOM.getTableRows(table),
-                // the number of rows updated at a time
-                chunkLength = 0,
                 // whether this update was triggered by a resize of table (column width)
                 isTableResize = false,
                 // whether the table has a 'simple' style
                 isSimpleTableStyle = false,
-                // whether this update was triggered by an GUI event
-                isGUIOperation = false,
                 // whether cells were added to the table
                 isGUIInsertOperation = false,
                 // whether cells were removed from the table
                 isGUIRemoveOperation = false,
                 // the number of the first row, that requires a formatting update
                 minUpdateRow = null,
-                // the maximum number of cells to be updated in asynchronous mode
-                maxCells = 40,
                 // the result Deferred object
                 def = null;
-            
+
             // checking if the attributes have table geometric dependencies
             // -> then a complete reformatting of the table is required.
             // Otherwise only the new cells (insert operations) or no cells
@@ -429,49 +424,49 @@ define('io.ox/office/text/format/tablestyles',
                 return isSimple;
             }
 
-            // updates the formatting of the passed table rows
-            function updateRowsFormatting(currentRowNodes) {
-                currentRowNodes.each(function () {
+            // updates the formatting of the passed table row
+            function updateRowFormatting(rowNode) {
 
-                    // update the table row itself
-                    tableRowStyles.updateElementFormatting(this);
+                // update the table row itself
+                tableRowStyles.updateElementFormatting(rowNode);
 
-                    // update table cells (do NOT pass merged attributes passed
-                    // to the updateTableFormatting() method as base attributes,
-                    // attributes must be recalculated for each single cell!)
-                    $(this).children('td').each(function () {
-                        var cellFormatting = true;
-                        // Performance: Updating only new cells, if this update was triggered from a GUI event
-                        // and if the table has a simple table style -> otherwise all cells need to be updated
-                        // Performance: Simplified cell formatting for deleteColumn and deleteRow GUI operations.
-                        // For the insert operations (insertRow or insertColumn) cell (and paragraph) formatting
-                        // is always required.
-                        if (isGUIRemoveOperation) {
-                            if (isSimpleTableStyle) {
-                                cellFormatting = false;  // using simplified cell formatting update (only taking care of tab stops)
-                                // TODO: For not simple table styles the full cell formatting is only required for those cells
-                                // that are behind the new/removed cell in the row for insertColumn and deleteColumn
-                            }
+                // update table cells (do NOT pass merged attributes passed
+                // to the updateTableFormatting() method as base attributes,
+                // attributes must be recalculated for each single cell!)
+                $(rowNode).children('td').each(function () {
+
+                    var cellFormatting = true;
+
+                    // Performance: Updating only new cells, if this update was triggered from a GUI event
+                    // and if the table has a simple table style -> otherwise all cells need to be updated
+                    // Performance: Simplified cell formatting for deleteColumn and deleteRow GUI operations.
+                    // For the insert operations (insertRow or insertColumn) cell (and paragraph) formatting
+                    // is always required.
+                    if (isGUIRemoveOperation) {
+                        if (isSimpleTableStyle) {
+                            cellFormatting = false;  // using simplified cell formatting update (only taking care of tab stops)
+                            // TODO: For not simple table styles the full cell formatting is only required for those cells
+                            // that are behind the new/removed cell in the row for insertColumn and deleteColumn
                         }
+                    }
 
-                        // Performance: Resizing the table (column width) requires only tab stop recalculation
-                        if (isTableResize) { cellFormatting = false; }
+                    // Performance: Resizing the table (column width) requires only tab stop recalculation
+                    if (isTableResize) { cellFormatting = false; }
 
-                        if (cellFormatting) {
-                            // complete formatting update of the cell -> this is necessary for complex table styles, but also for simple
-                            // table styles, because paragraph formatting is also necessary. For example determines the paragraph height the
-                            // height of the row, if the table row height is set to 'auto'.
-                            tableCellStyles.updateElementFormatting(this);
-                        } else {
-                            // checking if a cell contains tab stops, at least these need to be updated, if cell width was changed
-                            if ($(this).find(DOM.TAB_NODE_SELECTOR).length > 0) {
-                                // updating the tabs is always required, because the width of a cell might have changed -> updating all paragraphs in the table cell
-                                Utils.iterateSelectedDescendantNodes(DOM.getCellContentNode(this), DOM.PARAGRAPH_NODE_SELECTOR, function (paragraph) {
-                                    paragraphStyles.updateTabStops(paragraph);
-                                }, undefined, { children: true });
-                            }
+                    if (cellFormatting) {
+                        // complete formatting update of the cell -> this is necessary for complex table styles, but also for simple
+                        // table styles, because paragraph formatting is also necessary. For example determines the paragraph height the
+                        // height of the row, if the table row height is set to 'auto'.
+                        tableCellStyles.updateElementFormatting(this);
+                    } else {
+                        // checking if a cell contains tab stops, at least these need to be updated, if cell width was changed
+                        if ($(this).find(DOM.TAB_NODE_SELECTOR).length > 0) {
+                            // updating the tabs is always required, because the width of a cell might have changed -> updating all paragraphs in the table cell
+                            Utils.iterateSelectedDescendantNodes(DOM.getCellContentNode(this), DOM.PARAGRAPH_NODE_SELECTOR, function (paragraph) {
+                                paragraphStyles.updateTabStops(paragraph);
+                            }, undefined, { children: true });
                         }
-                    });
+                    }
                 });
             }
 
@@ -481,10 +476,9 @@ define('io.ox/office/text/format/tablestyles',
             }
 
             if (table.data('gui')) {
-                isGUIOperation = true;
                 if (table.data('gui') === 'insert') {
                     isGUIInsertOperation = true;
-                } else if  (table.data('gui') === 'remove') {
+                } else if (table.data('gui') === 'remove') {
                     isGUIRemoveOperation = true;
                 }
                 table.removeData('gui');
@@ -509,7 +503,7 @@ define('io.ox/office/text/format/tablestyles',
             if ((_.browser.WebKit) && (table.closest('div.cellcontent').length > 0) && (table.css('max-width') === '100%') && (table.width() > table.closest('div.cellcontent').width())) {
                 table.css('width', '');
             }
-            
+
             // Performance: Immediately return, if no update required for simple tables (for example after deleteRows).
             // Even update of tab stops is not required for insertRow and deleteRow. But insertRow requires update of
             // paragraph attributes.
@@ -523,22 +517,24 @@ define('io.ox/office/text/format/tablestyles',
 
             // Performance check for very slow devices. In this case an asynchronous update is
             // required, so that the user gets faster feed back
-            if (Utils.IPAD && !async && Utils.SLOW_DEVICE) {
-                async = true;
-                maxCells = 5;
+            async = async || (Utils.IPAD && Utils.SLOW_DEVICE);
+
+            // Performance: Reducing the number of rows, because upper rows require no update
+            if (_.isNumber(minUpdateRow)) {
+                rowNodes = rowNodes.slice(minUpdateRow);
             }
 
+            // change track attribute handling
+            app.getModel().getChangeTrack().updateChangeTrackAttributes(table, mergedAttributes);
+
             // update rows, cells, ... , but only if required
-            if (async === true) {
-                // process the table asynchronously (typically about 40 cells at a time)
-                chunkLength = Math.max(1, Math.round(maxCells / Math.max(1, tableGrid.length)));
-                def = app.processArrayDelayed(updateRowsFormatting, rowNodes, { chunkLength: chunkLength });
+            if (async) {
+                // process the table asynchronously
+                def = app.iterateArraySliced(rowNodes, updateRowFormatting);
             } else {
-                // Performance: Reducing the number of rows, because upper rows require no update
-                if (minUpdateRow !== null) { rowNodes = rowNodes.slice(minUpdateRow); }
                 // update all rows at once (but asynchronous, so that for example a new column is visible immediately)
                 def = app.executeDelayed(function () {
-                    updateRowsFormatting(rowNodes);
+                    rowNodes.each(function () { updateRowFormatting(this); });
                 });
             }
 
@@ -559,7 +555,7 @@ define('io.ox/office/text/format/tablestyles',
         function updateTablePreviewFormatting(table/*, mergedAttributes*/) {
 
             var // the table cell styles/formatter
-                tableCellStyles = documentStyles.getStyleSheets('cell');
+                tableCellStyles = documentStyles.getStyleCollection('cell');
 
             // iterate over all cells in the table to set the cell attributes
             DOM.getTableCells(table).each(function () {
@@ -752,7 +748,7 @@ define('io.ox/office/text/format/tablestyles',
 
     // exports ================================================================
 
-    // derive this class from class StyleSheets
-    return StyleSheets.extend({ constructor: TableStyles });
+    // derive this class from class StyleCollection
+    return StyleCollection.extend({ constructor: TableStyles });
 
 });
