@@ -15,13 +15,12 @@ define('io.ox/office/text/hyperlink',
     ['io.ox/office/tk/utils',
      'io.ox/office/tk/keycodes',
      'io.ox/office/tk/dialogs',
-     'io.ox/office/editframework/model/format/stylesheets',
+     'io.ox/office/editframework/utils/attributeutils',
+     'io.ox/office/editframework/view/hyperlinkutil',
      'io.ox/office/text/dom',
      'io.ox/office/text/position',
-     'io.ox/office/text/operations',
-     'gettext!io.ox/office/text',
-     'io.ox/office/editframework/view/hyperlinkutil'
-    ], function (Utils, KeyCodes, Dialogs, StyleSheets, DOM, Position, Operations, gt, HyperlinkUtil) {
+     'io.ox/office/text/operations'
+    ], function (Utils, KeyCodes, Dialogs, AttributeUtils, HyperlinkUtil, DOM, Position, Operations) {
 
     'use strict';
 
@@ -146,8 +145,8 @@ define('io.ox/office/text/hyperlink',
             }
 
             if (obj && obj.node && DOM.isTextSpan(obj.node.parentNode)) {
-                var attributes = StyleSheets.getExplicitAttributes(obj.node.parentNode, 'character');
-                if ((attributes) && (attributes.url) && (attributes.url.length > 0)) {
+                var attributes = AttributeUtils.getExplicitAttributes(obj.node.parentNode, { family: 'character', direct: true });
+                if (attributes.url) {
                     // Now we have to check some edge cases to prevent to show
                     // the popup for a paragraph which contains only an empty span
                     // having set the url attribute.
@@ -162,9 +161,9 @@ define('io.ox/office/text/hyperlink',
                 }
 
                 // Check next special case: Before a hyperlink we always want to show the popup
-                if (splitOperation && selection.getInsertTextInfo()) {
+                if (splitOperation && selection.getParagraphCache()) {
                     // Performance: Using previously cached paragraph and text position
-                    obj = Position.getDOMPosition(selection.getInsertTextInfo().node, [selection.getInsertTextInfo().offset + 1]);
+                    obj = Position.getDOMPosition(selection.getParagraphCache().node, [selection.getParagraphCache().offset + 1]);
                 } else {
                     var nextPosition = selection.getStartPosition();
                     nextPosition[nextPosition.length - 1] += 1;
@@ -172,8 +171,8 @@ define('io.ox/office/text/hyperlink',
                 }
 
                 if (obj && obj.node && DOM.isTextSpan(obj.node.parentNode)) {
-                    var nextAttributes = StyleSheets.getExplicitAttributes(obj.node.parentNode, 'character');
-                    if ((nextAttributes) && (nextAttributes.url) && (nextAttributes.url.length > 0) && (attributes.url !== nextAttributes.url)) {
+                    var nextAttributes = AttributeUtils.getExplicitAttributes(obj.node.parentNode, { family: 'character', direct: true });
+                    if (nextAttributes.url && (attributes.url !== nextAttributes.url)) {
                         result.url = nextAttributes.url;
                         result.beforeHyperlink = true;
                     }
@@ -253,7 +252,7 @@ define('io.ox/office/text/hyperlink',
             endNode = null,
             attributes = null,
             obj = null,
-            characterStyles = editor.getStyleSheets('character'),
+            characterStyles = editor.getStyleCollection('character'),
             result = { start: null, end: null };
 
         obj = Position.getDOMPosition(editor.getNode(), startPosition);
@@ -455,7 +454,7 @@ define('io.ox/office/text/hyperlink',
             endPos = -1,
             textFound = false,
             url = null,
-            characterStyles = editor.getStyleSheets('character'),
+            characterStyles = editor.getStyleCollection('character'),
             selection = { start: null, end: null, text: null, url: null };
 
         Position.iterateParagraphChildNodes(paragraph, function (node, nodeStart, nodeLength, nodeOffset, offsetLength) {
@@ -574,7 +573,7 @@ define('io.ox/office/text/hyperlink',
      */
     Hyperlink.insertHyperlink = function (editor, start, end, url) {
         var hyperlinkStyleId = editor.getDefaultUIHyperlinkStylesheet(),
-            characterStyles = editor.getStyleSheets('character'),
+            characterStyles = editor.getStyleCollection('character'),
             generator = editor.createOperationsGenerator();
 
         if (characterStyles.isDirty(hyperlinkStyleId)) {
@@ -687,11 +686,6 @@ define('io.ox/office/text/hyperlink',
         if (Hyperlink.getPopupNode(editor).length === 0) {
             var hyperlinkPopup = HyperlinkUtil.createPopupNode(app);
 
-            // Add keydown handler to handle TAB/SHIFT+TAB within popup
-            // as one of our parents (editdiv) installs a handler by it's
-            // own. We want to have keyboard handling for our functions.
-            hyperlinkPopup.on('keydown', { nextFocus: page }, Hyperlink.popupProcessKeydown);
-
             page.append(hyperlinkPopup);
 
             var updateEditMode = function (options) {
@@ -699,66 +693,14 @@ define('io.ox/office/text/hyperlink',
             };
 
             selection.on('change', function (event, options) {
-                if ((Utils.getBooleanOption(options, 'insertOperation', false)) && (hyperlinkPopup.css('display') === 'none')) {
-                    return;  // Performance: Do nothing for insertText operations, if no hyperlink popup is visible
+                if (Utils.getBooleanOption(options, 'insertOperation', false) && (hyperlinkPopup[0].style.display === 'none')) {
+                    // Performance: Use 'hyperlinkPopup[0].style.display' instead of jQuery, because of performance problems in Firefox
+                    return false;  // Performance: Do nothing for insertText operations, if no hyperlink popup is visible
                 }
                 updateEditMode(options);
             });
 
             HyperlinkUtil.addEditModeListener(editor, hyperlinkPopup, updateEditMode);
-
-        }
-    };
-
-    /**
-     * Process keyboard events used to travel in a container and sets the
-     * focus to the next/previous element.
-     *
-     * The function expects that the element which gets the focus on ESCAPE to
-     * be in the [data.nextFocus] part of the event. Therefore this function
-     * must be set using .on('keydown', {nextFocus: focus}, popupProcessKeydown)
-     *
-     * @param {Object} event
-     *  A keydown event sent by the browser.
-     *
-     * @returns {Boolean}
-     *  Returns false if the browser shouldn't process the keyboard
-     *  event.
-     */
-    Hyperlink.popupProcessKeydown = function (event) {
-        var items, focus, index, down;
-
-        if (event.keyCode === KeyCodes.TAB || DOM.isCursorKey(event.keyCode)) {
-            event.preventDefault();
-
-            items = $(this).find('[tabindex]');
-            focus = $(document.activeElement);
-            down = (((event.keyCode === KeyCodes.TAB) && !event.shiftKey) ||
-                    (event.keyCode === KeyCodes.DOWN_ARROW) ||
-                    (event.keyCode === KeyCodes.RIGHT_ARROW));
-
-            index = (items.index(focus) || 0);
-            if (!down) {
-                index--;
-            } else {
-                index++;
-            }
-
-            if (index >= items.length) {
-                index = 0;
-            } else if (index < 0) {
-                index = items.length - 1;
-            }
-            items[index].focus();
-            event.preventDefault();
-            return false;
-        } else if (event.keyCode === KeyCodes.ESCAPE) {
-            $(this).hide();
-            if (event.data && event.data.nextFocus && event.data.nextFocus.length) {
-                $(event.data.nextFocus).focus();
-                event.preventDefault();
-                return false;
-            }
         }
     };
 
