@@ -13,11 +13,12 @@
 
 define('io.ox/office/text/table',
     ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/format/stylesheets',
+     'io.ox/office/editframework/utils/attributeutils',
      'io.ox/office/text/dom',
      'io.ox/office/text/position',
-     'io.ox/office/text/operations'
-    ], function (Utils, StyleSheets, DOM, Position, Operations) {
+     'io.ox/office/text/operations',
+     'io.ox/office/text/utils/config'
+    ], function (Utils, AttributeUtils, DOM, Position, Operations, Config) {
 
     'use strict';
 
@@ -250,6 +251,53 @@ define('io.ox/office/text/table',
         return (new DOM.Range(new DOM.Point(activeCellNode, startOffset), new DOM.Point(activeCellNode, endOffset)));
     }
 
+    /**
+     * Returns whether the passed table cell elements contain equal border formatting attributes.
+     *  Table borders are implemented in a way that border attributes are assigned inside each table cell.
+     *  We compare top upper left and bottom right cells of each table.
+     *
+     * @param {HTMLElement|jQuery} cell1
+     *  The first DOM element whose formatting attributes will be compared with
+     *  the attributes of the other passed DOM element. If this object is a
+     *  jQuery collection, uses the first DOM node it contains.
+     *
+     * @param {HTMLElement|jQuery} cell2
+     *  The second DOM element whose formatting attributes will be compared
+     *  with the attributes of the other passed DOM element. If this object is
+     *  a jQuery collection, uses the first DOM node it contains.
+     *
+     * @returns {Boolean}
+     *  Whether both DOM elements contain equal explicit formatting attributes.
+     */
+    function hasEqualTableBordersAttributes(cell1, cell2) {
+
+        function getCellBorderAttributes(cell) {
+            var // deep copy of complete attribute map
+                attributes = _.copy($(cell).data('attributes'), true);
+
+            if ( _.isObject(attributes) && ('cell' in attributes)) {
+                _.each(attributes.cell, function (attribute, name) {
+                    // we are interested only in border attributes, kick out others
+                    if (_.indexOf(['borderBottom', 'borderLeft', 'borderRight', 'borderTop'], name) === -1) {
+                        delete attributes.cell[name];
+                    }
+                });
+
+                if (_.isEmpty(attributes.cell)) {
+                    delete attributes.cell;
+                }
+            }
+
+            // return attributes as a deep clone
+            return _.isObject(attributes) ? attributes : {};
+        }
+
+        return _.isEqual(
+            getCellBorderAttributes(cell1),
+            getCellBorderAttributes(cell2)
+        );
+    }
+
     // static functions =======================================================
 
     /**
@@ -271,12 +319,35 @@ define('io.ox/office/text/table',
             tableGrid = null;
 
         if (tablePosition) {
-            tableGrid = StyleSheets.getExplicitAttributes(tablePosition.node, 'table').tableGrid;
+            tableGrid = Table.getTableGridFromNode(tablePosition.node);
         }
 
         return tableGrid || [];
     };
 
+    /**
+     * Receiving the table grid attribute from a specified table node.
+     *
+     * @param {HTMLElement|jQuery} tableNode
+     *  The DOM element whose table grid attribute will be returned.
+     *  If object is a jQuery collection, uses the first node it
+     *  contains.
+     *
+     * @returns {Number[]}
+     *  Array of grid widths of the table as relative values.
+     */
+    Table.getTableGridFromNode = function (tableNode) {
+
+        var tableGrid = null;
+
+        if (DOM.isTableNode(tableNode)) {
+            tableGrid = AttributeUtils.getExplicitAttributes(tableNode, { family: 'table' }).tableGrid;
+        }
+
+        return tableGrid || [];
+
+    };
+
     /**
      * Calculation the width of a table as the some of the width of all
      * grid positions.
@@ -560,13 +631,26 @@ define('io.ox/office/text/table',
      *  The calculated cell position depends from the insert mode concerning
      *  the grid position. Allowed values are 'behind' and 'before'.
      *
+     * @param {Node} rootNode
+     *  The root node corresponding to the logical position.
+     *
+     * @param {Object} [options]
+     *  A map with additional options. The following options are supported:
+     *  @param {Boolean} [fullPosition=false]
+     *      If set to true, the complete logical position of the cells is returned.
+     *
      * @return {any[]} allInsertPositions
      *  An array, that contains the integer insert positions for each row in the
-     *  correct order.
+     *  correct order. If fullPosition is set to true, all logical cell positions
+     *  of the column are returned.
      */
-    Table.getAllInsertPositions = function (allRows, gridposition, insertmode) {
+    Table.getAllInsertPositions = function (allRows, gridposition, insertmode, rootNode, options) {
 
-        var allInsertPositions = [];
+        var allInsertPositions = [],
+            // whether the complete logical position shall be returned
+            fullPosition = Utils.getBooleanOption(options, 'fullPosition', false),
+            // the logical position of the row
+            rowPosition = null;
 
         allRows.each(function () {
 
@@ -576,6 +660,12 @@ define('io.ox/office/text/table',
                 insertPosition++;
             }
 
+            if (fullPosition) {
+                rowPosition = Position.getOxoPosition(rootNode, this, 0);
+                rowPosition.push(insertPosition);
+                insertPosition = _.clone(rowPosition);
+            }
+
             allInsertPositions.push(insertPosition);
         });
 
@@ -679,7 +769,7 @@ define('io.ox/office/text/table',
     Table.getColumnCount = function (tableNode) {
         // do not use the number of <col> elements in <colgroup>, these elements
         // are inserted/deleted deferred while formatting the table
-        var attributes = StyleSheets.getExplicitAttributes(tableNode);
+        var attributes = AttributeUtils.getExplicitAttributes(tableNode);
         if (_.isObject(attributes) && _.isObject(attributes.table) && _.isArray(attributes.table.tableGrid)) {
             return attributes.table.tableGrid.length;
         }
@@ -687,6 +777,65 @@ define('io.ox/office/text/table',
         return 0;
     };
 
+    /**
+     * Returning an array with all logical positions of the cells inside a column, specified
+     * by a given cell node.
+     *
+     * @param {Node} rootNode
+     *  The root node corresponding to the logical position.
+     *
+     * @param {HTMLElement|jQuery} cellNode
+     *  The table cell element.
+     *
+     *  @returns {[]}
+     *  An array with all logical positions of all cells in the column
+     */
+    Table.getAllCellPositionsInColumn = function (rootNode, cellNode) {
+
+        var // the table node containing the specified cell
+            tableNode = $(cellNode).closest(DOM.TABLE_NODE_SELECTOR),
+            // the row node containing the specified cell
+            rowNode = $(cellNode).closest(DOM.TABLE_ROWNODE_SELECTOR),
+            // all rows in the table
+            allRows = DOM.getTableRows(tableNode),
+            // the logical position of the cell
+            cellPos = Position.getOxoPosition(rootNode, cellNode, 0),
+            // the grid position of the cell inside the table
+            gridPosition = Table.getGridPositionFromCellPosition(rowNode, _.last(cellPos)).start;
+
+        return Table.getAllInsertPositions(allRows, gridPosition, null, rootNode, { fullPosition: true });
+    };
+
+    /**
+     * Returning an array with all cell nodes inside a column, specified
+     * by a given cell node.
+     *
+     * @param {Node} rootNode
+     *  The root node corresponding to the logical position.
+     *
+     * @param {HTMLElement|jQuery} cellNode
+     *  The table cell element.
+     *
+     *  @returns {HTMLTableCellElement[]}
+     *  An array with all html table cells.
+     */
+    Table.getAllCellsNodesInColumn = function (rootNode, cellNode) {
+
+        var // a collector for all table cells
+            allCellNodes = [],
+            // a collector with all logical cell positions of a specified column
+            allCellPositions = Table.getAllCellPositionsInColumn(rootNode, cellNode);
+
+        _.each(allCellPositions, function (onePosition) {
+            var oneCell = Position.getDOMPosition(rootNode, onePosition).node;
+            if (DOM.isTableCellNode(oneCell)) {
+                allCellNodes.push(oneCell);
+            }
+        });
+
+        return allCellNodes;
+    };
+
     /**
      * Forcing the browser to rerender the table. This is required for Firefox
      * that has often a problem with the right table border.
@@ -717,8 +866,7 @@ define('io.ox/office/text/table',
      *  The grid array.
      *
      * @param {Object} [options]
-     *  A map with additional options controlling the update. The
-     *  following options are supported:
+     *  Optional parameters:
      *  @param {Boolean} [options.isGUIInsertOperation=false]
      *      If set to true, this update was triggered by an insertColumn or
      *      insertRow operation from the GUI.
@@ -772,7 +920,7 @@ define('io.ox/office/text/table',
      */
     Table.checkForLateralTableStyle = function (generator, editor, tableStyleId) {
 
-        var tableStyles = editor.getStyleSheets('table');
+        var tableStyles = editor.getStyleCollection('table');
 
         if (_.isString(tableStyleId) && tableStyles.isDirty(tableStyleId)) {
 
@@ -795,8 +943,8 @@ define('io.ox/office/text/table',
     };
 
     /**
-     * Changes the horizontal cell selection in an OX Text table using the 'Shift'-Key and the
-     * cursor-Keys.
+     * Changes the horizontal cell selection in a table using the SHIFT key and
+     * the cursor keys.
      *
      * @param {TextApplication} app
      *  The application instance.
@@ -805,13 +953,14 @@ define('io.ox/office/text/table',
      *  The selection object of the text application
      *
      * @param {Object} browserSelection
-     * A selection object containing an array of DOM ranges representing the current browser selection.
+     *  A selection object containing an array of DOM ranges representing the
+     *  current browser selection.
      *
      * @param {Object} options
-     *  A map with options. The following options are supported:
+     *  Optional parameters:
      *  @param {Boolean} [options.backwards=false]
-     *      If set to true, the current selection will be modified from right to left
-     *      or from bottom to top. The cursor key 'KeyCodes.LEFT_ARROW' was pressed.
+     *      If set to true, the current selection will be modified from right
+     *      to left or from bottom to top (LEFT cursor key).
      */
     Table.changeCellSelectionHorz = function (app, selection, browserSelection, options) {
 
@@ -932,8 +1081,8 @@ define('io.ox/office/text/table',
     };
 
     /**
-     * Changes the vertical cell selection in an OX Text table using the 'Shift'-Key and the
-     * cursor-Keys.
+     * Changes the vertical cell selection in a table using the SHIFT key and
+     * the cursor keys.
      *
      * @param {TextApplication} app
      *  The application instance.
@@ -942,13 +1091,14 @@ define('io.ox/office/text/table',
      *  The selection object of the text application
      *
      * @param {Object} browserSelection
-     * A selection object containing an array of DOM ranges representing the current browser selection.
+     *  A selection object containing an array of DOM ranges representing the
+     *  current browser selection.
      *
      * @param {Object} options
-     *  A map with options. The following options are supported:
+     *  Optional parameters:
      *  @param {Boolean} [options.backwards=false]
-     *      If set to true, the current selection will be modified from right to left
-     *      or from bottom to top. The cursor key 'KeyCodes.UP_ARROW' was pressed.
+     *      If set to true, the current selection will be modified from right
+     *      to left or from bottom to top (UP cursor key).
      */
     Table.changeCellSelectionVert = function (app, selection, browserSelection, options) {
 
@@ -1226,7 +1376,7 @@ define('io.ox/office/text/table',
      *  (Can be a jQuery object for performance reasons.)
      *
      * @param {Object} options
-     *  A map with options. The following options are supported:
+     *  Optional parameters:
      *  @param {Boolean} [options.up=false]
      *      Whether KeyCodes.UP_ARROW was pressed.
      *  @param {Boolean} [options.down=false]
@@ -1350,7 +1500,7 @@ define('io.ox/office/text/table',
      *  (Can be a jQuery object for performance reasons.)
      *
      * @param {Object} options
-     *  A map with options. The following options are supported:
+     *  Optional parameters:
      *  @param {Boolean} [options.up=false]
      *      Whether KeyCodes.UP_ARROW was pressed.
      *  @param {Boolean} [options.down=false]
@@ -1423,8 +1573,11 @@ define('io.ox/office/text/table',
      *
      * @param {Generator} generator
      *  The operations generator
+     *
+     * @param {TextModel} model
+     *  The text model instance.
      */
-    Table.removeTableAttributes = function (tableNode, position, documentStyles, rootNode, generator) {
+    Table.removeTableAttributes = function (tableNode, position, documentStyles, rootNode, generator, model) {
 
         var // the options for the table operation
             tableOperationOptions = {},
@@ -1452,7 +1605,9 @@ define('io.ox/office/text/table',
             var // the options for the cell operation
                 cellOperationOptions = {},
                 // remove all explicit cell attributes
-                cellAttributes = documentStyles.buildNullAttributes('cell');
+                cellAttributes = documentStyles.buildNullAttributes('cell'),
+                // the old element attributes
+                oldAttrs = null;
 
             // exclude some cell attributes
             delete cellAttributes.cell.gridSpan;
@@ -1465,6 +1620,17 @@ define('io.ox/office/text/table',
             cellOperationOptions.start = _.clone(Position.getOxoPosition(rootNode, this, 0));
             cellOperationOptions.attrs = cellAttributes;
 
+            // Defining the oldAttrs, if change tracking is active
+            if (model.getChangeTrack().isActiveChangeTracking()) {
+                // Expanding operation for change tracking with old explicit attributes
+                oldAttrs = model.getChangeTrack().getOldNodeAttributes(this);
+                // adding the old attributes for change tracking
+                if (oldAttrs) {
+                    oldAttrs = _.extend(oldAttrs, model.getChangeTrack().getChangeTrackInfo());
+                    cellOperationOptions.attrs.changes = { 'modified': oldAttrs };
+                }
+            }
+
             // generate the 'setAttributes' operation for each cell, removing cell attributes
             generator.generateOperation(Operations.SET_ATTRIBUTES, cellOperationOptions);
         });
@@ -1481,7 +1647,7 @@ define('io.ox/office/text/table',
      *  A map of attribute value maps (name/value pairs), keyed by
      *  attribute families.
      *
-     * @param {StyleSheets} tableCellStyles
+     * @param {TableCellStyles} tableCellStyles
      *  The available table cell styles
      *
      * @param {Node} rootNode
@@ -1489,8 +1655,11 @@ define('io.ox/office/text/table',
      *
      * @param {Generator} generator
      *  The operations generator
+     *
+     * @param {TextModel} model
+     *  The text model instance.
      */
-    Table.setBorderWidthToVisibleCells = function (tableNode, attributes, tableCellStyles, rootNode, generator) {
+    Table.setBorderWidthToVisibleCells = function (tableNode, attributes, tableCellStyles, rootNode, generator, model) {
 
         DOM.getTableCells(tableNode).each(function () {
 
@@ -1499,7 +1668,8 @@ define('io.ox/office/text/table',
                 cellOperationOptions = {},
                 allCellOperationOptions = {},
                 border = null,
-                createOperation = false;
+                createOperation = false,
+                oldAttrs = null;
 
             operationOptions.start = _.clone(Position.getOxoPosition(rootNode, this, 0));
 
@@ -1515,6 +1685,18 @@ define('io.ox/office/text/table',
             if (createOperation) {
                 allCellOperationOptions.cell = cellOperationOptions;
                 operationOptions.attrs = allCellOperationOptions;
+
+                // Defining the oldAttrs, if change tracking is active
+                if (model.getChangeTrack().isActiveChangeTracking()) {
+                    // Expanding operation for change tracking with old explicit attributes
+                    oldAttrs = model.getChangeTrack().getOldNodeAttributes(this);
+                    // adding the old attributes for change tracking
+                    if (oldAttrs) {
+                        oldAttrs = _.extend(oldAttrs, model.getChangeTrack().getChangeTrackInfo());
+                        operationOptions.attrs.changes = { 'modified': oldAttrs };
+                    }
+                }
+
                 // generate the 'setAttributes' operation for each cell, removing cell attributes
                 generator.generateOperation(Operations.SET_ATTRIBUTES, operationOptions);
             }
@@ -1545,8 +1727,11 @@ define('io.ox/office/text/table',
      *
      * @param {Generator} generator
      *  The operations generator
+     *
+     * @param {TextModel} model
+     *  The text model instance.
      */
-    Table.setBorderModeToCells = function (tableNode, attributes, borderWidth, rootNode, generator) {
+    Table.setBorderModeToCells = function (tableNode, attributes, borderWidth, rootNode, generator, model) {
 
         var tableAttributes = attributes.table;
 
@@ -1568,7 +1753,8 @@ define('io.ox/office/text/table',
                 cellOperationOptions = {},
                 allCellOperationOptions = {},
                 border = null,
-                createOperation = false;
+                createOperation = false,
+                oldAttrs = null;
 
             operationOptions.start = _.clone(Position.getOxoPosition(rootNode, this, 0));
 
@@ -1589,12 +1775,116 @@ define('io.ox/office/text/table',
             if (createOperation) {
                 allCellOperationOptions.cell = cellOperationOptions;
                 operationOptions.attrs = allCellOperationOptions;
+
+                // Defining the oldAttrs, if change tracking is active
+                if (model.getChangeTrack().isActiveChangeTracking()) {
+                    // Expanding operation for change tracking with old explicit attributes
+                    oldAttrs = model.getChangeTrack().getOldNodeAttributes(this);
+                    // adding the old attributes for change tracking
+                    if (oldAttrs) {
+                        oldAttrs = _.extend(oldAttrs, model.getChangeTrack().getChangeTrackInfo());
+                        operationOptions.attrs.changes = { 'modified': oldAttrs };
+                    }
+                }
+
                 // generate the 'setAttributes' operation for each cell
                 generator.generateOperation(Operations.SET_ATTRIBUTES, operationOptions);
             }
         });
     };
 
+    /**
+     * Returns whether merge operation can be applied on two tables.
+     * Maximum number of rows and cells cannot be exceeded.
+     * The maximum number of rows and cells can
+     * be defined in the configuration.
+     *
+     * @param {jQuery|Node} firstTable
+     *  First of two table nodes for which we are checking merge possibility
+     *
+     * @param {jQuery|Node} secondTable
+     *  Second table node
+     *
+     * @returns {Boolean}
+     *  If merge is possible or not
+     */
+    Table.mergingTableHasAllowedSize = function (firstTable, secondTable) {
+
+        var // the number of rows in the first table
+            rowCountFirstTable = Table.getRowCount(firstTable),
+            // the number of rows in the second table
+            rowCountSecondTable = Table.getRowCount(secondTable),
+            // the number of columns in the first table
+            colCountFirstTable = Table.getColumnCount(firstTable),
+            // the number of columns in the second table
+            colCountSecondTable = Table.getColumnCount(secondTable),
+            // summed row count of both tables
+            rowCount = rowCountFirstTable + rowCountSecondTable,
+            // summed col count of both tables
+            colCount = colCountFirstTable + colCountSecondTable;
+
+        // check maximum row count and maximum cell count
+        return _.isNumber(rowCount) && _.isNumber(colCount) &&
+            (rowCount < Config.MAX_TABLE_ROWS) &&
+            ((rowCount + 1) * colCount <= Config.MAX_TABLE_CELLS);
+    };
+
+    /**
+     * Returns whether merge operation can be applied on two tables. This
+     * includes five tests:
+     *
+     * - both tables have a table grid with equal length.
+     * - both tables have same table style
+     * - both tables have same border attributes
+     * - both tables are not exceeded-size tables.
+     * - the new created table is not an exceeded-size table.
+     *
+     * @param {jQuery|Node} firstTable
+     *  First of two table nodes for which we are checking merge possibility
+     *
+     * @param {jQuery|Node} secondTable
+     *  Second of two table nodes for which we are checking merge possibility
+     *
+     * @returns {Boolean}
+     *  If merge of two specified tables is possible or not
+     */
+    Table.mergeableTables = function (firstTable, secondTable) {
+        var
+            firstTableStyleId = AttributeUtils.getElementStyleId(firstTable),
+            secondTableStyleId = AttributeUtils.getElementStyleId(secondTable),
+            // collection of all first level children cells in first passed table
+            tableCellsFirstTable = DOM.getTableCells(firstTable),
+            // collection of all first level children cells in second passed table
+            tableCellsSecondTable = DOM.getTableCells(secondTable),
+            // first cell node of first table
+            firstTableFirstCell = tableCellsFirstTable.first(),
+            // first cell node of second table
+            secondTableFirstCell = tableCellsSecondTable.first(),
+            // last cell node of first table
+            firstTableLastCell = tableCellsFirstTable.last(),
+            // last cell node of second table
+            secondTableLastCell = tableCellsSecondTable.last();
+
+        // comparing table grid of both tables
+        if (Table.getTableGridFromNode(firstTable).length !== Table.getTableGridFromNode(secondTable).length) { return false; }
+
+        // comparing table style of both tables
+        if (firstTableStyleId !== secondTableStyleId) { return false; }
+
+        // comparing borders of both tables - we compare top upper left and bottom right cells of each table
+        if (!hasEqualTableBordersAttributes(firstTableFirstCell, secondTableFirstCell) || !hasEqualTableBordersAttributes(firstTableLastCell, secondTableLastCell)) {
+            return false;
+        }
+
+        // excluding exceeded-size tables
+        if ((DOM.isExceededSizeTableNode(firstTable) || DOM.isExceededSizeTableNode(secondTable))) { return false; }
+
+        // checking that new table is not an exceeded size table
+        if (!Table.mergingTableHasAllowedSize(firstTable, secondTable)) { return false; }
+
+        return true;
+    };
+
     // exports ================================================================
 
     return Table;
