@@ -165,7 +165,7 @@ define('io.ox/office/text/tableResize',
                     // calculating maxLeftShift and maxRightShift
                     lastCell = cellNode[0].nextSibling ? false : true;
                     tablePosition = Position.getOxoPosition(editor.getNode(), tableNode.get(0), 0);
-                    tableNodeAttrs = editor.getStyleSheets('table').getElementAttributes(tableNode).table;
+                    tableNodeAttrs = editor.getStyleCollection('table').getElementAttributes(tableNode).table;
                     oldTableGrid = tableNodeAttrs.tableGrid;
                     oldTableWidth = tableNodeAttrs.width;
                     maxTableWidth = tableNode.parent().width();
@@ -296,7 +296,11 @@ define('io.ox/office/text/tableResize',
                 var generator = editor.createOperationsGenerator(),
                     rowPosition = null,
                     rowHeight = 0,
-                    newRowHeight = 0;
+                    newRowHeight = 0,
+                    // the old attributes
+                    oldAttrs = null,
+                    // the attributes for the setAttributes operation
+                    attrs = null;
 
                 // taking care of scroll node
                 scrollX = scrollNode.scrollLeft() - startScrollX;
@@ -344,14 +348,27 @@ define('io.ox/office/text/tableResize',
                             tableGrid.push(Math.round(gridSum * pixelGrid[i] / newTableWidth));  // only ints
                         }
 
-                        if ((! lastCell) && (editor.getStyleSheets('table').getElementAttributes(tableNode).table.width === 'auto')) {
+                        if ((! lastCell) && (editor.getStyleCollection('table').getElementAttributes(tableNode).table.width === 'auto')) {
                             newTableWidth = 'auto';
                         } else {
                             newTableWidth = Utils.convertLengthToHmm(newTableWidth, 'px');
                         }
 
+                        // the table attributes for the setAttributes operation
+                        attrs = { table: { tableGrid: tableGrid, width: newTableWidth } };
+
+                        if (editor.getChangeTrack().isActiveChangeTracking()) {
+                            // Expanding operation for change tracking with old explicit attributes
+                            oldAttrs = editor.getChangeTrack().getOldNodeAttributes(tableNode);
+                            // adding the old attributes, author and date for change tracking
+                            if (oldAttrs) {
+                                oldAttrs = _.extend(oldAttrs, editor.getChangeTrack().getChangeTrackInfo());
+                                attrs.changes = { 'modified': oldAttrs };
+                            }
+                        }
+
                         generator.generateOperation(Operations.SET_ATTRIBUTES, {
-                            attrs: { table: { tableGrid: tableGrid, width: newTableWidth } },
+                            attrs: attrs,
                             start: tablePosition
                         });
 
@@ -376,8 +393,21 @@ define('io.ox/office/text/tableResize',
                         newRowHeight = Utils.convertLengthToHmm(rowHeight, 'px');
                         rowPosition = Position.getOxoPosition(editor.getNode(), rowNode.get(0), 0);
 
+                        // the row attributes for the setAttributes operation
+                        attrs = { row: { height: newRowHeight } };
+
+                        if (editor.getChangeTrack().isActiveChangeTracking()) {
+                            // Expanding operation for change tracking with old explicit attributes
+                            oldAttrs = editor.getChangeTrack().getOldNodeAttributes(rowNode);
+                            // adding the old attributes, author and date for change tracking
+                            if (oldAttrs) {
+                                oldAttrs = _.extend(oldAttrs, editor.getChangeTrack().getChangeTrackInfo());
+                                attrs.changes = { 'modified': oldAttrs };
+                            }
+                        }
+
                         generator.generateOperation(Operations.SET_ATTRIBUTES, {
-                            attrs: { row: { height: newRowHeight } },
+                            attrs: attrs,
                             start: rowPosition
                         });
 
