@@ -14,15 +14,13 @@
 define('io.ox/office/tk/control/group',
     ['io.ox/office/tk/utils',
      'io.ox/office/tk/keycodes',
+     'io.ox/office/tk/forms',
      'io.ox/office/tk/object/triggerobject'
-    ], function (Utils, KeyCodes, TriggerObject) {
+    ], function (Utils, KeyCodes, Forms, TriggerObject) {
 
     'use strict';
 
-    var // DOM event that will cause a 'group:change' event from a group
-        INTERNAL_TRIGGER_EVENT = 'private:trigger',
-
-        // all existing groups and their focusable DOM nodes (stored globally  to be able
+    var // all existing groups and their focusable DOM nodes (stored globally  to be able
         // to use a single 'change:focus' listener instead of one listener per group instance)
         focusableNodesMap = {},
 
@@ -31,11 +29,6 @@ define('io.ox/office/tk/control/group',
 
     // global focus handling ==================================================
 
-    function getGroupKey(group) {
-        var key = group.getNode().attr('data-key');
-        return key ? ('"' + key + '"') : '<none>';
-    }
-
     function registerFocusGroup(group) {
         focusableNodesMap[group.getUid()] = { group: group, nodes: group.getNode() };
     }
@@ -47,7 +40,7 @@ define('io.ox/office/tk/control/group',
 
     function unregisterFocusGroup(group) {
         delete focusableNodesMap[group.getUid()];
-        focusStack = _(focusStack).without(group.getUid());
+        focusStack = _.without(focusStack, group.getUid());
     }
 
     Utils.on('change:focus', function (event, focusNode) {
@@ -55,8 +48,7 @@ define('io.ox/office/tk/control/group',
         // update CSS focus class, trigger event
         function changeFocusState(group, focused) {
             var type = focused ? 'group:focus' : 'group:blur';
-            group.getNode().toggleClass(Utils.FOCUSED_CLASS, focused);
-            Utils.FocusLogger.log('type="' + type + '" key=' + getGroupKey(group));
+            group.getNode().toggleClass(Forms.FOCUSED_CLASS, focused);
             group.trigger(type);
         }
 
@@ -95,37 +87,51 @@ define('io.ox/office/tk/control/group',
      * functionality to the inserted controls.
      *
      * Instances of this class trigger the following events:
-     * - 'group:change': If the control has been activated in a special way
-     *      depending on the type of the control group. The event handler
-     *      receives the new value of the activated control.
-     * - 'group:cancel': When the focus needs to be returned to the application
-     *      (e.g. when the ESCAPE key is pressed, or when a click on a
-     *      drop-down button closes the opened drop-down menu).
-     * - 'group:show': After the control has been shown or hidden. The event
-     *      handler receives the new visibility state.
-     * - 'group:enable': After the control has been enabled or disabled. The
-     *      event handler receives the new state.
-     * - 'group:layout': After child nodes have been inserted into this group
-     *      using the method Group.addChildNodes().
-     * - 'group:focus': After the group has been focused, by initially focusing
-     *      any of its focusable child nodes. As long as the focus remains
+     * - 'group:change'
+     *      If the control has been activated in a special way depending on the
+     *      type of the control group. The event handler receives the new value
+     *      of the activated control.
+     * - 'group:cancel'
+     *      When the focus needs to be returned to the application (e.g. when
+     *      the ESCAPE key is pressed, or when a click on a drop-down button
+     *      closes the opened drop-down menu).
+     * - 'group:beforeshow'
+     *      Before the control will be shown (while it is currently hidden).
+     * - 'group:show'
+     *      After the control has been shown (and it was hidden before).
+     * - 'group:beforehide'
+     *      Before the control will be hidden (while it is currently visible).
+     * - 'group:hide'
+     *      After the control has been hidden (and it was visible before).
+     * - 'group:enable'
+     *      After the control has been enabled or disabled. The event handler
+     *      receives the new state.
+     * - 'group:resize'
+     *      After the size of the root node has been changed. The event handler
+     *      receives the new width and height of the root node, in pixels.
+     * - 'group:focus'
+     *      After the group has been focused, by initially focusing any of its
+     *      focusable child nodes. As long as the focus remains inside the
+     *      group (even if the focus moves to another DOM node in the group),
+     *      no further 'group:focus' event will be triggered.
+     * - 'group:blur'
+     *      After the group has lost the browser focus, after focusing any
+     *      other DOM node outside the group. As long as the focus remains
      *      inside the group (even if the focus moves to another DOM node in
-     *      the group), no further 'group:focus' event will be triggered.
-     * - 'group:blur': After the group has lost the browser focus, after
-     *      focusing any other DOM node outside the group. As long as the focus
-     *      remains inside the group (even if the focus moves to another DOM
-     *      node in the group), the 'group:blur' event will not be triggered.
+     *      the group), the 'group:blur' event will not be triggered.
      *
      * @constructor
      *
      * @extends TriggerObject
      *
      * @param {Object} [initOptions]
-     *  A map of options to control the properties of the new group. The
-     *  following options are supported:
+     *  Optional parameters:
      *  @param {String} [initOptions.classes]
      *      Additional CSS classes that will be set at the root DOM node of
      *      this instance.
+     *  @param {String} [initOptions.role]
+     *      An ARIA role attribute that will be set at the root DOM node of
+     *      this instance.
      */
     function Group(initOptions) {
 
@@ -135,6 +141,9 @@ define('io.ox/office/tk/control/group',
             // create the group container element
             groupNode = $('<div>').addClass('group'),
 
+            // the last cached size of the root node, used to detect layout changes
+            nodeSize = null,
+
             // the current value of this control group (must be undefined initially, NOT null)
             groupValue,
 
@@ -142,7 +151,10 @@ define('io.ox/office/tk/control/group',
             groupOptions = null,
 
             // update handler functions
-            updateHandlers = [];
+            updateHandlers = [],
+
+            // special options for a drop-down-version of this group
+            dropDownVersion = Utils.getOption(initOptions, 'dropDownVersion');
 
         // base constructor ---------------------------------------------------
 
@@ -151,29 +163,145 @@ define('io.ox/office/tk/control/group',
         // private methods ----------------------------------------------------
 
         /**
-         * Keyboard handler for the entire group.
+         * Reads the current outer size of the root node of this group, and
+         * triggers a 'group:resize' event if the group is visible and its size
+         * has changed. Does nothing if the group is currently hidden.
          */
-        function keyHandler(event) {
-            if (event.keyCode === KeyCodes.ESCAPE) {
-                if (event.type === 'keydown') {
-                    $.cancelTracking();
-                    if (!event.isDefaultPrevented()) { self.triggerCancel(); }
-                }
-                return false;
+        function updateNodeSize() {
+
+            // do nothing if the group is not visible, or is inside a hidden container node
+            if (!Forms.isDeeplyVisibleNode(groupNode)) { return; }
+
+            // read current node size, notify listeners if size has changed
+            var newSize = { width: groupNode.outerWidth(), height: groupNode.outerHeight() };
+            if (!_.isEqual(nodeSize, newSize)) {
+                nodeSize = newSize;
+                self.trigger('group:resize', nodeSize.width, nodeSize.height);
             }
         }
 
         /**
-         * Triggers a 'group:cancel' event and suppresses the default action of
-         * the passed event, if this group is disabled.
+         * Returns an object with focus options for the passed source event,
+         * intended to be inserted into a 'group:change' or 'group:cancel'
+         * event triggered by this control group.
+         *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {jQuery.Event} [options.sourceEvent]
+         *      The source event that has caused the 'group:change' or
+         *      'group:cancel' event. According to the type of this source
+         *      event, the returned options object will contain specific focus
+         *      options.
+         *  @param {Boolean} [options.preserveFocus]
+         *      If specified, overrides the 'preserveFocus' option that would
+         *      be created from the passed source event.
+         *
+         * @returns {Object}
+         *  An options object with the following properties:
+         *  - {HTMLElement} sourceNode
+         *      The target node of the passed event. Defaults to the own root
+         *      node, if no source event has been passed.
+         *  - {String} sourceType
+         *      An abstract type identifier for the source event: 'click' for
+         *      regular mouse click events; 'keyboard' for keyboard events
+         *      (including click events originating from SPACE or RETURN keys),
+         *      or 'custom' for all other events.
+         *  - {Boolean} preserveFocus
+         *      Whether the passed source event leads to preserving the current
+         *      browser focus after performing the action associated to this
+         *      control group (usually, after SPACE or TAB keys).
          */
-        function disabledHandler(event) {
-            if (!self.isEnabled()) {
-                event.preventDefault();
-                self.triggerCancel();
+        function getEventOptions(options) {
+
+            var // resulting properties for the event object
+                eventOptions = {
+                    sourceNode: groupNode[0],
+                    sourceType: 'custom',
+                    preserveFocus: false
+                },
+                // the source event object
+                sourceEvent = Utils.getOption(options, 'sourceEvent');
+
+            // special handling for specific events
+            if (_.isObject(sourceEvent)) {
+
+                // use target of the passed source event as source node
+                eventOptions.sourceNode = sourceEvent.currentTarget;
+
+                // handle specific event types
+                switch (sourceEvent.type) {
+                case 'click':
+                    // Forms.setButtonKeyHandler() triggers 'click' events with 'keyCode' field
+                    eventOptions.sourceType = ('keyCode' in sourceEvent) ? 'keyboard' : 'click';
+                    // click events from keyboard: keep focus on button after SPACE or TAB key (see Forms.setButtonKeyHandler() method)
+                    eventOptions.preserveFocus = (sourceEvent.keyCode === KeyCodes.SPACE) || (sourceEvent.keyCode === KeyCodes.TAB);
+                    break;
+                case 'keydown':
+                case 'keypress':
+                case 'keyup':
+                    eventOptions.sourceType = 'keyboard';
+                    break;
+                }
             }
+
+            // override the 'preserveFocus' option
+            eventOptions.preserveFocus = Utils.getBooleanOption(options, 'preserveFocus', eventOptions.preserveFocus);
+
+            return eventOptions;
         }
 
+        // protected methods --------------------------------------------------
+
+        /**
+         * Registers the passed update handler function. These handlers will be
+         * called from the method Group.setValue() in order of their
+         * registration.
+         *
+         * @attention
+         *  Intended to be used by the internal implementations of sub class
+         *  constructors. DO NOT CALL from external code!
+         *
+         * @param {Function} updateHandler
+         *  The update handler function. Will be called in the context of this
+         *  group. Receives the following parameters:
+         *  (1) {Any} value
+         *      The value passed to the method Group.setValue().
+         *  (2) {Object} [options]
+         *      The options map passed to the method Group.setValue(). Will be
+         *      undefined if no options have been passed.
+         *  (3) {Any} oldValue
+         *      The old value of the group.
+         *
+         * @returns {Group}
+         *  A reference to this group.
+         */
+        this.registerUpdateHandler = function (updateHandler) {
+            updateHandlers.push(updateHandler);
+            return this;
+        };
+
+        /**
+         * Registers one or more DOM nodes located outside the root node of
+         * this group instance, which contain other control nodes that will be
+         * included into the focus handling and generation of 'group:focus' and
+         * 'group:blur' events. While the browser focus is located inside the
+         * passed nodes, this group will not trigger a 'group:blur' event.
+         *
+         * @attention
+         *  Intended to be used by the internal implementations of sub class
+         *  constructors. DO NOT CALL from external code!
+         *
+         * @param {HTMLElement|jQuery} nodes
+         *  The DOM node(s) to be added to the internal focus handling.
+         *
+         * @returns {Group}
+         *  A reference to this instance.
+         */
+        this.registerFocusableNodes = function (nodes) {
+            registerFocusableNodes(this, nodes);
+            return this;
+        };
+
         // methods ------------------------------------------------------------
 
         /**
@@ -194,176 +322,16 @@ define('io.ox/office/tk/control/group',
         };
 
         /**
-         * Registers the passed update handler function. These handlers will be
-         * called from the method Group.setValue() in order of their
-         * registration.
-         *
-         * @param {Function} updateHandler
-         *  The update handler function. Will be called in the context of this
-         *  group. Receives the following parameters:
-         *  (1) {Any} value
-         *      The value passed to the method Group.setValue().
-         *  (2) {Object|Undefined} options
-         *      The options map passed to the method Group.setValue().
-         *  (3) {Any} oldValue
-         *      The old value of the group.
-         *
-         * @returns {Group}
-         *  A reference to this group.
-         */
-        this.registerUpdateHandler = function (updateHandler) {
-            updateHandlers.push(updateHandler);
-            return this;
-        };
-
-        /**
-         * Registers an event handler for specific DOM control nodes, that will
-         * trigger a 'group:change' event for this group instance. The value of
-         * the 'group:change' event will be determined by the passed value
-         * resolver function.
-         *
-         * @param {String} [type]
-         *  The type of the event handler that will be bound to the selected
-         *  DOM node(s), e.g. 'click' or 'change'. If omitted, the
-         *  'group:change' event can only be triggered manually with the method
-         *  'Group.triggerChange()'.
-         *
-         * @param {Object} [options]
-         *  A map of options to control the behavior of this method. The
-         *  following options are supported:
-         *  @param {HTMLElement|jQuery} [options.source]
-         *      The DOM node that catches the specified event. May be a single
-         *      DOM control element, or any other container element that
-         *      catches the events bubbled up from several descending controls.
-         *      In this case, the set of source controls can be filtered with a
-         *      jQuery selector (option 'selector', see below). If omitted,
-         *      defaults to the root node of this group.
-         *  @param {String} [options.selector]
-         *      If specified, selects the descendant elements of the source
-         *      node, which are actually triggering the events. If omitted, the
-         *      event handler will be bound directly to the source node.
-         *  @param {Function} [options.eventFilter]
-         *      A predicate function that can specify to ignore specific DOM
-         *      events. Receives the jQuery event object as first parameter,
-         *      and must return a Boolean specifying whether to process the
-         *      event. If omitted, all DOM events will be processed.
-         *  @param {Function} [options.valueResolver]
-         *      A callback function that will return the value of this group
-         *      instance associated with the source node and/or event type.
-         *      Will be called in the context of this group. Receives the
-         *      control node that has triggered the event, as jQuery object.
-         *      Must return the current value of the control (e.g. the Boolean
-         *      state of a toggle button, the value of a list item, or the
-         *      current text in a text input field). May return null to
-         *      indicate that a 'group:cancel' event should be triggered
-         *      instead of a 'group:change' event. If omitted, defaults to the
-         *      static method Utils.getControlValue() that extracts a constant
-         *      value from the event's source element. The value will be passed
-         *      in the 'group:change' event triggered by this instance.
-         *  @param {Function} [options.optionsResolver]
-         *      A callback function that will return the additional options for
-         *      the value of this group instance associated with the source
-         *      node and/or event type. Will be called in the context of this
-         *      group. Receives the control node that has triggered the event,
-         *      as jQuery object. Must return the current value options of the
-         *      control. The value options will be passed in the 'group:change'
-         *      and 'group:cancel' events triggered by this instance.
-         *
-         * @returns {Group}
-         *  A reference to this group.
-         */
-        this.registerChangeHandler = function (type, options) {
-
-            var // the node to attach the event handler to
-                sourceNode = Utils.getOption(options, 'source', groupNode),
-                // the jQuery selector for ancestor nodes
-                selector = Utils.getStringOption(options, 'selector', ''),
-                // a predicate function to filter DOM events
-                eventFilter = Utils.getFunctionOption(options, 'eventFilter'),
-                // the resolver function for the group value to be passed in the 'group:change' event
-                valueResolver = Utils.getFunctionOption(options, 'valueResolver', Utils.getControlValue),
-                // the resolver function for the additional options to be passed in the group events
-                optionsResolver = Utils.getFunctionOption(options, 'optionsResolver', $.noop);
-
-            // handler for all received DOM events
-            function eventHandler(event, data) {
-
-                var value = null, valueOptions = null;
-
-                // ignore events if specified by the event filter callback function
-                if (_.isFunction(eventFilter) && (eventFilter.call(self, event) !== true)) {
-                    return;
-                }
-
-                // get value and options from callback functions
-                value = self.isEnabled() ? valueResolver.call(self, $(this)) : null;
-                valueOptions = self.isEnabled() ? optionsResolver.call(self, $(this)) : undefined;
-
-                // special handling for specific events
-                switch (event.type) {
-                case 'click':
-                    // click events from keyboard: keep focus on button after SPACE or TAB key (see Utils.setButtonKeyHandler() method)
-                    if ((event.keyCode === KeyCodes.SPACE) || (event.keyCode === KeyCodes.TAB)) {
-                        valueOptions = Utils.extendOptions(valueOptions, { preserveFocus: true });
-                    }
-                    break;
-                case INTERNAL_TRIGGER_EVENT:
-                    // internal trigger event may pass own options as separate parameter
-                    valueOptions = Utils.extendOptions(valueOptions, data);
-                    break;
-                }
-
-                if (_.isNull(value)) {
-                    self.triggerCancel(valueOptions);
-                } else {
-                    self.setValue(value, valueOptions);
-                    valueOptions = Utils.extendOptions(valueOptions, { source: Utils.getDomNode(sourceNode) });
-                    self.trigger('group:change', value, valueOptions);
-                }
-            }
-
-            // the resulting event type names
-            type = _.isString(type) ? (type + ' ' + INTERNAL_TRIGGER_EVENT) : INTERNAL_TRIGGER_EVENT;
-
-            // attach event handler to the node
-            if (selector.length > 0) {
-                $(sourceNode).on(type, selector, eventHandler);
-            } else {
-                $(sourceNode).on(type, eventHandler);
-            }
-
-            return this;
-        };
-
-        /**
-         * Registers one or more DOM nodes located outside the root node of
-         * this group instance, which contain other control nodes that will be
-         * included into the focus handling and generation of 'group:focus' and
-         * 'group:blur' events. While the browser focus is located inside the
-         * passed nodes, this group will not trigger a 'group:blur' event.
-         *
-         * @param {HTMLElement|jQuery} nodes
-         *  The DOM node(s) to be added to the internal focus handling.
-         *
-         * @returns {Group}
-         *  A reference to this instance.
-         */
-        this.registerFocusableNodes = function (nodes) {
-            registerFocusableNodes(this, nodes);
-            return this;
-        };
-
-        /**
-         * Triggers a 'group:layout' event that causes parent objects to
-         * refresh their internal layout according to the new contents and size
-         * of this group.
+         * Triggers a 'group:resize' event to all listeners, if the size of the
+         * group has been changed due to changed contents.
          *
          * @returns {Group}
          *  A reference to this instance.
          */
         this.layout = function () {
-            Utils.LayoutLogger.log('type="group:layout" key=' + getGroupKey(this));
-            return this.trigger('group:layout');
+            // notify listeners if the group size has changed
+            updateNodeSize();
+            return this;
         };
 
         /**
@@ -387,51 +355,27 @@ define('io.ox/office/tk/control/group',
          * 'group:layout' event notifying all listeners about the changed
          * layout of this group.
          *
-         * @param {jQuery} nodes
-         *  The nodes to be inserted into this group, as jQuery object.
+         * @param {HTMLElement|jQuery|String} [...]
+         *  The DOM nodes to be inserted into this group. Can be an arbitrary
+         *  number of HTML elements, jQuery collections, and/or HTML mark-up as
+         *  string.
          *
          * @returns {Group}
          *  A reference to this group.
          */
-        this.addChildNodes = function (nodes) {
-            groupNode.append(nodes);
+        this.addChildNodes = function () {
+            groupNode.append.apply(groupNode, arguments);
             return this.layout();
         };
 
-        /**
-         * Inserts the passed control into this group, and marks it to be
-         * included into keyboard focus navigation.
-         *
-         * @param {jQuery} control
-         *  The control to be inserted into this group, as jQuery object.
-         *
-         * @returns {Group}
-         *  A reference to this group.
-         */
-        this.addFocusableControl = function (control) {
-            return this.addChildNodes(control.addClass(Utils.FOCUSABLE_CLASS));
-        };
-
-        /**
-         * Returns whether this group contains any focusable controls.
-         */
-        this.hasFocusableControls = function () {
-            return this.getFocusableControls().length > 0;
-        };
-
-        /**
-         * Returns all control nodes contained in this group that need to be
-         * included into keyboard focus navigation.
-         */
-        this.getFocusableControls = function () {
-            return groupNode.find(Utils.FOCUSABLE_SELECTOR + Utils.REALLY_VISIBLE_SELECTOR);
-        };
-
         /**
          * Returns whether this group is currently in focused state, either
          * because the browser focus is located inside the group itself, or it
          * is located in any of the additional DOM nodes that have been
          * registered with the method 'Group.registerFocusableNodes()'.
+         *
+         * @returns {Boolean}
+         *  Whether the group currently owns the browser focus.
          */
         this.hasFocus = function () {
             return _.indexOf(focusStack, this.getUid()) >= 0;
@@ -445,29 +389,16 @@ define('io.ox/office/tk/control/group',
          *  A reference to this group.
          */
         this.grabFocus = function () {
-            if (!this.hasFocus()) {
-                this.getFocusableControls().first().focus();
-            }
+            if (!this.hasFocus()) { Forms.grabFocus(groupNode); }
             return this;
         };
 
         /**
-         * Returns whether this control group is configured to be visible (via
-         * the methods Group.show(), Group.hide(), or Group.toggle()). Does not
-         * check the effective visibility depending on the visibility of all
-         * parent DOM nodes (see method Group.isReallyVisible() to do that).
+         * Returns whether this control group is configured to be visible via
+         * the methods Group.show(), Group.hide(), or Group.toggle().
          */
         this.isVisible = function () {
-            return !groupNode.hasClass(Utils.HIDDEN_CLASS);
-        };
-
-        /**
-         * Returns whether this control group is effectively visible (it must
-         * not be hidden by itself, it must be inside the DOM tree, and all its
-         * parent nodes must be visible too).
-         */
-        this.isReallyVisible = function () {
-            return groupNode.is(Utils.REALLY_VISIBLE_SELECTOR);
+            return Forms.isVisibleNode(groupNode);
         };
 
         /**
@@ -506,9 +437,10 @@ define('io.ox/office/tk/control/group',
                 visible = (state === true) || ((state !== false) && this.isVisible());
 
             if (this.isVisible() !== visible) {
-                groupNode.toggleClass(Utils.HIDDEN_CLASS, !visible);
-                Utils.LayoutLogger.log('type="group:show" key=' + getGroupKey(this) + ' state=' + visible);
-                this.trigger('group:show', visible);
+                this.trigger(visible ? 'group:beforeshow' : 'group:beforehide');
+                Forms.showNodes(groupNode, visible);
+                this.trigger(visible ? 'group:show' : 'group:hide');
+                updateNodeSize();
             }
             return this;
         };
@@ -517,7 +449,7 @@ define('io.ox/office/tk/control/group',
          * Returns whether this control group is enabled.
          */
         this.isEnabled = function () {
-            return groupNode.is(Utils.ENABLED_SELECTOR);
+            return Forms.isEnabledNode(groupNode);
         };
 
         /**
@@ -535,10 +467,16 @@ define('io.ox/office/tk/control/group',
             var // whether to enable the group instance
                 enabled = _.isUndefined(state) || (state === true);
 
+            // returns all buttons inside the group
+            // we need to apply 'aria-disabled' not only on the group, but also on the buttons inside the group
+            function getButtonNodes () {
+                return groupNode.find(Forms.BUTTON_SELECTOR);
+            }
+
             // enable/disable the entire group node with all its descendants
             if (this.isEnabled() !== enabled) {
-                groupNode.toggleClass(Utils.DISABLED_CLASS, !enabled).attr('aria-disabled', !enabled);
-                Utils.LayoutLogger.log('type="group:enable" key=' + getGroupKey(this) + ' state=' + enabled);
+                Forms.enableNodes(groupNode, enabled);
+                if (state) { getButtonNodes().removeAttr('aria-disabled'); } else { getButtonNodes().attr('aria-disabled', true); }
                 this.trigger('group:enable', enabled);
             }
             return this;
@@ -562,7 +500,7 @@ define('io.ox/office/tk/control/group',
          *  The new value to be displayed in the controls of this group.
          *
          * @param {Object} [options]
-         *  A map with options that will be passed as second parameter to all
+         *  Optional parameters that will be passed as second parameter to all
          *  registered update handlers.
          *
          * @returns {Group}
@@ -570,14 +508,12 @@ define('io.ox/office/tk/control/group',
          */
         this.setValue = function (value, options) {
             var oldValue = groupValue;
-            if (!_.isUndefined(value)) {
-                groupValue = value;
-                groupOptions = options;
-                _(updateHandlers).each(function (updateHandler) {
-                    updateHandler.call(this, value, options, oldValue);
-                }, this);
-            }
-            return this;
+            groupValue = value;
+            groupOptions = options;
+            _.each(updateHandlers, function (updateHandler) {
+                updateHandler.call(this, value, options, oldValue);
+            }, this);
+            return this.layout();
         };
 
         /**
@@ -587,60 +523,92 @@ define('io.ox/office/tk/control/group',
          */
         this.refresh = _.debounce(function () {
             // prevent calling method on destroyed object in debounced callback
-            if (!self.destroyed) {
+            if (self && !self.destroyed) {
                 self.setValue(groupValue, groupOptions);
             }
         });
 
         /**
-         * Triggers a special event at the passed DOM control node contained in
-         * this Group instance. The group will catch the event, and will
-         * trigger a 'group:change' event by itself with the value associated
-         * to the DOM control node.
-         *
-         * @param {HTMLElement|jQuery}
-         *  A DOM control node of this group that must have been configured as
-         *  source of change events with the Group.registerChangeHandler()
-         *  method. If this object is a jQuery collection, uses the first DOM
-         *  node it contains.
-         *
-         * @param {Object} [options]
-         *  A map with additional options that will be passed to the
-         *  'group:change' event listeners of this class.
+         * Triggers a 'group:change' event at this control group instance, and
+         * inserts special focus options into the event object. If the control
+         * group is disabled, a 'group'cancel' event will be triggered instead.
          *
          * @returns {Group}
          *  A reference to this group.
          */
-        this.triggerChange = function (control, options) {
-            $(control).first().trigger(INTERNAL_TRIGGER_EVENT, options);
+        this.triggerChange = function (value, options) {
+            if (this.isEnabled() && !_.isNull(value)) {
+                this.setValue(value);
+                this.trigger(new $.Event('group:change', getEventOptions(options)), value);
+            } else {
+                this.triggerCancel(options);
+            }
             return this;
         };
 
         /**
-         * Triggers a 'group:cancel' event at this Group instance.
+         * Triggers a 'group:cancel' event at this Group instance, and inserts
+         * special focus options into the event object, according to the passed
+         * source event.
          *
-         * @param {Object} [options]
-         *  A map with additional options that will be passed to the
-         *  'group:cancel' event listeners of this class.
+         * @param {jQuery.Event} [sourceEvent]
+         *  The source event that causes the 'group:cancel' event. According to
+         *  the type of this source event, the cancel event will contain
+         *  specific focus options.
          *
          * @returns {Group}
          *  A reference to this group.
          */
         this.triggerCancel = function (options) {
-            return this.trigger('group:cancel', options);
+            this.trigger(new $.Event('group:cancel', getEventOptions(options)));
+            return this;
         };
 
+        /**
+		 * Needs to be overwritten by controls, if there were some special
+         * small version options.
+         */
+        this.activateSmallVersion = function(){ };
+        this.deactivateSmallVersion = function(){ };
+
+        /**
+         * Decides, whether the group should be shown in the current state
+         *
+         * @param {Boolean} [isShrinked]
+         *  With this option and the current shrinked-state, it will be
+         *  decided whether the dropdown-version should be visible or not.
+         */
+        this.dropDownVisible = function(isShrinked){
+            if (_.isObject(dropDownVersion) && _.has(dropDownVersion, 'visible')){
+                if ((isShrinked === true && dropDownVersion.visible === true) || (isShrinked === false && dropDownVersion.visible === false)) {
+                    // ANZEIGEN
+                    this.show();
+                } else if((isShrinked === true && dropDownVersion.visible === false) || (isShrinked === false && dropDownVersion.visible === true)) {
+                    // AUSBLENDEN
+                    this.hide();
+                }
+            }
+        };
+
+
         // initialization -----------------------------------------------------
 
+        // show/hide initially this group
+        this.dropDownVisible(false);
+
         // register for global focus handling
         registerFocusGroup(this);
 
         // formatting and tool tip
         groupNode.addClass(Utils.getStringOption(initOptions, 'classes', ''));
 
-        // add event handlers
-        self.listenTo(groupNode, 'keydown keypress keyup', keyHandler);
-        self.listenTo(groupNode, 'mousedown touchstart dragover drop contextmenu', disabledHandler);
+        // ARIA role
+        groupNode.attr('role', Utils.getStringOption(initOptions, 'role', null, true));
+
+        // log focus and layout events
+        Utils.FocusLogger.logEvent(this, 'group:focus group:blur');
+        Utils.LayoutLogger.logEvent(this, 'group:resize', 'width', 'height');
+        Utils.LayoutLogger.logEvent(this, 'group:beforeshow group:show group:beforehide group:hide');
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
