@@ -14,8 +14,9 @@
 define('io.ox/office/tk/control/spinfield',
     ['io.ox/office/tk/utils',
      'io.ox/office/tk/keycodes',
+     'io.ox/office/tk/forms',
      'io.ox/office/tk/control/textfield'
-    ], function (Utils, KeyCodes, TextField) {
+    ], function (Utils, KeyCodes, Forms, TextField) {
 
     'use strict';
 
@@ -30,14 +31,13 @@ define('io.ox/office/tk/control/spinfield',
      * @extends TextField
      *
      * @param {Object} [initOptions]
-     *  A map of options to control the properties of the control. Supports all
-     *  options of the TextField base class. If the option 'options.validator'
-     *  exists, it MUST be an instance of TextField.NumberValidator (or a sub
-     *  class). Alternatively, the options supported by the constructor of the
-     *  class TextField.NumberValidator can be passed. In this case, a new
-     *  instance of TextField.NumberValidator will be created automatically
-     *  based on these options. Additionally, the following options are
-     *  supported:
+     *  Optional parameters. Supports all options of the TextField base class.
+     *  If the option 'validator' exists, it MUST be an instance of
+     *  TextField.NumberValidator (or a sub class). Alternatively, the options
+     *  supported by the constructor of the class TextField.NumberValidator can
+     *  be passed. In this case, a new instance of TextField.NumberValidator
+     *  will be created automatically based on these options. Additionally, the
+     *  following options are supported:
      *  @param {Number} [initOptions.smallStep=1]
      *      The distance of small steps that will be used to increase or
      *      decrease the current value of the spin field when using the spin
@@ -99,7 +99,7 @@ define('io.ox/office/tk/control/spinfield',
         function createSpinButton(increase) {
 
             var // create the spin button element
-                spinButton = Utils.createButton().addClass(increase ? 'spin-up' : 'spin-down');
+                spinButton = $(Forms.createButtonMarkup({ focusable: false })).addClass(increase ? 'spin-up' : 'spin-down');
 
             // enables or disables node tracking according to own enabled state
             function initializeTracking() {
@@ -110,8 +110,8 @@ define('io.ox/office/tk/control/spinfield',
                 }
             }
 
-            // add the caret icon
-            spinButton.append($('<span>').addClass('caret-span').append(Utils.createIcon(increase ? 'fa-caret-up' : 'fa-caret-down')));
+            // add the caret symbol
+            spinButton.append(Forms.createCaretMarkup(increase ? 'up' : 'down'));
 
             // enable/disable node tracking according to own enabled state
             self.on('group:enable', initializeTracking);
@@ -120,13 +120,13 @@ define('io.ox/office/tk/control/spinfield',
             // register tracking event listeners to change the current value
             spinButton.on({
                 'tracking:start tracking:repeat': function () { changeValue(increase ? smallStep : -smallStep); },
-                'tracking:end tracking:cancel': function () { _.defer(function () { self.getTextFieldNode().focus(); }); }
+                'tracking:end tracking:cancel': function () { _.defer(function () { self.getInputNode().focus(); }); }
             });
 
             // enable/disable the button according to the current value
             self.registerUpdateHandler(function (value) {
                 var enabled = increase ? (value < validator.getMax()) : (value > validator.getMin());
-                spinButton.toggleClass(Utils.DISABLED_CLASS, !enabled);
+                Forms.enableNodes(spinButton, enabled);
             });
 
             return spinButton;
@@ -135,27 +135,33 @@ define('io.ox/office/tk/control/spinfield',
         /**
          * Handles keyboard events in the text field.
          */
-        function textFieldKeyHandler(event) {
+        function inputKeyDownHandler(event) {
 
-            function applyStep(step) {
-                if ((event.type === 'keydown') && KeyCodes.matchModifierKeys(event)) {
-                    changeValue(step);
-                    return false;
-                }
-            }
+            if (KeyCodes.hasModifierKeys(event)) { return; }
 
             switch (event.keyCode) {
             case KeyCodes.UP_ARROW:
-                return applyStep(smallStep);
+                changeValue(smallStep);
+                return false;
             case KeyCodes.DOWN_ARROW:
-                return applyStep(-smallStep);
+                changeValue(-smallStep);
+                return false;
             case KeyCodes.PAGE_UP:
-                return applyStep(largeStep);
+                changeValue(largeStep);
+                return false;
             case KeyCodes.PAGE_DOWN:
-                return applyStep(-largeStep);
+                changeValue(-largeStep);
+                return false;
             }
         }
 
+        /**
+         * Handles updates of the input text field
+         */
+        function updateHandler(value) {
+            this.getInputNode().attr({ 'aria-valuenow': value, 'aria-valuetext': this.getInputNode().val() });
+        }
+
         // initialization -----------------------------------------------------
 
         // add special marker class used to adjust formatting, add spin buttons
@@ -163,8 +169,14 @@ define('io.ox/office/tk/control/spinfield',
             $('<div>').addClass('spin-wrapper').append(createSpinButton(true), createSpinButton(false))
         );
 
+        // add ARIA attributes (note: role 'spinbutton' would be correct but doesn't work with VoiceOver)
+        this.getInputNode().attr({ role: 'slider', 'aria-valuemin': validator.getMin(), 'aria-valuemax': validator.getMax() });
+
         // register event handlers
-        this.getTextFieldNode().on('keydown keypress keyup', textFieldKeyHandler);
+        this.getInputNode().on('keydown', inputKeyDownHandler);
+
+        // register input field update handler
+        this.registerUpdateHandler(updateHandler);
 
     } // class SpinField
 
