@@ -14,8 +14,11 @@
 define('io.ox/office/tk/control/textfield',
     ['io.ox/office/tk/utils',
      'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/control/group'
-    ], function (Utils, KeyCodes, Group) {
+     'io.ox/office/tk/forms',
+     'io.ox/office/tk/object/baseobject',
+     'io.ox/office/tk/control/group',
+     'io.ox/office/tk/control/widthmixin'
+    ], function (Utils, KeyCodes, Forms, BaseObject, Group, WidthMixin) {
 
     'use strict';
 
@@ -30,17 +33,18 @@ define('io.ox/office/tk/control/textfield',
      * @constructor
      *
      * @extends Group
+     * @extends WidthMixin
      *
      * @param {Object} [initOptions]
-     *  A map of options to control the properties of the text field. Supports
-     *  all options of the Group base class, and all generic formatting options
-     *  of input fields (see method Utils.createTextField() for details).
-     *  Additionally, the following options are supported:
+     *  Optional parameters. Supports all options of the Group base class
+     *  constructor, all formatting options for input elements supported by the
+     *  method Forms.createInputMarkup(), and all options of the mix-in class
+     *  WidthMixin. Additionally, the following options are supported:
      *  @param {Boolean} [initOptions.select=false]
      *      If set to true, the entire text will be selected after the text
-     *      field has been clicked. Note that the text will always be selected
-     *      independent from this option, if the text field gets focus via
-     *      keyboard navigation.
+     *      field has been clicked, or if the method TextField.grabFocus() has
+     *      been called. The text will always be selected (independently from
+     *      this option), if the text field gets focus via keyboard navigation.
      *  @param {TextField.Validator} [initOptions.validator]
      *      A text validator that will be used to convert the values from the
      *      control values (passed to the Group.setValue() method) to the text
@@ -49,17 +53,20 @@ define('io.ox/office/tk/control/textfield',
      *      control value (returned by the Group.getValue() method). If no
      *      validator has been specified, a default validator will be used that
      *      does not perform any conversions.
+     *  @param {Object} [initOptions.smallerVersion]
+     *      If specified, the Textfield will known, how it should act, if there
+     *      will be not enough free place to show normal view
      */
     function TextField(initOptions) {
 
         var // self reference
             self = this,
 
-            // create the input field control (width has to be set at the wrapper node)
-            fieldNode = Utils.createTextField(Utils.extendOptions(initOptions, { width: undefined })).addClass(Utils.FOCUSABLE_CLASS),
+            // the <input> element
+            inputNode = $(Forms.createInputMarkup(initOptions)),
 
             // create the wrapper node for the input field (IE renders input fields with padding wrong)
-            wrapperNode = $('<div>').addClass('input-wrapper').width(Utils.getOption(initOptions, 'width', '')).append(fieldNode),
+            wrapperNode = $('<div>').addClass('input-wrapper').append(inputNode),
 
             // whether to select the entire text on click
             select = Utils.getBooleanOption(initOptions, 'select', false),
@@ -68,71 +75,69 @@ define('io.ox/office/tk/control/textfield',
             validator = Utils.getObjectOption(initOptions, 'validator', defaultValidator),
 
             // saved state of the text field, used to restore while validating
-            fieldState = null,
+            inputState = null,
 
             // initial value of text field when focused, needed for ESCAPE key handling
             initialText = null,
 
             // whether the group is focused (needed to detect whether to restore selection after focus change inside the group)
-            groupFocused = false;
+            groupFocused = false,
+
+            // saves the (old) styles in case of switching to small version
+            savedStyles = {},
+
+            // smaller version of group (to fit in smaller resolutions)
+            smallerVersion = Utils.getOption(initOptions, 'smallerVersion', false);
 
         // base constructor ---------------------------------------------------
 
         Group.call(this, initOptions);
+        WidthMixin.call(this, wrapperNode, initOptions);
 
         // private methods ----------------------------------------------------
 
         /**
          * Saves the current value and selection of the text field node.
          */
-        function saveFieldState() {
-            fieldState = Utils.getTextFieldSelection(fieldNode);
-            fieldState.value = fieldNode.val();
+        function saveInputState() {
+            inputState = Forms.getInputSelection(inputNode);
+            inputState.text = inputNode.val();
         }
 
         /**
          * Restores the current value and selection of the text field.
          */
-        function restoreFieldState() {
-            fieldNode.val(fieldState.value);
-            Utils.setTextFieldSelection(fieldNode, fieldState.start, fieldState.end);
+        function restoreInputState() {
+            inputNode.val(inputState.text);
+            Forms.setInputSelection(inputNode, inputState.start, inputState.end);
         }
 
         /**
          * The update handler for this text field.
          */
         function updateHandler(value) {
-            value = (_.isUndefined(value) || _.isNull(value)) ? '' : validator.valueToText(value);
-            fieldNode.val(value);
-            self.getNode().attr('data-value', value);
-            saveFieldState();
-        }
-
-        /**
-         * Returns the current value associated to the text field.
-         */
-        function resolveValueHandler() {
-            return self.getFieldValue();
+            value = _.isNull(value) ? '' : validator.valueToText(value);
+            // do not access the input element if the value does not change (browsers will destroy current selection)
+            if (inputNode.val() !== value) {
+                inputNode.val(value);
+                self.getNode().attr('data-value', value);
+                saveInputState();
+            }
         }
 
         /**
          * Handles all focus events of the text field.
          */
         function focusHandler(event) {
+
             switch (event.type) {
             case 'group:focus':
                 // save current value, if this group receives initial focus
-                initialText = fieldState.value = fieldNode.val();
+                initialText = inputState.text = inputNode.val();
                 // wait until the <input> element gets focused (see below)
                 groupFocused = false;
                 break;
-            case 'focus':
-                // IE9 does not trigger 'input' events when deleting characters
-                // or pasting text, use a timer interval as a workaround
-                if (Utils.IE9) {
-                    fieldNode.data('update-timer', window.setInterval(fieldInputHandler, 250));
-                }
-                break;
+
             case 'beforedeactivate':
                 // Bug 27912: IE keeps text selection visible when losing focus (text
                 // is still highlighted although the control is not focused anymore).
@@ -141,28 +146,24 @@ define('io.ox/office/tk/control/textfield',
                 // in 'blur' events, e.g. Chrome gets confused when changing browser
                 // text selection while it currently changes the focus node.
                 if (_.browser.IE) {
-                    saveFieldState();
-                    Utils.setTextFieldSelection(fieldNode, 0);
+                    saveInputState();
+                    Forms.setInputSelection(inputNode, 0);
                 }
                 break;
+
             case 'blur':
                 // save field state to be able to restore selection (but not in IE where the
                 // selection has been cleared, see the 'beforedeactivate' event handler above)
                 if (!_.browser.IE) {
-                    saveFieldState();
-                }
-                // IE9 does not trigger 'input' events when deleting characters or pasting
-                // text, remove the timer interval that has been started as a workaround
-                if (Utils.IE9) {
-                    window.clearInterval(fieldNode.data('update-timer'));
-                    fieldNode.data('update-timer', null);
+                    saveInputState();
                 }
                 break;
+
             case 'group:blur':
                 // Bug 27175: always commit value when losing focus
-                if (_.isString(initialText) && (initialText !== fieldNode.val())) {
+                if (_.isString(initialText) && (initialText !== inputNode.val())) {
                     // pass preserveFocus option to not interfere with current focus handling
-                    self.triggerChange(fieldNode, { preserveFocus: true });
+                    self.triggerChange(self.getFieldValue(), { sourceEvent: event, preserveFocus: true });
                 }
                 initialText = null;
                 groupFocused = false;
@@ -179,11 +180,11 @@ define('io.ox/office/tk/control/textfield',
         function globalFocusHandler(event, focusNode, blurNode, keyEvent) {
 
             // the <input> element has received browser focus
-            if (fieldNode.is(focusNode)) {
+            if (inputNode.is(focusNode)) {
 
                 // <input> element focused again (e.g. from drop-down menu): restore selection
                 if (groupFocused) {
-                    restoreFieldState();
+                    restoreInputState();
                     return;
                 }
 
@@ -192,27 +193,27 @@ define('io.ox/office/tk/control/textfield',
                 groupFocused = true;
 
                 // remove previous 'mouseup' handler (see below)
-                fieldNode.off('mouseup');
+                inputNode.off('mouseup');
 
                 // always select entire text when reaching the field with keyboard
                 if (_.isObject(keyEvent)) {
-                    fieldNode.select();
+                    inputNode.select();
                 } else if (select) {
                     // select entire text on mouse click when specified via option
-                    fieldNode.select();
+                    inputNode.select();
                     // suppress next 'mouseup' event to prevent that the browser changes the selection
-                    fieldNode.one('mouseup', false);
+                    inputNode.one('mouseup', false);
                 }
 
                 // save current contents and selection
-                saveFieldState();
+                saveInputState();
                 return;
             }
 
             // the <input> element has lost browser focus
-            if (fieldNode.is(blurNode)) {
+            if (inputNode.is(blurNode)) {
                 // always remove the dummy 'mouseup' handler when losing focus (see above)
-                fieldNode.off('mouseup');
+                inputNode.off('mouseup');
             }
         }
 
@@ -221,9 +222,9 @@ define('io.ox/office/tk/control/textfield',
          */
         function enableHandler(event, state) {
             if (state) {
-                fieldNode.removeAttr('readonly');
+                inputNode.removeAttr('readonly');
             } else {
-                fieldNode.attr('readonly', 'readonly');
+                inputNode.attr('readonly', 'readonly');
             }
         }
 
@@ -241,20 +242,25 @@ define('io.ox/office/tk/control/textfield',
         /**
          * Handles keyboard events.
          */
-        function fieldKeyHandler(event) {
+        function keyHandler(event) {
 
             switch (event.keyCode) {
             case KeyCodes.ENTER:
                 if (event.type === 'keyup') {
-                    self.triggerChange(fieldNode);
+                    self.triggerChange(self.getFieldValue(), { sourceEvent: event });
                 }
                 return false;
             case KeyCodes.ESCAPE:
                 if (event.type === 'keydown') {
-                    fieldNode.val(initialText);
-                    fieldInputHandler();
+                    inputNode.val(initialText);
+                    inputHandler();
                 }
                 break;
+            case KeyCodes.HOME:
+            case KeyCodes.END:
+                // do not let these key codes bubble up, just do the default action
+                event.stopPropagation();
+                break;
             }
         }
 
@@ -262,37 +268,68 @@ define('io.ox/office/tk/control/textfield',
          * Handles input events triggered when the text changes while typing.
          * Performs live validation with the current validator.
          */
-        function fieldInputHandler() {
+        function inputHandler() {
 
-            var // current value of the text field
-                value = fieldNode.val(),
-                // original field state
-                oldFieldState = null,
+            var // current text in the input element
+                currText = inputNode.val(),
+                // previous text contents of the input element
+                prevText = inputState.text,
                 // result of the text field validation
                 result = null;
 
             // do not perform validation if nothing has changed
-            if (value === fieldState.value) { return; }
+            if (currText === prevText) { return; }
 
-            // save old field state, validate the current field text
-            oldFieldState = _.clone(fieldState);
-            result = validator.validate(value);
+            // validate the current field text
+            result = validator.validate(currText);
 
             // update the text field according to the validation result
             if (result === false) {
-                // false: restore the old field state stored in validationFieldState
-                restoreFieldState();
-            } else if (_.isString(result) && (result !== value)) {
+                // false: restore the old field state
+                restoreInputState();
+            } else if (_.isString(result) && (result !== currText)) {
                 // insert the validation result and restore the old selection
-                fieldState.value = result;
-                restoreFieldState();
+                inputState.text = result;
+                restoreInputState();
             }
 
-            // trigger 'textfield:validated' event to all listeners, pass old field state
-            self.trigger('textfield:validated', oldFieldState);
+            // trigger 'group:validate' event to listeners, pass old text
+            self.trigger('group:validate', prevText);
 
             // update current state of the text field
-            saveFieldState();
+            saveInputState();
+        }
+
+        /**
+         * Activates/Deactivates a smaller version of the textfield
+         *
+         * @param {Boolean} value
+         *  decides wether the textfield should be small or wide
+         */
+        function smallTextfield(value){
+            // show the small version
+            if (value === true) {
+                // set new css if exists
+                if (_.isObject(smallerVersion.css)) {
+                    if (_.isEmpty(savedStyles.css)) { savedStyles.css = wrapperNode.attr('style'); }
+                    wrapperNode.css(smallerVersion.css);
+                }
+
+                // hide complete label
+                if (smallerVersion.hide) { self.getNode().addClass('hidden'); }
+
+            // show the default
+            } else {
+                // re-set the old (saved) styles
+                if (_.has(savedStyles, 'css') && !_.isNull(savedStyles.css)) {
+                    wrapperNode.removeAttr('style');
+                    wrapperNode.attr('style',savedStyles.css);
+                    savedStyles.css = null;
+                }
+
+                // hide complete label
+                if (smallerVersion.hide) { self.getNode().removeClass('hidden'); }
+            }
         }
 
         // methods ------------------------------------------------------------
@@ -300,57 +337,109 @@ define('io.ox/office/tk/control/textfield',
         /**
          * Returns the text control element, as jQuery object.
          */
-        this.getTextFieldNode = function () {
-            return fieldNode;
+        this.getInputNode = function () {
+            return inputNode;
         };
 
         /**
-         * Returns the value represented by the text in the text control.
+         * Returns the effective value represented by the current text in the
+         * text input control.
+         *
+         * @returns {Any}
+         *  The value represented by the current text, as returned by the
+         *  validator of this control group.
          */
         this.getFieldValue = function () {
-            return validator.textToValue(fieldNode.val());
+            return validator.textToValue(inputNode.val());
         };
 
         /**
-         * Sets the value to be displayed in the text control.
+         * Sets the effective value to be displayed in the text input control.
+         *
+         * @param {Any} value
+         *  The value to be displayed. The input control will display the text
+         *  for this value as returned by the validator of this control group.
+         *
+         * @returns {TextField}
+         *  A reference to this instance.
          */
         this.setFieldValue = function (value) {
-            fieldNode.val(validator.valueToText(value));
+            inputNode.val(validator.valueToText(value));
+            return this;
         };
 
         /**
-         * Converts the passed value to a text using the current validator.
+         * Is applying a new validator 
+         */
+        this.setValidator = function(value) {
+            validator = value;
+            return this;
+        };
+        
+        /**
+         * Converts the passed value to a text using the validator of this
+         * control group.
          */
         this.valueToText = function (value) {
             return validator.valueToText(value);
         };
 
         /**
-         * Converts the passed text to a value using the current validator.
+         * Converts the passed text to a value using the validator of this
+         * control group.
          */
         this.textToValue = function (text) {
             return validator.textToValue(text);
         };
 
+        /**
+         * Sets the browser focus into the text input element.
+         *
+         * @returns {TextField}
+         *  A reference to this instance.
+         */
+        this.grabFocus = function () {
+            inputNode.focus();
+            return this;
+        };
+
+        /**
+         * Overwrites the base-methods (group.js) to
+         * activate/deactivate the small version of the button
+         */
+        this.activateSmallVersion = function(){
+            if (_.isObject(smallerVersion)) {
+                smallTextfield(true);
+            }
+        };
+        this.deactivateSmallVersion = function(){
+            if (_.isObject(smallerVersion)) {
+                smallTextfield(false);
+            }
+        };
+
         // initialization -----------------------------------------------------
 
         // add special marker class used to adjust formatting
         this.getNode().addClass('text-field');
 
-        // insert the text field into this group, and register event handlers
-        this.addChildNodes(wrapperNode)
-            .registerUpdateHandler(updateHandler)
-            .registerChangeHandler(null, { source: fieldNode, valueResolver: resolveValueHandler })
-            .on({
-                'group:focus group:blur': focusHandler,
-                'group:enable': enableHandler,
-                'group:change': changeHandler
-            });
+        // insert the text field into this group
+        this.addChildNodes(wrapperNode);
 
-        fieldNode.on({
+        // update text contents in the input field
+        this.registerUpdateHandler(updateHandler);
+
+        // register event handlers
+        this.on({
+            'group:focus group:blur': focusHandler,
+            'group:enable': enableHandler,
+            'group:change': changeHandler
+        });
+
+        inputNode.on({
             'focus beforedeactivate blur': focusHandler,
-            'keydown keypress keyup': fieldKeyHandler,
-            'input': fieldInputHandler
+            'keydown keypress keyup': keyHandler,
+            input: inputHandler
         });
 
         // listen to global 'change:focus' events, to be able to distinguish between mouse and keyboard focus
@@ -359,21 +448,18 @@ define('io.ox/office/tk/control/textfield',
         // forward clicks on the wrapper node (left/right padding beside the input field)
         wrapperNode.on('click', function (event) {
             if (self.isEnabled() && wrapperNode.is(event.target)) {
-                fieldNode.focus();
+                inputNode.focus();
             }
         });
 
+        // initialize field state
+        saveInputState();
+
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            fieldNode.remove();
-            wrapperNode.remove();
-
-            self = fieldNode = wrapperNode = initialText = validator = null;
+            initOptions = self = inputNode = wrapperNode = validator = null;
         });
 
-        // initialize field state
-        saveFieldState();
-
     } // class TextField
 
     // class TextField.Validator ==============================================
@@ -384,8 +470,10 @@ define('io.ox/office/tk/control/textfield',
      * default implementation of all methods that do not restrict editing.
      *
      * @constructor
+     *
+     * @extends BaseObject
      */
-    TextField.Validator = _.makeExtendable(function () {
+    TextField.Validator = BaseObject.extend({ constructor: function () {
 
         // methods ------------------------------------------------------------
 
@@ -445,7 +533,7 @@ define('io.ox/office/tk/control/textfield',
          */
         this.validate = $.noop;
 
-    }); // class TextField.Validator
+    }}); // class TextField.Validator
 
     // global instance of the default validator
     defaultValidator = new TextField.Validator();
@@ -461,8 +549,7 @@ define('io.ox/office/tk/control/textfield',
      * @extends TextField.Validator
      *
      * @param {Object} [initOptions]
-     *  A map of options to control the properties of the text validator. The
-     *  following options are supported:
+     *  Optional parameters:
      *  @param {Number} [initOptions.maxLength=0x7FFFFFFF]
      *      The maximum number of characters to be inserted into the text
      *      field. All attempts to insert more characters will be rejected.
@@ -499,8 +586,7 @@ define('io.ox/office/tk/control/textfield',
      * @extends TextField.Validator
      *
      * @param {Object} [initOptions]
-     *  A map of options to control the properties of the validator. The
-     *  following options are supported:
+     *  Optional parameters:
      *  @param {Number} [initOptions.min]
      *      The minimum value allowed to enter. If omitted, defaults to
      *      -Number.MAX_VALUE.
