@@ -32,16 +32,13 @@ define('io.ox/office/tk/control/unitfield',
      * @extends SpinField.NumberValidator
      *
      * @param {Object} [initOptions]
-     *  A map of options to control the properties of the validator. Supports
-     *  all options that are supported by the base class NumberValidator.
+     *  Optional parameters. Supports all options that are supported by the
+     *  base class NumberValidator.
      */
     var UnitValidator = SpinField.NumberValidator.extend({ constructor: function (initOptions) {
 
         var // the regular expression for the unit field
-            regex = null,
-
-            // the supported units
-            supportedUnits = ['px', 'pc', 'pt', 'in', 'cm', 'mm'];
+            regex = null;
 
         // base constructor ---------------------------------------------------
 
@@ -87,7 +84,7 @@ define('io.ox/office/tk/control/unitfield',
 
             value = (value.length > 0) ? parseFloat(value.replace(/,/g, '.')) : null;
             if (_.isNumber(value) && _.isFinite(value)) {
-                unit = _(supportedUnits).contains(unit) ? unit : 'mm';
+                unit = /^(px|pc|pt|in|cm|mm)$/.test(unit) ? unit : 'mm';
                 return this.restrictValue(Utils.convertLengthToHmm(value, unit));
             }
 
@@ -124,10 +121,10 @@ define('io.ox/office/tk/control/unitfield',
      * @extends SpinField
      *
      * @param {Object} [initOptions]
-     *  A map of options to control the properties of the unit field. Supports
-     *  all options of the SpinField base class and the UnitValidator class (a
-     *  sub class of TextField.NumberValidator), except the option
-     *  'options.validator' which will be set to an instance of UnitValidator.
+     *  Optional parameters. Supports all options of the SpinField base class
+     *  and the UnitValidator class (a sub class of TextField.NumberValidator),
+     *  except the option 'validator' which will be set to an instance of
+     *  UnitValidator.
      */
     function UnitField(initOptions) {
 
