@@ -14,15 +14,32 @@
 define('io.ox/office/tk/dialogs',
     ['io.ox/core/tk/dialogs',
      'io.ox/office/tk/utils',
+     'io.ox/office/tk/forms',
+     'io.ox/office/tk/keycodes',
      'gettext!io.ox/office/main'
-    ], function (CoreDialogs, Utils, gt) {
+    ], function (CoreDialogs, Utils, Forms, KeyCodes, gt) {
 
     'use strict';
 
+    var // the number of visible modal dialogs
+        modalDialogs = 0;
+
     // static class Dialogs ===================================================
 
     var Dialogs = {};
 
+    // methods ----------------------------------------------------------------
+
+    /**
+     * Returns whether a modal dialog is currently open.
+     *
+     * @returns {Boolean}
+     *  Whether a modal dialog is currently open.
+     */
+    Dialogs.isModalDialogOpen = function () {
+        return modalDialogs > 0;
+    };
+
     // class ModalDialog ======================================================
 
     /**
@@ -32,19 +49,21 @@ define('io.ox/office/tk/dialogs',
      *
      * @extends CoreDialogs.ModalDialog
      *
-     * @param {Object} [options]
-     *  Additional options that control the appearance and behavior of the
+     * @param {Object} [initOptions]
+     *  Optional parameters that control the appearance and behavior of the
      *  dialog. The following options are supported:
      *  @param {String} [initOptions.title]
      *      The title of the dialog window that will be shown in a larger font.
      *  @param {Number} [initOptions.width=400]
      *      The width of the dialog, in pixels.
-     *  @param {String} [options.okLabel=gt('OK')]
+     *  @param {String} [initOptions.okLabel=gt('OK')]
      *      The label of the primary button that triggers the default action of
-     *      the dialog.
-     *  @param {String} [options.cancelLabel=gt('Cancel')]
-     *      The label of the 'Cancel' button that closes the dialog without
-     *      performing any action.
+     *      the dialog. If omitted, the button will be created with the
+     *      translated default label 'OK'.
+     *  @param {String} [initOptions.cancelLabel=gt('Cancel')]
+     *      The label of the cancel button that closes the dialog without
+     *      performing any action. If omitted, the button will be created with
+     *      the translated default label 'Cancel'.
      *  @param {Boolean} [initOptions.async=false]
      *      Whether the dialog will be opened in asynchronous mode.
      */
@@ -87,6 +106,13 @@ define('io.ox/office/tk/dialogs',
             return !triggerAction;
         }
 
+        function keyDownHandler(event) {
+            if (event.keyCode === KeyCodes.ENTER) {
+                inputEnterHandler();
+                return false;
+            }
+        }
+
         // methods ------------------------------------------------------------
 
         /**
@@ -206,8 +232,9 @@ define('io.ox/office/tk/dialogs',
                         def[(action === 'cancel') ? 'reject' : 'resolve'](action, self);
                     }
                 }
+                modalDialogs += 1;
                 self.on('triggered', actionHandler);
-                def.always(function () { self.off('triggered', actionHandler); self = null; });
+                def.always(function () { modalDialogs -= 1; self.off('triggered', actionHandler); self = null; });
                 // show the dialog, ignore the promise it returns
                 show.apply(self, arguments);
                 // return the own promise that will always be resolved/rejected
@@ -237,6 +264,9 @@ define('io.ox/office/tk/dialogs',
             this.addButton('cancel', cancelLabel, 'cancel', { tabIndex: 1 });
         }
 
+        // listen to key events for additional keyboard handling
+        this.getBody().on('keydown', keyDownHandler);
+
     }}); // class ModalDialog
 
     // class ModalTabDialog ===================================================
@@ -273,8 +303,8 @@ define('io.ox/office/tk/dialogs',
      *
      * @extends ModalDialog
      *
-     * @param {Object} [options]
-     *  A map with options that control the appearance and behavior of the
+     * @param {Object} [initOptions]
+     *  Optional parameters that control the appearance and behavior of the
      *  dialog. Supports all options also supported by the base class
      *  ModalDialog.
      */
@@ -290,7 +320,10 @@ define('io.ox/office/tk/dialogs',
             tabPanelNode = $('<div>').addClass('tab-content').css('margin-top', '13px'),
 
             // maps tab panels by user identifiers
-            tabPanels = {},
+            tabPanelsMap = {},
+
+            // all tab panel identifiers in insertion order
+            tabPanelIds = [],
 
             // the identifier of the visible tab panel
             activeId = null;
@@ -299,6 +332,45 @@ define('io.ox/office/tk/dialogs',
 
         Dialogs.ModalDialog.call(this, initOptions);
 
+        // private methods ----------------------------------------------------
+
+        /**
+         * Sets the browser focus into the button element of the active tab
+         * panel.
+         */
+        function focusActiveTabButton() {
+           tabBarNode.find('li.active a').first().focus();
+        }
+
+        /**
+         * Handles keyboard events for browser focus navigation.
+         */
+        function keyDownHandler(event) {
+
+            var // index of the active tab panel
+                activeIndex = _.indexOf(tabPanelIds, activeId),
+                // identifier of the new tab panel to be activated
+        		newTabId = null;
+
+        	switch (event.keyCode) {
+        	case KeyCodes.RIGHT_ARROW:
+    			newTabId = tabPanelIds[activeIndex + 1];
+        		break;
+        	case KeyCodes.LEFT_ARROW:
+        	    newTabId = tabPanelIds[activeIndex - 1];
+        		break;
+        	case KeyCodes.SPACE:
+        	    newTabId = $(event.target).attr('data-user-id');
+        		break;
+        	}
+
+        	if (newTabId) {
+        		self.activateTabPanel(newTabId);
+        		focusActiveTabButton();
+        		return false;
+        	}
+        }
+
         // methods ------------------------------------------------------------
 
         /**
@@ -310,8 +382,7 @@ define('io.ox/office/tk/dialogs',
          *  created for this dialog must be different.
          *
          * @param {Object} [options]
-         *  A map with options for the new tab panel. The following options are
-         *  supported:
+         *  Optional parameters:
          *  @param {String} [options.label]
          *      The text label to be shown in the tab button activating the new
          *      tab panel. If omitted, the tab button will not contain a text
@@ -332,38 +403,19 @@ define('io.ox/office/tk/dialogs',
                 // the unique DOM identifier for the tab panel container
                 panelId = _.uniqueId('panel'),
 
-                // the text label for the tab button
-                label = Utils.getStringOption(options, 'label', ''),
-                // the icon for the tab button
-                icon = Utils.getStringOption(options, 'icon', ''),
-
                 // the button element in the tab bar
-                tabButton = $('<a>', {
-                    href: '#' + panelId,
-                    id: buttonId,
-                    tabindex: 0,
-                    role: 'tab',
-                    'data-toggle': 'tab',
-                    'data-user-id': tabId,
-                    'aria-selected': false,
-                    'aria-controls': panelId
-                }),
+                tabButton = $('<a href="#' + panelId + '" id="' + buttonId + '" tabindex="1" role="tab" data-toggle="tab" data-user-id="' + tabId + '" aria-selected="false" aria-controls="' + panelId + '">'),
                 // the container element for the tab panel
-                tabPanel = $('<div>', {
-                    id: panelId,
-                    role: 'tabpanel',
-                    'data-user-id': tabId,
-                    'aria-labelledby': buttonId
-                }).addClass('tab-pane row-fluid');
+                tabPanel = $('<div class="tab-pane row-fluid" id="' + panelId + '" role="tabpanel" data-user-id="' + tabId + '" aria-labelledby="' + buttonId + '">');
 
             // set button caption
-            if (icon.length > 0) { tabButton.append(Utils.createIcon(icon)); }
-            if (label.length > 0) { tabButton.append($('<span>').text(label)); }
+            tabButton.append(Forms.createCaptionMarkup(options));
 
             // insert the tab button and tab panel
             tabBarNode.append($('<li>').append(tabButton));
             tabPanelNode.append(tabPanel);
-            tabPanels[tabId] = tabPanel;
+            tabPanelsMap[tabId] = tabPanel;
+            tabPanelIds.push(tabId);
 
             // return the tab panel for further initialization
             return tabPanel;
@@ -382,7 +434,7 @@ define('io.ox/office/tk/dialogs',
          *  identifier is invalid, returns an empty jQuery collection.
          */
         this.getTabPanel = function (tabId) {
-            return (tabId in tabPanels) ? tabPanels[tabId] : $();
+            return (tabId in tabPanelsMap) ? tabPanelsMap[tabId] : $();
         };
 
         /**
@@ -416,13 +468,16 @@ define('io.ox/office/tk/dialogs',
         // insert the nodes into the dialog body
         this.append(tabBarNode, tabPanelNode);
 
+        // initially focus the active tab button
+        this.on('show', focusActiveTabButton);
+
         // trigger 'tab:hide' event before the active tab panel will be hidden
         tabBarNode.on('show.bs.tab', function (event) {
             var tabButton = $(event.relatedTarget).closest('a'),
                 tabId = tabButton.attr('data-user-id');
             if (tabButton.length > 0) {
                 tabButton.attr('aria-selected', false);
-                self.trigger('tab:hide', tabId, tabPanels[tabId], tabButton);
+                self.trigger('tab:hide', tabId, tabPanelsMap[tabId], tabButton);
             }
         });
 
@@ -431,14 +486,17 @@ define('io.ox/office/tk/dialogs',
             var tabButton = $(event.target).closest('a');
             activeId = tabButton.attr('data-user-id');
             tabButton.attr('aria-selected', true);
-            self.trigger('tab:show', activeId, tabPanels[activeId], tabButton);
+            self.trigger('tab:show', activeId, tabPanelsMap[activeId], tabButton);
         });
 
+        // handle keyboard events for focus navigation
+        tabBarNode.on('keydown', keyDownHandler);
+
         // destroy members
         this.on('close', function () {
             tabBarNode.remove();
             tabPanelNode.remove();
-            self = tabBarNode = tabPanelNode = tabPanels = null;
+            self = tabBarNode = tabPanelNode = tabPanelsMap = null;
         });
 
     }}); // class ModalTabDialog
@@ -456,7 +514,7 @@ define('io.ox/office/tk/dialogs',
      *  The message text shown in the dialog body element.
      *
      * @param {Object} [initOptions]
-     *  A map with options that control the appearance and behavior of the
+     *  Optional parameters that control the appearance and behavior of the
      *  dialog. Supports all options also supported by the base class
      *  ModalDialog. The default value of the option 'okLabel' will be changed
      *  to the translated string 'Yes', and the default value of the option
@@ -487,27 +545,46 @@ define('io.ox/office/tk/dialogs',
      * @extends ModalDialog
      *
      * @param {Object} [initOptions]
-     *  A map with options that control the appearance and behavior of the
+     *  Optional parameters that control the appearance and behavior of the
      *  dialog. Supports all options also supported by the base class
      *  ModalDialog. Additionally, the following options are supported:
-     *  @param {String} [options.value='']
+     *  @param {String} [initOptions.value='']
      *      The initial value of the text field.
-     *  @param {String} [options.placeholder='']
+     *  @param {String} [initOptions.placeholder='']
      *      The place-holder text that will be shown in the empty text field.
+     *  @param {Boolean} [initOptions.allowEmpty=false]
+     *      If set to true, empty text will be considered valid. Otherwise, the
+     *      primary button will be disabled as long as the text input field is
+     *      empty.
      */
     Dialogs.ModalInputDialog = Dialogs.ModalDialog.extend({ constructor: function (initOptions) {
 
-        var inputNode = $('<input>', {
-                type: 'text',
+        var // self reference
+            self = this,
+
+            // the text field control
+            inputNode = $('<input type="text" class="form-control" tabindex="1">').attr({
                 placeholder: Utils.getStringOption(initOptions, 'placeholder', ''),
-                value: Utils.getStringOption(initOptions, 'value', ''),
-                tabindex: 1
-            }).addClass('form-control');
+                value: Utils.getStringOption(initOptions, 'value', '')
+            }),
+
+            // whether to allow empty text
+            allowEmpty = Utils.getBooleanOption(initOptions, 'allowEmpty', false);
 
         // base constructor ---------------------------------------------------
 
         Dialogs.ModalDialog.call(this, initOptions);
 
+        // private functions --------------------------------------------------
+
+        /**
+         * Enables or disables the primary button according to the contents of
+         * the text field.
+         */
+        function updateOkButton() {
+            self.enableOkButton(allowEmpty || (inputNode.val().length > 0));
+        }
+
         // methods ------------------------------------------------------------
 
         /**
@@ -534,24 +611,19 @@ define('io.ox/office/tk/dialogs',
             return this;
         };
 
-        /**
-         * Provides the input node as jQuery.
-         *
-         * @returns {jQuery}
-         *  The input node used inside the dialog.
-         */
-        this.getInputNode = function () {
-            return inputNode;
-        };
-
         // initialization -----------------------------------------------------
 
         // add the text field
         this.append(inputNode);
 
+        // enable/disable the primary button
+        inputNode.on('input', updateOkButton);
+        updateOkButton();
+
         // focus and select the text in the input field
-        this.on('show', function () {
-            inputNode.focus().select();
+        this.on({
+            show: function () { inputNode.focus().select(); },
+            close: function () { inputNode.remove(); }
         });
 
     }}); // class ModalInputDialog
