@@ -44,44 +44,7 @@ define('io.ox/office/tk/object/baseobject', ['io.ox/office/tk/utils'], function
             // all destructor callbacks
             destructors = null;
 
-        // methods ------------------------------------------------------------
-
-        /**
-         * Returns the globally unique identifier for this instance.
-         *
-         * @returns {String}
-         *  The globally unique identifier for this instance.
-         */
-        this.getUid = function () {
-            return uid;
-        };
-
-        /**
-         * Registers an event listener at the specified event source object. On
-         * destruction of this instance, all event listeners registered with
-         * this method will be removed from the source object.
-         *
-         * @param {Object} source
-         *  The event source object. Can be any object that provides a method
-         *  'on()' to register an event listener (for example, jQuery objects,
-         *  or any object extended with the core Events mix-in class).
-         *
-         * @param {String} type
-         *  The type of the event to listen to. Can be a space-separated list
-         *  of event type names.
-         *
-         * @param {Function} listener
-         *  The event listener that will be invoked for the events triggered by
-         *  the event source object.
-         *
-         * @returns {BaseObject}
-         *  A reference to this instance.
-         */
-        this.listenTo = function (source, type, listener) {
-            (listeners || (listeners = [])).push({ source: source, type: type, listener: listener });
-            source.on(type, listener);
-            return this;
-        };
+        // protected methods --------------------------------------------------
 
         /**
          * Registers a callback function used when cloning this instance with
@@ -146,6 +109,109 @@ define('io.ox/office/tk/object/baseobject', ['io.ox/office/tk/utils'], function
             return this;
         };
 
+        /**
+         * Registers a destructor callback function that will be invoked when
+         * the method BaseObject.destroy() has been called.
+         *
+         * @attention
+         *  Intended to be used by the internal implementations of sub classes.
+         *  DO NOT CALL from external code!
+         *
+         * @param {Function} destructor
+         *  A destructor callback function. The BaseObject.destroy() method
+         *  will invoke all registered destructor callbacks in reverse order of
+         *  registration. The callback will be invoked in the context of this
+         *  instance.
+         *
+         * @returns {BaseObject}
+         *  A reference to this instance.
+         */
+        this.registerDestructor = function (destructor) {
+            (destructors || (destructors = [])).unshift(destructor);
+            return this;
+        };
+
+        // methods ------------------------------------------------------------
+
+        /**
+         * Returns the globally unique identifier for this instance.
+         *
+         * @returns {String}
+         *  The globally unique identifier for this instance.
+         */
+        this.getUid = function () {
+            return uid;
+        };
+
+        /**
+         * Registers an event listener at the specified event source object, or
+         * a callback function for a specific event at a promise object. On
+         * destruction of this instance, all event listeners registered with
+         * this method will be removed from the source object, and the promise
+         * callbacks will not be invoked anymore (it is not possible to remove
+         * callbacks from promises explicitly).
+         *
+         * @param {jQuery|Events|jQuery.Promise} source
+         *  The event source object. Can be any object that provides a method
+         *  'on()' to register an event listener (for example, jQuery objects,
+         *  or any object extended with the core Events mix-in class), or the
+         *  promise of a Deferred object.
+         *
+         * @param {String} type
+         *  The type of the event to listen to. In case the passed source
+         *  object is an event source, this value can be a space-separated list
+         *  of event type names. In case the passed source object is a promise,
+         *  this type MUST be the name of one of the callback registration
+         *  methods provided by the promise (e.g. 'done', 'fail', 'always',
+         *  'progress', etc.).
+         *
+         * @param {Function} listener
+         *  The event listener that will be invoked for the events triggered by
+         *  the event source object, or the callback function that will be
+         *  invoked when the promise will be resolved, rejected, or notified.
+         *
+         * @returns {BaseObject}
+         *  A reference to this instance.
+         */
+        this.listenTo = function (source, type, listener) {
+
+            if (_.isFunction(source.on)) {
+                // register an event listener (prefer 'on' over 'promise' as jQuery collections provide a 'promise' method too)
+                (listeners || (listeners = [])).push({ source: source, type: type, listener: listener });
+                source.on(type, listener);
+            } else if (_.isFunction(source.promise)) {
+                // register promise callbacks
+                source.promise()[type](_.bind(function () {
+                    if (!this.destroyed) { return listener.apply(this, arguments); }
+                }, this));
+            }
+
+            return this;
+        };
+
+        /**
+         * Removes all event handlers that have been registered for the passed
+         * event source object using the method BaseObject.listenTo().
+         *
+         * @param {jQuery|Events} source
+         *  The event source object. Note that it is not possible to detach
+         *  callback handlers from promises.
+         *
+         * @returns {BaseObject}
+         *  A reference to this instance.
+         */
+        this.stopListeningTo = function (source) {
+
+            // detach all event listeners for the passed object
+            _.each(listeners, function (data) {
+                if (data.source === source) {
+                    source.off(data.type, data.listener);
+                }
+            });
+
+            return this;
+        };
+
         /**
          * Returns a clone of this instance. If a clone constructor has been
          * registered (see method BaseObject.registerCloneConstructor() above),
@@ -179,7 +245,7 @@ define('io.ox/office/tk/object/baseobject', ['io.ox/office/tk/utils'], function
 
             // invoke all registered clone handlers
             if (_.isArray(cloneGetters) && _.isArray(setters) && (cloneGetters.length === setters.length)) {
-                _(cloneGetters).each(function (getterFunc, index) {
+                _.each(cloneGetters, function (getterFunc, index) {
                     var setterFunc = setters[index];
                     setterFunc.call(cloned, getterFunc.call(this), this);
                 }, this);
@@ -190,28 +256,6 @@ define('io.ox/office/tk/object/baseobject', ['io.ox/office/tk/utils'], function
             return cloned;
         };
 
-        /**
-         * Registers a destructor callback function that will be invoked when
-         * the method BaseObject.destroy() has been called.
-         *
-         * @attention
-         *  Intended to be used by the internal implementations of sub classes.
-         *  DO NOT CALL from external code!
-         *
-         * @param {Function} destructor
-         *  A destructor callback function. The BaseObject.destroy() method
-         *  will invoke all registered destructor callbacks in reverse order of
-         *  registration. The callback will be invoked in the context of this
-         *  instance.
-         *
-         * @returns {BaseObject}
-         *  A reference to this instance.
-         */
-        this.registerDestructor = function (destructor) {
-            (destructors || (destructors = [])).unshift(destructor);
-            return this;
-        };
-
         /**
          * Destroys this object. Invokes all destructor callback functions that
          * have been registered for this instance in reverse order. Afterwards,
@@ -222,7 +266,7 @@ define('io.ox/office/tk/object/baseobject', ['io.ox/office/tk/utils'], function
         this.destroy = function () {
 
             // unregister all event listeners
-            _(listeners).each(function (data) {
+            _.each(listeners, function (data) {
                 // source object may have been destroyed already
                 if (data.source.off) { data.source.off(data.type, data.listener); }
             });
@@ -232,11 +276,11 @@ define('io.ox/office/tk/object/baseobject', ['io.ox/office/tk/utils'], function
             cloneConstructor = cloneGetters = cloneSetters = null;
 
             // invoke all destructor callbacks
-            _(destructors).each(function (destructor) { destructor.call(this); }, this);
+            _.each(destructors, function (destructor) { destructor.call(this); }, this);
             destructors = null;
 
             // delete all public members, to detect any misuse after destruction
-            _(this).each(function (member, name) { delete this[name]; }, this);
+            _.each(this, function (member, name) { delete this[name]; }, this);
             this.destroyed = true;
             this.uid = uid;
         };
@@ -267,7 +311,7 @@ define('io.ox/office/tk/object/baseobject', ['io.ox/office/tk/utils'], function
      */
     BaseObject.cloneArray = function (array) {
         var args = _.toArray(arguments).splice(1);
-        return _(array).map(function (entry) { return entry.clone.apply(entry, args); });
+        return _.map(array, function (entry) { return entry.clone.apply(entry, args); });
     };
 
     /**
@@ -288,7 +332,7 @@ define('io.ox/office/tk/object/baseobject', ['io.ox/office/tk/utils'], function
      */
     BaseObject.cloneMap = function (map) {
         var args = _.toArray(arguments).splice(1), result = {};
-        _(map).each(function (entry, key) { result[key] = entry.clone.apply(entry, args); });
+        _.each(map, function (entry, key) { result[key] = entry.clone.apply(entry, args); });
         return result;
     };
 
