@@ -25,19 +25,25 @@ define('io.ox/office/tk/popup/basepopup',
      * application window, and relative to an arbitrary DOM node.
      *
      * Instances of this class trigger the following events:
-     * - 'popup:beforeshow': Before the pop-up node will be shown.
-     * - 'popup:show': After the pop-up node has been shown.
-     * - 'popup:beforehide': Before the pop-up node will be hidden.
-     * - 'popup:hide': After the pop-up node has been hidden.
-     * - 'popup:beforelayout': Before the contents of the pop-up node, the
-     *      absolute position of the anchor node, or the browser window size
-     *      has changed, and the position and size of the pop-up node needs to
-     *      be adjusted.
-     * - 'popup:layout': After the contents of the pop-up node, the absolute
-     *      position of the anchor node or the browser window size has changed,
-     *      and the position and size of the pop-up node has been adjusted (in
+     * - 'popup:beforeshow'
+     *      Before the pop-up node will be shown.
+     * - 'popup:show'
+     *      After the pop-up node has been shown.
+     * - 'popup:beforehide'
+     *      Before the pop-up node will be hidden.
+     * - 'popup:hide'
+     *      After the pop-up node has been hidden.
+     * - 'popup:beforelayout'
+     *      Before the contents of the pop-up node, the absolute position of
+     *      the anchor node, or the browser window size has changed, and the
+     *      position and size of the pop-up node needs to be adjusted.
+     * - 'popup:layout'
+     *      After the contents of the pop-up node, the absolute position of the
+     *      anchor node or the browser window size has changed, and the
+     *      position and size of the pop-up node has been adjusted (in
      *      auto-layout mode).
-     * - 'popup:beforeclear': Before the pop-up node will be cleared.
+     * - 'popup:beforeclear'
+     *      Before the pop-up node will be cleared.
      *
      * @constructor
      *
@@ -141,17 +147,25 @@ define('io.ox/office/tk/popup/basepopup',
      *  @param {Number} [initOptions.anchorPadding=1]
      *      The distance between pop-up node and anchor position (or border
      *      position if specified, see option 'border' above).
+     *  @param {String} [initOptions.role]
+     *      The ARIA role attribute that will be set at the content DOM node.
+     *  @param {HTMLElement|jQuery} [initOptions.rootContainerNode]
+     *      The DOM container node for the pop-up node. By default, the pop-up
+     *      node will be appended into the <body> element of the page.
+     *      ATTENTION: Currently, it is not possible to use automatic layout
+     *      (see option 'autoLayout') when using a custom container node.
+     *      Manual positioning and sizing of the pop-up node is necessary.
      */
     function BasePopup(initOptions) {
 
         var // self reference (the Group instance)
             self = this,
 
-            // the root pop-up DOM node
-            rootNode = $('<div>').addClass('io-ox-office-main popup-container'),
+            // the root pop-up DOM node (attribute unselectable='on' is needed for IE to prevent focusing the node when clicking on a scroll bar)
+            rootNode = $('<div class="io-ox-office-main popup-container" unselectable="on">'),
 
             // the content node inside the root node used to calculate the effective node size
-            contentNode = $('<div>').addClass('popup-content').appendTo(rootNode),
+            contentNode = $('<div class="popup-content">').appendTo(rootNode),
 
             // automatic position and size of the pop-up node
             autoLayout = Utils.getBooleanOption(initOptions, 'autoLayout', true),
@@ -181,7 +195,13 @@ define('io.ox/office/tk/popup/basepopup',
             windowPadding = Utils.getIntegerOption(initOptions, 'windowPadding', 6, 0),
 
             // distance between pop-up node and anchor node
-            anchorPadding = Utils.getIntegerOption(initOptions, 'anchorPadding', 1, 0),
+            anchorPadding = Utils.getIntegerOption(initOptions, 'anchorPadding', 1),
+
+            // aria role
+            ariaRole = Utils.getStringOption(initOptions, 'role', ''),
+
+            // the container node for this pop-up node
+            rootContainerNode = $(Utils.getOption(initOptions, 'rootContainerNode', window.document.body)),
 
             // an interval timer to refresh the the pop-up node while it is visible
             refreshTimer = null,
@@ -202,38 +222,6 @@ define('io.ox/office/tk/popup/basepopup',
 
         // private methods ----------------------------------------------------
 
-        /**
-         * Returns the position and size of the visible area in the browser
-         * window, relative to the entire document page.
-         *
-         * @param {Number} [padding=0]
-         *  Additional padding size that will be removed from the resulting
-         *  visible area.
-         */
-        function getVisibleWindowArea(padding) {
-
-            var // the visible area of the browser window
-                visibleArea = { left: window.pageXOffset, top: window.pageYOffset },
-                // the size of the entire document page
-                pageSize = Utils.getPageSize();
-
-            padding = _.isNumber(padding) ? Math.max(padding, 0) : 0;
-            visibleArea.left += padding;
-            visibleArea.top += padding;
-
-            // The properties window.inner(Width|Height) for their own are not
-            // reliable when virtual keyboard on touch devices is visible. This
-            // 'hack' works for applications with fixed page size.
-            visibleArea.width = Math.max(Math.min(window.innerWidth, pageSize.width - keyboardWidth) - 2 * padding, 0);
-            visibleArea.height = Math.max(Math.min(window.innerHeight, pageSize.height - keyboardHeight) - 2 * padding, 0);
-
-            // add right/bottom distance to the result
-            visibleArea.right = pageSize.width - visibleArea.width - visibleArea.left;
-            visibleArea.bottom = pageSize.height - visibleArea.height - visibleArea.top;
-
-            return visibleArea;
-        }
-
         /**
          * Returns current position of the passed specifier as contained in the
          * constructor options 'anchor' or 'border', relative to the document
@@ -256,13 +244,15 @@ define('io.ox/office/tk/popup/basepopup',
             var // the size of the entire document page
                 pageSize = Utils.getPageSize(),
                 // the visible area of the browser window
-                visibleArea = windowArea ? getVisibleWindowArea() : null,
+                visibleArea = windowArea ? self.getVisibleWindowArea() : null,
                 // the resulting position
                 position = null;
 
             // resolve position of DOM element or jQuery object
             if ((positionValue instanceof HTMLElement) || (positionValue instanceof $)) {
-                position = Utils.getNodePositionInPage(positionValue);
+                if ($(positionValue).length > 0) {
+                    position = Utils.getNodePositionInPage(positionValue);
+                }
             } else if (_.isObject(positionValue)) {
                 position = _.clone(positionValue);
                 // add right/bottom distance to the rectangle
@@ -272,7 +262,7 @@ define('io.ox/office/tk/popup/basepopup',
 
             // restrict to visible area of the browser window
             if (position && visibleArea) {
-                _(['left', 'top', 'right', 'bottom']).each(function (border) {
+                _.each(['left', 'top', 'right', 'bottom'], function (border) {
                     position[border] -= visibleArea[border];
                 });
             }
@@ -370,6 +360,10 @@ define('io.ox/office/tk/popup/basepopup',
 
             // restricts the size of the content node
             function updateContentNodeSize(size, useMin) {
+
+                var // the exact node size (floating-points)
+                    contentSize = null;
+
                 switch (restrictSize) {
                 case 'width':
                     contentNode.css({ minWidth: useMin ? minSize : '', maxWidth: (minSize <= size.width) ? Math.min(maxSize, size.width) : '', minHeight: '', maxHeight: '' });
@@ -380,29 +374,28 @@ define('io.ox/office/tk/popup/basepopup',
                 default:
                     contentNode.css({ minWidth: '', maxWidth: '', minHeight: '', maxHeight: '' });
                 }
-                contentWidth = contentNode.outerWidth();
-                contentHeight = contentNode.outerHeight();
-                // Workaround for IE which reports node sizes rounded *down* to integers (e.g.: content width
-                // of a tooltip according to its text is is 123.45 pixels, but IE reports 123 pixels. When
-                // setting this width, the text will break into two text lines).
-                if (_.browser.IE) {
-                    contentWidth += 1;
-                    contentHeight += 1;
-                }
+
+                // Browsers usually report node sizes rounded to integers when using properties like
+                // 'offsetWidth', 'scrollWidth', etc. (example: content width of a tooltip according
+                // to its text contents is 123.45 pixels, but browsers report 123 pixels). When using
+                // this rounded width at the outer node, the text will break into two text lines.
+                contentSize = Utils.getExactNodeSize(contentNode);
+                contentWidth = Math.ceil(contentSize.width);
+                contentHeight = Math.ceil(contentSize.height);
             }
 
             // set all sizes to auto to get effective size of the pop-up node contents
-            rootNode.css({ left: 0, top: 0, right: '', bottom: '', width: 'auto', height: 'auto' });
+            rootNode.css({ left: 0, top: '', right: '', bottom: '-100%', width: 'auto', height: 'auto' });
             contentNode.css({ width: 'auto', height: 'auto' });
 
             // calculate the ratio of the menu node being visible at every side of the group
-            _(availableSizes).each(function (size) {
+            _.each(availableSizes, function (size) {
                 updateContentNodeSize(size, true);
                 size.ratio = ((contentWidth > 0) && (contentHeight > 0)) ? (Math.min(size.width / contentWidth, 1) * Math.min(size.height / contentHeight, 1)) : 0;
             });
 
             // prefer the best side that can take the menu node, in the given order
-            _(position.split(/\s+/)).any(function (pos) {
+            _.any(position.split(/\s+/), function (pos) {
                 var ratio = availableSizes[pos].ratio;
                 if (ratio >= 1) {
                     preferredPos = pos;
@@ -472,14 +465,15 @@ define('io.ox/office/tk/popup/basepopup',
                 }
             }
 
-//            Utils.info('BasePopup.setAutomaticPosition()');
-//            Utils.log(' anchor: ' + JSON.stringify(anchorPosition).replace(/"/g, '').replace(/,/g, ', '));
-//            Utils.log(' border: ' + JSON.stringify(borderPosition).replace(/"/g, '').replace(/,/g, ', '));
-//            Utils.log(' window: ' + JSON.stringify(getVisibleWindowArea()).replace(/"/g, '').replace(/,/g, ', '));
-//            Utils.log(' a-in-w: ' + JSON.stringify(getAnchorPositionInWindow()).replace(/"/g, '').replace(/,/g, ', '));
-//            Utils.log(' avail:  ' + JSON.stringify(availableSizes).replace(/"/g, '').replace(/,/g, ', '));
-//            Utils.log(' prefer: ' + preferredPos);
-//            Utils.log(' result: ' + JSON.stringify(cssProps).replace(/"/g, '').replace(/,/g, ', '));
+            if (Utils.LayoutLogger.isLoggingActive()) {
+                Utils.LayoutLogger.info('BasePopup.setAutomaticPosition()');
+                Utils.LayoutLogger.log(' anchor: ' + JSON.stringify(anchorPosition).replace(/"/g, '').replace(/,/g, ', '));
+                Utils.LayoutLogger.log(' border: ' + JSON.stringify(borderPosition).replace(/"/g, '').replace(/,/g, ', '));
+                Utils.LayoutLogger.log(' window: ' + JSON.stringify(self.getVisibleWindowArea()).replace(/"/g, '').replace(/,/g, ', '));
+                Utils.LayoutLogger.log(' avail:  ' + JSON.stringify(availableSizes).replace(/"/g, '').replace(/,/g, ', '));
+                Utils.LayoutLogger.log(' prefer: ' + preferredPos);
+                Utils.LayoutLogger.log(' result: ' + JSON.stringify(cssProps).replace(/"/g, '').replace(/,/g, ', '));
+            }
 
             // apply final CSS formatting
             self.setNodePosition(cssProps);
@@ -511,11 +505,18 @@ define('io.ox/office/tk/popup/basepopup',
                 if ($(lastActiveElement).is('input,textarea')) {
                     var pageX = window.pageXOffset, pageY = window.pageYOffset;
                     window.scrollTo(10000, 10000);
-                    keyboardWidth = window.pageXOffset;
-                    keyboardHeight = window.pageYOffset;
+                    // if the pageX- and pageY-Offset higher than the window-dimensions
+                    // there is no keyboard detected. Switch keyboard width/height to "0"
+                    keyboardWidth = (window.pageXOffset > window.innerWidth)?0:window.pageXOffset;
+                    keyboardHeight = (window.pageYOffset > window.innerHeight)?0:window.pageYOffset;
                     window.scrollTo(pageX, pageY);
                 } else {
                     keyboardWidth = keyboardHeight = 0;
+                    // FIX for IOS 8 (Safari)
+                    // IOS 8 let the body scroll up.
+                    // Don't know why, there is no keyboard or something else.
+                    // However, we scroll back to zero.
+                    window.scrollTo(0,0);
                 }
             }
 
@@ -542,7 +543,39 @@ define('io.ox/office/tk/popup/basepopup',
             refreshNode(false);
         }
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
+
+        /**
+         * Returns the position and size of the visible area in the browser
+         * window, relative to the entire document page.
+         *
+         * @param {Number} [padding=0]
+         *  Additional padding size that will be removed from the resulting
+         *  visible area.
+         */
+        this.getVisibleWindowArea = function (padding) {
+
+            var // the visible area of the browser window
+                visibleArea = { left: window.pageXOffset, top: window.pageYOffset },
+                // the size of the entire document page
+                pageSize = Utils.getPageSize();
+
+            padding = _.isNumber(padding) ? Math.max(padding, 0) : 0;
+            visibleArea.left += padding;
+            visibleArea.top += padding;
+
+            // The properties window.inner(Width|Height) for their own are not
+            // reliable when virtual keyboard on touch devices is visible. This
+            // 'hack' works for applications with fixed page size.
+            visibleArea.width = Math.max(Math.min(window.innerWidth, pageSize.width - keyboardWidth) - 2 * padding, 0);
+            visibleArea.height = Math.max(Math.min(window.innerHeight, pageSize.height - keyboardHeight) - 2 * padding, 0);
+
+            // add right/bottom distance to the result
+            visibleArea.right = pageSize.width - visibleArea.width - visibleArea.left;
+            visibleArea.bottom = pageSize.height - visibleArea.height - visibleArea.top;
+
+            return visibleArea;
+        };
 
         /**
          * Returns the available sizes for the pop-up node at every side of the
@@ -562,7 +595,7 @@ define('io.ox/office/tk/popup/basepopup',
             var // boundary position in the visible area of the browser window
                 borderPosition = resolveBorderPosition(true),
                 // visible area of the browser window, reduced by border padding
-                availableArea = getVisibleWindowArea(windowPadding),
+                availableArea = this.getVisibleWindowArea(windowPadding),
 
                 // vertical space above, below, left of, and right of the anchor node
                 totalPadding = windowPadding + anchorPadding,
@@ -602,7 +635,7 @@ define('io.ox/office/tk/popup/basepopup',
         this.setNodePosition = function (cssProps) {
 
             var // visible area of the browser window, reduced by border padding
-                availableArea = getVisibleWindowArea(windowPadding),
+                availableArea = this.getVisibleWindowArea(windowPadding),
                 // the width and height of the pop-up node (may be missing in passed properties)
                 nodeWidth = _.isNumber(cssProps.width) ? cssProps.width : rootNode.outerWidth(),
                 nodeHeight = _.isNumber(cssProps.height) ? cssProps.height : rootNode.outerHeight();
@@ -650,11 +683,17 @@ define('io.ox/office/tk/popup/basepopup',
 
             // show and position the pop-up node, notify all listeners
             this.trigger('popup:beforeshow');
-            $('body').append(rootNode);
+            rootContainerNode.append(rootNode);
             lastActiveElement = lastAnchorPosition = null;
             refreshNode(true);
             this.trigger('popup:show');
 
+            // 'show' event handles must not hide the menu
+            if (!this.isVisible()) {
+                Utils.error('BasePopup.show(): "show" event handler has hidden the pop-up menu');
+                return this;
+            }
+
             // start a timer that regularly refreshes the position of the pop-up node, and
             // hides the pop-up node automatically when the anchor node becomes inaccessible
             refreshTimer = window.setInterval(function () { refreshNode(false); }, Modernizr.touch ? 200 : 50);
@@ -746,7 +785,7 @@ define('io.ox/office/tk/popup/basepopup',
          *  A reference to this instance.
          */
         this.appendContentNodes = function () {
-            contentNode.append.apply(contentNode, _.toArray(arguments));
+            contentNode.append.apply(contentNode, arguments);
             this.refresh();
             return this;
         };
@@ -796,8 +835,8 @@ define('io.ox/office/tk/popup/basepopup',
          *  contains.
          *
          * @param {Object} [options]
-         *  A map of options to control the scroll action. Supports all options
-         *  that are supported by the method Utils.scrollToChildNode().
+         *  Optional parameters. Supports all options that are supported by the
+         *  method Utils.scrollToChildNode().
          *
          * @returns {BasePopup}
          *  A reference to this instance.
@@ -809,6 +848,17 @@ define('io.ox/office/tk/popup/basepopup',
             return this;
         };
 
+        /**
+         * Sets the anchor specification of this pop up.
+         * This function can be used if the anchor could not be specified during construction time.
+         *
+         * @param {Object|HTMLElement|jQuery|Function} anchor
+         * Identical to the initOptions.anchor specification of this class.
+         */
+        this.setAnchorSpec = function (anchor) {
+            anchorSpec = anchor || null;
+        };
+
         // initialization -----------------------------------------------------
 
         // additional CSS classes passed to constructor
@@ -820,16 +870,21 @@ define('io.ox/office/tk/popup/basepopup',
         });
 
         // disable dragging of controls (otherwise, it is possible to drag buttons and other controls around)
-        rootNode.on('contextmenu dragstart', Utils.BUTTON_SELECTOR + ',input,textarea,label', false);
+        rootNode.on('contextmenu drop dragstart dragenter dragexit dragover dragleave', false);
+
+        // log layout events
+        Utils.LayoutLogger.logEvent(this, 'popup:beforeshow popup:show popup:beforehide popup:hide popup:beforelayout popup:layout popup:beforeclear');
+
+        // ARIA role
+        if (ariaRole.length > 0) {
+            contentNode.attr('role', ariaRole);
+        }
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
             this.hide();
-
             rootNode.remove();
-
             self = rootNode = contentNode = anchorSpec = borderSpec = null;
-
         });
 
     } // class BasePopup
