@@ -14,8 +14,9 @@
 define('io.ox/office/tk/popup/listmenu',
     ['io.ox/office/tk/utils',
      'io.ox/office/tk/keycodes',
+     'io.ox/office/tk/forms',
      'io.ox/office/tk/popup/basemenu'
-    ], function (Utils, KeyCodes, BaseMenu) {
+    ], function (Utils, KeyCodes, Forms, BaseMenu) {
 
     'use strict';
 
@@ -54,11 +55,9 @@ define('io.ox/office/tk/popup/listmenu',
      *      (2) {jQuery} buttonNode
      *          The button element representing the new list item, as jQuery
      *          object.
-     *      (3) {String} sectionId
-     *          The identifier of the menu section containing the new button.
-     *      (4) {Any} value
+     *      (3) {Any} value
      *          The value of the new list item.
-     *      (5) {Object} [options]
+     *      (4) {Object} [options]
      *          The options passed to the method ListMenu.createItemNode().
      *
      * @constructor
@@ -69,10 +68,16 @@ define('io.ox/office/tk/popup/listmenu',
      *  Initial options for the item list menu element. Supports all options
      *  also supported by the base class BaseMenu. Additionally, the following
      *  options are supported:
-     *  @param {Function} [initOptions.equality=_.isEqual]
-     *      A comparison function that returns whether an arbitrary value
-     *      should be considered being equal to the value of a list item. If
-     *      omitted, uses _.isEqual() which compares arrays and objects deeply.
+     *  @param {Function} [initOptions.itemMatcher=_.isEqual]
+     *      A comparison function that returns whether a list item should be
+     *      selected, deselected, or kept unmodified according to a specific
+     *      value. Receives an arbitrary value as first parameter, and the
+     *      value of the current list item element as second parameter. The
+     *      function must return the Boolean value true to select the
+     *      respective list item element, the Boolean value false to deselect
+     *      the list item element, or any other value to leave the list item
+     *      element unmodified. If omitted, uses _.isEqual() which compares
+     *      arrays and objects deeply, and does not skip any list item.
      *  @param {String} [initOptions.itemDesign='list']
      *      The design mode of the list item elements:
      *      - 'list': All item elements will be stacked vertically, and will
@@ -81,17 +86,11 @@ define('io.ox/office/tk/popup/listmenu',
      *      - 'grid': All item elements will be arranged in a tabular layout
      *          per menu section. Highlighted items will be drawn with a thick
      *          border, leaving the background untouched.
-     *  @param {Boolean|Function} [initOptions.sortItems=false]
-     *      If set to true, the item elements will be sorted inside their menu
-     *      section by their label texts, ignoring case; items without text
-     *      label will be inserted in no special order. If set to a function,
-     *      it must return an ordinal value for each item element. The item
-     *      elements will be sorted according to these values, either by number
-     *      or lexicographically by strings. The function receives the button
-     *      element representing the drop-down menu item as jQuery object, and
-     *      will be called in the context of this item list instance. If
-     *      omitted or set to false, new item elements will be appended to the
-     *      section node.
+     *  @param {Boolean} [initOptions.sortItems=false]
+     *      If set to true, the list items will be sorted according to the sort
+     *      order that has been specified via the 'sortIndex' options of each
+     *      inserted list item (see method ListMenu.createItemNode() for
+     *      details).
      *  @param {Number} [options.gridColumns=10]
      *      The default number of columns in each menu section in 'grid' design
      *      mode (see option 'itemDesign' above). This value can be overridden
@@ -104,23 +103,87 @@ define('io.ox/office/tk/popup/listmenu',
             self = this,
 
             // comparator for list item values
-            equality = Utils.getFunctionOption(initOptions, 'equality', _.isEqual),
+            itemMatcher = Utils.getFunctionOption(initOptions, 'itemMatcher', _.isEqual),
 
             // the design mode
             design = Utils.getStringOption(initOptions, 'itemDesign', 'list'),
 
-            // functor used to sort the list items
-            sortFunctor = Utils.getFunctionOption(initOptions, 'sortItems'),
+            // whether the list items will be sorted
+            sorted = Utils.getBooleanOption(initOptions, 'sortItems', false),
 
             // default number of columns in each section node ('grid' design mode)
             defColumns = Utils.getIntegerOption(initOptions, 'gridColumns', 10, 1);
 
         // base constructor ---------------------------------------------------
 
-        BaseMenu.call(this, Utils.extendOptions({ preferFocusFilter: Utils.SELECTED_SELECTOR }, initOptions));
+        BaseMenu.call(this, Utils.extendOptions({ preferFocusFilter: Forms.SELECTED_SELECTOR }, initOptions));
 
         // private methods ----------------------------------------------------
 
+        /**
+         * Sorts all list items in the specified section node.
+         */
+        function sortSectionNode(sectionNode) {
+
+            var // all existing item buttons in the section, as plain array (for sorting)
+                buttonNodes = sectionNode.find(Forms.BUTTON_SELECTOR).get(),
+                // table cell nodes (grid design)
+                cellNodes = null;
+
+            // sort by fixed index, default to label text of the button
+            buttonNodes = _.sortBy(buttonNodes, function (btnNode) {
+                var index = $(btnNode).data('index');
+                return _.isNull(index) ? $(btnNode).text().toLowerCase() : index;
+            });
+
+            // insert button nodes depending on design mode
+            switch (design) {
+            case 'list':
+                sectionNode.append.apply(sectionNode, buttonNodes);
+                break;
+            case 'grid':
+                cellNodes = sectionNode.find('>table>tbody>tr>td');
+                _.each(buttonNodes, function (btnNode, index) {
+                    cellNodes.eq(index).append(btnNode);
+                });
+                break;
+            }
+        }
+
+        /**
+         * Sorts all list items in the specified section node debounced.
+         */
+        var sortSectionNodeDebounced = (function () {
+
+            var // all section nodes to be sorted
+                sectionNodes = $();
+
+            // sorts all registered sections debounced
+            var sortAllSectionNodes = _.debounce(function () {
+
+                // do not run debounced code in a destroyed instance
+                if (!self || self.destroyed) { return; }
+
+                // protect the focused list item
+                self.guardFocusedListItem(function () {
+                    // process all registered section nodes
+                    sectionNodes.each(function () { sortSectionNode($(this)); });
+                    // reset the cache of dirty section nodes
+                    sectionNodes = $();
+                });
+            }, 10);
+
+            // the actual sortSectionNodeDebounced() method to be returned from the local scope
+            function sortSectionNodeDebounced(sectionNode) {
+                // jQuery keeps the collection unique
+                sectionNodes = sectionNodes.add(sectionNode);
+                // sort all registered sections debounced
+                sortAllSectionNodes();
+            }
+
+            return sortSectionNodeDebounced;
+        }());
+
         /**
          * Handles 'keydown' events in the root node of the pop-up menu.
          */
@@ -137,7 +200,42 @@ define('io.ox/office/tk/popup/listmenu',
             }
         }
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
+
+        /**
+         * Invokes the passed callback function which wants to modify the
+         * contents of this drop-down list. The previously focused list item
+         * will be focused again afterwards.
+         *
+         * @param {Function} callback
+         *  The callback function that will be invoked while remembering the
+         *  list item currently focused.
+         *
+         * @returns {ListMenu}
+         *  A reference to this instance.
+         */
+        this.guardFocusedListItem = function (callback, context) {
+
+            var // the focused list item
+                focusNode = this.getItemNodes().filter(':focus'),
+                // value of the focused list item (bug 33737, 33747: preserve browser focus)
+                value = Forms.getButtonValue(focusNode.first());
+
+            // invoke passed callback function
+            callback.call(context);
+
+            // restore browser focus by looking up the original focus node, or
+            // by selecting a new focus node via the value of the old node
+            if (focusNode.length > 0) {
+                // first try to find the original node, resolve by value if this fails
+                if (this.getItemNodes().filter(focusNode).length === 0) {
+                    focusNode = this.findItemNodes(value);
+                }
+                focusNode.focus();
+            }
+
+            return this;
+        };
 
         /**
          * Adds a new section to this pop-up menu, if it does not exist yet.
@@ -146,13 +244,9 @@ define('io.ox/office/tk/popup/listmenu',
          *  The unique identifier of the menu section.
          *
          * @param {Object} [options]
-         *  A map of options to control the appearance of the section. The
-         *  following options are supported:
+         *  Optional parameters:
          *  @param {String} [options.label]
          *      If specified, a heading label will be created for the section.
-         *  @param {String} [options.classes]
-         *      Additional CSS classes that will be added to the section root
-         *      node.
          *  @param {Number} [options.gridColumns]
          *      The number of columns in the menu section, if this menu is in
          *      'grid' design mode (see option 'itemDesign' passed to the
@@ -165,24 +259,22 @@ define('io.ox/office/tk/popup/listmenu',
         this.createSectionNode = function (sectionId, options) {
 
             var // try to find an existing section node
-                sectionNode = this.getSectionNode(sectionId),
-                // the header label for the section
-                label = null;
+                sectionNode = this.getSectionNode(sectionId);
 
             if (sectionNode.length === 0) {
 
-                // create the section root node
-                sectionNode = Utils.createContainerNode(SECTION_CLASS + ' group')
-                    .attr('data-section', sectionId)
-                    .addClass(Utils.getStringOption(options, 'classes', ''))
-                    .data('grid-columns', Utils.getIntegerOption(options, 'gridColumns', defColumns, 1));
-
-                // set the label text as attribute (added as :before pseudo element by CSS)
-                label = Utils.getStringOption(options, 'label', '');
-                if (label.length > 0) {
-                    sectionNode.attr('data-section-label', label);
+                // create the header label mark-up
+                var headerMarkup = Utils.getStringOption(options, 'label', '');
+                if (headerMarkup.length > 0) {
+                    headerMarkup = '<div class="header-label" aria-hidden="true">' + Utils.escapeHTML(headerMarkup) + '</div>';
                 }
 
+                // create the section root node
+                sectionNode = $('<div class="' + SECTION_CLASS + ' group ' + Forms.HIDDEN_CLASS + '" data-section="' + sectionId + '">' + headerMarkup + '</div>');
+
+                // add the number of grid columns as jQuery data attribute
+                sectionNode.data('grid-columns', Utils.getIntegerOption(options, 'gridColumns', defColumns, 1));
+
                 // insert the new section into the drop-down menu, notify listeners
                 this.appendContentNodes(sectionNode);
                 this.trigger('create:section', sectionNode, sectionId, options);
@@ -219,142 +311,159 @@ define('io.ox/office/tk/popup/listmenu',
          * (see the sort options passed to the constructor), the item element
          * will be inserted according to these settings.
          *
-         * @param {String} sectionId
-         *  The unique identifier of the menu section the new list item will be
-         *  inserted into. If a section with this identifier does not exist
-         *  yet, it will be created without a header label.
-         *
          * @param {Any} value
-         *  The unique value associated to the list item. MUST NOT be null or
-         *  undefined.
+         *  The unique value associated to the list item. MUST NOT be null.
          *
          * @param {Object} [options]
-         *  A map of options to control the properties of the button DOM node
-         *  representing the new list item. See method Utils.createButton() for
-         *  details, except the option 'value' which will be set to the 'value'
-         *  parameter passed to this method.
+         *  Optional parameters. Supports all formatting options for button
+         *  elements supported by the method Forms.createButtonMarkup().
+         *  Additionally, the following options are supported:
+         *  @param {String} [options.section='']
+         *      The unique identifier of the menu section the new list item
+         *      will be inserted into. If a section with this identifier does
+         *      not exist yet, it will be created without a header label. Can
+         *      be omitted for example for simple list menus without sections.
+         *  @param {String} [options.dataValue]
+         *      A string that will be inserted into the 'data-value' attribute
+         *      of the button node. If omitted, the JSON string representation
+         *      of the 'value' parameter will be used instead (all double-quote
+         *      characters will be removed from the string though), unless the
+         *      value is a function.
          *
          * @returns {jQuery}
          *  The button element representing the new list item, as jQuery
          *  object.
          */
-        this.createItemNode = function (sectionId, value, options) {
+        this.createItemNode = function (value, options) {
 
             var // the root node of the menu section
-                sectionNode = this.createSectionNode(sectionId),
-                // all existing item buttons in the current section
-                buttonNodes = sectionNode.find(Utils.BUTTON_SELECTOR),
+                sectionNode = this.createSectionNode(Utils.getStringOption(options, 'section', '')),
                 // create the button element representing the item
-                buttonNode = Utils.createButton(Utils.extendOptions(options, { value: value })).addClass(Utils.FOCUSABLE_CLASS),
-                // insertion index for sorted items
-                sortIndex = -1;
+                buttonNode = $(Forms.createButtonMarkup(options));
 
-            // inserts the button node in a list
-            function insertListItemNode() {
-                if ((0 <= sortIndex) && (sortIndex < buttonNodes.length)) {
-                    buttonNode.insertBefore(buttonNodes[sortIndex]);
-                } else {
-                    sectionNode.append(buttonNode);
-                }
-            }
+            // make section node visible (initially hidden)
+            Forms.showNodes(sectionNode, true);
 
-            // inserts the button node in a table
-            function insertGridItemNode() {
-
-                var // the number of columns in the table
-                    columns = sectionNode.data('grid-columns'),
-                    // the table element containing the grid items
-                    tableNode = null,
-                    // the last table row
-                    rowNode = null;
-
-                // create a new table element for the button if required
-                tableNode = sectionNode.find('>table');
-                if (tableNode.length === 0) {
-                    tableNode = $('<table>').attr('role', 'grid').appendTo(sectionNode);
-                }
-
-                // create a new table row element for the button if required
-                rowNode = tableNode.find('>tbody>tr').last();
-                if ((rowNode.length === 0) || (rowNode.children().last().children().length > 0)) {
-                    // IE9 does not allow to write to the 'innerHTML' property for <TR> nodes :-O
-                    rowNode = $('<tr role="row">' + Utils.repeatString('<td role="gridcell"></td>', columns) + '</tr>').appendTo(tableNode);
-                }
-
-                // insert the new button into the array, and reinsert all buttons into the table
-                buttonNodes = buttonNodes.get();
-                buttonNodes.splice(sortIndex, 0, buttonNode);
-                tableNode.find('>tbody>tr>td').each(function (index) {
-                    $(this).append(buttonNodes[index]);
-                });
-            }
-
-            // find insertion index for sorted items
-            if (_.isFunction(sortFunctor)) {
-                sortIndex = _.chain(buttonNodes.get())
-                    // convert array of button elements to strings returned by sort functor
-                    .map(function (node) { return sortFunctor.call(self, $(node)); })
-                    // calculate the insertion index of the new list item
-                    .sortedIndex(sortFunctor.call(this, buttonNode))
-                    // exit the call chain, returns result of sortedIndex()
-                    .value();
-            } else {
-                // else: append to existing items
-                sortIndex = buttonNodes.length;
-            }
+            // initialize button node
+            Forms.setButtonValue(buttonNode, value, options);
+            buttonNode.data('index', Utils.getOption(options, 'sortIndex', null));
+            buttonNode.attr('role', 'option');
 
             // insert the new button according to the design mode
             switch (design) {
             case 'list':
-                insertListItemNode();
+                sectionNode.append(buttonNode);
                 break;
             case 'grid':
-                insertGridItemNode();
+                // create a new table element for the button if required
+                var tableNode = sectionNode.find('>table');
+                if (tableNode.length === 0) {
+                    tableNode = $('<table>').attr({ role: 'grid', 'aria-label': Utils.getStringOption(initOptions, 'tooltip', null, true) }).appendTo(sectionNode);
+                }
+                // create a new table row element for the button if required
+                var rowNode = tableNode.find('>tbody>tr').last();
+                if ((rowNode.length === 0) || (rowNode.children().last().children().length > 0)) {
+                    // IE9 does not allow to write to the 'innerHTML' property for <TR> nodes :-O
+                    rowNode = $('<tr role="row">' + Utils.repeatString('<td></td>', sectionNode.data('grid-columns')) + '</tr>').appendTo(tableNode);
+                }
+                // insert the button into the first empty cell
+                var cellNode = rowNode.find('>td:empty').first();
+                cellNode.attr({ role: 'gridcell', 'aria-label': buttonNode.attr('aria-label') || '' }).append(buttonNode);
+                break;
             }
 
-            // notify listeners
-            this.trigger('create:item', buttonNode, sectionId, value, options);
+            // sort the button nodes in the section
+            if (sorted) { sortSectionNodeDebounced(sectionNode); }
 
+            // notify listeners
+            this.trigger('create:item', buttonNode, value, options);
             return buttonNode;
         };
 
+        /**
+         * Removes an item element from this list.
+         *
+         * @param {Any} value
+         *  The unique value associated to the list item. MUST NOT be null.
+         *
+         * @returns {ListMenu}
+         *  A reference to this instance.
+         */
+        this.deleteItemNodes = function (value) {
+
+            var // all item buttons to be deleted
+                buttonNodes = this.findItemNodes(value),
+                // all modified section nodes
+                sectionNodes = $();
+
+            // collect modified section nodes, notify listeners
+            buttonNodes.each(function () {
+                var buttonNode = $(this);
+                sectionNodes = sectionNodes.add(buttonNode.closest('.' + SECTION_CLASS));
+                self.trigger('delete:item', buttonNode, value);
+            });
+
+            // remove button nodes from DOM
+            buttonNodes.remove();
+
+            // update tables in grid design mode, hide empty sections
+            sectionNodes.each(function () {
+
+                var // current section node
+                    sectionNode = $(this);
+
+                // all existing buttons in the section
+                buttonNodes = sectionNode.find(Forms.BUTTON_SELECTOR);
+
+                // reinsert buttons in grid design mode, update or remove table
+                if (design === 'grid') {
+                    var tableNode = sectionNode.find('>table');
+                    if (buttonNodes.length === 0) {
+                        tableNode.remove();
+                    } else {
+                        // redistribute all buttons
+                        var cellNodes = tableNode.find('>tbody>tr>td');
+                        buttonNodes.each(function (index) { cellNodes.eq(index).append(this); });
+                        // delete last empty row
+                        var rowNode = cellNodes.last().parent();
+                        if (rowNode.find(Forms.BUTTON_SELECTOR).length === 0) {
+                            rowNode.remove();
+                        }
+                    }
+                }
+
+                // hide empty sections
+                Forms.showNodes(sectionNode, buttonNodes.length > 0);
+            });
+
+            return this;
+        };
+
         /**
          * Returns all button elements representing the menu items of this
          * list.
          *
-         * @param {String} [sectionId]
-         *  If specified, the result will contain the item elements of that
-         *  menu section only.
-         *
          * @returns {jQuery}
          *  All button elements representing the menu items in this item list,
-         *  or the buttons of a single menu section, as jQuery collection.
+         *  as jQuery collection.
          */
-        this.getItemNodes = function (sectionId) {
-            var node = _.isString(sectionId) ? this.getSectionNode(sectionId) : this.getNode();
-            return node.find(Utils.BUTTON_SELECTOR);
+        this.getItemNodes = function () {
+            return this.getNode().find(Forms.BUTTON_SELECTOR);
         };
 
         /**
          * Returns the button elements representing the list item with the
-         * specified value or selected by the specified predicate function.
+         * specified value.
          *
-         * @param {Any|Function} selector
-         *  The value whose list item buttons will be returned, or a predicate
-         *  function that receives the button node as jQuery object in the
-         *  first parameter, and the list item value in the second parameter,
-         *  and that must return a Boolean value whether to include the list
-         *  item in the result.
+         * @param {Any} value
+         *  The value whose list item buttons will be returned.
          *
          * @returns {jQuery}
          *  The button elements representing the list items with the specified
          *  value, as jQuery collection.
          */
-        this.findItemNodes = function (selector) {
-            return (_.isUndefined(selector) || _.isNull(selector)) ? $() : this.getItemNodes().filter(function () {
-                var value = Utils.getControlValue($(this));
-                return _.isFunction(selector) ? selector.call(self, $(this), value) : equality(selector, value);
-            });
+        this.findItemNodes = function (value) {
+            return Forms.filterButtonNodes(this.getItemNodes(), value, itemMatcher);
         };
 
         /**
@@ -366,7 +475,7 @@ define('io.ox/office/tk/popup/listmenu',
          *  collection.
          */
         this.getSelectedItemNodes = function () {
-            return Utils.getSelectedButtons(this.getItemNodes());
+            return Forms.filterSelectedNodes(this.getItemNodes());
         };
 
         /**
@@ -377,8 +486,7 @@ define('io.ox/office/tk/popup/listmenu',
          *  The zero-based index of the list item to be selected.
          *
          * @param {Object} [options]
-         *  A map with additional options controlling the behavior of this
-         *  method. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.scroll=false]
          *      If set to true, the menu will scroll to the selected item node
          *      automatically.
@@ -401,8 +509,7 @@ define('io.ox/office/tk/popup/listmenu',
          *  jQuery collection that may contain multiple list items.
          *
          * @param {Object} [options]
-         *  A map with additional options controlling the behavior of this
-         *  method. The following options are supported:
+         *  Optional parameters:
          *  @param {Boolean} [options.scroll=false]
          *      If set to true, the menu will scroll to the selected item node
          *      automatically.
@@ -413,8 +520,8 @@ define('io.ox/office/tk/popup/listmenu',
          */
         this.selectItemNode = function (buttonNode, options) {
             buttonNode = $(buttonNode);
-            Utils.selectButtons(this.getItemNodes(), false);
-            Utils.selectButtons(buttonNode, true);
+            Forms.selectButtonNodes(this.getItemNodes(), false);
+            Forms.selectButtonNodes(buttonNode, true);
             if ((buttonNode.length > 0) && Utils.getBooleanOption(options, 'scroll', false)) {
                 this.scrollToChildNode(buttonNode);
             }
@@ -423,29 +530,20 @@ define('io.ox/office/tk/popup/listmenu',
 
         /**
          * Selects the button elements representing the list item with the
-         * specified value or selected by the specified predicate function, and
-         * deselects all other list items.
+         * passed value, and deselects all other list items.
          *
-         * @param {Any|Function} selector
-         *  The value whose list item buttons will be selected, or a predicate
-         *  function that receives the button node as jQuery object in the
-         *  first parameter, and the list item value in the second parameter,
-         *  and that must return a Boolean value whether to select the list
-         *  item.
-         *
-         * @param {Object} [options]
-         *  A map with additional options controlling the behavior of this
-         *  method. The following options are supported:
-         *  @param {Boolean} [options.scroll=false]
-         *      If set to true, the menu will scroll to the selected item node
-         *      automatically.
+         * @param {Any} value
+         *  The value whose list item buttons will be selected. List items will
+         *  be selected or deselected according to the item matcher callback
+         *  function passed to the constructor (option 'itemMatcher').
          *
          * @returns {jQuery}
          *  The button elements representing the selected list items with the
          *  specified value, as jQuery collection.
          */
-        this.findAndSelectItemNodes = function (selector, options) {
-            return this.selectItemNode(this.findItemNodes(selector), options);
+        this.selectMatchingItemNodes = function (value) {
+            Forms.selectMatchingButtonNodes(this.getItemNodes(), value, itemMatcher);
+            return this.getSelectedItemNodes();
         };
 
         /**
@@ -457,19 +555,12 @@ define('io.ox/office/tk/popup/listmenu',
          *
          * @param {HTMLElement|jQuery|Null} buttonNode
          *  The button element representing the source list item the cursor
-         *  movement originates from. MUST be a button element representing a
-         *  list item of this list menu.
-         *
-         * @param {Object} [options]
-         *  A map with additional options controlling the behavior of this
-         *  method. The following options are supported:
-         *  @param {Boolean} [options.selectEmpty=false]
-         *      If set to true, and the passed button node is invalid (null, or
-         *      an empty jQuery collection), selects the first or last list
-         *      item instead. All keys in 'down' orientation (CURSOR_DOWN,
-         *      PAGE_DOWN) and the HOME key will select the first list item,
-         *      all keys in 'up' orientation (CURSOR_UP, PAGE_UP) and the END
-         *      key will select the last list item.
+         *  movement originates from. If omitted, the first or last list item
+         *  will be selected instead according to the key code. All keys in
+         *  'down' orientation (CURSOR_DOWN, PAGE_DOWN) and the HOME key will
+         *  select the first list item, all keys in 'up' orientation
+         *  (CURSOR_UP, PAGE_UP) and the END key will select the last list
+         *  item.
          *
          * @returns {jQuery}
          *  The target list item to be focused or selected, as jQuery object.
@@ -477,24 +568,26 @@ define('io.ox/office/tk/popup/listmenu',
          *  cursor movement, or the passed button node is invalid, an empty
          *  jQuery collection will be returned instead.
          */
-        this.getItemNodeForKeyEvent = function (keyEvent, buttonNode, options) {
+        this.getItemNodeForKeyEvent = function (keyEvent, buttonNode) {
+
+            var // all visible button nodes
+                buttonNodes = Forms.filterVisibleNodes(self.getItemNodes());
 
             // returns the number of list items currently visible
             function getListPageSize() {
-                var itemHeight = self.getItemNodes().filter(':visible').first().outerHeight();
+                var itemHeight = buttonNodes.first().outerHeight();
                 return (itemHeight > 0) ? Math.max(1, Math.floor(self.getNode()[0].clientHeight / itemHeight) - 1) : 1;
             }
 
             // returns the previous or next item node according to the passed distance
             function getNextItemNode(diff, wrap) {
 
-                var // all button nodes in this list
-                    buttonNodes = self.getItemNodes(),
-                    // index of the focused button
+                var // index of the focused button
                     index = buttonNodes.index(buttonNode);
 
-                if (index < 0) { return $(); }
-                if (wrap) {
+                if (index < 0) {
+                    index = (diff < 0) ? (buttonNodes.length - 1) : 0;
+                } else if (wrap) {
                     index += (diff % buttonNodes.length) + buttonNodes.length;
                     index %= buttonNodes.length;
                 } else {
@@ -528,7 +621,7 @@ define('io.ox/office/tk/popup/listmenu',
                     // the table row node containing the focused button
                     rowNode = cellNode.parent(),
                     // all button elements in the focused row
-                    buttonNodes = rowNode.children().children(),
+                    rowButtonNodes = rowNode.children().children(),
                     // all row nodes in all tables
                     rowNodes = self.getSectionNodes().find('>table>tbody>tr'),
                     // the column index of the cell in the row
@@ -536,16 +629,18 @@ define('io.ox/office/tk/popup/listmenu',
                     // the row index of the cell in the table
                     rowIndex = rowNodes.index(rowNode),
                     // the column index of the cell in the table
-                    colRatio = (colIndex >= 0) ? (colIndex / buttonNodes.length) : 0;
+                    colRatio = (colIndex >= 0) ? (colIndex / rowButtonNodes.length) : 0;
 
-                if ((rowNodes.length === 0) || (rowIndex < 0)) { return $(); }
+                if ((rowNodes.length === 0) || (rowIndex < 0)) {
+                    return (diff < 0) ? buttonNodes.last() : buttonNodes.first();
+                }
 
                 // move focus to the specified row, try to keep column position constant inside row
                 rowIndex += (diff % rowNodes.length) + rowNodes.length;
                 rowNode = rowNodes.eq(rowIndex % rowNodes.length);
-                buttonNodes = rowNode.children().children();
-                colIndex = Math.min(Math.floor(colRatio * rowNode.children().length), buttonNodes.length - 1);
-                return buttonNodes.eq(colIndex);
+                rowButtonNodes = rowNode.children().children();
+                colIndex = Math.min(Math.floor(colRatio * rowNode.children().length), rowButtonNodes.length - 1);
+                return rowButtonNodes.eq(colIndex);
             }
 
             // returns the target item node for 'grid' design
@@ -568,32 +663,12 @@ define('io.ox/office/tk/popup/listmenu',
             // convert to jQuery collection
             buttonNode = $(buttonNode);
 
-            // passed button node may be invalid (null, or empty jQuery collection)
-            if (buttonNode.length === 0) {
-
-                // fall-back to first/last list item if specified
-                if (Utils.getBooleanOption(options, 'selectEmpty', false)) {
-                    switch (keyEvent.keyCode) {
-                    case KeyCodes.HOME:
-                    case KeyCodes.DOWN_ARROW:
-                    case KeyCodes.PAGE_DOWN:
-                        return this.getItemNodes().first();
-                    case KeyCodes.END:
-                    case KeyCodes.UP_ARROW:
-                    case KeyCodes.PAGE_UP:
-                        return this.getItemNodes().last();
-                    }
-                }
-
-                return $();
-            }
-
             // generic key shortcuts
             switch (keyEvent.keyCode) {
             case KeyCodes.HOME:
-                return this.getItemNodes().first();
+                return buttonNodes.first();
             case KeyCodes.END:
-                return this.getItemNodes().last();
+                return buttonNodes.last();
             }
 
             // special shortcuts dependent on design mode
@@ -616,9 +691,9 @@ define('io.ox/office/tk/popup/listmenu',
          *  first node it contains.
          *
          * @param {Object} [options]
-         *  A map of options to control the scroll action. Supports all options
-         *  that are supported by the method Utils.scrollToChildNode(). The
-         *  option 'padding' is defaulted to the value 5.
+         *  Optional parameters. Supports all options that are supported by the
+         *  method Utils.scrollToChildNode(). The option 'padding' is defaulted
+         *  to the value 5.
          *
          * @returns {ListMenu}
          *  A reference to this instance.
@@ -633,21 +708,13 @@ define('io.ox/office/tk/popup/listmenu',
         // initialization -----------------------------------------------------
 
         // add CSS marker for the item design
-        this.getNode().addClass('list-menu').attr('data-design', design);
+        this.getNode().addClass('list-menu').attr({ 'data-design': design, role: 'listbox' });
 
         // register event handlers
         this.getNode().on('keydown', keyDownHandler);
 
         // convert ENTER, SPACE, and TAB keys in the menu to click events
-        Utils.setButtonKeyHandler(this.getNode(), { tab: true });
-
-        // default sort functor: sort by button label text, case insensitive
-        if (Utils.getBooleanOption(initOptions, 'sortItems', false)) {
-            sortFunctor = function (buttonNode) {
-                var label = Utils.getControlLabel(buttonNode);
-                return _.isString(label) ? label.toLowerCase() : '';
-            };
-        }
+        Forms.setButtonKeyHandler(this.getNode(), { tab: true });
 
         // special handling for tables in 'grid' design
         if (design === 'grid') {
@@ -657,12 +724,9 @@ define('io.ox/office/tk/popup/listmenu',
             });
         }
 
+        // destroy all class members on destruction
         this.registerDestructor(function () {
-            //if (initOptions) {
-            //    delete initOptions.equality;
-            //    delete initOptions.sortItems;
-            //}
-            equality = sortFunctor = initOptions = null;
+            initOptions = self = itemMatcher = null;
         });
 
     } // class ListMenu
